<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:xs="http://www.w3.org/2001/XMLSchema" 
	xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" 
	xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
	xmlns:els="http://www.lefebvre-sarrut.eu/ns/els" 
	xmlns:dalloz="http://www.lefebvre-sarrut.eu/ns/dalloz" 
	xmlns:h="http://www.w3.org/1999/xhtml" 
	xmlns:functx="http://www.functx.com"
	xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
	xmlns:xfSAS="http://www.lefebvre-sarrut.eu/ns/xmlfirst/sas" 
	exclude-result-prefixes="#all" 
	version="3.0">
  
  <xsl:import href="../../xf-sas-common.xsl"/>
  
  <xsl:param name="creationUser" select="''" as="xs:string"/>
  <xsl:param name="LOG.URI" required="no" select="resolve-uri('log', base-uri(/))" as="xs:string"/>
  
  <xsl:function name="dalloz:metaOldId" as="element(xf:meta)">
    <xsl:param name="element" as="element()"/>
    <meta code="DZ_META_idOld">
      <xsl:choose>
        <xsl:when test="$element/@ID_LOGIC">
          <value><xsl:value-of select="$element/@ID_LOGIC"/></value>
        </xsl:when>
        <xsl:when test="$element/@ID_OLD">
          <value><xsl:value-of select="$element/@ID_OLD"/></value>
        </xsl:when>
        <xsl:when test="$element/@id">
          <value><xsl:value-of select="$element/@id"/></value>
        </xsl:when>
        <xsl:otherwise>
          <value><xsl:value-of select="dalloz:dallozId($element)"/></value>
        </xsl:otherwise>
      </xsl:choose>
    </meta>
  </xsl:function>
   
  <xsl:function name="dalloz:dallozId" as="xs:string">
    <xsl:param name="context" as="element()"/>
    <xsl:choose>
      <!-- prendre l'attribut ID_ALT s'il est présent -->
      <xsl:when test="$context/@ID_ALT">
        <xsl:value-of select="$context/@ID_ALT"/>
      </xsl:when>
      <!-- prendre l'attribut ID s'il est présent -->
      <xsl:when test="$context/@id">
        <xsl:value-of select="$context/@id"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="localName" select="local-name($context)" as="xs:string"/>
        <xsl:choose>
          <!-- if root element (but this should not happen) take its name -->
          <xsl:when test="$context/.. instance of document-node()">
            <xsl:value-of select="$localName"/>
            <xsl:message>[WARNING] No @ID en root element <xsl:value-of select="$localName"/> !</xsl:message>
          </xsl:when>
          <xsl:otherwise>
            <!-- else concatenate codes:dallozId(parent) with its name and ".type-rank" if not unique -->
            <xsl:variable name="nbBefore" select = "count($context/preceding-sibling::*[local-name() = $localName])" as="xs:integer"/>
            <xsl:variable name="nbAfter" select = "count($context/following-sibling::*[local-name() = $localName])" as="xs:integer"/>
            <xsl:choose>
              <xsl:when test="$nbBefore = 0 and $nbAfter = 0">
                <xsl:value-of select="concat(dalloz:dallozId($context/..), '/', $localName)"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="concat(dalloz:dallozId($context/..), '/', $localName, '.', string(1 + $nbBefore))"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="dalloz:ucfirst">
    <xsl:param name="in"/>
    <xsl:copy-of select="concat(upper-case(substring($in, 1, 1)), lower-case(substring($in, 2)))"/>
  </xsl:function>
  
  <xsl:function name="dalloz:historicalId" as="xs:string">
    <xsl:param name="context" as="element()"/>
    <!--<xsl:value-of select="xfSAS:makeHistoricIdAttribute($context, dalloz:dallozId($context))"/>-->
    <xsl:value-of select="xf:makeHistoricalIdAttributeValue($context, dalloz:dallozId($context), $xfSAS:company)"/>
  </xsl:function>
  
  <xsl:function name="dalloz:historicalId" as="xs:string">
    <xsl:param name="context" as="element()"/>
    <xsl:param name="suffix" as="xs:string"/>
    <!--<xsl:value-of select="xfSAS:makeHistoricIdAttribute($context, concat(dalloz:dallozId($context), '/', $suffix))"/>-->
    <xsl:value-of select="xf:makeHistoricalIdAttributeValue($context, concat(dalloz:dallozId($context), '/', $suffix), $xfSAS:company)"/>
  </xsl:function>
  
  <xsl:template name="xfSAS:addAttributeCreationUser" as="attribute(creationUser)*"> 
    <xsl:choose>
      <xsl:when test="$creationUser != ''">
        <xsl:attribute name="creationUser" select="$creationUser"/>
      </xsl:when>
      <xsl:when test="environment-variable('USERNAME')">
        <xsl:attribute name="creationUser" select="environment-variable('USERNAME')"/>
      </xsl:when>
      <xsl:when test="environment-variable('USER')">
        <xsl:attribute name="creationUser" select="environment-variable('USER')"/>
      </xsl:when>
      <xsl:when test="environment-variable('LOGNAME')">
        <xsl:attribute name="creationUser" select="environment-variable('LOGNAME')"/>
      </xsl:when>
      <xsl:otherwise/>
    </xsl:choose>
  </xsl:template>
  
  <xsl:function name="dalloz:dataDate-to-dateTime" as="xs:dateTime">
    <!-- en sortie: yyyy-mm-dd'T'hh:mm:ss'Z' e.g. 2017-05-09T00:00:00Z-->
    <xsl:param name="contentDate" as="xs:string"/>
    <xsl:variable name="date" select="translate($contentDate, '/', '-')"/>
    <xsl:choose>
      <!-- &#x2013; : n dash et &#x2014; m dash. On en trouve dans les sources en lieu et place de - 
           Merci le copier-coller word... 
      -->
      <xsl:when test="matches($date, '^(\d\d\d\d)((-|&#x2013;|&#x2014;)(\d\d?)((-|&#x2013;|&#x2014;)(\d\d?))?)?$')">
        <xsl:analyze-string select="$date" regex="^(\d\d\d\d)((-|&#x2013;|&#x2014;)(\d\d?)((-|&#x2013;|&#x2014;)(\d\d?))?)?$">
          <xsl:matching-substring>
            <xsl:variable name="year" select="regex-group(1)"/>
            <xsl:variable name="month" select="functx:pad-integer-to-length(functx:if-empty(regex-group(4), 1), 2)"/>
            <xsl:variable name="day"   select="functx:pad-integer-to-length(functx:if-empty(regex-group(7), 1), 2)"/>
            <xsl:value-of select="concat($year, '-', $month, '-', $day, 'T00:00:00Z')"/>
          </xsl:matching-substring>
        </xsl:analyze-string>
      </xsl:when>
      <!-- format spécifique aux encyclopédies Dalloz -->
      <xsl:when test="matches($date, '^(janvier|février|mars|avril|mai|juin|juillet|août|septembre|octobre|novembre|décembre)\s+(\d\d\d\d)$')">
        <xsl:analyze-string select="$date" regex="^(janvier|février|mars|avril|mai|juin|juillet|août|septembre|octobre|novembre|décembre)\s+(\d\d\d\d)$">
          <xsl:matching-substring>
            <xsl:variable name="year" select="regex-group(2)"/>
            <xsl:variable name="month" select="functx:pad-integer-to-length(index-of(('janvier','février','mars','avril','mai','juin','juillet','août','septembre','octobre','novembre','décembre'), regex-group(1)), 2)"/>
            <xsl:variable name="day"   select="'01'"/>
            <xsl:value-of select="concat($year, '-', $month, '-', $day, 'T00:00:00Z')"/>
          </xsl:matching-substring>
        </xsl:analyze-string>
      </xsl:when>
      <!-- pour les actualités Dalloz -->
      <xsl:when test="matches($date, '^(\d\d\d\d)(\d\d)(\d\d)$')">
        <xsl:analyze-string select="$date" regex="^(\d\d\d\d)(\d\d)(\d\d)$">
          <xsl:matching-substring>
            <xsl:variable name="year" select="regex-group(1)"/>
            <xsl:variable name="month" select="regex-group(2)"/>
            <xsl:variable name="day"   select="regex-group(3)"/>
            <xsl:value-of select="concat($year, '-', $month, '-', $day, 'T00:00:00Z')"/>
          </xsl:matching-substring>
        </xsl:analyze-string>
      </xsl:when>
      <xsl:otherwise>
        <xsl:message terminate="yes">[ERROR] format de date incorrect pour la date <xsl:value-of select="$contentDate"/></xsl:message>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:template name="dalloz:logMessage">
    <xsl:param name="log.source" as="xs:string"/>
    <xsl:param name="log.message" as="xs:string"/>
    <xsl:param name="log.level" as="xs:string"/>
    <xsl:call-template name="els:log">
      <xsl:with-param name="xsltName" select="$log.source"/>
      <xsl:with-param name="level" select="$log.level"/>
      <xsl:with-param name="code" select="$els:log.level.code"/>
      <xsl:with-param name="alert" select="if ($xfSAS:debug) then (true()) else (false())"/>
      <xsl:with-param name="markup" select="false()"/>
      <xsl:with-param name="description" select="$log.message"/>
    </xsl:call-template>
    <xsl:if test="$xfSAS:debug and ($log.level = 'fatal')">
      <xsl:message terminate="yes">Transformation failed!</xsl:message>
    </xsl:if>
  </xsl:template> 
  
  <xsl:template name="dalloz:logDocument">
    <xsl:param name="log.source" as="xs:string"/>
    <xsl:param name="log.document" as="document-node()"/>
    <xsl:param name="log.filename" as="xs:string"/>
    <xsl:if test="$xfSAS:debug">
      <xsl:variable name="log.uri" select="resolve-uri($log.filename, concat($LOG.URI,'/'))" as="xs:anyURI"/>
      <xsl:call-template name="dalloz:logMessage">
        <xsl:with-param name="log.source" select="$log.source"/>
        <xsl:with-param name="log.message"> writing <xsl:value-of select="$log.uri"/></xsl:with-param>
        <xsl:with-param name="log.level" select="'info'"/>
      </xsl:call-template>
      <xsl:result-document href="{$log.uri}">
        <xsl:sequence select="$log.document"/>
      </xsl:result-document>
    </xsl:if>
  </xsl:template> 
  
</xsl:stylesheet>
