<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:xs="http://www.w3.org/2001/XMLSchema" 
	xmlns:map="http://www.w3.org/2005/xpath-functions/map"
	xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" 
	xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
	xmlns:els="http://www.lefebvre-sarrut.eu/ns/els" 
	xmlns:dalloz="http://www.lefebvre-sarrut.eu/ns/dalloz" 
	xmlns:h="http://www.w3.org/1999/xhtml" 
	xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
	xmlns:xfSAS="http://www.lefebvre-sarrut.eu/ns/xmlfirst/sas" 
	exclude-result-prefixes="#all" 
	version="3.0">
  
  <xsl:import href="../common/dalloz_common.xsl"/>
  
  <xsl:param name="codePrefixMeta" select="'DZ_META_'" as="xs:string"/>
  <xsl:param name="codeModele" select="'ouvrages'" as="xs:string"/>

	<!--==================================================-->
	<!-- MODE METADATA-->
	<!--==================================================-->

	<xd:doc scope="stylesheet">
		<xd:desc>
			<xd:p>XSLT pour gérer les métadonnées générées (1 seule fois dans le SAS et ensuite éditable dans l'ECM)</xd:p>
			<xd:p>A ne pas confondre avec les métadonnées calculées à partir du contenu (gérées dans sie-xf-ecm-resources), ici les éléments xml qui servent au métadonnées générées ne sont pas conservés. (ici on peut aussi déclarer des métadonnées "générées" qui ne sont pas liées au contenu xml, par exemple une date d'import)</xd:p>
		</xd:desc>
	</xd:doc>

	<!--==================-->
	<!-- -->
  <!--==================-->
  <xsl:template match="@id" mode="metadata"/>
  <xsl:template match="@ID_OLD" mode="metadata"/>
  
  <xsl:template match="METADONNEES|METAS" mode="metadata">
    <xsl:apply-templates mode="metadata"/>
  </xsl:template>
  
  <xsl:template match="ISBN" mode="metadata">
    <meta code="{concat($codePrefixMeta, $codeModele, 'Isbn')}">
      <value>
        <xsl:value-of select="."/>
      </value>
    </meta>
  </xsl:template>
  
  <xsl:template match="DATES" mode="metadata">
    <xsl:apply-templates select="*" mode="metadata"/>
  </xsl:template>  
  
  <xsl:template match="DATE-CREA" mode="metadata">
    <meta code="{concat($codePrefixMeta, $codeModele, 'Date2')}">
      <value>
        <xsl:value-of select="dalloz:dataDate-to-dateTime(.)"/>
      </value>
    </meta>
  </xsl:template>  
  
  <xsl:template match="DATE-MAJ" mode="metadata">
    <meta code="{concat($codePrefixMeta, $codeModele, 'Date3')}">
      <value>
        <xsl:value-of select="."/>
      </value>
    </meta>  
  </xsl:template>  
  
  <xsl:template match="DATE-PUBLI" mode="metadata">
    <meta code="{concat($codePrefixMeta, $codeModele, 'Annee')}">
      <value>
        <xsl:value-of select="."/>
      </value>
    </meta>
  </xsl:template>  
  
  <xsl:template match="DATE-APPLI" mode="metadata">
    <meta code="{concat($codePrefixMeta, $codeModele, 'Date4')}">
      <value>
        <xsl:value-of select="dalloz:dataDate-to-dateTime(.)"/>
      </value>
    </meta>
  </xsl:template>
  
  <xsl:template match="TITRE" mode="metadata">
    <xsl:apply-templates select="*" mode="metadata"/>
  </xsl:template>  
  
  <xsl:template match="TIT-NUMERO" mode="metadata">
    <xsl:apply-templates select="*" mode="metadata"/>
  </xsl:template>  
  
  <xsl:template match="TIT-NIV" mode="metadata">
    <meta code="{concat($codePrefixMeta, $codeModele, 'TitreNiv')}">
      <value><xsl:value-of select="."/></value>
    </meta>
  </xsl:template> 
  
  <xsl:template match="TIT-NUM" mode="metadata">
    <meta code="{concat($codePrefixMeta, $codeModele, 'TitreNum')}">
      <value><xsl:value-of select="."/></value>
    </meta>
  </xsl:template>  
  
  <xsl:template match="METADONNEES/TITRE/TIT-INTITULE" mode="metadata"/>
  
  <xsl:template match="TIT-INTITULE" mode="metadata">
    <meta code="{concat($codePrefixMeta, $codeModele, 'TitreInt')}">
      <value><xsl:copy-of select="TIT-INTT/node()"/></value>
    </meta>
  </xsl:template>
  
  <xsl:template match="NUME" mode="metadata">
    <meta code="{concat($codePrefixMeta, $codeModele, 'ParaNume')}">
      <value><xsl:value-of select="."/></value>
    </meta>
  </xsl:template> 
  
  <xsl:template match="AUTEUR" mode="metadata">
    <xsl:apply-templates select="*" mode="metadata"/>
  </xsl:template> 
  
  <xsl:strip-space elements="AUTEUR-NOM-COMPLET"/>
  <xsl:template match="AUTEUR-NOM-COMPLET" mode="metadata">
    <meta code="{concat($codePrefixMeta, $codeModele, 'AuteurPrenomNom')}">
      <xsl:variable name="AuteurPrenomNom"><xsl:value-of select="*|text()" separator="&#160;"/></xsl:variable>
      <value><xsl:value-of select="normalize-space($AuteurPrenomNom)"/></value>
    </meta>
  </xsl:template>
  
  <xsl:template match="AUTEUR-QUALITES" mode="metadata">
    <meta code="{concat($codePrefixMeta, $codeModele, 'AuteurQualite')}">
      <xsl:apply-templates select="*" mode="metadata"/>
    </meta>
  </xsl:template>
  
  <xsl:template match="AUT-QUALITE" mode="metadata">
    <xsl:choose>
      <xsl:when test="local-name(..) = 'AUTEUR-QUALITES'">
        <value><xsl:value-of select="."/></value>
      </xsl:when>
      <xsl:otherwise>
        <meta code="{concat($codePrefixMeta, $codeModele, 'AuteurQualite')}">
          <value><xsl:value-of select="."/></value>
        </meta>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="THEMATISATION" mode="metadata">
    <meta code="{concat($codePrefixMeta, $codeModele, 'MotcleNiv1')}">
      <xsl:apply-templates mode="#current"/>
    </meta>
  </xsl:template>
  
  <xsl:template match="THM-NIV1" mode="metadata">
    <xsl:variable name="thm1" as="xs:string"><xsl:value-of select="THM-LIB"/></xsl:variable>
    <xsl:choose>
      <xsl:when test="THM-NIV2">
        <xsl:for-each select="THM-NIV2">
          <value><xsl:value-of select="concat($thm1, ' / ', THM-LIB)"/></value>
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
        <value><xsl:value-of select="$thm1"/></value>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="INDX" mode="metadata" />
  
  <xsl:function name="dalloz:metaCollectionType" as="element(xf:meta)*">
    <xsl:param name="root" as="element(OUVRAGE-DALLOZ)"/>
    <xsl:if test="$root/@id">
      <xsl:variable name="tokens" select="tokenize($root/@id, '/')"/>
      <xsl:variable name="collec" select="if ($tokens[1]='DZ') then $tokens[2] else $tokens[1]"/>
      <xsl:if test="$collec">
        <xsl:variable name="TypeCollec">
          <xsl:choose>
            <xsl:when test="$collec='ACTION'"><xsl:value-of select="'Dalloz Action'"/></xsl:when>
            <xsl:when test="$collec='DELMAS'"><xsl:value-of select="'Delmas'"/></xsl:when>
            <xsl:when test="$collec='JURIS-CORPUS'"><xsl:value-of select="'Juris Corpus'"/></xsl:when>
            <xsl:when test="$collec='REFERENCE'"><xsl:value-of select="'Dalloz Référence'"/></xsl:when>
            <xsl:when test="$collec='JURIS-CORPUS'"><xsl:value-of select="'Juris Corpus'"/></xsl:when>
            <xsl:when test="$collec='PRAXIS'"><xsl:value-of select="'Praxis Dalloz'"/></xsl:when>
            <!-- en attente de repose Jira, je mets Praxis Dalloz mais ce devrait etre Dalloz Professionnel-->
            <xsl:when test="$collec='PCA'"><xsl:value-of select="'Praxis Dalloz'"/></xsl:when>
            <!-- et il manque les codes pour Praxis Dalloz et Guides Dalloz -->
            <xsl:otherwise>
              <xsl:message>[WARNING] Code de collection inconnu <xsl:value-of select="$collec"/></xsl:message>
              <xsl:value-of select="$collec"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <meta code="{concat($codePrefixMeta, 'ouvrages', 'Collection')}">
          <value><xsl:value-of select="$TypeCollec"/></value>
        </meta>
      </xsl:if>
      <xsl:variable name="type" select="if ($tokens[1]='DZ') then $tokens[3] else $tokens[2]"/>
      <xsl:if test="$type">
        <meta code="{concat($codePrefixMeta, 'ouvrages', 'Type')}">
          <value><xsl:value-of select="$type"/></value>
        </meta>
      </xsl:if>
    </xsl:if>
  </xsl:function>
  
  <xsl:function name="dalloz:metaTitle" as="element(xf:meta)">
    <xsl:param name="root" as="element(OUVRAGE-DALLOZ)"/>
    <meta code="{concat($codePrefixMeta, $codeModele, 'TitreInt')}">
      <xsl:choose>
        <xsl:when test="$root/METADONNEES/TITRE/TIT-INTITULE/TIT-INTT/text()">
          <value><xsl:value-of select="$root/METADONNEES/TITRE/TIT-INTITULE/TIT-INTT"/></value>
        </xsl:when>
        <xsl:otherwise>
          <xsl:variable name="tokens" select="tokenize(replace($root/@id, '^DZ/', ''), '/')"/>
          <xsl:choose>
            <xsl:when test="$tokens[2]">
              <xsl:variable name="apos">'</xsl:variable>
              <xsl:variable name="dalloz:collections" select="map { 
                (: Dalloz action: le code apparait sous la forme ACTION/XXXX :)
                'CASSATION-PENALE' : 'La cassation en matière pénale',
                'COPROPRIETE' : 'La copropriété',
                'DROIT-AUTEUR' : concat('Droit d', $apos, 'auteur'),
                'CONSTRUCTION' : 'Droit de la construction',
                'EXECUTION-PEINES' : concat('Droit de l', $apos, 'exécution des peines'),
                'EXPERTISE' : concat('Droit de l', $apos, 'expertise'),
                'FAMILLE' : 'Droit de la famille',
                'BAUX-COMMERCIAUX' : 'Droit et pratique des baux commerciaux',
                'INSTRUCTION-PREPARATOIRE' : concat('Droit et pratique de l', $apos, 'instruction préparatoire'),
                'PROCEDURE-CIVILE' : 'Droit et pratique de la procédure civile',
                'PATRIMONIAL' : 'Droit patrimonial de la famille',
                'RESPONSABILITE' : 'Droit de la responsabilité et des contrats',
                'FONDS-COMMERCE' : 'Fonds de commerce',
                (: Dalloz référence: le code apparait sous la forme REFERENCE/XXXX :)
                'CABINET-AVOCATS' : concat('Cabinet d', $apos, 'avocats'),
                'AFFAIRES' : 'Pratique du contentieux administratif',
                'DIVORCE' : 'Droit et pratique du divorce',
                (: cas particulier :)
                (: Juris corpus: le code apparait sous la forme JURIS-CORPUS/XXXX :)
                'ASSOCIATIONS-FONDATIONS' : 'Droit des associations et fondations',
                (: Dalloz Professionnel: le code apparait sous la forme PCA/XXXX :)
                'CONTENTIEUX-ADMINISTRATIF' : 'Pratique du contentieux administratif',
                (:  Delmas: le code apparait sous la forme DELMAS/XXXX :)
                'PROCEDURES-COLLECTIVES' : 'Procédures collectives'
                }"/>  
              <xsl:choose>
                <xsl:when test="map:contains($dalloz:collections, $tokens[2])">
                  <value><xsl:value-of select="map:get($dalloz:collections, $tokens[2])"/></value>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:message>[WARNING] Code d'ouvrage inconnu <xsl:value-of select="$tokens[2]"/></xsl:message>
                  <xsl:message>Veuillez adapter le code du SAS d'import</xsl:message>
                  <value><xsl:value-of select="$tokens[2]"/></value>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
              <xsl:message>[ERROR] Ouvrage sans code!</xsl:message>
              <value><xsl:value-of select="'Ouvrage sans titre'"/></value>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </meta>
  </xsl:function>

	<xsl:template match="*" mode="metadata" priority="-1">
		<xsl:message>[ERROR] élément non matché en mode "metadata" : <xsl:value-of select="local-name(.)"/></xsl:message>
	</xsl:template>
  
</xsl:stylesheet>
