<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" 
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  xmlns:functx="http://www.functx.com" 
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els" 
  xmlns:xfSAS="http://www.lefebvre-sarrut.eu/ns/xmlfirst/sas"
  xmlns:sas.utils="http://www.lefebvre-sarrut.eu/ns/els/utils" 
  xmlns:dalloz="http://www.lefebvre-sarrut.eu/ns/dalloz" 
  xmlns:ouvrages="http://www.lefebvre-sarrut.eu/ns/dalloz/ouvrages" 
  xmlns:h="http://www.w3.org/1999/xhtml" 
  xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  exclude-result-prefixes="#all" version="3.0">

  <xsl:import href="../../xf-sas-common.xsl"/>
	
  <xsl:param name="codePrefixMeta" select="'DZ_META_'" as="xs:string"/>
  <!-- Allows to continue processing despite major errors in the source document -->
  <xsl:param name="relaxedChecks" select="false()" as="xs:boolean"/>
  
  <xsl:key name="idkey" match="*[@id]" use="@id"/>
  <xsl:key name="xfidkey" match="*[@xf:id]" use="@xf:id"/>

	<xsl:template match="editorialEntity/body" mode="makeRelations">
		<xsl:next-match/>
		<relations>
			<xsl:apply-templates select="contentNode/content//ouvrages:REF[@idRef]" mode="makeRelationElement"/>
		</relations>
	</xsl:template>

	<!--=====================================-->
	<!--Ouvrages / Codes : REF-->
	<!--=====================================-->

	<xsl:template match="ouvrages:REF[@idRef]" mode="makeRelationElement">
		<xsl:call-template name="makeRelationElement">
			<xsl:with-param name="element" select="." as="element(ouvrages:REF)"/>
			<xsl:with-param name="type" select="@TYPE" as="xs:string?"/>
			<xsl:with-param name="refid" select="@idRef" as="xs:string"/>
		</xsl:call-template>
	</xsl:template>

	<!--=====================================-->
	<!--makeRelations-->
	<!--=====================================-->

	<xsl:template name="makeRelationElement">
		<xsl:param name="element" as="element()"/>
		<xsl:param name="type" as="xs:string?"/>
		<xsl:param name="refid" as="xs:string"/>

	  <xsl:choose>
	    <!-- Reference interne a l'ouvrage -->
	    <xsl:when test="upper-case($type) = ('INTRA','INTER')">
	      <xsl:if test="upper-case($type) = 'INTER'">
	        <xsl:message>[WARNING] Référence de type INTER inattendue. Traitement comme une référence INTRA. </xsl:message>
	      </xsl:if>
	      <xsl:variable name="referedPara" select="key('idkey', $refid)"/>
	      <xsl:variable name="historicid" select="xf:makeHistoricalIdAttributeValue($element, $refid, $xfSAS:company)"/>
	      <xsl:variable name="referedNode" select="key('xfidkey', $historicid)"/>
	      <xsl:choose>
	        <!-- duplicated id is a failure, unless we have relaxedChecks enabled, for testing purposes -->
	        <xsl:when test="count($referedPara) gt 1 and $relaxedChecks = false()">
	          <xsl:message>[ERROR] Duplicate ID with value <xsl:value-of select="@idRef"/></xsl:message>
	          <!-- QQiao : pour avancer le test XD, j'ai mis le message en commentaire -->
	          <!--<xsl:message terminate="yes">Exiting program</xsl:message>-->
	        </xsl:when>
	        <!-- if there is no target, there is no relation -->
	        <xsl:when test="count($referedPara) + count($referedNode)= 0">
	          <xsl:message>[ERROR] Invalid reference target: <xsl:value-of select="upper-case($type)"/> : <xsl:value-of select="$refid"/></xsl:message>
	      </xsl:when>
	        <!-- on est dans le même contentNode => adressage par idRef on ne touche à rien -->
	        <xsl:when test="$referedPara and (generate-id($element/ancestor::content[1]) = generate-id($referedPara[1]/ancestor::content[1]))"/>
	        <xsl:otherwise>
	          <!-- on est dans un contentNode différent=> on va passer par une relation pour l'adressage -->
	          <relation xf:id="{xf:generateRelationId($element)}" code="DZ_TR_renvoi_OuvragePara_GrpOuvrage" calculated="true">
	            <metadata/>
	            <xsl:choose>
	              <!-- on est dans la même editorial entity => adressage xf:idRef pour l'adresse du para cible -->
	              <xsl:when test="$referedPara and ($element/ancestor::editorialEntity[1]/@xf:id = $referedPara[1]/ancestor::editorialEntity[1]/@xf:id)">
	                <ref 
	                  xf:targetResType="editorialEntity"
	                  xf:idRef="{$refid}" 
	                  xf:refType="xf:referenceNode"/>
	              </xsl:when>
	              <xsl:otherwise>
	                <!-- on est dans une editorial distincte => adressage xf:targetResId pour l'adresse de l'EE cible -->
	                <!-- et (si necessaire) adressage xf:id pour l'adresse du para cible -->
	                <xsl:variable name="idtokens" select="tokenize($refid, '/')"/>
	                <xsl:variable name="EErefered" as="xs:boolean">
	                  <xsl:choose>
	                    <xsl:when test="(count($idtokens) = 2) 
	                      or (count($idtokens) &gt; 3) and ($idtokens[last()-1] = ('PARA', 'PARA-LEGIS', 'PARA-JURIS', 'PARA-QUEST', 'PARA-BIBLIO', 'PARA-PLAN', 'RESUME', 'ENCADRE', 'CITATION'))">
	                      <xsl:value-of select="true()"/>
	                    </xsl:when>
	                    <xsl:otherwise>
	                      <xsl:value-of select="false()"/>
	                    </xsl:otherwise>
	                  </xsl:choose>
	                </xsl:variable>
	                <xsl:choose>
	                  <xsl:when test="$referedPara">
	                    <ref xf:targetResType="editorialEntity"
	                      xf:targetResId="{$referedPara[1]/ancestor::editorialEntity[1]/@xf:id}"
	                      xf:refType="xf:referenceNode">
	                      <xsl:if test="not($EErefered)">
	                        <xsl:attribute name="idRef" select="$refid"/>
	                      </xsl:if>
	                    </ref>
	                  </xsl:when>
	                  <xsl:otherwise>
	                    <xsl:variable name="targetid" select="xf:makeHistoricalIdAttributeValue($element, $refid, $xfSAS:company)"/>
	                    <xsl:variable name="referedNode" select="key('xfidkey', $targetid)"/>
	                    <!--<xsl:message>id: <xsl:value-of select="$refid"/> - tid: <xsl:value-of select="$targetid"/></xsl:message>-->
	                    <xsl:choose>
	                      <xsl:when test="$referedNode">
	                        <ref xf:targetResType="editorialEntity"
	                          xf:targetResId="" 
	                          xf:idRef="{$targetid}" 
	                          xf:refType="xf:referenceNode">
	                        </ref>
	                      </xsl:when>
	                      <xsl:otherwise>
	                        <xsl:message>[ERROR] Invalid reference target: <xsl:value-of select="upper-case($type)"/> : <xsl:value-of select="$refid"/></xsl:message>
	                        <!--<ref xf:targetResType="editorialEntity"
	                          xf:targetResId="{xf:makeHistoricalIdAttributeValue($element, $refid, $xfSAS:company)}" 
	                          xf:refType="xf:referenceNode">
	                          <xsl:if test="not($EErefered)">
	                            <xsl:attribute name="idRef" select="$refid"/>
	                          </xsl:if>-->
	                        <!--</ref>-->
	                      </xsl:otherwise>
	                    </xsl:choose>
	                  </xsl:otherwise>
	                </xsl:choose>
	              </xsl:otherwise>
	            </xsl:choose>
	          </relation>
	        </xsl:otherwise>
	      </xsl:choose>
	    </xsl:when>
	    <!-- Reference externe à un autre ouvrage -->
	    <xsl:when test="upper-case($type) = 'EXTRA'">
	      <relation xf:id="{xf:generateRelationId($element)}" code="DZ_TR_renvoi_OuvragePara_GrpOuvrage" calculated="true">
	        <metadata/>
	        <ref xf:targetResType="editorialEntity"
	          xf:targetResId="{xf:makeHistoricalIdAttributeValue($element, $refid, $xfSAS:company)}" 
	          xf:refType="xf:referenceNode">
	          <xsl:variable name="idtokens" select="tokenize($refid, '/')"/>
	          <xsl:variable name="EErefered" as="xs:boolean">
	            <xsl:choose>
	              <xsl:when test="(count($idtokens) = 2) 
	                or (count($idtokens) &gt; 3) and ($idtokens[last()-1] = ('PARA', 'PARA-LEGIS', 'PARA-JURIS', 'PARA-QUEST', 'PARA-BIBLIO', 'PARA-PLAN', 'RESUME', 'ENCADRE', 'CITATION'))">
	                <xsl:value-of select="true()"/>
	              </xsl:when>
	              <xsl:otherwise>
	                <xsl:value-of select="false()"/>
	              </xsl:otherwise>
	            </xsl:choose>
	          </xsl:variable>
	          <xsl:if test="not($EErefered)">
	            <xsl:attribute name="idRef" select="$refid"/>
	          </xsl:if>
	        </ref>
	      </relation>
	    </xsl:when>
	    <!-- Reference externe à une jurisprudence -->
	    <xsl:when test="upper-case($type) = 'JUR'">
	      <!-- a implémenter dès que possible -->
	    </xsl:when>
	    <xsl:when test="upper-case($type) = 'FORM'">
	      <relation xf:id="{xf:generateRelationId($element)}" code="DZ_TR_renvoi_GrpOuvrage_GrpFormule" calculated="true">
	        <ref xf:targetResType="editorialEntity"
	          xf:targetResId="{xf:makeHistoricalIdAttributeValue($element, $refid, $xfSAS:company)}" 
	          xf:refType="xf:referenceNode">
	        </ref>
	      </relation>
	    </xsl:when>
	    <!-- Reference externe à une revue -->
	    <xsl:when test="upper-case($type) = 'REV'">
	      <xsl:variable name="tokens" select="tokenize(replace($refid, '^(\s*\|)|(\|\s*)$', ''), '\|')"/>
	      <xsl:variable name="revid" select="normalize-space(upper-case($tokens[1]))"/>
	      <xsl:choose>
	        <xsl:when test="matches($revid, '^(X+|VIDE-AUTO)$')"/>
	        <xsl:when test="matches($revid, '^[A-Z]+/[A-Z0-9]+/\d\d\d\d/\d\d\d\d?\d?$')">
	          <xsl:choose>
	            <xsl:when test="$tokens[2]">
	              <xsl:message>[WARNING] Multiple revues identifier: &quot;<xsl:value-of select="$refid"/>&quot; Processsing first identifier only.</xsl:message>
	            </xsl:when>
	            <xsl:when test="matches($refid, '^\s|\s$')">
	              <xsl:message>[WARNING] Bad revue identifier: &quot;<xsl:value-of select="$refid"/>&quot; Space before or after identifier will be removed.</xsl:message>
	            </xsl:when>
	          </xsl:choose>
	          <xsl:variable name="revidparts" select="tokenize($revid, '/')"/>
	          <xsl:variable name="revlist" select="('AJCA', 'AJCT', 'AJDA', 'AJDI', 'AJFAM', 'AJFP', 'AJPEN', 'AVOCATS', 'CAHCONST', 'CAHJU', 'CONSTIT', 'DIPIT', 'DS', 
	            'JA', 'JAC', 'JCASS', 'JS', 'JT', 'LEBON', 'RDI', 'RDSS', 'RECUEIL', 'REVDIP', 'REVSOC', 'REVTRAV', 'RFDA', 'RMC', 'RSC', 'RTDCIV', 'RTDCOM', 'RTDEUR')"/>
	          <xsl:choose>
	            <xsl:when test="not($revidparts[1]=$revlist)">
	              <xsl:message>[WARNING] Unknown revue code: &quot;<xsl:value-of select="$revidparts[1]"/>&quot;.</xsl:message>
	            </xsl:when>
	            <xsl:when test="not($revidparts[2]=('AJ', 'CHRON', 'COMALD', 'CONCL', 'DOSS', 'IR', 'JURIS', 'LEGIS', 'NOTA', 'NOTE', 'OBS', 'RAPP', 'RM', 'SC', 'VALEURS'))">
	              <xsl:message>[WARNING] Unknown revue sub-code: &quot;<xsl:value-of select="$revidparts[2]"/>&quot;.</xsl:message>
	            </xsl:when>
	          </xsl:choose>
	          <relation xf:id="{xf:generateRelationId($element)}" code="DZ_TR_renvoi_GrpOuvrages_GrpRevues" calculated="true">
	            <ref xf:targetResType="editorialEntity"
	              xf:targetResId="{xf:makeHistoricalIdAttributeValue($element, $revid, $xfSAS:company)}" 
	              xf:refType="xf:referenceNode">
	            </ref>
	          </relation>
	        </xsl:when>
	        <xsl:otherwise>
	          <xsl:message>[WARNING] Bad revue identifier: <xsl:value-of select="$refid"/></xsl:message>
	        </xsl:otherwise>
	      </xsl:choose>
	    </xsl:when>
	    <!-- Reference externe à d'autres fonds identifiés, mais sans occurence pour le moment dans nos données -->
	    <!-- En stand-by pour le moment -->
	    <xsl:when test="upper-case($type) = ('BROC', 'LEG', 'ENCY')">
	      <xsl:message>Unhandled reference @type found : <xsl:value-of select="$type"/>.</xsl:message>
	      <xsl:message>Please adapt the sas conversuion code for this value.</xsl:message>
	      <xsl:message><xsl:value-of select="$element"/></xsl:message>
	    </xsl:when>
	    <!-- 'LEX', 'FORM', 'FICH' et autres fonds théoriquement possibles -->
	    <!-- A implémenter au cas par cas -->
	    <xsl:otherwise>
	      <xsl:message>Unexpected reference @type found : <xsl:value-of select="$type"/> !</xsl:message>
	      <xsl:message>Please adapt the sas conversuion code for this new value.</xsl:message>
	      <xsl:message><xsl:value-of select="$element"/></xsl:message>
	      <!-- QQiao : pour avancer le test XD, j'ai mis le message en commentaire -->
	      <!--<xsl:message terminate="yes">Exiting program</xsl:message>-->
	    </xsl:otherwise>
	  </xsl:choose>
	</xsl:template>

  <xsl:template match="contentNode/content//ouvrages:REF[@idRef]" mode="makeRelations">
    <xsl:copy>
      <xsl:choose>
        <xsl:when test="upper-case(@TYPE) = ('INTRA','INTER')">
          <!-- lien sur une para de le meme ouvrage -->
          <xsl:variable name="referedPara" select="key('idkey', @idRef)"/>
          <xsl:variable name="historicid" select="xf:makeHistoricalIdAttributeValue(., @idRef, $xfSAS:company)"/>
          <xsl:variable name="referedNode" select="key('xfidkey', $historicid)"/>
          <xsl:if test="count($referedPara) gt 1 and $relaxedChecks = false()">
            <xsl:message>[ERROR] Duplicate ID with value <xsl:value-of select="@idRef"/></xsl:message>
            <!-- QQiao : pour avancer le test XD, j'ai mis le message en commentaire -->
            <!--<xsl:message terminate="yes">Exiting program</xsl:message>-->
          </xsl:if>
          <xsl:choose>
            <!-- if there is no target, there is no relation -->
            <xsl:when test="count($referedPara) + count($referedNode)= 0">
              <xsl:apply-templates select="@idRef" mode="#current"/>
            </xsl:when>
            <xsl:when test="generate-id(ancestor::content[1]) = generate-id($referedPara[1]/ancestor::content[1])">
              <xsl:attribute name="idRef" select="translate(@idRef, '/', '_slash_')"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:attribute name="xf:idRef" select="xf:generateRelationId(.)"/>
              <xsl:attribute name="xf:refType" select="'xf:relationRef'"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:when test="upper-case(@TYPE) = 'EXTRA'">
          <xsl:attribute name="xf:idRef" select="xf:generateRelationId(.)"/>
          <xsl:attribute name="xf:refType" select="'xf:relationRef'"/>
        </xsl:when>
        <xsl:when test="upper-case(@TYPE) = 'FORM'">
          <xsl:attribute name="xf:idRef" select="xf:generateRelationId(.)"/>
          <xsl:attribute name="xf:refType" select="'xf:relationRef'"/>
        </xsl:when>
        <xsl:when test="upper-case(@TYPE) = 'REV'">
          <xsl:variable name="tokens" select="tokenize(replace(@idRef, '^(\s*\|)|(\|\s*)$', ''), '\|')"/>
          <xsl:choose>
            <xsl:when test="matches(upper-case($tokens[1]), '^[A-Z]+/[A-Z0-9]+/\d\d\d\d/\d\d\d\d?\d?$')">
              <xsl:attribute name="xf:idRef" select="xf:generateRelationId(.)"/>
              <xsl:attribute name="xf:refType" select="'xf:relationRef'"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="@idRef" mode="#current"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <!-- lien d'un autre type -->
        <xsl:otherwise>
          <xsl:apply-templates select="@idRef" mode="#current"/>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="@* except @idRef" mode="#current"/>
      <xsl:apply-templates select="node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template name="makeRelation">
    <!-- pas traite pour le moment et ne devrait pas survenir-->
    <xsl:param name="element" as="element()"/>
    <xsl:copy-of select="."/>
  </xsl:template>

	<!--=====================================-->
	<!--COPY-->
	<!--=====================================-->

	<xsl:template match=" node() | @*" mode="makeRelations">
		<xsl:copy>
			<xsl:apply-templates select="node() | @*" mode="#current"/>
		</xsl:copy>
	</xsl:template>

</xsl:stylesheet>
