﻿<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:map="http://www.w3.org/2005/xpath-functions/map" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" 
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els" 
  xmlns:dalloz="http://www.lefebvre-sarrut.eu/ns/dalloz" 
  xmlns:ouvrages="http://www.lefebvre-sarrut.eu/ns/dalloz/ouvrages"
  xmlns:h="http://www.w3.org/1999/xhtml" 
  xmlns:xfSAS="http://www.lefebvre-sarrut.eu/ns/xmlfirst/sas"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst"  
  exclude-result-prefixes="#all" 
  version="3.0">
  
  <xsl:import href="ouvragesDalloz2editorialEntity.makeRelations.xsl"/>
  <xsl:import href="ouvragesDalloz.getGeneratedMetadata.xsl"/>
  <xsl:import href="../../../xf-ecmRes/dalloz/_common/title2html.xsl"/>
  <xsl:import href="ouvragesDallozAdjustIds.xsl"/>
  
  <xsl:output method="xml" indent="no"/>
  
  <xsl:param name="treeEntityOnly"  select="false()" as="xs:boolean"/>
  
  <xsl:param name="rel.to.schema" select="'xf-models:/editorialEntity'" as="xs:string"/>
  <xsl:param name="xfSAS:company" select="'dalloz'" as="xs:string"/>
  <xsl:param name="outDir" select="base-uri()" as="xs:string"/>

  <xsl:variable name="codeModele" select="'ouvrages'" as="xs:string"/>
  <xsl:variable name="codePrefixTee" select="'DZ_TEE_'" as="xs:string"/>
  <xsl:variable name="codePrefixTr" select="'DZ_TR_'" as="xs:string"/>
  <xsl:variable name="codePrefixMeta" select="'DZ_META_'" as="xs:string"/>
  <xsl:variable name="contentNode.namespace.uri" select="'http://www.lefebvre-sarrut.eu/ns/dalloz/ouvrages'" as="xs:string"/>
  
  <xsl:variable name="idroot" select="/OUVRAGE-DALLOZ/@ID"/>
  
  <xsl:template match="/">
    <xsl:choose>
      <!-- permet d'éviter de tenter de transformer des documents inadéquats -->
      <xsl:when test="/xf:editorialEntity | /xf:editorialEntities/xf:editorialEntity">
        <xsl:message>[WARNING] Le document est déjà transformé!</xsl:message>
      </xsl:when>
      <xsl:when test="/OUVRAGE-DALLOZ">
        <xsl:variable name="sourcefile" select="els:getFileName(static-base-uri())"/>
        <!-- Step -1 -->
        <xsl:variable name="step" as="document-node()">
          <xsl:document>
            <xsl:apply-templates select="." mode="adjustAllIds"/>
          </xsl:document>
        </xsl:variable>
        <xsl:call-template name="dalloz:logDocument">
          <xsl:with-param name="log.source" select="$sourcefile"/>
          <xsl:with-param name="log.document" select="$step" as="document-node()"/>
          <xsl:with-param name="log.filename" select="'step00-xml2ee.log.xml'" as="xs:string"/>
        </xsl:call-template>
        <!-- Step 0 -->
        <xsl:variable name="step" as="document-node()">
          <xsl:document>
            <xsl:apply-templates select="$step" mode="adjust1"/>
          </xsl:document>
        </xsl:variable>
        <xsl:call-template name="dalloz:logDocument">
          <xsl:with-param name="log.source" select="$sourcefile"/>
          <xsl:with-param name="log.document" select="$step" as="document-node()"/>
          <xsl:with-param name="log.filename" select="'step0-xml2ee.log.xml'" as="xs:string"/>
        </xsl:call-template>
        <!-- Step 1 -->
        <xsl:variable name="step" as="document-node()">
          <xsl:document>
            <xsl:apply-templates select="$step" mode="fix-paras"/>
          </xsl:document>
        </xsl:variable>
        <xsl:call-template name="dalloz:logDocument">
          <xsl:with-param name="log.source" select="$sourcefile"/>
          <xsl:with-param name="log.document" select="$step" as="document-node()"/>
          <xsl:with-param name="log.filename" select="'step1-xml2ee.log.xml'" as="xs:string"/>
        </xsl:call-template>
        <!-- Step 2 -->
        <xsl:variable name="step" as="document-node()">
          <xsl:document>
            <xsl:apply-templates select="$step" mode="structuralNode"/>
          </xsl:document>
        </xsl:variable>
        <xsl:call-template name="dalloz:logDocument">
          <xsl:with-param name="log.source" select="$sourcefile"/>
          <xsl:with-param name="log.document" select="$step" as="document-node()"/>
          <xsl:with-param name="log.filename" select="'step2-xml2ee.log.xml'" as="xs:string"/>
        </xsl:call-template>
        <!-- Step 3 Normalisation des métas -->
        <xsl:variable name="step" as="document-node()">
          <xsl:document>
            <xsl:apply-templates select="$step" mode="xfSAS:fixMeta"/>        
          </xsl:document>
        </xsl:variable>
        <xsl:call-template name="dalloz:logDocument">
          <xsl:with-param name="log.source" select="$sourcefile"/>
          <xsl:with-param name="log.document" select="$step" as="document-node()"/>
          <xsl:with-param name="log.filename" select="'step3-fixMeta.log.xml'" as="xs:string"/>
        </xsl:call-template>
        <!-- Step 4 Gestion des relations -->
        <xsl:variable name="step" as="document-node()">
          <xsl:document>
            <xsl:apply-templates select="$step" mode="makeRelations"/>
          </xsl:document>
        </xsl:variable>
        <xsl:call-template name="dalloz:logDocument">
          <xsl:with-param name="log.source" select="$sourcefile"/>
          <xsl:with-param name="log.document" select="$step" as="document-node()"/>
          <xsl:with-param name="log.filename" select="'step4-xml2ee.log.xml'" as="xs:string"/>
        </xsl:call-template>
        <!-- Step 5 -->
        <xsl:variable name="step" as="document-node()">
          <xsl:document>
            <xsl:apply-templates select="$step" mode="fixIds"/>
          </xsl:document>
        </xsl:variable>
        <xsl:call-template name="dalloz:logDocument">
          <xsl:with-param name="log.source" select="$sourcefile"/>
          <xsl:with-param name="log.document" select="$step" as="document-node()"/>
          <xsl:with-param name="log.filename" select="'step5-xml2ee.log.xml'" as="xs:string"/>
        </xsl:call-template>
        <!-- en sortie: le document final -->
        <xsl:call-template name="xfSAS:makeXmlModelPi"/>
        <xsl:variable name="deleted" select="ouvrages:getDeletedIds()"/>
        <xsl:choose>
          <xsl:when test="count($deleted) != 0">
            <editorialEntities>
              <xsl:for-each select="$deleted">
                <editorialEntity code="DZ_TEE_ouvragesParagraphe">
                  <xsl:attribute name="xf:id" select="concat('{historicalId:', $xfSAS:company, '_',  . , '}')"/>
                  <xsl:sequence select="xfSAS:addAttributeOwnerGroup('DZ_TEE_ouvragesParagraphe', ouvrages:getProduct($idroot))"/>
                  <xsl:attribute name="archived" select="'true'"/>
                </editorialEntity>
              </xsl:for-each>
              <xsl:sequence select="$step"/>
            </editorialEntities>
          </xsl:when>
          <xsl:otherwise>
            <xsl:sequence select="$step"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:message>[ERROR] Le document a transformer n'est pas un ouvrage!</xsl:message>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:variable name="idRoot" select="/OUVRAGE-DALLOZ/@ID" as="xs:string?"/>
  
  <xsl:template match="@ID" mode="adjust1">
    <xsl:attribute name="id" select="."/>
  </xsl:template>
  
  <xsl:template match="@REFID" mode="adjust1">
    <xsl:attribute name="idRef" select="."/>
  </xsl:template>
  
  <xsl:template match="@*|node()" mode="adjust1">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()" mode="adjust1"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="IMG" mode="fix-paras">
    <xsl:copy>
      <xsl:apply-templates select="@* except @FICHIER" mode="#current"/>
      <xsl:if test="@FICHIER">
        <xsl:attribute name="FICHIER" 
          select="concat('DZ_TEE_ouvragesParagraphe/', tokenize(replace(/OUVRAGE-DALLOZ/@id, '^DZ/', ''), '/')[2], '/', xfSAS:normalizeMediaFileName(@FICHIER))"/>
      </xsl:if>
      <xsl:apply-templates select="node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="TABLEAU[parent::NIV-TEXTE]" mode="fix-paras">
    <PARA xmlns="" id="{dalloz:dallozId(.)}">
      <PARA-TXT>
        <xsl:copy>
          <xsl:apply-templates select="@*|node()" mode="#current"/>
        </xsl:copy>
      </PARA-TXT>
    </PARA>
  </xsl:template>
  
  <xsl:template match="LISTE[parent::NIV-TEXTE]" mode="fix-paras">
    <PARA xmlns="" id="{dalloz:dallozId(.)}">
      <PARA-TXT>
        <xsl:copy>
          <xsl:apply-templates select="@*|node()" mode="#current"/>
        </xsl:copy>
      </PARA-TXT>
    </PARA>
  </xsl:template>
  
  <xsl:template match="(NIVO-CORPS|NIV-COMPL)/NIV-TEXTE/PARA[1][not(@id)]" mode="fix-paras" priority="1">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="PARA[not(@id)]|PARA-LEGIS[not(@id)]|PARA-JURIS[not(@id)]|PARA-QUEST[not(@id)]|PARA-BIBLIO[not(@id)]|PARA-PLAN[not(@id)]" mode="fix-paras">
    <xsl:copy>
      <xsl:attribute name="id" select="dalloz:dallozId(.)"/>
      <xsl:apply-templates select="@*|node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="REF" mode="fix-paras">
    <xsl:copy>
      <xsl:attribute name="TYPE" select="upper-case(@TYPE)"/>
      <xsl:apply-templates select="@* except @TYPE" mode="#current"/>
      <xsl:apply-templates select="node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="@*|node()" mode="fix-paras">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--===============================-->
  <!--STRUCTURAL NODES -->
  <!--===============================-->
  
  <xsl:template match="*" mode="structuralNode">
    <xsl:message>Structural Mode Called on <xsl:value-of select="local-name(.)"></xsl:value-of></xsl:message>
  </xsl:template>
  
  <xsl:template match="text()" mode="structuralNode"/>

  <xsl:template match="OUVRAGE-DALLOZ" mode="structuralNode">
    <xsl:variable name="entityCode" select="concat($codePrefixTee, $codeModele)" as="xs:string"/>
    <editorialEntity code="{$entityCode}"
      xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
      xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst">
      <xsl:attribute name="xf:id" select="dalloz:historicalId(.)"/>
      <xsl:sequence select="xfSAS:addAttributeOwnerGroup($entityCode, ouvrages:getProduct($idroot))"/>
      <xsl:call-template name="xfSAS:addAttributeCreationUser"/>
      <metadata>
        <meta code="{concat($codePrefixMeta, 'ouvrages', 'Origine')}">
          <value><xsl:value-of select="local-name(.)"/></value>
        </meta>
        <xsl:copy-of select="dalloz:metaCollectionType(.)"/>
        <xsl:copy-of select="dalloz:metaTitle(.)"/>
        <xsl:copy-of select="dalloz:metaOldId(.)"/>
        <xsl:apply-templates select="@*" mode="metadata"/>
        <xsl:apply-templates select="METADONNEES" mode="metadata"/>
      </metadata>
      <body>
        <xsl:apply-templates select="* except METADONNEES" mode="structuralNode"/>
      </body>
    </editorialEntity>
  </xsl:template>
  
  <xsl:template match="CORPUS" mode="structuralNode">
    <structuralNode>
      <xsl:attribute name="xf:id" select="dalloz:historicalId(.)"/>
      <metadata>
        <meta code="{concat($codePrefixMeta, 'ouvrages', 'Origine')}">
          <value><xsl:value-of select="local-name(.)"/></value>
        </meta>
        <xsl:copy-of select="dalloz:metaOldId(.)"/>
        <xsl:apply-templates select="@*" mode="metadata"/>
      </metadata>
      <title>
        <div xmlns="http://www.w3.org/1999/xhtml">
          <span class="{local-name(.)}" xmlns="http://www.w3.org/1999/xhtml">
              <xsl:value-of select="local-name(.)"/>
            </span>
        </div>
      </title>
      <xsl:apply-templates select="* except METAS" mode="structuralNode"/>
    </structuralNode>
  </xsl:template>
  
  <xsl:template match="PRELIMINAIRES" mode="structuralNode">
    <!-- 
    <structuralNode>
      <xsl:attribute name="xf:id" select="dalloz:historicalId(.)"/>
      <metadata>
        <meta code="{concat($codePrefixMeta, 'ouvrages', 'Origine')}">
          <value><xsl:value-of select="local-name(.)"/></value>
        </meta>
        <xsl:copy-of select="dalloz:metaOldId(.)"/>
        <xsl:apply-templates select="@*" mode="metadata"/>
      </metadata>
      <title>
        <div xmlns="http://www.w3.org/1999/xhtml">
          <span class="{local-name(.)}" xmlns="http://www.w3.org/1999/xhtml">
              <xsl:value-of select="local-name(.)"/>
            </span>
        </div>
      </title>
      <xsl:apply-templates select="* except METAS" mode="structuralNode"/>
    </structuralNode>
    -->
    <xsl:if test="PGTITRE">
      <xsl:message>[WARNING] Il y a un element PGTITRE a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
    <xsl:if test="PGAUTEURS">
      <xsl:message>[WARNING] Il y a un element PGAUTEURS a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
    <xsl:if test="PGCOLLEC">
      <xsl:message>[WARNING] Il y a un element PGCOLLEC a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
    <xsl:if test="PGINFOS">
      <xsl:message>[WARNING] Il y a un element PGINFOS a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
    <xsl:if test="COPYRIGHT">
      <xsl:message>[WARNING] Il y a un element COPYRIGHT a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
    <xsl:if test="INDEX">
      <xsl:message>[WARNING] Il y a un element INDEX a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
    <xsl:if test="SOMMAIRE">
      <xsl:message>[WARNING] Il y a un element SOMMAIRE a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
    <xsl:if test="TBL-MAT">
      <xsl:message>[WARNING] Il y a un element TBL-MAT a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
    <xsl:if test="TBL-REF">
      <xsl:message>[WARNING] Il y a un element TBL-REF a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
    <xsl:if test="BIBLIOGRAPHIE">
      <xsl:message>[WARNING] Il y a un element BIBLIOGRAPHIE a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
    <xsl:if test="REFERENCES">
      <xsl:message>[WARNING] Il y a un element REFERENCES a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
    <xsl:if test="LEXIQUE">
      <xsl:message>[WARNING] Il y a un element LEXIQUE a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
    <xsl:if test="LIST-GEN">
      <xsl:message>[WARNING] Il y a un element LIST-GEN a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
    <xsl:if test="LIST-TOM">
      <xsl:message>[WARNING] Il y a un element LIST-TOM a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
    <xsl:if test="VOLUME">
      <xsl:message>[WARNING] Il y a un element VOLUME a traiter dans la section PRELIMINAIRES.</xsl:message>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="COMPLEMENTS" mode="structuralNode">
    <xsl:choose>
      <!-- Pour le moment on ne traite que ces éléments -->
      <xsl:when test="ANNEXES|ANNEXE">
        <structuralNode>
          <xsl:attribute name="xf:id" select="dalloz:historicalId(.)"/>
          <metadata>
            <meta code="{concat($codePrefixMeta, 'ouvrages', 'Origine')}">
              <value><xsl:value-of select="local-name(.)"/></value>
            </meta>
            <xsl:copy-of select="dalloz:metaOldId(.)"/>
            <xsl:apply-templates select="@*" mode="metadata"/>
          </metadata>
          <title>
            <div xmlns="http://www.w3.org/1999/xhtml">
              <span class="{local-name(.)}" xmlns="http://www.w3.org/1999/xhtml">
                <xsl:value-of select="local-name(.)"/>
              </span>
            </div>
          </title>
          <xsl:apply-templates select="* except METAS" mode="structuralNode"/>
        </structuralNode>
      </xsl:when>
      <xsl:otherwise>
        <xsl:if test="CONCLUSION">
          <xsl:message>[WARNING] Il y a un element CONCLUSION a traiter dans la section COMPLEMENTS.</xsl:message>
        </xsl:if>
        <xsl:if test="PEDAGO">
            <xsl:message>[WARNING] Il y a un element PEDAGO a traiter dans la section COMPLEMENTS.</xsl:message>
        </xsl:if>
        <xsl:if test="INDEX">
          <xsl:message>[WARNING] Il y a un element INDEX a traiter dans la section COMPLEMENTS.</xsl:message>
        </xsl:if>
        <xsl:if test="SOMMAIRE">
          <xsl:message>[WARNING] Il y a un element SOMMAIRE a traiter dans la section COMPLEMENTS.</xsl:message>
        </xsl:if>
        <xsl:if test="TBL-MAT">
          <xsl:message>[WARNING] Il y a un element TBL-MAT a traiter dans la section COMPLEMENTS.</xsl:message>
        </xsl:if>
        <xsl:if test="TBL-REF">
          <xsl:message>[WARNING] Il y a un element TBL-REF a traiter dans la section COMPLEMENTS.</xsl:message>
        </xsl:if>
        <xsl:if test="BIBLIOGRAPHIE">
          <xsl:message>[WARNING] Il y a un element BIBLIOGRAPHIE a traiter dans la section COMPLEMENTS.</xsl:message>
        </xsl:if>
        <xsl:if test="REFERENCES">
          <xsl:message>[WARNING] Il y a un element REFERENCES a traiter dans la section COMPLEMENTS.</xsl:message>
        </xsl:if>
        <xsl:if test="LEXIQUE">
          <xsl:message>[WARNING] Il y a un element LEXIQUE a traiter dans la section COMPLEMENTS.</xsl:message>
        </xsl:if>
        <xsl:if test="ACHEVE">
          <xsl:message>[WARNING] Il y a un element ACHEVE a traiter dans la section COMPLEMENTS.</xsl:message>
        </xsl:if>
        <xsl:if test="INFOLEG">
          <xsl:message>[WARNING] Il y a un element INFOLEG a traiter dans la section COMPLEMENTS.</xsl:message>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="ANNEXES" mode="structuralNode">
    <structuralNode>
      <xsl:attribute name="xf:id" select="dalloz:historicalId(.)"/>
      <metadata>
        <meta code="{concat($codePrefixMeta, 'ouvrages', 'Origine')}">
          <value><xsl:value-of select="local-name(.)"/></value>
        </meta>
        <xsl:copy-of select="dalloz:metaOldId(.)"/>
        <xsl:apply-templates select="@*" mode="metadata"/>
        <!--<xsl:apply-templates select="TITRE" mode="metadata"/>-->
      </metadata>
      <title>
        <div xmlns="http://www.w3.org/1999/xhtml">
          <xsl:choose>
            <xsl:when test="TITRE">
              <xsl:for-each select="TITRE//(TIT-NIV|TIT-NUM), TITRE/TIT-INTITULE/(TIT-INTT|TIT-COMPL)">
                <xsl:if test="position() &gt; 1">
                  <xsl:choose>
                    <xsl:when test="self::TIT-INTT or self::TIT-COMPL[not(preceding-sibling::TIT-INTT)]">
                      <xsl:value-of select="'&#160;-&#160;'"/>
                    </xsl:when>
                    <xsl:otherwise>
                      <xsl:value-of select="'&#160;'"/>
                    </xsl:otherwise>
                  </xsl:choose>
                </xsl:if>
                <xsl:apply-templates select="." mode="xfRes:title2html"/>
              </xsl:for-each>
            </xsl:when>
            <xsl:otherwise>
              <span class="{local-name(.)}" xmlns="http://www.w3.org/1999/xhtml">
                <xsl:value-of select="local-name(.)"/>
              </span>
            </xsl:otherwise>
          </xsl:choose>
        </div>
      </title>
      <xsl:apply-templates select="ANNEXE" mode="structuralNode"/>
    </structuralNode>
  </xsl:template> 
  
  <xsl:template match="NIVEAU1|NIVEAU2|NIVEAU3|NIVEAU4|NIVEAU5|NIVEAU6" mode="structuralNode">
    <structuralNode>
      <xsl:attribute name="xf:id" select="dalloz:historicalId(.)"/>
      <metadata>
        <meta code="{concat($codePrefixMeta, 'ouvrages', 'Origine')}">
          <value><xsl:value-of select="local-name(.)"/></value>
        </meta>
        <xsl:copy-of select="dalloz:metaOldId(.)"/>
        <xsl:apply-templates select="@*" mode="metadata"/>
        <xsl:apply-templates select="METAS" mode="metadata"/>
        <xsl:apply-templates select="TITRE" mode="metadata"/>
        <xsl:apply-templates select="NIVO-PRELIM/AUTEUR" mode="metadata"/>
      </metadata>
      <xsl:if test="TITRE">
        <title>
          <div xmlns="http://www.w3.org/1999/xhtml">
            <xsl:for-each select="TITRE//(TIT-NIV|TIT-NUM), TITRE/TIT-INTITULE/(TIT-INTT|TIT-COMPL)">
              <xsl:if test="position() &gt; 1">
                <xsl:choose>
                  <xsl:when test="self::TIT-INTT or self::TIT-COMPL[not(preceding-sibling::TIT-INTT)]">
                    <xsl:value-of select="'&#160;-&#160;'"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="'&#160;'"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:if>
              <xsl:apply-templates select="." mode="xfRes:title2html"/>
              <!--<span class="{local-name(.)}"><xsl:sequence select="xfRes:makeTitle(.)"/></span>-->
            </xsl:for-each>
          </div>
        </title>
      </xsl:if>
      <xsl:apply-templates select="* except METAS" mode="structuralNode"/>
    </structuralNode>
  </xsl:template>
  
  <xsl:template match="NIVO-CORPS" mode="structuralNode">
      <xsl:apply-templates select="* except METAS" mode="structuralNode"/>
  </xsl:template>
  
  <xsl:template match="NIVO-PRELIM" mode="structuralNode">
    <xsl:apply-templates select="RESUME|ENCADRE|ORIENTEUR|INTRO|CITATION" mode="structuralNode"/>
  </xsl:template>
  
  <xsl:template match="NIVO-COMPL" mode="structuralNode">
    <xsl:apply-templates select="* except METAS" mode="structuralNode"/>
  </xsl:template>
  
  <xsl:template match="TITRE" mode="structuralNode">
    <!-- TBD -->
  </xsl:template>
  
  <xsl:template match="ORIENTEUR|INTRO|NIV1|NIV2|NIV3|NIV4|NIV5|NIV6|NIV7|NIV8|NIV9|NIV10|NIV11|NIV12|CONCL" mode="structuralNode">
    <structuralNode>
      <xsl:attribute name="xf:id" select="dalloz:historicalId(.)"/>
      <metadata>
        <meta code="{concat($codePrefixMeta, 'ouvrages', 'Origine')}">
          <value><xsl:value-of select="local-name(.)"/></value>
        </meta>
        <xsl:copy-of select="dalloz:metaOldId(.)"/>
        <xsl:apply-templates select="@*" mode="metadata"/>
        <xsl:apply-templates select="METAS" mode="metadata"/>
        <xsl:apply-templates select="TITRE" mode="metadata"/>
      </metadata>
      <xsl:if test="TITRE">
        <title>
          <div xmlns="http://www.w3.org/1999/xhtml">
            <xsl:for-each select="TITRE//(TIT-NIV|TIT-NUM), TITRE/TIT-INTITULE/(TIT-INTT|TIT-COMPL)">
              <xsl:if test="position() &gt; 1">
                <xsl:choose>
                  <xsl:when test="self::TIT-INTT or self::TIT-COMPL[not(preceding-sibling::TIT-INTT)]">
                    <xsl:value-of select="'&#160;-&#160;'"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="'&#160;'"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:if>
              <xsl:apply-templates select="." mode="xfRes:title2html"/>
              <!--<span class="{local-name(.)}"><xsl:sequence select="xfRes:makeTitle(.)"/></span>-->
            </xsl:for-each>
          </div>
        </title>
      </xsl:if>
      <xsl:apply-templates select="* except METAS" mode="structuralNode"/>
    </structuralNode>
  </xsl:template>
  
  <xsl:template match="NIV-TEXTE" mode="structuralNode"/>
  
  <xsl:template match="NIV-TEXTE[PARA|PARA-LEGIS|PARA-JURIS|PARA-QUEST|PARA-BIBLIO|PARA-PLAN]" mode="structuralNode">
    <xsl:apply-templates select="* except METAS" mode="structuralNode"/>
  </xsl:template>
 
  <xsl:template match="PGTITRE" mode="structuralNode">
    <!-- TBD -->
  </xsl:template>
  
  <xsl:template match="INDEX" mode="structuralNode">
    <!-- TBD -->
  </xsl:template>
  
  <xsl:template match="ANNEXE" mode="structuralNode">
    <referenceNode code="{concat($codePrefixTr, 'accrochage_Ouvrage_OuvrageSecondaire')}">
      <ref xf:targetResId="{dalloz:historicalId(.)}" xf:refType="xf:referenceNode">
        <xsl:variable name="entityCode" select="concat($codePrefixTee, $codeModele, 'Secondaire')" as="xs:string"/>
        <editorialEntity code="{$entityCode}"
          xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
          xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst">
          <xsl:attribute name="xf:id" select="dalloz:historicalId(.)"/>
          <xsl:sequence select="xfSAS:addAttributeOwnerGroup($entityCode, ouvrages:getProduct($idroot))"/>
          <xsl:call-template name="xfSAS:addAttributeCreationUser"/>
          <metadata>
            <meta code="{concat($codePrefixMeta, 'ouvrages', 'Origine')}">
              <value><xsl:value-of select="local-name(.)"/></value>
            </meta>
            <!-- <xsl:copy-of select="dalloz:metaTitle(.)"/> -->
            <xsl:copy-of select="dalloz:metaOldId(.)"/>
            <xsl:apply-templates select="@*" mode="metadata"/>
            <xsl:apply-templates select="METAS" mode="metadata"/>
            <xsl:apply-templates select="TITRE" mode="metadata"/>
          </metadata>
          <body>
            <xsl:apply-templates select="* except METAS" mode="structuralNode"/>
          </body>
        </editorialEntity>
      </ref>
    </referenceNode>
  </xsl:template>
  
  <xsl:template match="ANX" mode="structuralNode">
    <referenceNode code="{concat($codePrefixTr, 'accrochage_Ouvrage_OuvrageSecondaire')}">
      <ref xf:targetResId="{dalloz:historicalId(.)}" xf:refType="xf:referenceNode">
        <xsl:variable name="entityCode" select="concat($codePrefixTee, $codeModele, 'Secondaire')" as="xs:string"/>
        <editorialEntity code="{$entityCode}"
          xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
          xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst">
          <xsl:attribute name="xf:id" select="dalloz:historicalId(.)"/>
          <xsl:sequence select="xfSAS:addAttributeOwnerGroup($entityCode, ouvrages:getProduct($idroot))"/>
          <xsl:call-template name="xfSAS:addAttributeCreationUser"/>
          <metadata>
            <meta code="{concat($codePrefixMeta, 'ouvrages', 'Origine')}">
              <value><xsl:value-of select="local-name(.)"/></value>
            </meta>
            <!-- <xsl:copy-of select="dalloz:metaTitle(.)"/> -->
            <xsl:copy-of select="dalloz:metaOldId(.)"/>
            <xsl:apply-templates select="@*" mode="metadata"/>
            <xsl:apply-templates select="METAS" mode="metadata"/>
            <xsl:apply-templates select="TITRE" mode="metadata"/>
          </metadata>
          <body>
            <xsl:apply-templates select="* except METAS" mode="structuralNode"/>
          </body>
        </editorialEntity>
      </ref>
    </referenceNode>
  </xsl:template>
  
  <xsl:template match="PARA|PARA-LEGIS|PARA-JURIS|PARA-QUEST|PARA-BIBLIO|PARA-PLAN|RESUME|ENCADRE|CITATION" mode="structuralNode">
    <referenceNode>
      <xsl:choose>
        <xsl:when test="ancestor::ANNEXE or ancestor::ANX">
          <xsl:attribute name="code" select="concat($codePrefixTr, 'accrochage_OuvrageSecondaire_OuvragePara')"/> 
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="code" select="concat($codePrefixTr, 'accrochage_Ouvrage_OuvragePara')"/> 
        </xsl:otherwise>
      </xsl:choose>
      <ref xf:targetResId="{dalloz:historicalId(.)}" xf:refType="xf:referenceNode">
        <xsl:if test="$treeEntityOnly=false()">
          <xsl:variable name="entityCode" select="concat($codePrefixTee, $codeModele, 'Paragraphe')" as="xs:string"/>
          <editorialEntity xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
            xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
            code="{$entityCode}">
            <xsl:attribute name="xf:id" select="dalloz:historicalId(.)"/>
            <xsl:sequence select="xfSAS:addAttributeOwnerGroup($entityCode, ouvrages:getProduct($idroot))"/>
            <xsl:call-template name="xfSAS:addAttributeCreationUser"/>
            <metadata>
              <meta code="{concat($codePrefixMeta, 'ouvrages', 'Origine')}">
                <value><xsl:value-of select="local-name(.)"/></value>
              </meta>
              <xsl:copy-of select="dalloz:metaOldId(.)"/>
              <xsl:apply-templates select="@*" mode="metadata"/>
              <xsl:apply-templates select="NUME" mode="metadata"/>
              <xsl:apply-templates select="METAS" mode="metadata"/>
            </metadata>
            <body>
              <xsl:apply-templates select="." mode="contentNode"/>
            </body>
          </editorialEntity>
        </xsl:if>
      </ref>
    </referenceNode>
  </xsl:template>
  
  <xsl:template match="PARA|PARA-LEGIS|PARA-JURIS|PARA-QUEST|PARA-BIBLIO|PARA-PLAN|RESUME|ENCADRE|CITATION" mode="contentNode">
    <contentNode>
      <content>
        <xsl:apply-templates select="." mode="convertNamespace">
          <xsl:with-param name="namespace.uri" select="$contentNode.namespace.uri" tunnel="yes"/>
        </xsl:apply-templates>
      </content>
    </contentNode>
  </xsl:template>
  
  <xsl:template match="*" mode="contentNode">
    <xsl:message>Content Mode Called on <xsl:value-of select="local-name(.)"></xsl:value-of></xsl:message>
  </xsl:template>
  
  <!--==================================================-->
  <!--MODE convertNamespace -->
  <!--==================================================-->
  
  <xsl:template match="METAS" mode="convertNamespace"/>
  
  <xsl:template match="PLAN" mode="convertNamespace">
    <xsl:param name="namespace.uri" required="yes" tunnel="yes"/>
    <xsl:element name="{local-name(.)}" namespace="{$namespace.uri}">
      <xsl:apply-templates select="@PROFONDEUR" mode="#current"/>
    </xsl:element>
  </xsl:template>
  
  
  <xsl:template match="TABLE-CALS" mode="convertNamespace">
    <xsl:param name="namespace.uri" required="yes" tunnel="yes"/>
    <xsl:element name="table" namespace="{$namespace.uri}">
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="*" mode="convertNamespace" priority="-1">
    <xsl:param name="namespace.uri" required="yes" tunnel="yes"/>
    <xsl:element name="{local-name(.)}" namespace="{$namespace.uri}">
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="convertNamespace" priority="-2">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*"/>
    </xsl:copy>
  </xsl:template>
  
  <!--==================================================-->
  <!--MODE fixIds -->
  <!--==================================================-->
  
  <xsl:mode name="fixIds" on-no-match="shallow-copy"/>
  
  <xsl:template match="(@id|@idRef)" mode="fixIds">
    <xsl:attribute name="{name()}" select="replace(., '/', '_slash_')"/>
  </xsl:template>
  
  <xsl:function name="ouvrages:getProduct" as="xs:string">
    <xsl:param name="idroot" as="xs:string"/>
    <xsl:variable name="productkey">
      <xsl:variable name="idrootparts" select="tokenize($idroot, '/')"/>
      <xsl:choose>
        <xsl:when test="$idrootparts[1]='DZ'">
          <xsl:value-of select="concat($idrootparts[2], '/', $idrootparts[3])"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat($idrootparts[1], '/', $idrootparts[2])"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ouvrages.groupes" select="map { 
      'ACTION/BAUX-COMMERCIAUX'                 : 'PBC',
      'ACTION/CASSATION-MATIERE-PENALE'         : 'DACASPEN',
      'ACTION/CONSTRUCTION'                     : 'CONSTR',
      'ACTION/COPROPRIETE'                      : 'COPRO',
      'ACTION/DROIT-AUTEUR'                     : 'DAAUTEUR',
      'ACTION/EXECUTION-PEINES'                 : 'DAEXPEINES',
      'ACTION/EXPERTISE'                        : 'DAEXPERTISE',
      'ACTION/FAMILLE'                          : 'DAFAMILLE',
      'ACTION/FONDS-COMMERCE'                   : 'DAFONCOM',
      'ACTION/INSTRUCTION-PREPARATOIRE'         : 'DAINSPREP',
      'ACTION/PATRIMONIAL-FAMILLE'              : 'DAPATRI',
      'ACTION/PROCEDURE-CIVILE'                 : 'DAPROCIV',
      'ACTION/PROCEDURES-COLLECTIVES'           : 'DAPROCOLL',
      'ACTION/RESPONSABILITE'                   : 'RESPONSABILITE',
      'ACTION/RESPONSABILITE-CONTRATS'          : 'RESPONSABILITE',
      'DELMAS/PROCEDURES-COLLECTIVES'           : 'DELPROCOLL',
      'DZ/ACTION/BAUX-COMMERCIAUX'              : 'PBC',
      'DZ/ACTION/CASSATION-MATIERE-PENALE'      : 'DACASPEN',
      'DZ/ACTION/CONSTRUCTION'                  : 'CONSTR',
      'DZ/ACTION/COPROPRIETE'                   : 'COPRO',
      'DZ/ACTION/DROIT-AUTEUR'                  : 'DAAUTEUR',
      'DZ/ACTION/EXECUTION-PEINES'              : 'DAEXPEINES',
      'DZ/ACTION/EXPERTISE'                     : 'DAEXPERTISE',
      'DZ/ACTION/FAMILLE'                       : 'DAFAMILLE',
      'DZ/ACTION/FONDS-COMMERCE'                : 'DAFONCOM',
      'DZ/ACTION/INSTRUCTION-PREPARATOIRE'      : 'DAINSPREP',
      'DZ/ACTION/PATRIMONIAL-FAMILLE'           : 'DAPATRI',
      'DZ/ACTION/PROCEDURE-CIVILE'              : 'DAPROCIV',
      'DZ/ACTION/PROCEDURES-COLLECTIVES'        : 'DAPROCOLL',
      'DZ/ACTION/RESPONSABILITE'                : 'RESPONSABILITE',
      'DZ/ACTION/RESPONSABILITE-CONTRATS'       : 'RESPONSABILITE',
      'DZ/DELMAS/PROCEDURES-COLLECTIVES'        : 'DELPROCOLL',
      'DZ/JURIS-CORPUS/ASSOCIATIONS-FONDATIONS' : 'JCDAF',
      'DZ/PCA/CONTENTIEUX-ADMINISTRATIF'        : 'PCA',
      'DZ/REFERENCE/AFFAIRES'                   : 'DRCONTRAFF', 
      'DZ/REFERENCE/APPEL'                      : 'APPEL',
      'DZ/REFERENCE/BAUX-HABITATION'            : 'DRBAUX',
      'DZ/REFERENCE/CABINET-AVOCAT'             : 'DRCABAV',
      'DZ/REFERENCE/CABINET-AVOCATS'            : 'DRCABAV',
      'DZ/REFERENCE/DIVORCE'                    : 'DRDIVORCE',
      'JURIS-CORPUS/ASSOCIATIONS-FONDATIONS'    : 'JCDAF',
      'PCA/CONTENTIEUX-ADMINISTRATIF'           : 'PCA',
      'REFERENCE/AFFAIRES'                      : 'DRCONTRAFF', 
      'REFERENCE/APPEL'                         : 'APPEL',
      'REFERENCE/BAUX-HABITATION'               : 'DRBAUX',
      'REFERENCE/CABINET-AVOCAT'                : 'DRCABAV',
      'REFERENCE/CABINET-AVOCATS'               : 'DRCABAV',
      'REFERENCE/DIVORCE'                       : 'DRDIVORCE'
      }"/>
    <xsl:choose>
      <xsl:when test="map:contains($ouvrages.groupes, $productkey)">
        <xsl:value-of select="map:get($ouvrages.groupes, $productkey)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="''"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

</xsl:stylesheet>
