﻿<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:map="http://www.w3.org/2005/xpath-functions/map" 
  xmlns:file="http://expath.org/ns/file"
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" 
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els" 
  xmlns:dalloz="http://www.lefebvre-sarrut.eu/ns/dalloz" 
  xmlns:ouvrages="http://www.lefebvre-sarrut.eu/ns/dalloz/ouvrages"
  xmlns:dz="http://lefebvre-sarrut.eu/ns/dalloz/mapping-id"
  xmlns:ml="fr:askjadev:xml:extfunctions" 
  xmlns:h="http://www.w3.org/1999/xhtml" 
  xmlns:xfSAS="http://www.lefebvre-sarrut.eu/ns/xmlfirst/sas"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst"  
  extension-element-prefixes="ml"
  exclude-result-prefixes="#all" 
  version="3.0">
  
  <xsl:param name="ml:server" as="xs:string" required="true" />
  <xsl:param name="ml:port" as="xs:string" required="true" />
  <xsl:param name="ml:user" as="xs:string" required="true" />
  <xsl:param name="ml:password" as="xs:string" required="true" />
  <xsl:param name="dz:env" as="xs:string" required="true"/>
  
  <xsl:output method="xml" indent="yes"/>
  
  <xsl:mode name="adjustAllIds" on-no-match="shallow-copy"/>
  <xsl:mode name="cleanIds" on-no-match="shallow-copy"/>
  
  <xsl:variable name="tableDoc" 
    select="concat('http://', $ml:server, ':', $ml:port, '/v1/documents?uri=/', lower-case($dz:env), '/dalloz/mapping-id/',  /OUVRAGE-DALLOZ/@ID, '.xml')"/>
  
  
  <xsl:template match="/" mode="adjustAllIds">
      <xsl:try>
        <xsl:variable name="idTable" select="doc($tableDoc)"/>
        <xsl:variable name="dateModif" select="$idTable/dz:mappings/@date" as="xs:string"/>
        <xsl:variable name="idTableEntry" select="function($id as xs:string) as item()* {
          let $entry:=$idTable/dz:mappings/dz:mapping[@ID=$id][@date=$dateModif] return $entry
          }"/>
        <xsl:variable name="step" as="document-node()">
          <xsl:apply-templates select="." mode="ajustIDs">
            <xsl:with-param name="idTableEntry" select="$idTableEntry" tunnel="yes"/>
          </xsl:apply-templates>
        </xsl:variable>
        <xsl:sequence select="$step"/>
      
      <xsl:catch>
        <xsl:apply-templates select="." mode="cleanIds"/>
      </xsl:catch>
        
      </xsl:try>
  </xsl:template>
  
  <xsl:template match="*[@ID]" mode="cleanIds">
    <xsl:copy>
      <xsl:attribute name="ID_LOGIC" select="@ID"/>
      <xsl:apply-templates select="@*" mode="cleanIds"/>
      <xsl:apply-templates select="node()" mode="cleanIds"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="@ID_PREC" mode="cleanIds"/>
  
  <xsl:template match="/|node()|@*" mode="ajustIDs">
    <xsl:param name="idTableEntry" tunnel="yes"/>
    <xsl:copy>
      <xsl:apply-templates select="node()|@*" mode="ajustIDs">
        <xsl:with-param name="idTableEntry" select="$idTableEntry" tunnel="yes"/>
      </xsl:apply-templates>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="*[@ID]" mode="ajustIDs">
    <xsl:param name="idTableEntry" tunnel="yes"/>
    <!-- <xsl:message><xsl:value-of select="name(.)"/></xsl:message>-->
    <xsl:choose>
      <xsl:when test="local-name()='OUVRAGE-DALLOZ'">
        <xsl:copy>
          <xsl:apply-templates select="*|@*" mode="ajustIDs">
            <xsl:with-param name="idTableEntry" select="$idTableEntry" tunnel="yes"/>
          </xsl:apply-templates>
        </xsl:copy>
      </xsl:when>
      <xsl:when test="count($idTableEntry(./@ID))!=0">
        <xsl:copy>
          <xsl:variable name="element" select="$idTableEntry(./@ID)[1]"/>
          <!--<xsl:message terminate="yes"><xsl:copy-of select="$idTableEntry(./@ID)"/></xsl:message>-->
            <xsl:attribute name="ID" select="$element/@ID_HISTORICAL"/>
            <xsl:attribute name="ID_LOGIC" select="@ID"/>
          <xsl:apply-templates select="*|@* except (@ID|@ID_ALT|@ID_PREC|@ID_OLD)" mode="ajustIDs">
            <xsl:with-param name="idTableEntry" select="$idTableEntry" tunnel="yes"/>
          </xsl:apply-templates>
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:copy>
          <!-- Nouvel élément on crée l'ID historique -->
          <xsl:attribute name="ID" select="@ID"/>
          <xsl:attribute name="ID_LOGIC" select="@ID"/>
          <xsl:apply-templates select="*|@* except (@ID|@ID_ALT|@ID_PREC|@ID_OLD)" mode="ajustIDs">
            <xsl:with-param name="idTableEntry" select="$idTableEntry" tunnel="yes"/>
          </xsl:apply-templates>
        </xsl:copy>
        <!--<xsl:message terminate="1">[ERROR] ID <xsl:value-of select="@ID"/> had no mapping</xsl:message>-->
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>
  
  <xsl:template match="*[@REFID]" mode="ajustIDs">
    <xsl:param name="idTableEntry" tunnel="yes"/>
    <xsl:choose>
      <xsl:when test="count($idTableEntry(./@REFID))!=0">
        <xsl:copy>
          <xsl:variable name="element" select="$idTableEntry(./@REFID)[1]"/>
          <xsl:attribute name="REFID" select="$element/@ID_HISTORICAL"/>
          <xsl:apply-templates select="*|@* except (@REFID)" mode="ajustIDs">
            <xsl:with-param name="idTableEntry" select="$idTableEntry" tunnel="yes"/>
          </xsl:apply-templates>
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:copy> 
          <xsl:apply-templates select="*|@*" mode="ajustIDs">
            <xsl:with-param name="idTableEntry" select="$idTableEntry" tunnel="yes"/>
          </xsl:apply-templates>
        </xsl:copy>
        <!--<xsl:message terminate="1">[ERROR] ID <xsl:value-of select="@ID"/> had no mapping</xsl:message>-->
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:function name="ouvrages:getDeletedIds">
    <xsl:try>
      <xsl:variable name="idTable" select="doc($tableDoc)"/>
      <xsl:variable name="dateModif" select="$idTable/dz:mappings/@date" as="xs:string"/>
      <xsl:variable name="idTableEntry" select="function($id as xs:string) as item()* {
        let $entry:=$idTable/dz:mappings/dz:mapping[@ID=$id][@date=$dateModif] return $entry
        }"/>
        <xsl:sequence>
          <xsl:for-each select="$idTable/dz:mappings/dz:mapping[@deleted=$dateModif]">
          <xsl:choose>
            <xsl:when test="@node-name=('PARA', 'PARA-LEGIS', 'PARA-JURIS', 'PARA-QUEST', 'PARA-BIBLIO', 'PARA-PLAN')">
              <xsl:value-of select="@ID_HISTORICAL"/>
            </xsl:when>
          </xsl:choose>
        </xsl:for-each>
        </xsl:sequence>
      
      <xsl:catch/>
      
      </xsl:try>
  </xsl:function>
  
</xsl:stylesheet>
