<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:map="http://www.w3.org/2005/xpath-functions/map"
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" 
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els" 
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  xmlns:xfSAS="http://www.lefebvre-sarrut.eu/ns/xmlfirst/sas"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all" 
  version="3.0">
  
  <xsl:import href="../xslLib/xslLib/els-common.xsl"/>
  <xsl:import href="../xslLib/xslLib/els-log.xsl"/>
  <!-- Projet ecmRes - beacause metadata will be pre-calcultated here-->  
  <xsl:import href="../xf-ecmRes/_common/getCalculatedMeta.xsl"/>
  
  <!--SAS utilities-->
  <xsl:import href="common/sas.utils.xsl"/>

  <!--KEY-->
  <xsl:key name="getRelationByXfId" match="relation" use="@xf:id"/>
  
  <!--PARAMETERS-->
  
  <xsl:param name="xfSAS:debug" select="false()" as="xs:boolean"/>
  <xsl:param name="xfSAS:LOG.URI" select="resolve-uri('log', base-uri(.))" as="xs:string"/>
  <xsl:variable name="xfSAS:log.uri" select="if(ends-with($xfSAS:LOG.URI, '/')) then ($xfSAS:LOG.URI) else(concat($xfSAS:LOG.URI, '/'))" as="xs:string"/>
  
  <!--dalloz, efl, el...-->
  <xsl:param name="xfSAS:company" required="yes" as="xs:string"/>
  <xsl:param name="xfSAS:rel.to.schema" select="'xf-models:/xf-models/editorialEntity_sas/'"/>
  <!-- pour activer ou descativer les requetes historicalId dans la fonction makeHistoricIdAttribute -->
  <xsl:param name="xfSAS:enableHistoricalId" required="no" as="xs:boolean" select="true()"/>
  
  <!--========================================================================================-->
  <!-- MODE "xfSAS:deduplicateRefNodeEntity" -->
  <!--========================================================================================-->
  <!--When 2 referenceNode contains the same EE, only keep the last one-->
  
  <xsl:template match="referenceNode/ref/editorialEntity" mode="xfSAS:deduplicateRefNodeEntity">
    <xsl:choose>
      <xsl:when test="following::referenceNode/ref/editorialEntity[@xf:id = current()/@xf:id]">
        <!--Suppression de l'EE en double-->
        <xsl:message>[INFO][xfSAS:deduplicateRefNodeEntity] deduplicate referenceNode to EE id="<xsl:value-of select="@xf:id"/>"</xsl:message>
      </xsl:when>
      <xsl:otherwise>
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xfSAS:deduplicateRefNodeEntity">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--========================================================================================-->
  <!-- MODE "xfSAS:fixMeta" -->
  <!--========================================================================================-->
  <!--Fixing metadata : clean and add calculated metadata from repo "sie-xf-ecm-resources"-->
  
  <!--Delete metadata element if empty -->
  <xsl:template match="*[not(self::editorialEntity)]/metadata[not(*)]" mode="xfSAS:fixMeta" priority="1"/>

  <xsl:template match="editorialEntity/metadata" mode="xfSAS:fixMeta">
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <xsl:call-template name="xfSAS:addEditorialEntityMeta">
        <xsl:with-param name="editorialEntity" select="parent::editorialEntity"/>
      </xsl:call-template>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="editorialEntity[not(metadata)]" mode="xfSAS:fixMeta">
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <metadata>
        <xsl:call-template name="xfSAS:addEditorialEntityMeta">
          <xsl:with-param name="editorialEntity" select="."/>
        </xsl:call-template>
      </metadata>
      <xsl:apply-templates mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--context node must be editorialEntity/metadata-->
  <xsl:template name="xfSAS:addEditorialEntityMeta">
    <xsl:param name="editorialEntity" as="element(editorialEntity)"/>
    <!--Add calculated metadata-->
    <xsl:comment>CALCULATED METADATA</xsl:comment>
    <xsl:variable name="calculatedMetadata" as="element(metadata)">
      <xsl:apply-templates select="$editorialEntity" mode="xfRes:getCalculatedMetadata"/>
    </xsl:variable>
    <xsl:sequence select="$calculatedMetadata/*"/>
    <xsl:variable name="generatedMetadata" as="element(metadata)">
      <xsl:apply-templates select="$editorialEntity/metadata" mode="xf:normalizeMetadata"/>
    </xsl:variable>
    <xsl:if test="count($generatedMetadata/meta) != 0">
      <xsl:comment>GENERATED METADATA</xsl:comment>
      <xsl:sequence select="$generatedMetadata/*"/>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="structuralNode/metadata" mode="xfSAS:fixMeta">
    <xsl:apply-templates select="." mode="xf:normalizeMetadata"/>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xfSAS:fixMeta">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>

  <!--========================================================================================-->
  <!--MODE "xfSAS:deduplicateRelations"-->
  <!--========================================================================================-->
  <!--supprime les relations en double d'une EE (non géré côté ECM)-->
  <!-- Note: l'import SAS de l'ECM ne gère pas correctement deux relations (cibles identiques) dans une même EE -->
  <!-- La conversion SAS doit alors faire le ménage pour ne pas avoir deux relations identiques                 -->
  <!-- Les liens sont modifiés en fonction                                                                      -->
  
  <xd:doc>
    <xd:desc>
      <xd:p>Gestion des liens dans le flux XML</xd:p>
      <xd:p>Si nécessaire, modification de l'attribut @xf:idRef si la relation correspondante est dupliquée.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="contentNode/content//*[@xf:refType = 'xf:relationRef']" mode="xfSAS:deduplicateRelations">
    <!-- note: root() est utilisé pour unqiuement géré les éléments du document courant -->        
    <xsl:variable name="relationElement" select="key('getRelationByXfId', @xf:idRef, ancestor::editorialEntity[1])" as="element(relation)"/>
    <!-- Récupération des relations identiques à celle ciblée par le lien -->
    <xsl:variable name="precedingSiblingEqualRelation" 
      select="$relationElement/preceding-sibling::relation[xfSAS:compareElement-exceptAttributes(., $relationElement, ('xf:id'))]" 
      as="element(xf:relation)*"/>
    <xsl:choose>
      <xsl:when test="count($precedingSiblingEqualRelation/node()) > 0">
        <!-- Récupération du xf:relation/@xf:id souhaité : première relation/@xf:id (préceding-sibling) équivalente-->
        <xsl:variable name="newXfIdRef" select="$precedingSiblingEqualRelation[1]/@xf:id" as="xs:string"/>
        <!-- Recopie l'élément XML et modifie l'attribut xf:idRef -->
        <xsl:copy>
          <!-- Ajour d'une trace de la modification de la cible du lien-->
          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
            <xsl:with-param name="level" select="'info'"/>
            <xsl:with-param name="code" select="$els:log.level.code"/>
            <xsl:with-param name="alert" select="true()"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="description">[RELATION][DEDUPLICATION] Modification de la cible (&lt;relation&gt;) du lien &lt;<xsl:value-of select="local-name(.)"/>&gt; : @xf:idRef <xsl:value-of select="@xf:idRef"/> =&lt; <xsl:value-of select="$newXfIdRef"/></xsl:with-param>      
          </xsl:call-template>                                        
          <!-- Si $newXfIdRef n'est pas trouvé, l'EE totale convertie est alors en erreur. D'où la présence de l'"except @xf:idRef" -->                    
          <xsl:copy-of select="@* except @xf:idRef"/>                    
          <xsl:attribute name="xf:idRef" select="$newXfIdRef"/>                   
          <xsl:apply-templates mode="#current"/>
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <!-- Recopie tel quel l'élément -->
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Suppression de la relation en double</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="relation[xfSAS:sequence-deep-equal-exceptAttributes(., preceding-sibling::relation, ('xf:id'))]" mode="xfSAS:deduplicateRelations"/>            
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne true() au moins un élément de $elementsList est égal à l'élément $element</xd:p>
      <xd:p>Inspiration du dev : http://www.xsltfunctions.com/xsl/functx_sequence-deep-equal.html</xd:p>
    </xd:desc>
    <xd:param name="element">élément</xd:param>
    <xd:param name="elementsList">liste d'éléments à comparer</xd:param>
    <xd:param name="exceptedAttributes">liste d'attributs de l'élément à ne pas comparer</xd:param>
  </xd:doc>
  <xsl:function name="xfSAS:sequence-deep-equal-exceptAttributes" as="xs:boolean">
    <xsl:param name="element" as="element()"/>
    <xsl:param name="elementsList" as="element()*"/>
    <xsl:param name="exceptedAttributes" as="xs:string*"/>        
    <xsl:sequence select="
      some $i in 1 to count($elementsList) satisfies (
      xfSAS:compareElement-exceptAttributes($element, $elementsList[$i], $exceptedAttributes)
      )"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne true() si les deux éléments XML sont identiques (hormis les attributs définis par $exceptedAttributes)</xd:p>
      <xd:p>Comparaison en deux temps :
        <xd:ul>
          <xd:li>Comparaison des éléments enfants</xd:li>
          <xd:li>Comparaison des attributs except les attributs définis par le paramètre $exceptedAttributes</xd:li>
        </xd:ul>
      </xd:p>
    </xd:desc>
    <xd:param name="element1">1ier élément à comparer</xd:param>
    <xd:param name="element2">2ième élément à comparer</xd:param>
    <xd:param name="exceptedAttributes">liste d'attributs de l'élément à ne pas comparer</xd:param>
  </xd:doc>
  <xsl:function name="xfSAS:compareElement-exceptAttributes" as="xs:boolean">
    <xsl:param name="element1" as="element()"/>
    <xsl:param name="element2" as="element()"/>
    <xsl:param name="exceptedAttributes" as="xs:string*"/>        
    <xsl:sequence select="
      (: comparaison des nodes enfants :)
      deep-equal($element1/node(), $element2/node())
      and (: comparaison des attributes except $exceptedAttributes :)
      deep-equal($element1/@* except $element1/@*[name(.) = $exceptedAttributes], $element2/@* except $element2/@*[name(.) = $exceptedAttributes])"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Traitement des autres noeuds et attributs (mode="xfSAS:deduplicateRelations")</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="node() | @*" mode="xfSAS:deduplicateRelations" priority="-1">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>

  <!--========================================================================================-->
  <!-- XF-SAS COMMON FUNCTIONS -->
  <!--========================================================================================-->
  
  <xsl:function name="xfSAS:makeHistoricIdAttribute" as="xs:string">
    <xsl:param name="e" as="element()"/>
    <xsl:param name="historicalId" as="xs:string?"/>
    <xsl:message>[WARNING] xfSAS:makeHistoricIdAttribute is deprecated, please use xf:makeHistoricalIdAttributeValue()</xsl:message>
    <xsl:variable name="prefix" select="if($xfSAS:company = 'els') then('') else(concat($xfSAS:company, '_'))" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="empty($historicalId)">
        <xsl:message>[WARNING] Pas d'id ici (<xsl:value-of select="els:get-xpath($e)"/>)!</xsl:message>
        <xsl:sequence select="''"/>
      </xsl:when>
      <xsl:when test="$xfSAS:enableHistoricalId">
        <xsl:value-of select="concat('{historicalId:', $prefix, $historicalId, '}')"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="concat($prefix, $historicalId)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <!--<xsl:function name="xfSAS:makeHistoricIdAttributeWithPrefix" as="xs:string">
    <xsl:param name="e" as="element()"/>
    <xsl:param name="historicalId" as="xs:string?"/>
    <xsl:param name="prefix" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="empty($historicalId)">
        <xsl:message>[WARNING] Pas d'id ici (<xsl:value-of select="els:get-xpath($e)"/>)!</xsl:message>
        <xsl:sequence select="''"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="if($xfSAS:enableHistoricalId) then concat('{historicalId:', $prefix, $historicalId, '}') else concat($prefix, $historicalId)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>-->

  <xsl:function name="xfSAS:makeQueryIdAttribute" as="xs:string">
    <xsl:param name="e" as="element()"/>
    <xsl:param name="metaValuePair" as="xs:string*"/>
    <!--ex : metaName1=value1, metaName2=value2, ...-->
    <xsl:message>[WARNING] xfSAS:makeQueryIdAttribute is deprecated, please use xf:makeQueryIdAttributeValue()</xsl:message>
    <xsl:choose>
      <xsl:when test="empty($metaValuePair)">
        <xsl:message>[WARNING] Aucune paire meta/valeur (<xsl:value-of select="els:get-xpath($e)"/>)!</xsl:message>
        <xsl:sequence select="''"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="result" as="xs:string*">
          <xsl:text>{query:</xsl:text>
          <xsl:for-each select="$metaValuePair[normalize-space(substring-before(., '='))!=''][normalize-space(substring-after(., '='))!='']">
            <xsl:value-of select="substring-before(., '=')"/>
            <xsl:text>=</xsl:text>
            <xsl:value-of select="encode-for-uri(substring-after(., '='))"/>
            <xsl:if test="not(position() = last())">
              <xsl:text>&amp;</xsl:text>
            </xsl:if>
          </xsl:for-each>
          <xsl:text>}</xsl:text>
        </xsl:variable>
        <xsl:sequence select="string-join($result, '')"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <xsl:template name="xfSAS:makeXmlModelPi">
    <xsl:value-of select="els:crlf()"/>
    <xsl:comment>
			<xsl:text>&lt;?xml-model type="application/xml" schematypens="http://purl.oclc.org/dsdl/schematron" href="</xsl:text>
			<xsl:value-of select="$xfSAS:rel.to.schema"/>
			<xsl:text>editorialEntity.sch"?&gt;</xsl:text>
    </xsl:comment>
    <xsl:value-of select="els:crlf()"/>
    <xsl:processing-instruction name="xml-model">
			<xsl:text>type="application/xml" schematypens="http://purl.oclc.org/dsdl/nvdl/ns/structure/1.0" href="</xsl:text>
			<xsl:value-of select="$xfSAS:rel.to.schema"/>
			<xsl:text>editorialEntity.nvdl"</xsl:text>
		</xsl:processing-instruction>
    <xsl:value-of select="els:crlf()"/>
    <xsl:comment>
			<xsl:text>&lt;?xml-model type="application/xml" schematypens="http://relaxng.org/ns/structure/1.0" href="</xsl:text>
			<xsl:value-of select="$xfSAS:rel.to.schema"/>
			<xsl:text>editorialEntity.rng"?&gt;</xsl:text>
		</xsl:comment>
    <xsl:value-of select="els:crlf()"/>
  </xsl:template>
  
  <xsl:template name="xfSAS:makeRngModelPi">
    <xsl:param name="path.to.rng.model" as="xs:string"/>
    <xsl:processing-instruction name="xml-model">
      <xsl:text>type="application/xml" schematypens="http://relaxng.org/ns/structure/1.0" href="</xsl:text>
      <xsl:value-of select="$path.to.rng.model"/>
      <xsl:text>"</xsl:text>
    </xsl:processing-instruction>
  </xsl:template>

  <xd:doc>
    <xd:desc>
      <p>Recopie des commentaires afin de les garder de step en step</p>
      <p>Besoin : commentaires des éléments &lt;els:log/&gt;</p>
    </xd:desc>    
  </xd:doc>
  <xsl:template match="comment()" mode="#all" priority="2">
    <xsl:copy/>
  </xsl:template>

  <xd:doc>
    <xd:desc>
      <xd:p><xd:b>Récupération du code ownerGroupe</xd:b></xd:p>
      <xd:p>Pour CXML : voir Jira FLAS-75</xd:p>
      <xd:p>Pour DZ : voir ...</xd:p>
    </xd:desc>
    <xd:param>codeTee : le codeTEE de l'editorialEntity</xd:param>
    <xd:return>Attribut ownerGroup avec la bonne valeur.</xd:return>
  </xd:doc>
  <xsl:function name="xfSAS:addAttributeOwnerGroup" as="attribute(ownerGroup)">
    <xsl:param name="codeTee" as="xs:string"/>
    <xsl:attribute name="ownerGroup">
      <xsl:choose>
        <!-- CXML -->
        <!-- mis en commentaire suite à la Jira FLAS-316 -->
<!--        <xsl:when test="$codeTee='EFL_TEE_bpim' or $codeTee='EFL_TEE_bdp' or $codeTee='EFL_TEE_baf'">
          <xsl:value-of select="'EFL_GRP_polePluridisciplinaire'"/>
        </xsl:when>
        <xsl:when test="$codeTee='EFL_TEE_acp' or $codeTee='EFL_TEE_frs' or $codeTee='EFL_TEE_bpci'">
          <xsl:value-of select="'EFL_GRP_redactionSociale'"/>
        </xsl:when>
        <xsl:when test="$codeTee='EFL_TEE_rjf' or $codeTee='EFL_TEE_frc' or $codeTee='EFL_TEE_frchs' or $codeTee='EFL_TEE_acdaf' or $codeTee='EFL_TEE_fr'">
          <xsl:value-of select="'EFL_GRP_redactionFiscale'"/>
        </xsl:when>
        <xsl:when test="$codeTee='EFL_TEE_brda' or $codeTee='EFL_TEE_brdaguide'">
          <xsl:value-of select="'EFL_GRP_redactionAffaires'"/>
        </xsl:when>-->
        <xsl:when test="$codeTee='EFL_TEE_srcUiTexte'">
          <!-- Jira FLAS-316 -->
          <xsl:value-of select="'EFL_GRP_sources'"/>
        </xsl:when>
        <!-- DZ -->
        <!-- revues -->
        <!-- POC -->
        <xsl:when test="$codeTee = ('DZ_TEE_revuesAJPEN_FLASDZ_279', 'DZ_TEE_revuesDossier_FLASDZ_279', 'DZ_TEE_revuesArticle_FLASDZ_279')">
          <xsl:value-of select="'DZ_GRP_revueAJPEN_FLASDZ_279'"/>
        </xsl:when>
        <xsl:when test="matches($codeTee, 'DZ_TEE_revues\p{Lu}{2,}')">
          <xsl:value-of select="concat('DZ_GRP_revue', tokenize($codeTee, 'DZ_TEE_revues')[2])"/>
        </xsl:when>
        <!-- ouvrages -->
        <xsl:when test="$codeTee = ('DZ_TEE_ouvrages', 'DZ_TEE_ouvragesParagraphe', 'DZ_TEE_ouvragesSecondaire', 'DZ_TEE_actualisationOuvrage')">
          <xsl:value-of select="'DZ_GRP_ouvrage'"/>
        </xsl:when>
        <!-- encyclo -->
        <xsl:when test="$codeTee = ('DZ_TEE_encycloRubrique', 'DZ_TEE_encycloParagraphe', 'DZ_TEE_colLoc')">
          <xsl:value-of select="'DZ_GRP_encyclopedie'"/>
        </xsl:when>
        <!-- formules -->
        <xsl:when test="$codeTee = ('DZ_TEE_formule', 'DZ_TEE_formuleRepliquee', 'DZ_TEE_formuleTexte')">
          <xsl:value-of select="'DZ_GRP_formule'"/>
        </xsl:when>
        <!-- actualite -->
        <xsl:when test="$codeTee = 'DZ_TEE_actualite'">
          <xsl:value-of select="'DZ_GRP_actualite'"/>
        </xsl:when>
        <!-- oasis -->
        <xsl:when test="$codeTee = 'DZ_TEE_ficheOasis'">
          <xsl:value-of select="'DZ_GRP_fichesOasis'"/>
        </xsl:when>
        <!-- codes -->
        <xsl:when test="$codeTee = ('DZ_TEE_codalloz', 'DZ_TEE_codes', 'DZ_TEE_codesArbreSecondaire', 'DZ_TEE_codesArti', 'DZ_TEE_codesElement')">
          <xsl:value-of select="'DZ_GRP_code'"/>
        </xsl:when>
        <!-- ELS -->
        <xsl:when test="$codeTee='ELS_TEE_jurisprudence'">
          <xsl:value-of select="'ELS_GRP_DSI'"/>
        </xsl:when>
        <!-- Autres maisons ...-->
        <xsl:otherwise>
          <xsl:value-of select="'NO_SPECIFIC_OWNER_GROUP_CODE'"/>
        </xsl:otherwise>
      </xsl:choose>    
    </xsl:attribute>
  </xsl:function>

  <xsl:function name="xfSAS:addAttributeOwnerGroup" as="attribute(ownerGroup)">
    <xsl:param name="codeTee" as="xs:string"/>
    <xsl:param name="produit" as="xs:string"/>
    <xsl:attribute name="ownerGroup">
      <xsl:choose>
        <xsl:when test="$produit=''">
          <xsl:sequence select="xfSAS:addAttributeOwnerGroup($codeTee)"/>
        </xsl:when>
        <!-- DZ -->
        <!-- encyclo -->
        <xsl:when test="$codeTee = ('DZ_TEE_encycloRubrique', 'DZ_TEE_encycloParagraphe', 'DZ_TEE_colLoc')">
          <xsl:value-of select="concat('DZ_GRP_encyclo', $produit)"/>
        </xsl:when>
        <!-- formules -->
        <xsl:when test="$codeTee = ('DZ_TEE_formule', 'DZ_TEE_formuleRepliquee', 'DZ_TEE_formuleTexte')">
          <xsl:value-of select="concat('DZ_GRP_form', $produit)"/>
        </xsl:when>
        <!-- ouvrages -->
        <xsl:when test="$codeTee = ('DZ_TEE_ouvrages', 'DZ_TEE_ouvragesParagraphe', 'DZ_TEE_ouvragesSecondaire', 'DZ_TEE_actualisationOuvrage')">
          <xsl:value-of select="concat('DZ_GRP_ouvrage', $produit)"/>
        </xsl:when>
        <!-- codes -->
        <xsl:when test="$codeTee = ('DZ_TEE_codalloz', 'DZ_TEE_codes', 'DZ_TEE_codesArbreSecondaire', 'DZ_TEE_codesArti', 'DZ_TEE_codesElement')">
          <xsl:choose>
            <xsl:when test="$produit='FRNK'">
              <xsl:sequence select="xfSAS:addAttributeOwnerGroup($codeTee)"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="concat('DZ_GRP_code', $produit)"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
          <xsl:sequence select="xfSAS:addAttributeOwnerGroup($codeTee)"/>
        </xsl:otherwise>
      </xsl:choose>   
    </xsl:attribute>
  </xsl:function>

  <xd:doc>
    <xd:desc>
      <xd:p>Convert media file name to be synchronised with the actions done by the ECM media import service</xd:p>
      <xd:p>See FLAS-360 Jira</xd:p>
      <xd:p>Remove all characters [^.\p{Z}\w_]</xd:p>
      <xd:p>Convert [.\p{Z}] to '_'</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:function name="xfSAS:normalizeMediaFileName" as="xs:string?">
    <xsl:param name="fileName" as="xs:string?"/>
    <xsl:variable name="els:replace-list" as="element(els:replace-list)">      
      <els:replace-list>
        <els:replace>
          <els:pattern>([^.<xsl:value-of select="$els:regAnySpace"/><xsl:value-of select="$els:regAnyLetter"/><xsl:value-of select="$els:regAnyNumber"/>_])</els:pattern>
          <els:replacement/>
        </els:replace>
        <els:replace>
          <els:pattern>([.<xsl:value-of select="$els:regAnySpace"/>])</els:pattern>
          <els:replacement>_</els:replacement>
        </els:replace>
      </els:replace-list>
    </xsl:variable>
    <xsl:sequence select="concat(
      els:replace-multiple(els:normalize-no-diacritic(els:getFileName($fileName, false())), $els:replace-list),
      '.', els:getFileExt($fileName))"/>
  </xsl:function>

</xsl:stylesheet>
