<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:ecm="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns="http://www.w3.org/1999/xhtml"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all"
  version="3.0">

  <xsl:import href="../../xslLib/xslLib/xml2simpleHtml.xsl"/>
  <xsl:import href="../_common/TEE_text.html.default.htmlSimplePreview.xsl"/>
  <xsl:import href="../_common/xf-common.htmlPreview.xsl"/>
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Generate a HTML simple preview for EE type tree.</xd:p>
      <xd:p>This xslt is not able to display contentNode properly, import from another xslt that knows how to display sepcific contentNode</xd:p>
    </xd:desc>
  </xd:doc>
  
  <!--indent="no" is important for xspec diff--> 
  <xsl:output method="xhtml" indent="no"/>
  
  <!--============================================================-->
  <!--INIT-->
  <!--============================================================-->
  
  <!--No modes for standalone execution-->
  <xsl:template match="/">
    <xsl:apply-templates select="." mode="xfRes:TEE_tree.default.htmlSimplePreview"/>
  </xsl:template>
  
  <!--Mode xfRes:htmlSimplePreview for using from insertMetaInsideBody.xsl-->
  <xsl:template match="/" mode="xfRes:htmlSimplePreview">
    <xsl:apply-templates select="." mode="xfRes:TEE_tree.default.htmlSimplePreview"/>
  </xsl:template>
  
  <!--============================================================-->
  <!--MAIN-->
  <!--============================================================-->
  
  <xsl:template match="/" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="." mode="xfRes:TEE_tree.default.htmlSimplePreview.adaptXML"/>
      </xsl:document>
    </xsl:variable>
    <xsl:if test="$xfRes:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri(concat(els:getFileName(static-base-uri()), '_step-1.log.xml'), $xfRes:log.uri)" as="xs:anyURI"/>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())" as="xs:string"/>
        <xsl:with-param name="code" select="'log.step'" as="xs:string"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template>
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <xsl:variable name="step" as="item()*">
      <xsl:apply-templates select="$step/editorialEntity" mode="#current"/>
    </xsl:variable>
    <xsl:choose>
      <!--When previewing a tree with $ecm:referenceNodeOnly='true' and referenced EE is a text EE, don't need to add <html>-->
      <xsl:when test="$step//html:body">
        <xsl:sequence select="$step"/>
      </xsl:when>
      <xsl:otherwise>
        <html>
          <xsl:call-template name="xfRes:htmlPreview.makeHtmlHead"/>
          <body>
            <xsl:call-template name="xfRes:htmlPreview.addDebugInfo"/>
            <xsl:sequence select="$step"/>
          </body>
        </html>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!--============================================================-->
  <!--MODE TEE_tree.default.htmlSimplePreview.adaptXML-->
  <!--============================================================-->
  
  <xd:p>SIEGEL-510 : When viewing only one referenceNode of a tree EE ignore the current EE:  display only the targeted EE</xd:p>
  <xsl:template match="/editorialEntity[$ecm:referenceNodeOnly = 'true']" mode="xfRes:TEE_tree.default.htmlSimplePreview.adaptXML">
    <xsl:apply-templates select="(body//referenceNode)[1]/ref/editorialEntity" mode="#current"/>
  </xsl:template>
  
  <xd:p>Override the EE systemTitle when it's called within a referenceNode which also have a systemTitle</xd:p>
  <xsl:template match="referenceNode/ref/editorialEntity/metadata/meta[@code = 'systemTitle']/value/html:div" mode="xfRes:TEE_tree.default.htmlSimplePreview.adaptXML">
    <xsl:variable name="referenceNode" select="ancestor::referenceNode[1]" as="element(referenceNode)"/>
    <xsl:choose>
      <xsl:when test="$ecm:referenceNodeOnly = 'true' and count($referenceNode/ancestor::editorialEntity) = 1">
        <xsl:variable name="referenceNode.systemTitle.value" select="xf:getMeta($referenceNode, 'systemTitle')/xf:getMetaValueContent(.)" as="item()*"/>
        <xsl:choose>
          <xsl:when test="not(els:is-empty-or-whitespace($referenceNode.systemTitle.value))">
            <xsl:copy>
              <xsl:apply-templates select="@*" mode="#current"/>
              <xsl:sequence select="$referenceNode.systemTitle.value/node()"/>
            </xsl:copy>
          </xsl:when>
          <xsl:otherwise>
            <xsl:next-match/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xfRes:TEE_tree.default.htmlSimplePreview.adaptXML">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--============================================================-->
  <!--MODE TEE_tree.default.htmlSimplePreview-->
  <!--============================================================-->

  <xsl:template match="editorialEntity" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:apply-templates select="metadata/meta[@code = 'systemTitle']" mode="#current"/>
    <xsl:apply-templates select="body" mode="#current"/>
  </xsl:template>
  
  <xsl:template match="editorialEntity/metadata/meta[@code = 'systemTitle']" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <!--FIXME SIEGEL-492: missing EE title as common behaviour ?-->
    <!--<div class="systemTitle level-{count(ancestor::editorialEntity)}">
      <xsl:variable name="editorialEntity" select="../../self::editorialEntity" as="element(editorialEntity)"/>
      <xsl:apply-templates select="$editorialEntity/@xf:id" mode="#current"/>
      <xsl:apply-templates select="value/html:div/node()" mode="#current"/>
    </div>-->
  </xsl:template>
  
  <xsl:template match="body" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <ul>
      <xsl:apply-templates mode="#current"/>
    </ul>
  </xsl:template>
  
  <xsl:template match="structuralNode" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:variable name="structuralNode.level" select="count(ancestor::structuralNode) + 1" as="xs:integer"/>
    <xsl:variable name="htmlElement.level" select="if($structuralNode.level le 6) then($structuralNode.level) else(6)" as="xs:integer"/>
    <xsl:variable name="htmlElement.name" select="concat('h', $htmlElement.level)" as="xs:string"/>
    <xsl:variable name="title.div" as="node()*">
      <xsl:apply-templates select="title/html:div" mode="#current"/>
    </xsl:variable>
    <li class="structuralNode level-{$structuralNode.level}">
      <xsl:apply-templates select="@xf:id" mode="#current"/>
      <xsl:element name="{$htmlElement.name}">
        <xsl:choose>
          <xsl:when test="normalize-space(string($title.div)) != ''">
            <xsl:copy-of select="$title.div/node()"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>[NO TITLE]</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:element>
      <xsl:if test="* except (metadata | title)">
        <ul>
          <xsl:apply-templates select="* except (metadata | title)" mode="#current"/>
        </ul>
      </xsl:if>
    </li>
  </xsl:template>
  
  <xsl:template match="queryNode" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <li class="queryNode">
      <xsl:text>[ERROR][queryNode] TODO</xsl:text>
    </li>
  </xsl:template>
  
  <xsl:template match="referenceNode" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <li class="referenceNode">
      <xsl:apply-templates select="@xf:id" mode="#current"/>
      <xsl:variable name="systemTitle.value" select="xf:getMeta(., 'systemTitle')/xf:getMetaValueContent(.)" as="item()*"/>
      <xsl:variable name="referenceNode.title" as="item()*">
        <xsl:choose>
          <xsl:when test="not(els:is-empty-or-whitespace($systemTitle.value))">
            <xsl:sequence select="$systemTitle.value/node()"/>
          </xsl:when>
          <xsl:when test="not(els:is-empty-or-whitespace(verbalization))">
            <xsl:apply-templates select="verbalization" mode="#current"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="(ref/@xf:targetResId, '[ERROR] No verbalization or id found')[1]"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <xsl:apply-templates select="ref" mode="#current">
        <xsl:with-param name="referenceNode.title" select="$referenceNode.title" as="item()*"/>
      </xsl:apply-templates>
      <!--in case there something (like els:log)-->
      <xsl:apply-templates select="ref/*" mode="#current"/>
    </li>
  </xsl:template>
  
  <!--only called for generating the referenceNode title, will be wrap into a link-->
  <xsl:template match="referenceNode/verbalization" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:choose>
      <xsl:when test="count(*/*) = 1  and */html:p">
        <xsl:apply-templates select="*/html:p/node()" mode="#current"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="*/node()" mode="#current"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="referenceNode/ref" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:param name="referenceNode.title" required="yes" as="item()*"/>
    <a href="{ecm:getEEpreviewUrl(@xf:targetResId)}" target="_blank">
      <xsl:sequence select="$referenceNode.title"/>
    </a>
  </xsl:template>
  
  <xsl:template match="contentNode" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <li class="contentNode">
      <xsl:apply-templates select="@xf:id" mode="#current"/>
      <xsl:apply-templates mode="#current"/>
    </li>
  </xsl:template>
  
  <xsl:template match="contentNode/content" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <div class="content">
      <xsl:apply-templates mode="#current"/>
    </div>
  </xsl:template>
  
  <xsl:template match="contentNode/content/*" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:choose>
      <xsl:when test="namespace-uri(.) = 'http://www.w3.org/1999/xhtml'">
        <xsl:apply-templates select="." mode="xfRes:TEE_text.html.default.htmlSimplePreview"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="msg" as="xs:string*">
          <xsl:text>ERROR : unable to display contentNode properly at </xsl:text>
          <xsl:value-of select="els:get-xpath(.)"/>
        </xsl:variable>
        <xsl:message><xsl:value-of select="$msg"/></xsl:message>
        <div class="error" style="background-color:red; color:white; padding:0.5em; font-family: monospace;">
          <h3 style="margin-bottom:0"><xsl:value-of select="$msg"/></h3>
          <div>
            <xsl:apply-templates select="content/*" mode="els:xml2simpleHtml"/>
          </div>
        </div>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!--+==================================================+
      | COMMON                                           |
      +==================================================+-->
  
  <xsl:template match="@xf:id" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:attribute name="id" select="."/>
  </xsl:template>
  
  <!--Default xf element : continue -->
  <xsl:template match="xf:*" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:apply-templates mode="#current"/>
  </xsl:template>
  
  <!--except metadata-->
  <xsl:template match="metadata" mode="xfRes:TEE_tree.default.htmlSimplePreview"/>
  
  <!--Default html element : copy-->
  <xsl:template match="html:*" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:copy copy-namespaces="no">
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
</xsl:stylesheet>