<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" 
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els" 
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all" 
  version="3.0">
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Get calculated metadata from any EE file</xd:p>
      <xd:p>If param $metaCode is not setted, then get all calculated metadat</xd:p>
      <xd:p>If param $metaCode is setted, then get the calculated metada whose code is asked for.</xd:p>
    </xd:desc>
  </xd:doc>
  
  <xsl:import href="xf-common.xsl"/>
  
  <!--Import all xsl to calculate metadata from any EE type-->
  <!--EFL-->
  <xsl:import href="../efl/EFL_TEE_uiCommentaire.getCalculatedMetadata.xsl"/>
  <xsl:import href="../efl/EFL_TEE_tree.default.getCalculatedMetadata.xsl"/>
  <xsl:import href="../efl/EFL_TEE_srcUiTexte.getCalculatedMetadata.xsl"/>
  <!--DALLOZ-->
  <xsl:import href="../dalloz/DZ_TEE_revues.getCalculatedMetadata.xsl"/>
  <xsl:import href="../dalloz/DZ_TEE_revues.poc.getCalculatedMetadata.xsl"/>
  <xsl:import href="../dalloz/DZ_TEE_ouvrages.getCalculatedMetadata.xsl"/>
  <xsl:import href="../dalloz/DZ_TEE_encyclo.getCalculatedMetadata.xsl"/>
  <xsl:import href="../dalloz/DZ_TEE_colLoc.getCalculatedMetadata.xsl"/>
  <xsl:import href="../dalloz/DZ_TEE_formule.getCalculatedMetadata.xsl"/>
  <xsl:import href="../dalloz/DZ_TEE_codes.getCalculatedMetadata.xsl"/>
  <xsl:import href="../dalloz/DZ_TEE_actualite.getCalculatedMetadata.xsl"/>
  <xsl:import href="../dalloz/DZ_TEE_ficheOasis.getCalculatedMetadata.xsl"/>
  <xsl:import href="../dalloz/DZ_TEE_guidanceTopicTree.getCalculatedMetadata.xsl"/>
  <xsl:import href="../dalloz/DZ_TEE_guidanceTopicPage.getCalculatedMetadata.xsl"/>
  <xsl:import href="../dalloz/DZ_TEE_guidancePracticeNote.getCalculatedMetadata.xsl"/>
  <xsl:import href="../dalloz/DZ_TEE_checklist.getCalculatedMetadata.xsl"/>
  <xsl:import href="../dalloz/DZ_TEE_auteur.getCalculatedMetadata.xsl"/>
  <!--EL-->
  <xsl:import href="../el/fichePratique.getCalculatedMetadata.xsl"/>
  <xsl:import href="../el/EL_TEE_fichePratique_FLASEL_171/EL_TEE_fichePratique_FLASEL_171.getCalculatedMetadata.xsl"/>
  <xsl:import href="../el/el_FICHE_QR.getCalculatedMetadata.xsl"/>
  <xsl:import href="../el/el_fichePro.getCalculatedMetadata.xsl"/>
  <xsl:import href="../el/el_ficheMT.getCalculatedMetadata.xsl"/>
  <xsl:import href="../el/el_etudes.getCalculatedMetadata.xsl"/>
  <xsl:import href="../el/el_actualites.getCalculatedMetadata.xsl"/>
  <xsl:import href="../el/el_outils.getCalculatedMetadata.xsl"/>
  <xsl:import href="../el/el_practiceNoteRedigee.getCalculatedMetadata.xsl"/>
	<xsl:import href="../el/el_tableauRecapitulatif.getCalculatedMetadata.xsl"/>
	<xsl:import href="../el/el_arbreGuidance.getCalculatedMetadata.xsl"/>
  <xsl:import href="../el/EL_TEE_pageGuidance/EL_TEE_pageGuidance.getCalculatedMetadata.xsl"/>
  <xsl:import href="../el/EL_TEE_arbreThemeGuidance/EL_TEE_arbreThemeGuidance.getCalculatedMetadata.xsl"/>
  <xsl:import href="../el/EL_TEE_practiceNoteAssemblee/EL_TEE_practiceNoteAssemblee.getCalculatedMetadata.xsl"/>
  <xsl:import href="../el/EL_TEE_checklist.getCalculatedMetadata.xsl"/>
  <xsl:import href="../el/el_outilsReferences.getCalculatedMetadata.xsl"/>
  <!--ELS-->
  <xsl:import href="../els/ELS_TEE_jrpJuri.getCalculatedMetatada.xsl"/>
  
  <xsl:output indent="yes"/>
  
  <!--$metaCode parameter : list of meta code as coma separated values, empty string means "all"-->  
  <xsl:param name="metaCode" as="xs:string" select="''"/>
  
  <!--version tokenizée-->
  <xsl:variable name="metaCode.token" select="tokenize(replace($metaCode, '\s', ''), ',')" as="xs:string*"/>
  
  <!--========================================================-->
  <!--INIT-->
  <!--========================================================-->
  
  <!--A way to launch the transformation without initial mode-->
  
  <xsl:template match="/" priority="-1">
    <xsl:apply-templates select="descendant::xf:editorialEntity" mode="xfRes:getCalculatedMetadata"/> 
  </xsl:template>
  
  <!--========================================================-->
  <!--MAIN-->
  <!--========================================================-->
  
  <!--No "/" because this xslt is also called from EE at SAS format--> 
  <xsl:template match="editorialEntity" mode="xfRes:getCalculatedMetadata">
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:choose>
          <!--=== EFL ===--> 
          <!--info EFL-->
          <xsl:when test="@code = 'EFL_TEE_uiCommentaire'">
            <xsl:apply-templates select="." mode="xfRes:EFL_TEE_infoCommentaire.getCalculatedMetatada"/>
          </xsl:when>
          <!--revues EFL-->
          <xsl:when test="@code = (
              'EFL_TEE_baf', 'EFL_TEE_bdp', 'EFL_TEE_bpat', 'EFL_TEE_bpim', 'EFL_TEE_brda', 'EFL_TEE_frc', 'EFL_TEE_frchs', 'EFL_TEE_rjf', 'EFL_TEE_acp', 'EFL_TEE_fr', 'EFL_TEE_frs', 'EFL_TEE_acdaf', 'EFL_TEE_brdaguide', 'EFL_TEE_bpci'
            , 'EFL_TEE_ACTAFF', 'EFL_TEE_ACTASSO', 'EFL_TEE_ACTCPT', 'EFL_TEE_ACTFISC', 'EFL_TEE_ACTIM', 'EFL_TEE_ACTPAT', 'EFL_TEE_ACTSOC'
            )">
            <xsl:apply-templates select="." mode="xfRes:EFL_TEE_treeRevueCxml.getCalculatedMetatada"/>
          </xsl:when>
          <!-- NewsLetter -->
          <xsl:when test="@code = 'EFL_TEE_newsletter'">
            <xsl:apply-templates select="." mode="xfRes:EFL_TEE_treeRevueCxml.getCalculatedMetatada"/>
          </xsl:when>
          <!-- Matiere -->
          <xsl:when test="@code = 'EFL_TEE_filActu'">
            <xsl:apply-templates select="." mode="xfRes:EFL_TEE_treeRevueCxml.getCalculatedMetatada4Matiere"/>
          </xsl:when>
          <!-- Quotidienne -->
          <xsl:when test="@code = 'EFL_TEE_quotidienne'">
            <xsl:apply-templates select="." mode="xfRes:EFL_TEE_treeRevueCxml.getCalculatedMetatada4Quotidienne"/>
          </xsl:when>          
          <!-- OM -->
          <xsl:when test="@code = ('EFL_TEE_filActusOmAvocat', 'EFL_TEE_filActusOmCEC', 'EFL_TEE_filActusOmCee', 'EFL_TEE_filActusOmDaf', 'EFL_TEE_filActusOmJur', 'EFL_TEE_filActusOmPaie', 'EFL_TEE_filActusOmRh')">
            <xsl:apply-templates select="." mode="xfRes:EFL_TEE_treeRevueCxml.getCalculatedMetatada4OM"/>
          </xsl:when>
          <!-- UI Source Texte -->
          <xsl:when test="@code = 'EFL_TEE_srcUiTexte'">
            <xsl:apply-templates select="." mode="xfRes:EFL_TEE_treeRevueCxml.getCalculatedMetatada4UISourceTexte"/>
          </xsl:when>
          <!--=== DALLOZ ===-->
          <!-- Revues -->
          <xsl:when test="@code = ('DZ_TEE_revuesAJPEN_FLASDZ_279', 'DZ_TEE_revuesDossier_FLASDZ_279', 'DZ_TEE_revuesArticle_FLASDZ_279')">
            <xsl:apply-templates select="." mode="xfRes:DZ_TEE_revues.poc.getCalculatedMetatada"/>           
          </xsl:when>
          <xsl:when test="matches(@code, 'DZ_TEE_revues\p{Lu}{2,}') or @code = ('DZ_TEE_revuesCahier', 'DZ_TEE_revuesDossier', 'DZ_TEE_revuesArticle')">
            <xsl:apply-templates select="." mode="xfRes:DZ_TEE_revues.getCalculatedMetatada"/>           
          </xsl:when>
          <!-- Ouvrages -->
          <xsl:when test="@code = ('DZ_TEE_ouvrages', 'DZ_TEE_ouvragesParagraphe', 'DZ_TEE_ouvragesSecondaire', 'DZ_TEE_actualisationOuvrage')">
            <xsl:apply-templates select="." mode="xfRes:DZ_TEE_ouvrages.getCalculatedMetatada"/>
          </xsl:when>
          <!-- Encyclopedies -->
          <xsl:when test="@code = ('DZ_TEE_encycloRubrique', 'DZ_TEE_encycloParagraphe')">
            <xsl:apply-templates select="." mode="xfRes:DZ_TEE_encyclo.getCalculatedMetatada"/>
          </xsl:when>
          <!-- Formules -->
          <xsl:when test="@code = ('DZ_TEE_formule', 'DZ_TEE_formuleRepliquee', 'DZ_TEE_formuleTexte')">
            <xsl:apply-templates select="." mode="xfRes:DZ_TEE_formule.getCalculatedMetatada"/>
          </xsl:when>
          <!-- Collectivités Locales -->
          <xsl:when test="@code = 'DZ_TEE_colLoc'">
            <xsl:apply-templates select="." mode="xfRes:DZ_TEE_colLoc.getCalculatedMetatada"/>
          </xsl:when>
          <!-- Codes -->
          <xsl:when test="@code = ('DZ_TEE_codalloz', 'DZ_TEE_codes', 'DZ_TEE_codesArbreSecondaire', 'DZ_TEE_codesArti', 'DZ_TEE_codesElement')">
            <xsl:apply-templates select="." mode="xfRes:DZ_TEE_codes.getCalculatedMetatada"/>
          </xsl:when>
          <!-- Actualité -->
          <xsl:when test="@code = 'DZ_TEE_actualite'">
            <xsl:apply-templates select="." mode="xfRes:DZ_TEE_actualite.getCalculatedMetatada"/>
          </xsl:when>
          <!-- Fiche Oasis -->
          <xsl:when test="@code = 'DZ_TEE_ficheOasis'">
            <xsl:apply-templates select="." mode="xfRes:DZ_TEE_ficheOasis.getCalculatedMetatada"/>
          </xsl:when>
          <!-- Guidance -->
          <xsl:when test="@code = 'DZ_TEE_guidanceTopicTree'">
            <xsl:apply-templates select="." mode="xfRes:DZ_TEE_guidanceTopicTree.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = 'DZ_TEE_guidanceTopicPage'">
            <xsl:apply-templates select="." mode="xfRes:DZ_TEE_guidanceTopicPage.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = 'DZ_TEE_guidancePracticeNote'">
            <xsl:apply-templates select="." mode="xfRes:DZ_TEE_guidancePracticeNote.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = 'DZ_TEE_checklist'">
            <xsl:apply-templates select="." mode="xfRes:DZ_TEE_checklist.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = 'DZ_TEE_auteur'">
            <xsl:apply-templates select="." mode="xfRes:DZ_TEE_auteur.getCalculatedMetatada"/>
          </xsl:when>
          <!--=== EL ===-->
          <xsl:when test="@code = 'EL_TEE_fichePratique'">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_FichePratique.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = 'EL_TEE_fichePratique_FLASEL_171'">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_FichePratique_FLASEL_171.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = 'EL_TEE_FICHE_QR' or @code = 'EL_TEE_ficheQR'">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_FICHE_QR.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = 'EL_TEE_fichePro'">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_fichePro.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = 'EL_TEE_ficheMT' or @code = 'EL_TEE_formDP'">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_ficheMT.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = 'EL_TEE_formDP'">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_ficheMT.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = 'EL_TEE_etudes'">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_etudes.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = 'EL_TEE_pnum'">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_etudes.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = ('EL_TEE_actualites','EL_TEE_actualite')">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_actualites.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = ('ELS_TEE_outils')">
            <xsl:apply-templates select="." mode="xfRes:ELS_TEE_outils.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = ('EL_TEE_outils','EL_TEE_practiceNoteRedigee')">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_practiceNote.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = ('EL_TEE_outils','EL_TEE_tableauRecapitulatif')">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_tableauRecapitulatif.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = ('EL_TEE_arbreGuidance')">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_arbreGuidance.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = ('EL_TEE_arbreThemeGuidance')">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_arbreThemeGuidance.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = ('EL_TEE_pageGuidance')">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_pageGuidance.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = ('EL_TEE_practiceNoteAssemblee')">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_practiceNoteAssemblee.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = ('EL_TEE_checklist')">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_checklist.getCalculatedMetatada"/>
          </xsl:when>
          <xsl:when test="@code = ('EL_TEE_outilsReferences')">
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_outilsReferences.getCalculatedMetatada"/>
          </xsl:when>
          <!--=== ELS ===-->
          <xsl:when test="@code = 'ELS_TEE_jurisprudence'">
            <xsl:apply-templates select="." mode="xfRes:ELS_TEE_jrpJuri.getCalculatedMetatada"/>            
          </xsl:when>
          <xsl:otherwise>
            <xsl:message terminate="yes">[FATAL] call mode="xfRes:getCalculatedMetadata" with editorialEntity whose @code="<xsl:value-of select="@code"/>" is not recognized</xsl:message>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:document>
    </xsl:variable>
    <xsl:variable name="step" as="element(xf:metadata)">
      <xsl:apply-templates select="$step" mode="xfRes:getCalculatedMetadata.filtering"/>
    </xsl:variable>
    <xsl:apply-templates select="$step" mode="xf:normalizeMetadata"/>
  </xsl:template>
  
  <xsl:template match="/xf:metadata" mode="xfRes:getCalculatedMetadata.filtering">
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:choose>
        <xsl:when test="$metaCode = ''">
          <xsl:apply-templates mode="#current"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates select="meta[@code = $metaCode.token] | processing-instruction() | comment()" mode="#current"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xfRes:getCalculatedMetadata.filtering">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
</xsl:stylesheet>