<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns:ecm="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm"
  xmlns="http://www.w3.org/1999/xhtml"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all" 
  version="3.0">
  
  <xsl:import href="xf-common.xsl"/>
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Common functions and templates for XMLfirst ECM Resources HTML previews</xd:p>
    </xd:desc>
  </xd:doc>
  
  <xsl:param name="xfRes:editorialEntity.code" select="/editorialEntity/@code" as="xs:string"/>
  
  <!--only one css allowed (ECM constraint), use css import if you need more-->
  <xsl:param name="xfRes:css.uri" select="xfRes:getCssRelativeUri($xfRes:editorialEntity.code, $xfRes:baseUri, 'ecm.htmlSimplePreview.style')" as="xs:string?"/>
  <!--CSS name and path can be overrided (usefull for xspec for example)-->
  <xsl:param name="xfRes:css.path" as="xs:string?"/>
  <xsl:param name="xfRes:css.name" as="xs:string?"/>
  
  <!--Normalize css path by adding a final "/"-->
  <xsl:variable name="xfRes:css.path.normalized" select="if(exists($xfRes:css.path) and not(ends-with($xfRes:css.path, '/'))) then(concat($xfRes:css.path, '/')) else($xfRes:css.path)" as="xs:string?"/>
  <!--Normalize css path from xfRes:css.uri by adding a final "/"--> 
  <xsl:variable name="xfRes:css.uri.path.normalized" select="concat(els:getFolderPath($xfRes:css.uri), '/')" as="xs:string?"/>
  
  <!--Get the final CSS uri after overriding-->
  <xsl:variable name="xfRes:css.final.uri" as="xs:string?">
    <xsl:choose>
      <xsl:when test="exists($xfRes:css.uri)">
        <xsl:sequence select="concat(
          ($xfRes:css.path.normalized, $xfRes:css.uri.path.normalized)[1],
          ($xfRes:css.name, els:getFileName($xfRes:css.uri))[1] 
           )"/>
      </xsl:when>
      <xsl:when test="exists($xfRes:css.path) or exists($xfRes:css.name)">
        <xsl:sequence select="concat($xfRes:css.path.normalized, $xfRes:css.name)"/>
      </xsl:when>
    </xsl:choose>
  </xsl:variable>
  
  <xsl:template name="xfRes:htmlPreview.makeHtmlHead">
    <head>
      <title><xsl:value-of select="els:getFileName($xfRes:baseUri)"/></title>
      <xsl:if test="not(empty($xfRes:css.final.uri))">
        <link type="text/css" rel="Stylesheet" href="{$xfRes:css.final.uri}"/>
      </xsl:if>
    </head>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>A template to add debuging informations : to be called from any htmlPreview for example</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template name="xfRes:htmlPreview.addDebugInfo">
    <xsl:if test="$xfRes:displayDebugInformations">
      <div class="ecm_debug" style="display:none;">
        <p>Generated at <xsl:value-of select="current-dateTime()"/></p>
        <p>server.url = <xsl:value-of select="$ecm:server.url"/></p>
        <p>ee.id = <xsl:value-of select="/editorialEntity/@xf:id"/></p>
        <p><a href="{concat($ecm:ee.url.begin, /editorialEntity/@xf:id)}" target="_blank">XML</a></p>
      </div>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="html:body" mode="xfRes:htmlPreview.normalizeHTML">
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <xsl:call-template name="xfRes:htmlPreview.addDebugInfo"/>
      <xsl:apply-templates select="node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="html:head" mode="xfRes:htmlPreview.normalizeHTML">
    <xsl:call-template name="xfRes:htmlPreview.makeHtmlHead"/>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xfRes:htmlPreview.normalizeHTML">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
</xsl:stylesheet>