<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:functx="http://www.functx.com"  
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns:xfResTeeConf="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources/TEE-conf"
  xmlns:ecm="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all" 
  version="3.0">
  
  <xsl:import href="../../xf-lib/_common/xf-common.xsl"/>
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Common functions and templates for XMLfirst ECM Resources</xd:p>
    </xd:desc>
  </xd:doc>
  
  <!-- Current xsl fileName for els:log-->
  <xsl:param name="els:currentResName" required="no" select="els:getFileName(static-base-uri())" as="xs:string"/>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Parameters "ecm:xxx" : catches every parameters sent by the ECM to all XSLT</xd:p>
    </xd:desc>
  </xd:doc>
  <!--PARAM sent by ECM (as string)-->
  <!--FIXME : delete default value when the param is sent for each env-->
  <xsl:param name="ecm:server.url" select="''" as="xs:string"/>
  <!--afficher ou non les métadonnées-->
  <xsl:param name="ecm:htmlProofing-metadata" select="'false'" as="xs:string"/>
  <!--ajoute des marges permettant d’annoter le document imprimé-->
  <xsl:param name="ecm:htmlProofing-margin" select="'false'" as="xs:string"/>
  <!--afficher ou non les suivis de modifications -->
  <xsl:param name="ecm:htmlProofing-revisedMode" select="'false'" as="xs:string"/>
  <!--affichage ou non des métadonnées des liens (disponible uniquement pour la prévisualisation d'une entité de type arbre ou d'un nœud de structure)-->
  <xsl:param name="ecm:htmlProofing-linkMetadata" select="'false'" as="xs:string"/>
  <!--affichage ou non du contenu des nœuds d’accrochage (disponible uniquement pour la prévisualisation d'une entité de type arbre ou d'un nœud de structure)-->
  <xsl:param name="ecm:htmlProofing-contentOfLinks" select="'false'" as="xs:string"/>
  <!--SIEGEL-510 : affichage d'un arbre avec l'option "surcharge"-->
  <xsl:param name="ecm:referenceNodeOnly" select="'false'" as="xs:string"/>
  
  <xsl:param name="xfRes:debug" select="false()" as="xs:boolean"/>
  <xsl:param name="xfRes:LOG.URI" select="resolve-uri('log', base-uri(.))" as="xs:string"/>
  <xsl:variable name="xfRes:log.uri" select="if(ends-with($xfRes:LOG.URI, '/')) then ($xfRes:LOG.URI) else(concat($xfRes:LOG.URI, '/'))" as="xs:string"/>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Variables</xd:p>
    </xd:desc>
  </xd:doc>
  <!--ecm:ee.url.begin should be a variable but it might be usefull to override it for XSpec-->
  <xsl:param name="ecm:ee.url.begin" select="concat($ecm:server.url, 'ecm-core/api/xmlExporter/exportToXML?flashId=')" as="xs:string"/>
  <!--Most of the time, image are called with a relativ URI, so we don't need to add this. 
  But in the case of proofing, the ECM open a new window with target="_blank" so we need to add it there.-->
  <xsl:variable name="ecm:img.url.begin" select="concat($ecm:server.url,'ecm-core')" as="xs:string"/>
  <xsl:variable name="ecm:ee.html.url.begin" select="'#/editorialEntity/'" as="xs:string"/>
  <!--baseUri may be lost in multi-step xslt so we declare a variable to be override and use at the good places--> 
  <xsl:variable name="xfRes:baseUri" select="base-uri(/)" as="xs:anyURI"/>
  <xsl:variable name="xfRes:xf-ecmRes.main.uri" as="xs:anyURI?">
    <xsl:if test="$ecm:server.url = ''"> <!--localy-->
      <xsl:sequence select="resolve-uri('../../', static-base-uri())"/>
    </xsl:if>
  </xsl:variable>
  <xsl:param name="xfRes:displayDebugInformations" as="xs:boolean">
    <xsl:choose>
      <!--Don't display debug information localy (because Xspec won't work with current-dateTime)-->
      <xsl:when test="$ecm:server.url = ''"><!--localy-->
        <xsl:sequence select="false()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select="true()"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:param>
  
  <!--=========================================================================-->
  <!--FUNCTIONS-->
  <!--=========================================================================-->
  
  <xsl:function name="ecm:getEEpreviewUrl" as="xs:string">
    <xsl:param name="id" as="xs:string?"/>
    <xsl:sequence select="concat($ecm:server.url, $ecm:ee.html.url.begin, $id)"/>
  </xsl:function>
  
  <xsl:function name="xf:getEEdocByFlashId" as="document-node()">
    <xsl:param name="id" as="xs:string"/>
    <xsl:param name="targetResVersion" as="xs:string?"/>
    <!-- Support for @targetResVersion is currently dropped -->
    <xsl:sequence select="xf:getEEdocByFlashId($id)"/>
  </xsl:function>
  
  <xsl:function name="xf:getEEdocByFlashId" as="document-node()">
    <xsl:param name="id" as="xs:string"/>
    <xsl:variable name="ee.uri" select="concat($ecm:ee.url.begin, $id)" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="doc-available($ee.uri)">
        <xsl:sequence select="doc($ee.uri)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:document>
          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="$els:currentResName" as="xs:string"/>
            <xsl:with-param name="xpathContext" select="''" as="xs:string"/>
            <xsl:with-param name="level" select="'error'" as="xs:string"/>
            <xsl:with-param name="code" select="'ECM.EE.UNAVAILABLE'" as="xs:string"/>
            <xsl:with-param name="description" as="xs:string*">EE with id '<xsl:value-of select="$id"/>' is not available at uri: <xsl:value-of select="$ee.uri"/> </xsl:with-param>
          </xsl:call-template>
        </xsl:document>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="xfRes:getTEEconfResource" as="element()*">
    <xsl:param name="editorialEntity.code" as="xs:string"/>
    <xsl:param name="resource.type" as="xs:string"/>
    <xsl:variable name="TEE-conf.name" select="concat($editorialEntity.code, '.ecm-resources.conf.xml')" as="xs:string"/>
    <xsl:variable name="TEE-conf.uri" select="resolve-uri($TEE-conf.name, concat($xfRes:xf-ecmRes.main.uri, 'TEE-conf/'))" as="xs:anyURI"/>
    <xsl:choose>
      <xsl:when test="doc-available($TEE-conf.uri)">
        <xsl:sequence select="doc($TEE-conf.uri)/xfResTeeConf:resources/xfResTeeConf:*[@type = $resource.type]"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:message>[ERROR] document <xsl:value-of select="$TEE-conf.uri"/> is not available</xsl:message>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <!--This function aimed at beeing execute localy but not server side (ECM will add the css automaticaly from TEE_conf file)-->
  <xsl:function name="xfRes:getCssRelativeUri" as="xs:string?">
    <xsl:param name="editorialEntity.code" as="xs:string"/>
    <xsl:param name="baseUri" as="xs:anyURI"/>
    <xsl:param name="resource.css.type" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="$ecm:server.url != ''">
        <xsl:message>[INFO][xfRes:getCssRelativeUri()] Do not try to get css path when executing this xslt on the ECM server</xsl:message>
      </xsl:when>
      <xsl:when test="empty($xfRes:xf-ecmRes.main.uri)">
        <xsl:message>[WARNING][xfRes:getCssRelativeUri()] $xfRes:xf-ecmRes.main.uri is empty, do not try to get css path</xsl:message>
      </xsl:when>
      <!--If not, try to read the TEE_conf file-->
      <xsl:otherwise>
        <xsl:variable name="resource.css" select="xfRes:getTEEconfResource($editorialEntity.code, $resource.css.type)" as="element()*"/>
        <xsl:variable name="resource.css.jar-href" select="$resource.css/@jar-href" as="xs:string?"/>
        <xsl:if test="exists($resource.css.jar-href) and starts-with($resource.css.jar-href, 'xf-ecmRes/')">
          <xsl:variable name="resource.css.uri" as="xs:string">
            <xsl:variable name="resource.css.uri.tmp" as="xs:string*">
              <xsl:value-of select="$xfRes:xf-ecmRes.main.uri"/>
              <xsl:choose>
                <!--src/main/web-resources will create a folder within the jar-->
                <xsl:when test="starts-with($resource.css.jar-href, 'xf-ecmRes/web-resources/')">
                  <xsl:sequence select="substring-after($resource.css.jar-href, 'xf-ecmRes/')"/>
                </xsl:when>
                <!--src/main/resources will not create a folder within the jar-->
                <!--let's consider this case-->
                <xsl:when test="starts-with($resource.css.jar-href, 'xf-ecmRes/')">
                  <xsl:sequence select="concat('resources/', substring-after($resource.css.jar-href, 'xf-ecmRes/'))"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:message>[WARNING] Unable to localte css resource for <xsl:value-of select="$resource.css.jar-href"/></xsl:message>
                  <xsl:sequence select="$resource.css.jar-href"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            <xsl:sequence select="string-join($resource.css.uri.tmp, '')"/>
          </xsl:variable>
          <xsl:sequence select="els:getRelativePath(els:getFolderPath(string($baseUri)), $resource.css.uri)"/>
          <!--DEBUG-->
          <!--<xsl:message>els:getFolderPath(string($baseUri))= <xsl:value-of select="els:getFolderPath(string($baseUri))"/></xsl:message>-->
          <!--<xsl:message>$resource.css.uri= <xsl:value-of select="$resource.css.uri"/></xsl:message>-->
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Update a meta verbalization element by looking for its current value in the appropriate referential database.</xd:p>
      <xd:p>FIXME : A implémenter quand le format des référentiels sera défini + ces derniers accessibles depuis une base XML. Pour le moment -> simple copie de la méta en entrée.</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(meta)] The meta element whose verbalization needs to be updated. It must contains a ref/@xf:idRef pointing to a referential database entry.</xd:param>
    <xd:return>[element(meta)] The updated meta element.</xd:return>
  </xd:doc>
  <xsl:function name="xfRes:updateMetaVerbalization" as="element(meta)">
    <xsl:param name="meta" as="element(meta)"/>
    <xsl:if test="not($meta/value/ref[@xf:refType = 'xf:referenceItemRef'])">
      <xsl:message>[ERROR][xfRes:updateMetaVerbalization] The meta must contain a reference to a referential database entry (ref/@xf:refType = 'xf:referenceItemRef').</xsl:message>
    </xsl:if>
    <!-- TO DO -->
    <xsl:sequence select="$meta"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Simple template to update all metadata verbalizations.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="meta[value/ref/@xf:refType = 'xf:referenceItemRef']" mode="xfRes:updateMetaVerbalization">
    <xsl:sequence select="xfRes:updateMetaVerbalization(.)"/>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Default copy for mode xfRes:updateMetaVerbalization.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="node() | @*" mode="xfRes:updateMetaVerbalization">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Returns the short/long label of an identified referential database entry.</xd:p>
      <xd:p>FIXME : A revoir lorsque les référentiels seront mis en place (la fonction ne doit pas être utilisée pour le moment).</xd:p>
    </xd:desc>
    <xd:param name="refXML">[element(xf:referenceTable)] XML serialization of the referential database (FLASH format).</xd:param>
    <xd:param name="entryId">[xs:string] The database entry ID.</xd:param>
    <xd:param name="labelType">[xs:string] Possible values: "short" or "long".</xd:param>
    <xd:return>[xs:string?] The short/long label, if it was found.</xd:return>
  </xd:doc>
  <xsl:function name="xfRes:getRefEntryLabel" as="xs:string?">
    <xsl:param name="refXML" as="element(xf:referenceTable)"/>
    <xsl:param name="entryId" as="xs:string"/>
    <xsl:param name="labelType" as="xs:string"/>
    <xsl:variable name="authorisedLabelTypes" select="('short','long')" as="xs:string+"/>
    <xsl:if test="not($labelType = $authorisedLabelTypes)">
      <xsl:message>[ERROR][xfRes:getRefEntryLabel] Unknown label type: <xsl:value-of select="$labelType"/>.</xsl:message>
    </xsl:if>
    <xsl:value-of select="$refXML//referenceItem[@id = $entryId]/label[@type = $labelType]"/>
  </xsl:function>
  
</xsl:stylesheet>