<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources" 
  xmlns:content="http://www.lefebvre-sarrut.eu/ns/dalloz/codes" 
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  exclude-result-prefixes="#all" 
  version="3.0">
  
  <xsl:import href="_common/xf-common-dalloz.xsl"/>
  <xsl:include href="_common/title2html.xsl"/>
  
  <xsl:variable name="codes" select="document('_common/codes.xml')"/>
  
  <xsl:variable name="systemMeta" select="('systemTitle', 'systemAbstract')" as="xs:string+"/>
  <xsl:variable name="calculatedMeta" select="('ELS_META_LUX_uri')" as="xs:string+"/>
  
  <xsl:param name="metaLuxia" select="'yes'"/>
  
  <xsl:template match="xf:editorialEntity" mode="xfRes:DZ_TEE_codes.getCalculatedMetatada">
    <metadata>
      <xsl:sequence select="for $codeMeta in ($systemMeta, $calculatedMeta) return xfRes:codes_dispatchGenMeta(., $codeMeta)"/>
    </metadata>
  </xsl:template>
  
  <xsl:function name="xfRes:codes_dispatchGenMeta">
    <xsl:param name="entity" as="element(xf:editorialEntity)"/>
    <xsl:param name="codeMeta" as="xs:string"/>
    <xsl:choose>
      <!-- System -->
      <xsl:when test="$codeMeta = 'systemTitle'">
        <xsl:choose>
          <xsl:when test="$entity/@code='DZ_TEE_codalloz'">
            <xsl:sequence select="xfRes:genHtmlMeta($codeMeta, xfRes:codes_createMainTitle($entity))"/>
          </xsl:when>
          <xsl:when test="$entity/@code='DZ_TEE_codes'">
            <xsl:sequence select="xfRes:genHtmlMeta($codeMeta, xfRes:codes_createTitle($entity))"/>
          </xsl:when>
          <xsl:when test="$entity/@code='DZ_TEE_codesArbreSecondaire'">
            <xsl:sequence select="xfRes:genHtmlMeta($codeMeta, xfRes:codes_createSecondaryTitle($entity))"/>
          </xsl:when>
          <xsl:when test="$entity/@code='DZ_TEE_codesArti'">
            <xsl:sequence select="xfRes:genHtmlMeta($codeMeta, xfRes:codes_createArtiTitle($entity))"/>
          </xsl:when>
          <xsl:when test="$entity/@code='DZ_TEE_codesElement'">
            <xsl:sequence select="xfRes:genHtmlMeta($codeMeta, xfRes:codes_createFeuilleTitle($entity))"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:message>[ERROR] System meta "title" undefined</xsl:message>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="$codeMeta = 'systemAbstract'"/>
      <xsl:when test="$codeMeta = 'ELS_META_LUX_uri'">
        <xsl:if test="$metaLuxia='yes'">
        <xsl:choose>
          <xsl:when test="$entity/@code='DZ_TEE_codalloz'">
            <xsl:sequence select="xfRes:codes_createMainLuxiaAri($entity)"/>
          </xsl:when>
          <xsl:when test="$entity/@code='DZ_TEE_codes'">
            <!-- <xsl:sequence select="xfRes:genHtmlMeta($codeMeta, xfRes:codes_createLuxiaAri($entity))"/> -->
          </xsl:when>
          <xsl:when test="$entity/@code='DZ_TEE_codesArti'">
            <xsl:sequence select="xfRes:codes_createArtiLuxiaAri($entity)"/>
          </xsl:when>
          <xsl:when test="$entity/@code='DZ_TEE_codesArbreSecondaire'"/>
          <xsl:when test="$entity/@code='DZ_TEE_codesElement'"/>
          <xsl:otherwise>
            <xsl:message>[ERROR] Calculated meta <xsl:value-of select="$codeMeta"/> undefined on <xsl:value-of select="$entity/@code"/></xsl:message>
          </xsl:otherwise>
        </xsl:choose>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <xsl:message>[ERROR] Metadata <xsl:value-of select="$codeMeta"/> undefined</xsl:message>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="xfRes:dallozCode2AriCode" as="xs:string?">
    <xsl:param name="dallozCode" as="xs:string?"/>
    <xsl:value-of select="$codes/CODES/LISTE-CODES/CODE[NOM=$dallozCode]/LUXIA"/>
  </xsl:function>
  
  <xsl:function name="xfRes:dallozCodeTitre" as="xs:string?">
    <xsl:param name="dallozCode" as="xs:string?"/>
    <xsl:value-of select="$codes/CODES/LISTE-CODES/CODE[NOM=$dallozCode]/INTT"/>
  </xsl:function>
  
  <xsl:function name="xfRes:codes_createMainLuxiaAri" as="element(xf:meta)?">
    <xsl:param name="entity" as="element(xf:editorialEntity)"/>
    <xsl:variable name="codeNom" select="$entity/xf:metadata/xf:meta[@code = 'DZ_META_codesNom']/xf:value" as="xs:string?"/>
    <xsl:choose>
      <xsl:when test="matches($codeNom, '^[A-Z]{4}$')">
        <xsl:variable name="luxiaARI" select="xfRes:dallozCode2AriCode($codeNom)" as="xs:string?"/>
        <xsl:choose>
          <xsl:when test="$luxiaARI">
            <meta code="ELS_META_LUX_uri">
              <value><xsl:value-of select="concat('fr/code/', $luxiaARI)"/></value>
            </meta>
          </xsl:when>
          <xsl:otherwise>
            <xsl:message>[WARNING] le code <xsl:value-of select="xfRes:dallozCodeTitre($codeNom)"/> n'a pas d'équivalent Luxia</xsl:message>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:message terminate="yes">[ERROR] L'identifiant du code n'est pas conforme.</xsl:message>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="xfRes:codes_createArtiLuxiaAri" as="element(xf:meta)?">
    <xsl:param name="entity" as="element(xf:editorialEntity)"/>
    <xsl:if test="$entity/xf:metadata/xf:meta[@code = 'DZ_META_codesOrigine']/xf:value = 'ARTI'">
    <xsl:variable name="codeNom" select="substring(substring-after($entity/@xf:id, '{historicalId:dalloz_'), 1, 4)" as="xs:string?"/>
    <xsl:choose>
      <xsl:when test="matches($codeNom, '^[A-Z]{4}$')">
        <xsl:variable name="luxiaARI" select="xfRes:dallozCode2AriCode($codeNom)" as="xs:string?"/>
        <xsl:choose>
          <xsl:when test="$luxiaARI">
            <xsl:variable name="nume" select="$entity/xf:metadata/xf:meta[@code = 'DZ_META_codesNume']/xf:value"/>
            <xsl:choose>
              <xsl:when test="$nume">
                <xsl:analyze-string select="normalize-space($nume)" 
                  regex="^Art.\s+(L|LO|R|D|A)?\.?([*]*)\s*(\d+(-\d+)*)(.*?)((\s+(à|et)\s+)(L|LO|R|D|A)?\.?([*]*)\s*(\d+(-\d+)*))?$">
                  <xsl:matching-substring>
                    <meta code="ELS_META_LUX_uri">
                      <value><xsl:value-of select="concat('fr/code/', $luxiaARI, '/', 
                        upper-case(regex-group(1)), upper-case(regex-group(2)), upper-case(regex-group(3)), translate(upper-case(regex-group(5)), ' ', '_'), 
                        translate(upper-case(regex-group(7)), ' ', '_'), upper-case(regex-group(9)), upper-case(regex-group(10)), upper-case(regex-group(11)))"/></value>
                    </meta>
                  </xsl:matching-substring>
                  <xsl:non-matching-substring>
                    <xsl:message>[WARNING] l'article d'ID <xsl:value-of select="substring(substring-after($entity/@xf:id, '{historicalId:dalloz_'), 1, 10)"/> n'a pas d'ARI Luxia calculable.</xsl:message>
                  </xsl:non-matching-substring>
                </xsl:analyze-string>
              </xsl:when>
              <xsl:otherwise>
                <xsl:message>[WARNING] l'article d'ID <xsl:value-of select="substring(substring-after($entity/@xf:id, '{historicalId:dalloz_'), 1, 10)"/> n'a pas de numerotation.</xsl:message>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:otherwise>
            <xsl:message>[WARNING] le code <xsl:value-of select="xfRes:dallozCodeTitre($codeNom)"/> n'a pas d'équivalent Luxia</xsl:message>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when> 
      <xsl:otherwise>
        <xsl:message terminate="yes">[ERROR] La numérotation de l'article n'est pas conforme.</xsl:message>
      </xsl:otherwise>
    </xsl:choose>
    </xsl:if>
  </xsl:function>
  
  <xsl:function name="xfRes:codes_createMainTitle" as="element(html:div)">
    <xsl:param name="entity" as="element(xf:editorialEntity)"/>
    <xsl:variable name="codeNom" select="$entity/xf:metadata/xf:meta[@code = 'DZ_META_codesNom']/xf:value" as="xs:string"/>
    <xsl:variable name="title" select="xfRes:dallozCodeTitre($codeNom)" as="xs:string?"/>
    <div xmlns="http://www.w3.org/1999/xhtml">
      <span class="ATT-NOM"><xsl:value-of select="if ($title) then $title else 'Code Inconnu'"/></span>
    </div>
  </xsl:function>
  
  <xsl:function name="xfRes:codes_createFeuilleTitle" as="element(html:div)">
    <xsl:param name="entity" as="element(xf:editorialEntity)"/>
    <xsl:variable name="codeNume" select="$entity/xf:metadata/xf:meta[@code = 'DZ_META_codesNume']/xf:value" as="xs:string?"/>
    <xsl:variable name="codeIntt" select="$entity/xf:metadata/xf:meta[@code = 'DZ_META_codesIntt']/xf:value" as="xs:string?"/>
    <xsl:choose>
      <xsl:when test="$codeIntt">
        <div xmlns="http://www.w3.org/1999/xhtml">
          <xsl:if test="$codeNume">
            <span class="NUME"><xsl:value-of select="$codeNume"/></span><xsl:value-of select="'&#160;'"/>
          </xsl:if>
          <span class="INTT"><xsl:value-of select="$codeIntt"/></span>
        </div>
      </xsl:when>
      <xsl:otherwise><!--
        <xsl:variable name="textContent" select="normalize-space(string-join($entity/xf:body/xf:contentNode/xf:content//text(), ' '))" as="xs:string?"/>
        <div xmlns="http://www.w3.org/1999/xhtml">
          <span><xsl:value-of select="concat(substring($textContent, 1, 20), '...')"/></span>
        </div>-->
        <xsl:choose>
          <xsl:when test="not($entity/xf:body/xf:contentNode/xf:content//text())">
            <div xmlns="http://www.w3.org/1999/xhtml">...</div>
          </xsl:when>
          <xsl:when test="$entity/xf:body/xf:contentNode/xf:content/*[1]">
            <div xmlns="http://www.w3.org/1999/xhtml">
              <xsl:sequence select="xfRes:makeTruncatedTitle($entity/xf:body/xf:contentNode/xf:content/*[1])"/>
            </div>
          </xsl:when>
          <xsl:otherwise>
            <!-- un titre vide ne convient pas, alors on utilise un blanc -->
            <div xmlns="http://www.w3.org/1999/xhtml">
              <xsl:value-of select="' '"/>
            </div>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="xfRes:codes_secondaryTitleFromElementName" as="element(html:span)">
    <xsl:param name="elementName" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="$elementName='COCO'">
        <span class="{$elementName}" xmlns="http://www.w3.org/1999/xhtml"><xsl:value-of select="'Commentaire'"/></span>
      </xsl:when>
      <xsl:when test="$elementName='COPILOT'">
        <span class="{$elementName}" xmlns="http://www.w3.org/1999/xhtml"><xsl:value-of select="'COPILOT'"/></span>
      </xsl:when>
      <xsl:when test="$elementName='JURI'">
        <span class="{$elementName}" xmlns="http://www.w3.org/1999/xhtml"><xsl:value-of select="'Jurisprudence'"/></span>
      </xsl:when>
      <xsl:when test="$elementName='BIBLLEG'">
        <span class="{$elementName}" xmlns="http://www.w3.org/1999/xhtml"><xsl:value-of select="'Bibliographie'"/></span>
      </xsl:when>
      <xsl:when test="$elementName=('APPE', 'CODEAPPE')">
        <span class="{$elementName}" xmlns="http://www.w3.org/1999/xhtml"><xsl:value-of select="'Appendice'"/></span>
      </xsl:when>
      <xsl:otherwise>
        <span xmlns="http://www.w3.org/1999/xhtml"><xsl:value-of select="''"/></span>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="xfRes:codes_createSecondaryTitle" as="element(html:div)">
    <xsl:param name="entity" as="element(xf:editorialEntity)"/>
    <xsl:variable name="metaOrigine" select="$entity/xf:metadata/xf:meta[@code = 'DZ_META_codesOrigine']/xf:value" as="xs:string?"/>
    <xsl:variable name="metaTitr" select="$entity/xf:metadata/xf:meta[@code = 'DZ_META_codesTitr']/xf:value" as="xs:string?"/>
    <div xmlns="http://www.w3.org/1999/xhtml">
      <xsl:choose>
        <xsl:when test="$metaTitr">
          <span class="TITR"><xsl:value-of select="$metaTitr"/></span>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="xfRes:codes_secondaryTitleFromElementName($metaOrigine)"/>
        </xsl:otherwise>
      </xsl:choose>
    </div>
  </xsl:function>
  
  <xsl:function name="xfRes:codes_createTitle" as="element(html:div)">
    <xsl:param name="entity" as="element(xf:editorialEntity)"/>
    <xsl:variable name="metaOrigine" select="$entity/xf:metadata/xf:meta[@code = 'DZ_META_codesOrigine']/xf:value" as="xs:string"/>
    <xsl:variable name="metaIntt" select="$entity/xf:metadata/xf:meta[@code = 'DZ_META_codesIntt']/xf:value" as="xs:string?"/>
    <xsl:variable name="metaTitr" select="$entity/xf:metadata/xf:meta[@code = 'DZ_META_codesTitr']/xf:value" as="xs:string?"/>
    <div xmlns="http://www.w3.org/1999/xhtml">
      <xsl:choose>
        <xsl:when test="$metaOrigine='LOID'">
          <!-- titre non réduit à deux lignes dans ce cas particulier on va mettre une classe spécifique -->
          <div class="SOUR DZ_TEE_codes_systemTitle"><xsl:apply-templates select="$entity/xf:body/xf:contentNode/xf:content/content:SOUR/*" mode="xfRes:title2html"/></div>
          <xsl:if test="$metaIntt">
            <div class="INTT DZ_TEE_codes_systemTitle"><xsl:value-of select="$metaIntt"/></div>
          </xsl:if>
        </xsl:when>
        <xsl:when test="$metaOrigine='ANNE'">
          <xsl:variable name="title">
            <xsl:choose>
              <xsl:when test="$metaIntt">
                <span class="INTT"><xsl:value-of select="$metaIntt"/></span>
              </xsl:when>
              <xsl:when test="$metaTitr">
                <span class="TITR"><xsl:value-of select="$metaTitr"/></span>
              </xsl:when>
              <xsl:when test="$entity/xf:body/xf:contentNode/xf:content/content:SOUR/(content:NATX|content:NUME|content:DATE|content:ARTE)">
                <xsl:for-each select="$entity/xf:body/xf:contentNode/xf:content/content:SOUR/(content:NATX|content:NUME|content:DATE|content:ARTE)">
                  <xsl:apply-templates select="." mode="xfRes:title2html"/>
                </xsl:for-each>
              </xsl:when>
              <xsl:otherwise>
                <span><xsl:value-of select="''"/></span>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <xsl:variable name="titleAsString"><xsl:value-of select="$title"/></xsl:variable>
          <xsl:if test="not(starts-with(upper-case($titleAsString), 'ANNEXE'))">
            <span>ANNEXE&#160;-&#160;</span>
          </xsl:if>
          <xsl:sequence select="$title"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <xsl:when test="$metaIntt">
              <span class="INTT"><xsl:value-of select="$metaIntt"/></span>
            </xsl:when>
            <xsl:when test="$metaTitr">
              <span class="TITR"><xsl:value-of select="$metaTitr"/></span>
            </xsl:when>
            <xsl:when test="$entity/xf:body/xf:contentNode/xf:content/content:SOUR/(content:NATX|content:NUME|content:DATE|content:ARTE)">
              <xsl:for-each select="$entity/xf:body/xf:contentNode/xf:content/content:SOUR/(content:NATX|content:NUME|content:DATE|content:ARTE)">
                <xsl:apply-templates select="." mode="xfRes:title2html"/>
              </xsl:for-each>
            </xsl:when>
            <xsl:otherwise>
              <span><xsl:value-of select="''"/></span>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </div>
  </xsl:function>
  
  <xsl:function name="xfRes:codes_createArtiTitle" as="element(html:div)">
    <xsl:param name="entity" as="element(xf:editorialEntity)"/>
    <xsl:variable name="metaNume" select="$entity/xf:metadata/xf:meta[@code = 'DZ_META_codesNume']/xf:value" as="xs:string?"/>
    <xsl:variable name="metaIntt" select="$entity/xf:metadata/xf:meta[@code = 'DZ_META_codesIntt']/xf:value" as="xs:string?"/>
    <xsl:variable name="origine" select="$entity/xf:metadata/xf:meta[@code = 'DZ_META_codesOrigine']/xf:value" as="xs:string?"/>
    <div xmlns="http://www.w3.org/1999/xhtml">
      <xsl:choose>
        <xsl:when test="$metaNume">
          <span class="NUME"><xsl:value-of select="$metaNume"/></span>
          <xsl:if test="$metaIntt">
            <xsl:value-of select="'&#160;'"/><span class="INTT"><xsl:value-of select="$metaIntt"/></span>
          </xsl:if>
        </xsl:when>
        <xsl:when test="$metaIntt">
          <span class="INTT"><xsl:value-of select="$metaIntt"/></span>
        </xsl:when>
        <xsl:otherwise>
          <span><xsl:value-of select="$origine"/></span>
        </xsl:otherwise>
      </xsl:choose>
    </div>
  </xsl:function>
  
</xsl:stylesheet>
