<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources" 
  xmlns:content="http://www.lefebvre-sarrut.eu/ns/dalloz/ouvrages" 
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  exclude-result-prefixes="#all" 
  version="3.0">
  
  <xsl:import href="_common/xf-common-dalloz.xsl"/>
  <xsl:include href="_common/title2html.xsl"/>
  
  <xsl:variable name="systemMeta" select="('systemTitle', 'systemAbstract')" as="xs:string+"/>
  
  <xsl:template match="xf:editorialEntity" mode="xfRes:DZ_TEE_encyclo.getCalculatedMetatada">
    <xsl:variable name="element" as="element()">
      <xsl:choose>
        <xsl:when test="@code = 'DZ_TEE_encycloRubrique'">
          <xsl:sequence select="xf:metadata"/>
        </xsl:when>
        <xsl:when test="@code = 'DZ_TEE_encycloParagraphe'">
          <xsl:sequence select="xf:body/xf:contentNode/xf:content/(content:PARA|content:PARA-LEGIS|content:PARA-JURIS|content:PARA-QUEST|content:PARA-BIBLIO|content:ENCADRE)"/>
        </xsl:when>
      </xsl:choose>
    </xsl:variable>
    <metadata>
      <xsl:sequence select="for $codeMeta in $systemMeta return xfRes:encyclo_dispatchGenMeta($element, $codeMeta)"/>
    </metadata>
  </xsl:template>
  
  <xsl:function name="xfRes:encyclo_dispatchGenMeta">
    <xsl:param name="element" as="element()"/>
    <xsl:param name="codeMeta" as="xs:string"/>
    <xsl:choose>
      <!-- System -->
      <xsl:when test="$codeMeta = 'systemTitle'">
        <xsl:choose>
          <xsl:when test="local-name($element) = 'metadata'">
            <xsl:choose>
              <xsl:when test="$element/xf:meta[@code = 'DZ_META_rubriqueTitre']">
                <xsl:sequence select="xfRes:genHtmlMeta($codeMeta, xfRes:encyclo_createTitle($element/xf:meta[@code = 'DZ_META_rubriqueTitre']))"/>
              </xsl:when>
              <xsl:when test="$element/xf:meta[@code = 'DZ_META_encycloTitreInt']">
                <xsl:sequence select="xfRes:genHtmlMeta($codeMeta, xfRes:encyclo_createTitle($element/xf:meta[@code = 'DZ_META_encycloTitreInt']))"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:message>[ERROR] System meta "title" undefined</xsl:message>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:otherwise>
            <xsl:sequence select="xfRes:genHtmlMeta($codeMeta, xfRes:encyclo_createTitleParagraphe($element))"/>
          </xsl:otherwise>
        </xsl:choose>  
      </xsl:when>
      <xsl:when test="$codeMeta = 'systemAbstract'"/>  
      
      <xsl:otherwise>
        <xsl:message>[ERROR] Metadata <xsl:value-of select="$codeMeta"/> undefined</xsl:message>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="xfRes:encyclo_createTitle" as="element(html:div)">
    <xsl:param name="element" as="element()"/>
    <div xmlns="http://www.w3.org/1999/xhtml">
      <xsl:value-of select="$element/xf:value"/>
    </div>
  </xsl:function>
  
  <xsl:function name="xfRes:encyclo_createAbstract" as="element(html:div)">
    <xsl:param name="element" as="element()"/>
    <div xmlns="http://www.w3.org/1999/xhtml">
      <xsl:value-of select="$element/xf:value"/>
    </div>
  </xsl:function>
  
  <xsl:function name="xfRes:encyclo_createTitleParagraphe" as="element(html:div)">
    <xsl:param name="entity" as="element()"/>
    <div xmlns="http://www.w3.org/1999/xhtml">
      <xsl:choose>
        <xsl:when test="$entity/content:TITP">
          <xsl:if test="$entity/content:NUME">
            <span class="NUME"><xsl:apply-templates select="$entity/content:NUME/node()" mode="xfRes:title2html"/></span>
            <xsl:value-of select="'&#160;-&#160;'"/>
          </xsl:if>
          <span class="TITP"><xsl:apply-templates select="$entity/content:TITP/node()" mode="xfRes:title2html"/></span>
        </xsl:when>
        <xsl:when test="$entity/content:NUME">
          <span class="NUME"><xsl:apply-templates select="$entity/content:NUME/node()" mode="xfRes:title2html"/></span>
        </xsl:when>
        <xsl:when test="$entity/(content:TIT1|content:TIT2|content:TIT3|content:TIT4|content:TIT5)">
          <span class="ENCADRE"><xsl:apply-templates select="$entity/(content:TIT1|content:TIT2|content:TIT3|content:TIT4|content:TIT5)" mode="xfRes:title2html"/></span>
        </xsl:when>
        <xsl:otherwise>
          <!-- un titre vide ne convient pas, alors on utilise un blanc -->
          <xsl:value-of select="' '"/>
        </xsl:otherwise>
      </xsl:choose>
    </div>
  </xsl:function>
  
  <xsl:function name="xfRes:encyclo_createAbstractParagraphe" as="element(html:div)">
    <xsl:param name="entity" as="element()"/>
    <div xmlns="http://www.w3.org/1999/xhtml">
      <xsl:choose>
        <xsl:when test="$entity/content:TITP">
          <span class="TITP"><xsl:apply-templates select="$entity/content:TITP/node()" mode="xfRes:title2html"/></span>
        </xsl:when>
        <xsl:otherwise>
          <!-- un abstract vide ne convient pas, alors on utilise un blanc -->
          <xsl:value-of select="' '"/>
        </xsl:otherwise>
      </xsl:choose>
    </div>
  </xsl:function>
  
</xsl:stylesheet>
