<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xdoc="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:exslt="http://exslt.org/dates-and-times"
  xmlns:file="http://expath.org/ns/file"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:ecm="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm"
  xmlns:xfe="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns:ouvrages="http://www.lefebvre-sarrut.eu/ns/dalloz/ouvrages"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns=""
  exclude-result-prefixes="#all"
  version="3.0">
  
  <xsl:import href="../_common/TEE_tree.default.htmlSimplePreview.xsl"/>
  <xsl:import href="../../xfe/xfe-XML2HTML.xsl"/>
  
  <!-- PARAMS -->
  <xsl:param name="xfe:css.name" select="els:getFileName($xfRes:css.uri)" as="xs:string"/>
  <xsl:param name="xfe:css.path" select="els:getFolderPath($xfRes:css.uri)" as="xs:string"/>  
  <!-- Paramètre obligatoire pour xfe:xml2html -->
  <xsl:param name="xfe:conf.uri" select="resolve-uri('_common/DZ_TEE_ouvrages.xfe-conf.xml',static-base-uri())" as="xs:string"/>
  <xsl:param name="xfe:addXmlModelPi" select="false()" as="xs:boolean"/>
  <xsl:param name="xfe:addHTML5DOCTYPE" select="false()" as="xs:boolean"/>
  <xsl:param name="xfe:debug" select="false()" as="xs:boolean"/>
  <xsl:param name="xfe:fillHeadTitle" select="true()" as="xs:boolean"/>
  <xsl:param name="xfe:allowUnValidXML" select="true()" as="xs:boolean"/>
  
  <!--indent="no" is important for xspec diff--> 
  <xsl:output method="xhtml" indent="no"/>
  
  <!-- LANCEMENT SANS MODE -->
  <xsl:template match="/">
    <html xmlns="http://www.w3.org/1999/xhtml">
      <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
        <title></title>
        <link type="text/css" rel="Stylesheet" href="{concat($xfe:css.path, if(normalize-space($xfe:css.path) != '' and not(ends-with($xfe:css.path, '/'))) then('/') else(''), $xfe:css.name)}" />
      </head>
      <body>
        <xsl:apply-templates select="*" mode="xfRes:htmlAggregate"/>
      </body>
    </html>
  </xsl:template>
  
  <xsl:template match="node()|@*" mode="xfRes:htmlAggregate"/>
  
  <!-- ********************************************************* -->
  <!-- NEW CODE STARTS HERE                                      -->
  <!-- ********************************************************* -->
  <xsl:template match="editorialEntity" mode="xfRes:htmlAggregate">
    <xsl:apply-templates select="body/*" mode="#current"/>
  </xsl:template>
  
  <xsl:template match="title" mode="xfRes:htmlAggregate"/>
  
  <xsl:template match="referenceNode[@code=('DZ_TR_accrochage_Encyclo_EncycloPara')]" mode="xfRes:htmlAggregate">
    <xsl:variable name="target.uri" select="concat($ecm:ee.url.begin, ref/@xf:targetResId)"/>
    <xsl:choose>
      <xsl:when test="doc-available($target.uri)">
        <xsl:apply-templates select="doc($target.uri)" mode="#current"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:copy>
          <html:p class="ecm_error">[ERROR] EE <xsl:value-of select="$target.uri"/> is not available !</html:p>
          <html:p class="ecm_debug">
            <xsl:choose>
              <xsl:when test="unparsed-text-available($target.uri)">
                <p>unparsed-text-available :</p>
                <textarea rows="10" cols="50">
                  <xsl:value-of select="unparsed-text($target.uri)"/>
                </textarea>
              </xsl:when>
              <xsl:otherwise>
                <p>unparsed-text is not available</p>
              </xsl:otherwise>
            </xsl:choose>
          </html:p>
        </xsl:copy>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
    
  <xsl:template match="structuralNode" mode="xfRes:htmlAggregate">
    <ul>
      <li>
        <xsl:copy-of select="title/*:div/node()"/>
        <xsl:apply-templates select="* except (metadata, title)" mode="#current"/>
      </li>
    </ul>
  </xsl:template>
  
  <!-- xfRes:xmlPublish mode on body/* -->
  <xsl:template match="queryNode" mode="xfRes:htmlAggregate"/>
  
  <!-- xfRes:xmlPublish mode on relations -->
  <xsl:template match="relations" mode="xfRes:htmlAggregate"/>
  
  <xsl:template match="contentNode" mode="xfRes:htmlAggregate">
    <!--<xsl:apply-templates select="metadata" mode="xfRes:xmlPublish"/>-->
    <xsl:apply-templates select="content/*" mode="xfRes:htmlAggregate"/>
  </xsl:template>
  
  <xsl:template match="contentNode/content/*" mode="xfRes:htmlAggregate">
    <xsl:choose>
      <xsl:when test="namespace-uri(.) = 'http://www.w3.org/1999/xhtml'">
        <xsl:apply-templates select="." mode="xfRes:TEE_text.html.default.htmlSimplePreview"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="full-translation">
          <xsl:apply-templates select="." mode="xfe:xml2html">
            <xsl:with-param name="xfe:contentNodeId" select="./@xf:id"></xsl:with-param>
          </xsl:apply-templates>
        </xsl:variable>
        <xsl:sequence select="$full-translation//html:body/*"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="title" mode="xfRes:htmlAggregate"/>
  
</xsl:stylesheet>