<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources" 
  xmlns:content="http://www.lefebvre-sarrut.eu/ns/dalloz/ouvrages" 
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  exclude-result-prefixes="#all" 
  version="3.0">
  
  <xsl:import href="_common/xf-common-dalloz.xsl"/>
  <xsl:include href="_common/title2html.xsl"/>
  
  <xsl:variable name="systemMeta" select="('systemTitle', 'systemAbstract')" as="xs:string+"/>
  
  <xsl:template match="xf:editorialEntity" mode="xfRes:DZ_TEE_ouvrages.getCalculatedMetatada">
    <xsl:variable name="element" as="element()">
      <xsl:choose>
        <xsl:when test="@code = 'DZ_TEE_ouvrages'">
          <xsl:sequence select="xf:metadata"/>
        </xsl:when>
        <xsl:when test="@code = 'DZ_TEE_ouvragesParagraphe'">
          <xsl:sequence select="xf:body/xf:contentNode/xf:content/(content:PARA|content:PARA-LEGIS|content:PARA-JURIS|content:PARA-QUEST|content:PARA-BIBLIO
            |content:PARA-PLAN|content:RESUME|content:ENCADRE|content:CITATION)"/>
        </xsl:when>
        <xsl:when test="@code = 'DZ_TEE_ouvragesSecondaire'">
          <xsl:sequence select="xf:metadata"/>
        </xsl:when>
        <xsl:when test="@code = 'DZ_TEE_actualisationOuvrage'">
          <xsl:sequence select="xf:metadata"/>
        </xsl:when>
      </xsl:choose>
    </xsl:variable>
    <metadata>
      <xsl:sequence select="for $codeMeta in $systemMeta return xfRes:ouvrages_dispatchGenMeta($element, $codeMeta)"/>
    </metadata>
  </xsl:template>
  
  <xsl:function name="xfRes:ouvrages_dispatchGenMeta">
    <xsl:param name="element" as="element()"/>
    <xsl:param name="codeMeta" as="xs:string"/>
    <xsl:choose>
      <!-- System -->
      <xsl:when test="$codeMeta = 'systemTitle'">
        <xsl:choose>
          <xsl:when test="local-name($element) = 'metadata'">
            <xsl:choose>
              <xsl:when test="$element/xf:meta[@code = 'DZ_META_ouvragesCollection']">
                <xsl:choose>
                  <xsl:when test="$element/xf:meta[@code = 'DZ_META_ouvragesTitreInt']">
                    <xsl:sequence select="xfRes:genHtmlMeta($codeMeta, xfRes:ouvrages_createTitle($element/xf:meta[@code = 'DZ_META_ouvragesTitreInt']))"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:message>[ERROR] System meta "title" undefined</xsl:message>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>
              <xsl:otherwise>
                <xsl:choose>
                  <xsl:when test="$element/xf:meta[@code = 'DZ_META_ouvragesTitreInt']">
                    <xsl:sequence select="xfRes:genHtmlMeta($codeMeta, xfRes:ouvrages_createSecondaryTitle($element/xf:meta[@code = 'DZ_META_ouvragesTitreInt'], 
                      $element/xf:meta[@code = 'DZ_META_ouvragesTitreNiv'], $element/xf:meta[@code = 'DZ_META_ouvragesTitreNum']))"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:message>[ERROR] System meta "title" undefined</xsl:message>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:otherwise>
            <xsl:sequence select="xfRes:genHtmlMeta($codeMeta, xfRes:ouvrages_createTitleParagraphe($element))"/>
          </xsl:otherwise>
        </xsl:choose>  
      </xsl:when>
      <xsl:when test="$codeMeta = 'systemAbstract'"/>  
      
      <xsl:otherwise>
        <xsl:message>[ERROR] Metadata <xsl:value-of select="$codeMeta"/> undefined</xsl:message>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="xfRes:ouvrages_createTitle" as="element(html:div)">
    <xsl:param name="titre" as="element()"/>
    <div xmlns="http://www.w3.org/1999/xhtml">
      <xsl:value-of select="$titre/xf:value"/>
    </div>
  </xsl:function>
  
  <xsl:function name="xfRes:ouvrages_createSecondaryTitle" as="element(html:div)">
    <xsl:param name="titre1" as="element()"/>
    <xsl:param name="titre2" as="element()?"/>
    <xsl:param name="titre3" as="element()?"/>
    <div xmlns="http://www.w3.org/1999/xhtml">
      <xsl:if test="$titre2">
        <xsl:value-of select="$titre2/xf:value"/><xsl:value-of select="'&#160;'"/>
      </xsl:if>
      <xsl:if test="$titre3">
        <xsl:value-of select="$titre3/xf:value"/><xsl:value-of select="'&#160;-&#160;'"/>
      </xsl:if>
      <xsl:value-of select="$titre1/xf:value"/>
    </div>
  </xsl:function>
  
  <xsl:function name="xfRes:ouvrages_createAbstract" as="element(html:div)">
    <xsl:param name="titre" as="element()"/>
    <xsl:param name="collection" as="element()"/>
    <div xmlns="http://www.w3.org/1999/xhtml">
      <xsl:value-of select="$titre/xf:value"/><xsl:value-of select="'&#160;-&#160;'"/><xsl:value-of select="$collection/xf:value"/>
    </div>
  </xsl:function>
  
  <xsl:function name="xfRes:ouvrages_createTitleParagraphe" as="element(html:div)">
    <xsl:param name="entity" as="element()"/>
    <div xmlns="http://www.w3.org/1999/xhtml">
      <xsl:choose>
        <xsl:when test="$entity/content:TITP">
          <xsl:if test="$entity/content:NUME">
            <span class="NUME"><xsl:apply-templates select="$entity/content:NUME/node()" mode="xfRes:title2html"/></span>
            <xsl:value-of select="'&#160;-&#160;'"/>
          </xsl:if>
          <span class="TITP"><xsl:apply-templates select="$entity/content:TITP/node()" mode="xfRes:title2html"/></span>
        </xsl:when>
        <xsl:when test="$entity/content:NUME">
          <span class="NUME"><xsl:apply-templates select="$entity/content:NUME/node()" mode="xfRes:title2html"/></span>
        </xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <xsl:when test="$entity/content:PARA-TXT">
              <xsl:sequence select="xfRes:makeTruncatedTitle($entity/content:PARA-TXT[1])"/>
            </xsl:when>
            <xsl:otherwise>
              <!-- un titre vide ne convient pas, alors on utilise un blanc -->
              <xsl:value-of select="' '"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </div>
  </xsl:function>
  
  <xsl:function name="xfRes:ouvrages_createAbstractParagraphe" as="element(html:div)">
    <xsl:param name="entity" as="element()"/>
    <div xmlns="http://www.w3.org/1999/xhtml">
      <xsl:choose>
        <xsl:when test="$entity/content:TITP">
          <span class="TITP"><xsl:apply-templates select="$entity/content:TITP/node()" mode="xfRes:title2html"/></span>
        </xsl:when>
        <xsl:otherwise>
          <!-- un abstract vide ne convient pas, alors on utilise un blanc -->
          <xsl:value-of select="' '"/>
        </xsl:otherwise>
      </xsl:choose>
    </div>
  </xsl:function>
  
</xsl:stylesheet>
