<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xdoc="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:exslt="http://exslt.org/dates-and-times"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:ecm="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm"
  xmlns:xfe="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns:ouvrages="http://www.lefebvre-sarrut.eu/ns/dalloz/ouvrages"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns=""
  exclude-result-prefixes="#all"
  version="3.0">
  
<xsl:output method="xml" doctype-public="DALLOZ//DTD OUVRAGES V1//FR" doctype-system="C:/DTD Dalloz/Ouvrages_V1/Ouvrages_V1.dtd" indent="no"/>
  
  <xsl:param name="server.name" select="'ecm'" as="xs:string"/>
  <xsl:variable name="query.base">
    <xsl:variable name="ecm.query.base" select="'http://10.16.14.46:8080/ecm-core/api/xmlExporter/exportToXML?flashId='"/>
    <xsl:variable name="MarkLogic.query.base" select="'http://fr000-devapp014.eu.els.local:8009/flash/getEEbyId.xqy?id='"/>
    <xsl:choose>
      <xsl:when test="lower-case($server.name)='marklogic'">
        <xsl:value-of select="$MarkLogic.query.base"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$ecm.query.base"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  
  <!-- LANCEMENT SANS MODE -->
  <xsl:template match="/">
    <xsl:apply-templates select="*" mode="xfRes:xmlPublish"/>
  </xsl:template>
  
  <xsl:template match="node()|@*" mode="xfRes:xmlPublish"/>
  
  <!-- ********************************************************* -->
  <!-- NEW CODE STARTS HERE                                      -->
  <!-- ********************************************************* -->
  <xsl:template match="editorialEntity" mode="xfRes:xmlPublish">
    <xsl:variable name="origine" select="metadata/meta[@code='DZ_META_ouvragesOrigine']/value"/>
    <xsl:if test="$origine!=''">
      <xsl:choose>
        <xsl:when test="@code=('DZ_TEE_ouvrages', 'DZ_TEE_ouvragesSecondaire', 'DZ_TEE_ouvragesParagraphe')">
          
          <xsl:element name="{$origine}">
            <xsl:apply-templates select="@*" mode="attribute"/>
            <xsl:apply-templates select="metadata/meta" mode="attribute"/>
            <xsl:apply-templates select="metadata" mode="#current">
              <xsl:with-param name="context" select="@code"/>
            </xsl:apply-templates>
            <xsl:apply-templates select="body/*" mode="#current"/>
            <xsl:apply-templates select="relations" mode="#current"/>
          </xsl:element>
        </xsl:when>
        <xsl:otherwise>
          <xsl:message terminate="yes">[ERROR] Unknown TEE found: <xsl:value-of select="@code"/> !</xsl:message>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:template>
  
  <!-- attribute mode -->
  <xsl:template match="@* except @xf:historicalId" mode="attribute"/>
  
  <xsl:template match="@xf:historicalId" mode="attribute">
    <xsl:attribute name="ID" select="substring-after(., 'dalloz_')"/>
  </xsl:template>
  
  <xsl:template match="meta" mode="attribute"/>  
  <xsl:template match="meta[@code='DZ_META_idOld']" mode="attribute">
    <xsl:attribute name="ID_OLD" select="value"/>
  </xsl:template>
  
  <!-- xfRes:xmlPublish mode on metadata -->
  <xsl:template match="metadata" mode="xfRes:xmlPublish">
    <xsl:param name="context"/>
    <xsl:choose>
      <xsl:when test="$context='DZ_TEE_ouvrages'">
        <xsl:element name="METADONNEES">
          <xsl:apply-templates select="meta[@code='DZ_META_ouvragesIsbn']" mode="#current"/>
          <xsl:variable name="dates">
            <xsl:apply-templates select="meta[@code='DZ_META_ouvragesAnnee']" mode="#current"/>
            <xsl:apply-templates select="meta[@code='DZ_META_ouvragesDate2']" mode="#current"/>
            <xsl:apply-templates select="meta[@code='DZ_META_ouvragesDate3']" mode="#current"/>
            <xsl:apply-templates select="meta[@code='DZ_META_ouvragesDate4']" mode="#current"/>
          </xsl:variable>
          <xsl:if test="$dates!=''">
            <xsl:element name="DATES">
              <xsl:sequence select="$dates"/>
            </xsl:element>
          </xsl:if>
          <xsl:variable name="titre">
            <xsl:variable name="titre.numero">
              <xsl:apply-templates select="meta[@code='DZ_META_ouvragesTitreNiv']" mode="#current"/>
              <xsl:apply-templates select="meta[@code='DZ_META_ouvragesTitreNum']" mode="#current"/>
            </xsl:variable>
            <xsl:if test="$titre.numero!=''">
              <xsl:element name="TIT-NUMERO">
                <xsl:sequence select="$titre.numero"/>
              </xsl:element>
            </xsl:if>
            <xsl:variable name="titre.intitule">
              <xsl:apply-templates select="meta[@code='DZ_META_ouvragesTitreInt']" mode="#current"/>
            </xsl:variable>
            <xsl:if test="$titre.intitule!=''">
              <xsl:element name="TIT-INTITULE">
                <xsl:sequence select="$titre.intitule"/>
              </xsl:element>
            </xsl:if>
          </xsl:variable>
          <xsl:if test="$titre!=''">
            <xsl:element name="TITRE">
              <xsl:sequence select="$titre"/>
            </xsl:element>
          </xsl:if>
        </xsl:element>
      </xsl:when>
      <xsl:when test="$context='DZ_TEE_ouvragesSecondaire'">
        <xsl:element name="METAS">
          <!-- TODO -->
        </xsl:element>
        <xsl:element name="TITRE">
          <!-- TODO -->
        </xsl:element>
      </xsl:when>
      <xsl:when test="$context='structuralNode'">
        <xsl:variable name="titre">
          <xsl:variable name="titre.numero">
            <xsl:apply-templates select="meta[@code='DZ_META_ouvragesTitreNiv']" mode="#current"/>
            <xsl:apply-templates select="meta[@code='DZ_META_ouvragesTitreNum']" mode="#current"/>
          </xsl:variable>
          <xsl:if test="$titre.numero!=''">
            <xsl:element name="TIT-NUMERO">
              <xsl:sequence select="$titre.numero"/>
            </xsl:element>
          </xsl:if>
          <xsl:variable name="titre.intitule">
            <xsl:apply-templates select="meta[@code='DZ_META_ouvragesTitreInt']" mode="#current"/>
          </xsl:variable>
          <xsl:if test="$titre.intitule!=''">
            <xsl:element name="TIT-INTITULE">
              <xsl:sequence select="$titre.intitule"/>
            </xsl:element>
          </xsl:if>
        </xsl:variable>
        <xsl:if test="$titre!=''">
          <xsl:element name="TITRE">
            <xsl:sequence select="$titre"/>
          </xsl:element>
        </xsl:if>
      </xsl:when>
    </xsl:choose>
  </xsl:template>
  
  <!-- xfRes:xmlPublish mode on meta -->
  <xsl:template match="meta" mode="xfRes:xmlPublish"/>
  
  <xsl:template match="meta[@code='DZ_META_ouvragesIsbn']" mode="xfRes:xmlPublish">
    <xsl:element name="ISBN"><xsl:value-of  select="value"/></xsl:element>
  </xsl:template>
  
  <xsl:template match="meta[@code='DZ_META_ouvragesAnnee']" mode="xfRes:xmlPublish">
    <xsl:element name="DATE-PUBLI"><xsl:value-of  select="value"/></xsl:element>
  </xsl:template>
  
  <xsl:template match="meta[@code='DZ_META_ouvragesDate2']" mode="xfRes:xmlPublish">
    <xsl:element name="DATE-CREA"><xsl:value-of  select="value"/></xsl:element>
  </xsl:template>
  
  <xsl:template match="meta[@code='DZ_META_ouvragesDate3']" mode="xfRes:xmlPublish">
    <xsl:element name="DATE-MAJ"><xsl:value-of  select="value"/></xsl:element>
  </xsl:template>
  
  <xsl:template match="meta[@code='DZ_META_ouvragesDate4']" mode="xfRes:xmlPublish">
    <xsl:element name="DATE-APPLI"><xsl:value-of  select="value"/></xsl:element>
  </xsl:template>
  
  <xsl:template match="meta[@code='DZ_META_ouvragesTitreNiv']" mode="xfRes:xmlPublish">
    <xsl:element name="TIT-NIV"><xsl:value-of  select="value"/></xsl:element>
  </xsl:template>
  
  <xsl:template match="meta[@code='DZ_META_ouvragesTitreNum']" mode="xfRes:xmlPublish">
    <xsl:element name="TIT-NUM"><xsl:value-of  select="value"/></xsl:element>
  </xsl:template>
  
  <xsl:template match="meta[@code='DZ_META_ouvragesTitreInt']" mode="xfRes:xmlPublish">
    <xsl:element name="TIT-INTT"><xsl:value-of  select="value"/></xsl:element>
  </xsl:template>
  
  <!-- xfRes:xmlPublish mode on body/* -->
  <xsl:template match="queryNode" mode="xfRes:xmlPublish"/>
  
  <!-- xfRes:xmlPublish mode on relations -->
  <xsl:template match="relations" mode="xfRes:xmlPublish"/>
  
 <xsl:template match="contentNode" mode="xfRes:xmlPublish">
    <!--<xsl:apply-templates select="metadata" mode="xfRes:xmlPublish"/>-->
   <xsl:apply-templates select="content" mode="xfRes:xmlPublish"/>
  </xsl:template>
  
  <xsl:template match="content" mode="xfRes:xmlPublish">
    <xsl:apply-templates select="*[1]/*" mode="content"/>
  </xsl:template>
  
  <xsl:template match="node() except element()|@*" mode="content">
    <xsl:copy/>
  </xsl:template>
  
  <xsl:template match="*" mode="content">
    <xsl:element name="{name()}" namespace="">
      <xsl:apply-templates select="@*|node()" mode="content" />
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="ouvrages:REF" mode="content">
    <xsl:element name="REF" namespace="">
      <xsl:copy-of select="@TYPE"/>
      <xsl:variable name="target" select="ancestor::editorialEntity[1]/relations/relation[@xf:id=current()/@xf:idRef]/ref/@xf:targetResId"/>
      <xsl:choose>
        <xsl:when test="$target and $target!='' and $target!='NULL'">
          <xsl:variable name="target.uri" select="concat($query.base, $target)"/>
          <xsl:variable name="targetid"  select="doc($target.uri)/editorialEntity/@xf:historicalId"/>
          <xsl:attribute name="REFID" select="substring-after($targetid, 'dalloz_')"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="REFID" select="'XXX'"/>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="node()" mode="content" />
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="ouvrages:table" mode="content">
    <xsl:element name="TABLE-CALS" namespace="">
      <xsl:apply-templates select="@*|node()" mode="content" />
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="structuralNode[metadata/meta[@code='DZ_META_ouvragesOrigine']/value='COMPLEMENTS']" mode="xfRes:xmlPublish"/>
  
  <xsl:template match="structuralNode" mode="xfRes:xmlPublish">
    <xsl:variable name="origine" select="metadata/meta[@code='DZ_META_ouvragesOrigine']/value"/>
    <xsl:element name="{$origine}">
      <xsl:apply-templates select="@*" mode="attribute"/>
      <xsl:apply-templates select="metadata/meta" mode="attribute"/>
      <xsl:apply-templates select="metadata" mode="#current">
        <xsl:with-param name="context" select="'structuralNode'"/>
      </xsl:apply-templates>
     <xsl:apply-templates select="title" mode="#current"/>
     <!--<xsl:apply-templates select="* except (metadata, title)" mode="#current"/>-->
      <xsl:for-each-group select="* except (metadata, title)" group-adjacent="local-name()">
        <xsl:choose>
        <xsl:when test="current-grouping-key()='structuralNode'">
          <xsl:for-each-group select="current-group()" group-adjacent="metadata/meta[@code='DZ_META_ouvragesOrigine']/value">
            <xsl:choose>
              <xsl:when test="current-grouping-key()=('NIV12', 'NIV11', 'NIV10', 'NIV9', 'NIV8', 'NIV7', 'NIV6', 'NIV5', 'NIV4', 'NIV3', 'NIV2', 'NIV1')">
                <xsl:choose>
                  <xsl:when test="$origine = ('NIVEAU6', 'NIVEAU5', 'NIVEAU4', 'NIVEAU3', 'NIVEAU2', 'NIVEAU1', 'ANNEXE')">
                    <xsl:element name="NIVO-CORPS">
                      <xsl:apply-templates select="current-group()" mode="#current"/>
                    </xsl:element>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:apply-templates select="current-group()" mode="#current"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>
              <xsl:otherwise>
                <xsl:apply-templates select="current-group()" mode="#current"/>
              </xsl:otherwise>
            </xsl:choose>
         </xsl:for-each-group>
        </xsl:when>
          <xsl:when test="current-grouping-key()='referenceNode'">
            <xsl:choose>
              <xsl:when test="current-group()[1]/@code=('DZ_TR_accrochage_Ouvrage_OuvragePara', 'DZ_TR_accrochage_OuvrageSecondaire_OuvragePara')">
                <xsl:element name="NIV-TEXTE">
                  <xsl:apply-templates select="current-group()" mode="xfRes:xmlPublish"/>
                </xsl:element>
              </xsl:when>
              <xsl:otherwise>
                <xsl:apply-templates select="current-group()" mode="xfRes:xmlPublish"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates select="current-group()" mode="#current"/>
        </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each-group>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="title" mode="xfRes:xmlPublish"/>
  
  <xsl:template match="referenceNode[@code=('DZ_TR_accrochage_Ouvrage_OuvrageSecondaire', 'DZ_TR_accrochage_Ouvrage_OuvragePara', 'DZ_TR_accrochage_OuvrageSecondaire_OuvragePara')]" mode="xfRes:xmlPublish">
    <xsl:variable name="target.uri" select="concat($query.base, ref/@xf:targetResId)"/>
    <xsl:apply-templates select="doc($target.uri)"  mode="xfRes:xmlPublish"/>
  </xsl:template>
  
</xsl:stylesheet>