<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns:saxon="http://saxon.sf.net/" 
  xmlns:html="http://www.w3.org/1999/xhtml" 
  xmlns="http://www.w3.org/1999/xhtml"
  exclude-result-prefixes="#all"
  version="3.0">
  
  <!--xpath-default-namespace="http://modeles.dalloz.fr/modeles/reference"-->
  <!--
    "*:" car cette xslt est utilisée dans le sas donc ns = celui de ouvrages
    mais aussi dans certaines conversion de xf-resources donc ns = celui assigné par xmlFirst
  -->
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Converti les élément de titres ou paragraphe simple en html.</xd:p>
    </xd:desc>
  </xd:doc>
  
  <!--==================================================-->
  <!--MODE "xfRes:title2html"-->
  <!--==================================================-->
  
  <!--FIXME : attributs ?-->
  
  <xsl:template match="*:FOOTNOTE" mode="xfRes:title2html"/>
  
  <xsl:template match="*:REF" mode="xfRes:title2html"/>
  
  <xsl:template match="*:EXP" mode="xfRes:title2html">
    <sup xmlns="http://www.w3.org/1999/xhtml"><xsl:apply-templates mode="#current"/></sup>
  </xsl:template>
  
  <!-- on ne veut pas de titre avec un saut de ligne -->
  <xsl:template match="*:BR" mode="xfRes:title2html">
    <span class="{local-name(.)}" xmlns="http://www.w3.org/1999/xhtml">&#x2003;</span>
  </xsl:template>
  
    <xsl:template match="*:GRAS" mode="xfRes:title2html">
    <span xmlns="http://www.w3.org/1999/xhtml" style="font-weight:bold;"><xsl:apply-templates mode="#current"/></span>
  </xsl:template>
  
  <xsl:template match="*:ITAL" mode="xfRes:title2html">
    <span xmlns="http://www.w3.org/1999/xhtml" style="font-style:italic;"><xsl:apply-templates mode="#current"/></span>
  </xsl:template>
  
  <xsl:template match="*:PTCAP" mode="xfRes:title2html">
    <span xmlns="http://www.w3.org/1999/xhtml" style="font-variant: small-caps;"><xsl:apply-templates mode="#current"/></span>
  </xsl:template>
  
  <xsl:template match="*:ROMA" mode="xfRes:title2html">
    <span xmlns="http://www.w3.org/1999/xhtml" style="font-style:normal;"><xsl:apply-templates mode="#current"/></span>
  </xsl:template>
  
  <xsl:template match="*:PICTO" mode="xfRes:title2html">
    <span class="{local-name(.)}" xmlns="http://www.w3.org/1999/xhtml"/>
  </xsl:template>
  
  <xsl:template match="*" mode="xfRes:title2html" priority="-1">
    <span class="{local-name(.)}" xmlns="http://www.w3.org/1999/xhtml"><xsl:apply-templates mode="#current"/></span>
  </xsl:template>
  
  <!-- Les SOUR des codes sont en italiques -->
  <xsl:template match="*:SOUR" mode="xfRes:title2html">
    <span class="{local-name(.)}" xmlns="http://www.w3.org/1999/xhtml" style="font-style:italic;"><xsl:apply-templates mode="#current"/></span>
  </xsl:template>
  
  <!-- 
  <xsl:template match="*:motRep" mode="xfRes:titre2html">
    <strong class="motRep">
      <xsl:apply-templates mode="#current"/>
    </strong>
  </xsl:template>
  
  <xsl:template match="*:e" mode="xfRes:titre2html">
    <sup>
      <xsl:apply-templates mode="#current"/>
    </sup>
  </xsl:template>
  
  <xsl:template match="*:ind" mode="xfRes:titre2html">
    <sub>
      <xsl:apply-templates mode="#current"/>
    </sub>
  </xsl:template>
  
  <xsl:template match="*:ital" mode="xfRes:titre2html">
    <em>
      <xsl:apply-templates mode="#current"/>
    </em>
  </xsl:template>
  
  <xsl:template match="*:url " mode="xfRes:titre2html">
    <a href="{@href}">
      <xsl:apply-templates mode="#current"/>
    </a>
  </xsl:template>
  
  <xsl:template match="*:graphique" mode="xfRes:titre2html">
    FIXME : xf:id ?-->
    <!-- <img href="{@href}">
      <xsl:apply-templates mode="#current"/>
    </img>
  </xsl:template>
  
  <xsl:template match="*:refDocEnrichie" mode="xfRes:titre2html">
    FIXME : xf:id ?-->
    <!-- <span class="refDocEnrichie">
      <xsl:apply-templates mode="#current"/>
    </span>
  </xsl:template>-->
  
  <!--==================================================-->
  <!-- Troncature pour génération du titre              -->
  <!--==================================================--> 
  
  <xsl:param name="defaultTruncateLength" select="xs:positiveInteger(30)" as="xs:positiveInteger"/>
  
  <xsl:template match="text()" mode="truncate_step0" priority="1">
    <xsl:sequence select="replace(., '\s+', ' ')"/>
  </xsl:template>
  
  <xsl:template match="node()|@*" mode="truncate_step0 truncate_step1 truncate_step2 truncate_step3">
    <xsl:param name="maxlength" select="$defaultTruncateLength" tunnel="yes" as="xs:positiveInteger"/>
    <xsl:copy>
      <xsl:apply-templates select="node()|@*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="text()" mode="truncate_step1" priority="1">
    <xsl:param name="maxlength" select="$defaultTruncateLength" tunnel="yes" as="xs:positiveInteger"/>
    <xsl:choose>
      <xsl:when test="sum(./preceding::text()/string-length()) ge $maxlength"/>
      <xsl:otherwise>
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="text()[not(following::text())]" mode="truncate_step2">
    <xsl:param name="maxlength" select="$defaultTruncateLength" tunnel="yes" as="xs:positiveInteger"/>
    <xsl:variable name="vRemaininingLength" select="$maxlength - sum(preceding::text()/string-length())"/>
    <xsl:variable name="regexp" select="concat('^(.{1,', $vRemaininingLength, '})\W.*$')"/>
    <xsl:choose>
      <xsl:when test="$vRemaininingLength ge ./string-length()">
        <xsl:sequence select="."/>
      </xsl:when>
      <xsl:when test="$vRemaininingLength gt 0 and matches(., $regexp)">
        <xsl:sequence select="replace(., $regexp, '$1')"/>
      </xsl:when>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="/*[1]" mode="truncate_step3">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()" mode="#current"/>
      <xsl:text>...</xsl:text>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="*" mode="truncate_step2" priority="1">
    <xsl:param name="maxlength" select="$defaultTruncateLength" tunnel="yes" as="xs:positiveInteger"/>
    <xsl:choose>
      <xsl:when test="not(.//text())"/>
      <xsl:when test="(.//text())[1][sum(preceding::text()/string-length()) ge $maxlength]"/>
      <xsl:otherwise>
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:function name="xfRes:truncateTitle" as="element()">
    <xsl:param name="element" as="element()"/>
    <xsl:param name="maxlength" as="xs:integer?"/>
    
    <xsl:variable name="truncateLength" as="xs:positiveInteger">
      <xsl:choose>
        <xsl:when test="$maxlength castable as xs:positiveInteger">
          <xsl:value-of select="$maxlength cast as xs:positiveInteger"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:message>[WARNING] Mauvaise valeur de parametre de troncature: <xsl:value-of select="$maxlength"/> !</xsl:message>
          <xsl:value-of select="$defaultTruncateLength"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <!-- On simplifie  les \s multiples, tout en tardant les blancs simples en début et fin de tags -->
    <xsl:variable name="step">
      <xsl:apply-templates select="$element" mode="truncate_step0"/>
    </xsl:variable>
    <xsl:variable name="totalSize" select="sum($step//text()/string-length())"/>
    <!-- on vide les noeuds textes dépassant la limite -->
    <xsl:variable name="step">
      <xsl:apply-templates select="$step" mode="truncate_step1">
        <xsl:with-param name="maxlength" select="$truncateLength" tunnel="yes"/>
      </xsl:apply-templates>
    </xsl:variable>
    <!-- On tronque et supprime les noeuds vides -->
    <xsl:variable name="step">
      <xsl:apply-templates select="$step" mode="truncate_step2">
        <xsl:with-param name="maxlength" select="$truncateLength" tunnel="yes"/>
      </xsl:apply-templates>
    </xsl:variable>
    <xsl:variable name="truncatedSize" select="sum($step//text()/string-length())"/>
    <!-- On ajoute ... si necessaire -->
    <xsl:choose>
      <xsl:when test="$truncatedSize=$totalSize">
        <xsl:sequence select="$step/*"/> 
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="step">
          <xsl:apply-templates select="$step" mode="truncate_step3"/>
        </xsl:variable>
        <xsl:sequence select="$step/*"/> 
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="xfRes:truncateTitle" as="element()">
    <xsl:param name="element" as="element()"/>
    <xsl:sequence select="xfRes:truncateTitle($element, $defaultTruncateLength)"/> 
  </xsl:function>
  
  <xsl:function name="xfRes:makeTitle">
    <xsl:param name="element" as="element()"/>
    <xsl:apply-templates select="$element/node()" mode="xfRes:title2html"/>
  </xsl:function>  
  
  <xsl:function name="xfRes:makeTruncatedTitle">
    <xsl:param name="element" as="element()"/>
    <xsl:param name="maxlength" as="xs:integer?"/>
    <xsl:variable name="truncatedTitle">
      <xsl:apply-templates select="xfRes:truncateTitle($element, $maxlength)/node()" mode="xfRes:title2html"/>
    </xsl:variable>
    <xsl:apply-templates select="$truncatedTitle" mode="xfRes:onePara"/>
  </xsl:function>
  
  <xsl:function name="xfRes:makeTruncatedTitle">
    <xsl:param name="element" as="element()"/>
    <xsl:variable name="truncatedTitle">
      <xsl:apply-templates select="xfRes:truncateTitle($element)/node()" mode="xfRes:title2html"/>
    </xsl:variable>
    <xsl:apply-templates select="$truncatedTitle" mode="xfRes:onePara"/>
  </xsl:function>
  
  <xsl:template match="node() | @*" mode="xfRes:onePara">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="html:p[preceding::html:p]" mode="xfRes:onePara"/>
  
</xsl:stylesheet>