<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/efl/infoCommentaire"
  xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all"
  version="3.0">
  
  <xsl:import href="../../xf-lib/_common/xf-common.xsl"/>
  <xsl:import href="EFL_TEE_srcUiTexte.common.xsl"/>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Default values for required metadatas</xd:p>
    </xd:desc>
  </xd:doc> 
  <xsl:variable name="defaultTxtNatureValue" select="'[Natures Textes]'" as="xs:string"/>
  <xsl:variable name="defaultTxtNomTexteValue" select="'[Nom des Textes]'" as="xs:string"/>
  <xsl:variable name="defaultTxtNumeroValue" select="'[Numéro de Décision]'" as="xs:string"/>
  <xsl:variable name="defaultTxtPubValue" select="'[Nom Publication]'" as="xs:string"/>
  <xsl:variable name="defaultTxtOrganismeEmetteurValue" select="'[Organisme Émetteur]'" as="xs:string"/>
  <xsl:variable name="defaultTxtDTValue" select="'[Date de Décision]'" as="xs:string"/> 
  
  <xsl:template match="xf:editorialEntity" mode="xfRes:EFL_TEE_treeRevueCxml.getCalculatedMetatada4UISourceTexte">
    <metadata>
      <meta code="systemTitle" as="element(html:div)">
        <value as="element(html:div)">
          <xsl:variable name="txtTitre" select="xf:getMetaValue(xf:getMeta(., 'ELS_META_txtTitre'))" as="xs:string?"/>
          <div xmlns="http://www.w3.org/1999/xhtml">
            <xsl:choose>
              <xsl:when test="normalize-space($txtTitre) != ''">
                <xsl:value-of select="$txtTitre"/>
              </xsl:when>
              <xsl:when test="xf:getMetaValue(xf:getMeta(., 'ELS_META_txtClasseRefDoc')) = $classRefDocUsed">
                <xsl:sequence select="xf:srcTxt-getTitleAccordingClassRefDoc(.)"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:message>[INFO][XF-ECM-RES] System meta "title" undefined for xf:id=<xsl:value-of select="@xf:id"/></xsl:message>
              </xsl:otherwise>
            </xsl:choose>
          </div>
        </value>
      </meta>
      <meta code="systemAbstract" as="element(html:div)">
        <value as="element(html:div)">
          <div xmlns="http://www.w3.org/1999/xhtml">
            <!-- nothing to write currently -->
          </div>
        </value>
      </meta>
    </metadata>
  </xsl:template>

  <xd:doc>
    <xd:desc>
      <xd:p>This function calculates the "systemTitle" metadata according the value of the 'ELS_META_txtClasseRefDoc' metadata</xd:p>
      <xd:p>This function must generate at best according to "meta4_generateUiSourcesFormats.xsl" from CXML project</xd:p>
      <xd:p>FIXME : we do not have access to referential files to get 'libelleAbrege' or 'libelleComplet'</xd:p>
      <xd:p>We only use the metadata verbalization to create the value of "systemTitle" metadata</xd:p>
      <xd:p>See Jira FLAS-85</xd:p>
    </xd:desc>
    <xd:param name="e">current xf:editorialEntity</xd:param>
    <xd:return>The calculated systemTitle value otherwise nothing</xd:return>
  </xd:doc>
  <!-- Correspondence between text field, metadata id, and metadata type 
    Natures Textes                            - ELS_META_txtNature              : list
    Organisme Émetteur                        - ELS_META_txtOrganismeEmetteur   : list
    Numéro de Décision                        - ELS_META_txtNumero              : text - todo (done)
    Numéro NOR                                - ELS_META_txtNor                 : text - todo (done)
    Nom des Textes                            - ELS_META_txtNomTexte            : list
    Date de Décision                          - ELS_META_txtDT                  : date
    Num Article                               - ELS_META_txtArt                 : text - todo (done)
    Date de Publication                       - ELS_META_txtDtPub               : date 
    Numéro Publication                        - ELS_META_txtNPub                : text - todo (done)
    Renseignement Complémentaires Publication - ELS_META_txtRcPub               : text - todo (done)
    Numéro Paragraphe                         - ELS_META_txtNPar                : text - todo (done)
    Subdivisions Articles Complément          - ELS_META_txtSubArt              : text - todo (done)
    Nom Publication                           - ELS_META_txtPub                 : list 
    Titre des Texte                           - ELS_META_txtTitreTexte          : text - todo (maybe ?)
    Format Système Titre                      - ELS_META_txtTitre               : text - nothing to do
    ClasseRefDoc                              - ELS_META_txtClasseRefDoc        : list    
  -->
  <xsl:function name="xf:srcTxt-getTitleAccordingClassRefDoc" as="xs:string*">         
    <xsl:param name="e" as="element(xf:editorialEntity)"/>
    <!-- get classRefDoc value -->    
    <xsl:variable name="classRefDoc" select="xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtClasseRefDoc'))" as="xs:string*"/>
    <xsl:variable name="calculatedSystemTitle" as="xs:string*">
      <xsl:choose>
        <xsl:when test="$classRefDoc = 'tbofip'"> 
          <xsl:sequence select="concat(
            if (xf:hasMeta($e, 'ELS_META_txtNature')) then xf:getMetaVerbalization(xf:getMeta($e, 'ELS_META_txtNature')) else $defaultTxtNatureValue, ' ',
            if (xf:hasMeta($e, 'ELS_META_txtNomTexte')) then xf:getMetaVerbalization(xf:getMeta($e, 'ELS_META_txtNomTexte')) else $defaultTxtNomTexteValue, ' - ',
            if (xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNumero')) != '') then xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNumero')) else $defaultTxtNumeroValue,
            if (xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNPar')) != '') then concat(' n°', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNPar'))) else '',
            if (xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtDateMAJ')) != '') then concat(' du ', els:date-number-slash(xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtDateMAJ')))) else ''
            )"/>
        </xsl:when>
        <xsl:when test="$classRefDoc = 'tcode'">
          <xsl:sequence select="concat(
            if (xf:hasMeta($e, 'ELS_META_txtNomTexte')) then xf:getMetaVerbalization(xf:getMeta($e, 'ELS_META_txtNomTexte')) else '',
            if (xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtArt')) != '') then concat(' art.&#160;', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtArt'))) else '',
            if (xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtSubArt')) != '') then concat(', ', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtSubArt'))) else '' 
            )"/>          
        </xsl:when>
        <xsl:when test="$classRefDoc = 'teur'">
          <xsl:sequence select="concat(
            if (xf:hasMeta($e, 'ELS_META_txtNature')) then xf:getMetaVerbalization(xf:getMeta($e, 'ELS_META_txtNature')) else $defaultTxtNatureValue, ' ',
            if (xf:hasMeta($e, 'ELS_META_txtOrganismeEmetteur')) then xf:getMetaVerbalization(xf:getMeta($e, 'ELS_META_txtOrganismeEmetteur')) else '',
            if (xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNumero')) != '') then concat(' n°', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNumero'))) else $defaultTxtNumeroValue,
            ' du ',
            if (xf:hasMeta($e, 'ELS_META_txtDT')) then els:date-number-slash(xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtDT'))) else $defaultTxtDTValue
            )"/>
        </xsl:when>
        <xsl:when test="$classRefDoc = 'tlegreg'">          
          <xsl:sequence select="concat(
            if (xf:hasMeta($e, 'ELS_META_txtNature')) then xf:getMetaVerbalization(xf:getMeta($e, 'ELS_META_txtNature')) else $defaultTxtNatureValue,
            if (xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNumero')) != '') then concat(' n°', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNumero'))) 
            else if(xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNor')) != '') then concat(' n°', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNor')))
            else '',
            if (xf:hasMeta($e, 'ELS_META_txtDT')) then concat(' du ', els:date-number-slash(xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtDT')))) else '',
            if (xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtArt')) != '') then concat(' art.&#160;', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtArt'))) else '',
            (: publication 'class' :)
            '&#160;: ', 
            if (xf:hasMeta($e, 'ELS_META_txtPub')) then xf:getMetaVerbalization(xf:getMeta($e, 'ELS_META_txtPub')) else $defaultTxtPubValue,
            if (xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNPub')) != '') then concat(' n°', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNPub'))) else '',                    
            if (xf:hasMeta($e, 'ELS_META_txtDtPub')) then concat(' du ', els:date-number-slash(xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtDtPub')))) else '',
            if (xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtRcPub')) != '') then concat(' ', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtRcPub'))) else '',
            (: /!\ pas de txtTitre :)
            if ((xf:getMeta($e, 'ELS_META_txtNature')/xf:value/xf:ref/@idRef='Arrete') and (xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtTitreTexte')) != '')) then concat(' ', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtTitreTexte'))) else ''                        
            )"/>
        </xsl:when>
        <xsl:when test="$classRefDoc = 'torgadm'">
          <xsl:sequence select="
            if (xf:hasMeta($e, 'ELS_META_txtNature')) then concat(
            xf:getMetaVerbalization(xf:getMeta($e, 'ELS_META_txtNature')), ' ',
              if (xf:hasMeta($e, 'ELS_META_txtOrganismeEmetteur')) then xf:getMetaVerbalization(xf:getMeta($e, 'ELS_META_txtOrganismeEmetteur')) else $defaultTxtOrganismeEmetteurValue,
              ' n°', if(xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNumero')) != '') then xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNumero')) else $defaultTxtNumeroValue,
              ' du ',
              if (xf:hasMeta($e, 'ELS_META_txtDT')) then els:date-number-slash(xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtDT'))) else $defaultTxtDTValue
              ) 
            else 
              if (xf:hasMeta($e, 'ELS_META_txtPub')) then concat( 
              if (xf:hasMeta($e, 'ELS_META_txtOrganismeEmetteur')) then xf:getMetaVerbalization(xf:getMeta($e, 'ELS_META_txtOrganismeEmetteur')) else $defaultTxtOrganismeEmetteurValue, 
                (: publication 'class' :)
                '&#160;: ',
                if (xf:hasMeta($e, 'ELS_META_txtPub')) then xf:getMetaVerbalization(xf:getMeta($e, 'ELS_META_txtPub')) else $defaultTxtPubValue,
                if (xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNPub')) != '') then concat(' n°', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNPub'))) else '',                    
                if (xf:hasMeta($e, 'ELS_META_txtDtPub') and els:isIsoDate(els:makeIsoDate(els:date-number-dash(xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtDtPub'))), '-'))) then concat(' du ', els:date-number-slash(xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtDtPub')))) else '',
                if ((xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtRcPub')) != '')) then concat(' ', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtRcPub'))) else ''
              )                
              else ''
            "/>
        </xsl:when>
        <xsl:when test="$classRefDoc = 'ttp'">
          <xsl:sequence select="concat(
            if(xf:hasMeta($e, 'ELS_META_txtNature')) then xf:getMetaVerbalization(xf:getMeta($e, 'ELS_META_txtNature')) else $defaultTxtNatureValue,
            if(xf:hasMeta($e, 'ELS_META_txtOrganismeEmetteur')) then concat(' ', xf:getMetaVerbalization(xf:getMeta($e, 'ELS_META_txtOrganismeEmetteur'))) else '',
            if(xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNumero')) != '') then concat(' n°', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNumero'))) else concat(' ', $defaultTxtNumeroValue),
            ' du ',
            if (xf:hasMeta($e, 'ELS_META_txtDT')) then els:date-number-slash(xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtDT'))) else $defaultTxtDTValue            
            )"/>
        </xsl:when>
        <xsl:when test="$classRefDoc = 'tint'">
          <xsl:sequence select="concat(
            if(xf:hasMeta($e, 'ELS_META_txtNature')) then xf:getMetaVerbalization(xf:getMeta($e, 'ELS_META_txtNature')) else $defaultTxtNatureValue,
            if (xf:hasMeta($e, 'ELS_META_txtDT')) then concat(' du ', els:date-number-slash(xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtDT')))) else '',
            if(xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNumero')) != '') then concat(' n°', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtNumero'))) else '',
            if(xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtArt')) != '') then concat(' art.&#160;', xf:getMetaValue(xf:getMeta($e, 'ELS_META_txtArt'))) else ''
            )"/>          
        </xsl:when>
        <xsl:otherwise>
          <xsl:message>[INFO][XF-ECM-RES] no classRefDoc defined for '<xsl:value-of select="$classRefDoc"/>' to calculate 'systemTitle' for EE @xf:id='<xsl:value-of select="$e/@xf:id"/>'</xsl:message>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:sequence select="normalize-space(string-join($calculatedSystemTitle, ''))"/>
  </xsl:function>

</xsl:stylesheet>