<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:efl="http://www.lefebvre-sarrut.eu/ns/efl"
  xmlns:infoCommentaire="http://www.lefebvre-sarrut.eu/ns/efl/infoCommentaire" 
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources" 
  xmlns:h="http://www.w3.org/1999/xhtml" 
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/efl/infoCommentaire" 
  xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  exclude-result-prefixes="#all" 
  version="3.0">

  <!-- Verbalization du code TEE -->
  <xsl:import href="EFL_TEE_tree.default.getProductVerbalization.xsl"/>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Generate default "systemTitle" metadata</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="xf:editorialEntity" mode="xfRes:EFL_TEE_treeRevueCxml.getCalculatedMetatada">
    <metadata>
      <meta code="systemTitle" as="element(html:div)">
        <value as="element(html:div)">
          <!-- Flas75 point 2 et 4 : suppression de la meta EFL_META_produitCx -->
          <xsl:variable name="shortVerbalizationCodeTee" select="xfRes:getVerbalizationCodeTee(@code, 'short')"/>
          <xsl:variable name="numeroOrdre" select="efl:getNumeroOrdre(xf:metadata/xf:meta[@code = 'EFL_META_numeroOrdre']/xf:value)" as="xs:string?"/>
          <xsl:variable name="anneePublication" select="xf:metadata/xf:meta[@code = 'EFL_META_anneePublication']/xf:value" as="xs:integer?"/>
          <!-- L'année est représentées sur les deux derniers digits -->
          <xsl:variable name="anneeTronquee" as="xs:integer?">
            <xsl:choose>
              <xsl:when test="$anneePublication > 1000">
                <xsl:value-of select="$anneePublication mod 1000"/>
              </xsl:when>
              <xsl:when test="$anneePublication > 100">
                <xsl:value-of select="$anneePublication mod 100"/>
              </xsl:when>
              <xsl:when test="$anneePublication > 10">
                <xsl:value-of select="$anneePublication"/>
              </xsl:when>
              <xsl:otherwise/>
            </xsl:choose>
          </xsl:variable>
          <div xmlns="http://www.w3.org/1999/xhtml">
            <xsl:variable name="systemTitle.calculated" select="concat($shortVerbalizationCodeTee, ' ', $numeroOrdre, '/', $anneeTronquee)" as="xs:string"/>
            <!--FLAS-47 : cas spécial où il y a une méta ELS_META_titreEE (non vide) : alors c'est elle qu'on mets dans le systemTitle-->
            <xsl:variable name="ELS_META_titreEE.value" select="xf:metadata/xf:meta[@code = 'ELS_META_titreEE']/xf:value" as="element(xf:value)?"/>
            <xsl:choose>
              <xsl:when test="string-join($ELS_META_titreEE.value/h:div//text(), '') != '' ">
                <xsl:sequence select="$ELS_META_titreEE.value/h:div/node()"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:sequence select="$systemTitle.calculated"/>
              </xsl:otherwise>
            </xsl:choose>
          </div>
        </value>
      </meta>
      <xsl:call-template name="xfRes:efl_getDefaultSystemAbstractMeta"/>
    </metadata>
  </xsl:template>   
   
  <xd:doc>
    <xd:desc>
      <xd:p>Generate "systemTitle" metadata for TEE (OM).</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="xf:editorialEntity" mode="xfRes:EFL_TEE_treeRevueCxml.getCalculatedMetatada4OM">
    <metadata>
      <meta code="systemTitle" as="element(html:div)">
        <value as="element(html:div)">
          <xsl:variable name="codeTee" select="@code" as="xs:string"/>
          <!-- Flas75 point 2 et 4 : suppression de la meta EFL_META_produitCx -->
          <xsl:variable name="shortVerbalizationCodeTee" select="xfRes:getVerbalizationCodeTee(@code, 'short')"/>
          <!-- HACK pour OM AVOCAT -> le code métier renseigné dans l'ECM est "AFF" et non "AVOCAT" (il correspond à ce qui est attendu côté EZPublish)
               Pour l'affichage du systemTitle, on substitue la verbalisation de "AFF" par "Avocat" pour afficher "le bon titre" dans l'ECM
               Exception à supprimer si les codes EZPublish sont modifiés -->
          <xsl:variable name="allMetiers"
                        select="for $metier in xf:metadata/xf:meta[@code = 'EFL_META_metier'] return
                                if ($codeTee = 'EFL_TEE_filActusOmAvocat' and $metier/xf:value/xf:ref/@idRef = 'AFF') then ('Avocat') else ($metier/xf:verbalization)"
                        as="xs:string*"/>
          <xsl:variable name="verbalizationMetier" select="string-join($allMetiers, ' - ')" as="xs:string?"/>
          <div xmlns="http://www.w3.org/1999/xhtml">
            <xsl:variable name="systemTitle.calculated" select="concat('Actualités OM – INNEO ', $verbalizationMetier)" as="xs:string"/>
            <xsl:sequence select="normalize-space($systemTitle.calculated)"/>
          </div>
        </value>
      </meta>      
      <xsl:call-template name="xfRes:efl_getDefaultSystemAbstractMeta"/>
    </metadata>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Generate "systemTitle" metadata for TEE (Matiere).</xd:p>      
    </xd:desc>
  </xd:doc>
  <xsl:template match="xf:editorialEntity" mode="xfRes:EFL_TEE_treeRevueCxml.getCalculatedMetatada4Matiere">
    <metadata>
      <meta code="systemTitle" as="element(html:div)">
        <value as="element(html:div)">
          <!-- Flas75 point 2 et 4 : suppression de la meta EFL_META_produitCx -->
          <xsl:variable name="shortVerbalizationCodeTee" select="xfRes:getVerbalizationCodeTee(@code, 'short')"/>
          <xsl:variable name="verbalizationMatiere" select="string-join(xf:metadata/xf:meta[@code = 'EFL_META_matiere']/xf:verbalization, ' - ')" as="xs:string?"/>
          <div xmlns="http://www.w3.org/1999/xhtml">
            <xsl:variable name="systemTitle.calculated" select="concat('Fil Info ', $verbalizationMatiere)" as="xs:string"/>
            <xsl:sequence select="normalize-space($systemTitle.calculated)"/>
          </div>
        </value>
      </meta>      
      <xsl:call-template name="xfRes:efl_getDefaultSystemAbstractMeta"/>
    </metadata>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Generate "systemTitle" metadata for TEE (Quotidienne).</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="xf:editorialEntity" mode="xfRes:EFL_TEE_treeRevueCxml.getCalculatedMetatada4Quotidienne">
    <metadata>
      <meta code="systemTitle" as="element(html:div)">
        <value as="element(html:div)">
          <!-- Flas75 point 2 et 4 : suppression de la meta EFL_META_produitCx -->
          <xsl:variable name="shortVerbalizationCodeTee" select="xfRes:getVerbalizationCodeTee(@code, 'short')"/>
          <xsl:variable name="readDateParution" select="xf:metadata/xf:meta[@code = 'EFL_META_dateParution']/xf:value/text()" as="xs:string"/>
          <xsl:variable name="dateParution" select="els:getIsoDateFromString($readDateParution)" as="xs:string"/>
          <div xmlns="http://www.w3.org/1999/xhtml">
            <xsl:variable name="systemTitle.calculated" select="concat($shortVerbalizationCodeTee, ' du ', els:getDayFromIsoDate($dateParution), ' ', els:verbalizeMonthFromNum(els:getMonthFromIsoDate($dateParution)), ' ', els:getYearFromIsoDate($dateParution))" as="xs:string"/>
            <xsl:variable name="ELS_META_titreEE.value" select="xf:metadata/xf:meta[@code = 'ELS_META_titreEE']/xf:value" as="element(xf:value)?"/>
            <xsl:choose>
              <xsl:when test="string-join($ELS_META_titreEE.value/h:div//text(), '') != '' ">
                <xsl:sequence select="$ELS_META_titreEE.value/h:div/node()"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:sequence select="$systemTitle.calculated"/>
              </xsl:otherwise>
            </xsl:choose>            
          </div>
        </value>
      </meta>      
      <xsl:call-template name="xfRes:efl_getDefaultSystemAbstractMeta"/>
    </metadata>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Generate default "systemAbstract" metadata.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template name="xfRes:efl_getDefaultSystemAbstractMeta">
    <meta code="systemAbstract" as="element(html:div)">
      <value as="element(html:div)">        
        <div xmlns="http://www.w3.org/1999/xhtml">
          <!--systemAbstract vide (nous ne savons pas quoi y mettre pour le moment)-->
        </div>
      </value>
    </meta>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Get the value of the "numeroOrdre" meta.</xd:p>
      <xd:p>Skip the '0' characters</xd:p>
    </xd:desc>
    <xd:param name="value">value of the "numeroOrdre" meta.</xd:param>
  </xd:doc>
  <xsl:function name="efl:getNumeroOrdre" as="xs:string*">
    <xsl:param name="value" as="xs:string*"/>       
    <xsl:variable name="seq.values" select="tokenize($value, '-')" as="xs:string*"/>
    <xsl:choose>
      <xsl:when test="count($seq.values) = 2 and string(number($seq.values[1])) != 'NaN' and string(number($seq.values[2])) != 'NaN'">
        <xsl:value-of select="concat(number($seq.values[1]), '-', number($seq.values[2]))"/>
      </xsl:when>
      <xsl:when test="count($seq.values) = 1 and string(number($seq.values[1])) != 'NaN'">
        <xsl:value-of select="number($seq.values[1])"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$value"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
</xsl:stylesheet>
