<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns:ecm="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm"
  xmlns:xfe="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all"
  xmlns="http://www.w3.org/1999/xhtml"
  version="3.0">
  
  
  <!--+===================================================+
      | INIT.                                             |
      +===================================================+-->
  
  <!-- IMPORTS -->
  <xsl:import href="EFL_TEE_uiCommentaire.htmlProofing.xsl"/>
  <xsl:import href="../_common/TEE_tree.default.htmlSimplePreview.xsl"/>
  <xsl:import href="../_common/extractMetaAsHtml.xsl"/>
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Génère une prévisualition HTML (vue dite d'épreuvage) de l'EE arbre ou du structuralNode reçu en entrée.</xd:p>
    </xd:desc>
  </xd:doc>
  
  <!-- PARAMS -->
  <xsl:param name="xfRes:css.name" select="'EFL_TEE_tree.default.htmlProofing.css'" as="xs:string"/>
  <xsl:param name="xfRes:editorialEntity.code" select="(/editorialEntity/@code,'EFL_TEE_dummy')[1]" as="xs:string"/>
  <xsl:param name="ecm:ee.url.begin" select="concat($ecm:server.url,'ecm-core/api/xmlExporter/exportToXML?flashId=')" as="xs:string"/>
  
  <xd:doc>
    <xd:desc>Lancement sans mode.</xd:desc>
  </xd:doc>
  <xsl:template match="/">
    <xsl:apply-templates select="." mode="xfRes:htmlProofing.tree"/>
  </xsl:template>
  
  
  <!--+===================================================+
      | MAIN / MODE xfRes:htmlProofing.tree               |
      +===================================================+-->
  
  <xd:doc>
    <xd:desc>Switch en fonction de l'élément racine + des paramètres donnés par l'ECM.</xd:desc>
  </xd:doc>
  <xsl:template match="/" mode="xfRes:htmlProofing.tree">
    <!-- L'élément racine peut être autre chose qu'une <editorialEntity> (ex. : un <structuralNode>).
         On l'encapsule dans une "fausse EE" si c'est le cas pour harmoniser les traitements. -->
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates mode="xfRes:htmlProofing.tree.normalizeRootElement"/>
      </xsl:document>
    </xsl:variable>
    <!-- Application d'une preview HTML avec ou sans métadonnées selon $ecm:htmlProofing-metadata.
         Surcharges spécifiques du mode xfRes:TEE_tree.default.htmlSimplePreview pour insérer les prévisualisations des noeuds d'accrochage. -->
    <xsl:apply-templates select="$step" mode="xfRes:TEE_tree.default.htmlSimplePreview"/>
  </xsl:template>
  
  
  <!--+===================================================+
      | MODE xfRes:htmlProofing.tree.normalizeRootElement |
      +===================================================+-->
  
  <xd:doc>
    <xd:desc>Encapsulation des éléments racines &lt;structuralNode&gt; dans une &lt;editorialEntity&gt;.</xd:desc>
  </xd:doc>
  <xsl:template match="/structuralNode" mode="xfRes:htmlProofing.tree.normalizeRootElement">
    <editorialEntity xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst" code="{$xfRes:editorialEntity.code}" xf:id="ee-{@xf:id}">
      <!-- Les métadonnées du <structuralNode> sont déplacées sous <editorialEntity> pour être présentées comme bloc de métadonnées principal. -->
      <xsl:apply-templates select="metadata" mode="#current"/>
      <body>
        <!-- Insertion du <structuralNode>. -->
        <xsl:copy>
          <xsl:apply-templates select="@* | node() except metadata" mode="#current"/>
        </xsl:copy>
      </body>
    </editorialEntity>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Copie par défaut en mode xfRes:htmlProofing.tree.normalizeRootElement.</xd:desc>
  </xd:doc>
  <xsl:template match="@* | node()" mode="xfRes:htmlProofing.tree.normalizeRootElement">
    <xsl:copy>
      <xsl:apply-templates select="@* | node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  
  <!--+===================================================+
      | MODE xfRes:TEE_tree.default.htmlSimplePreview     |
      | Surcharges spécifiques pour PROOFING              |
      +===================================================+-->
  
  <xd:doc>
    <xd:desc>
      <xd:p>Gestion du &lt;body&gt; pour :</xd:p>
      <xd:ul>
        <xd:li>- l'insertion des métadonnées de l'EE ;</xd:li>
        <xd:li>- générer la prévisualisation avec ou sans marge.</xd:li>
      </xd:ul>
    </xd:desc>
  </xd:doc>
  <xsl:template match="body" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <!--<body>-->
      <!-- Ajout d'un <div> wrapper sous <body> pour gérer les marges, cf. $ecm:htmlProofing-margin. -->
      <xsl:variable name="ecm-margin" select="if ($ecm:htmlProofing-margin = 'true') then (' ecm-margin') else ('')" as="xs:string"/>
      <!-- Ajout d'une information pour le style CSS, cf. $ecm:htmlProofing-contentOfLinks -->
      <xsl:variable name="ecm-contentOfLinks" select="if ($ecm:htmlProofing-contentOfLinks = 'false') then (' ecm-noContentOfLinks') else ('')" as="xs:string"/>
      <div class="htmlProofing-tree-container{$ecm-margin}{$ecm-contentOfLinks}">
        <xsl:call-template name="xfRes:htmlPreview.addDebugInfo"/>
        <!-- Insertion des métadonnées en fonction de $ecm:htmlProofing-metadata. -->
        <xsl:if test="$ecm:htmlProofing-metadata = 'true' and /editorialEntity/metadata/meta[xfRes:isMetaEligibleForExtraction(.)]">
          <xsl:apply-templates select="/" mode="xfRes:extractMeta"/>
        </xsl:if>
        <div class="ecm_content">
          <ul>
            <xsl:apply-templates mode="#current"/>
          </ul>
        </div>
      </div>
    <!--</body>-->
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Gestion des &lt;referenceNode&gt; de la prévisualisation.</xd:desc>
  </xd:doc>
  <xsl:template match="referenceNode" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <li class="referenceNode">
      <!-- Affichage d'informations sur l'EE pour mieux se repérer dans la prévisualisation. -->
      <div class="htmlProofing-text-infos">
        <p>→ Structure : <xsl:value-of select="string-join(ancestor::structuralNode/title/normalize-space(),' > ')"/></p>
        <p>→ EE ID : <xsl:value-of select="ref/@xf:targetResId"/></p>
      </div>
      <!-- Affichage des métadonnées des liens, en fonction de $ecm:htmlProofing-linkMetadata.
           On suppose qu'il s'agit des métadonnées du <referenceNode> -> à confirmer. -->
      <xsl:if test="$ecm:htmlProofing-linkMetadata = 'true' and metadata/meta[xfRes:isMetaEligibleForExtraction(.)]">
        <div class="ecm_metadata">
          <table>
            <xsl:apply-templates select="metadata/meta" mode="xfRes:extractMeta"/>
          </table>
        </div>
      </xsl:if>
      <!-- Envoi du <ref> uniquement si le contenu des liens doit être affiché.
           Sinon on affiche en priorité la verbalisation du <referenceNode>. -->
      <xsl:choose>
        <xsl:when test="$ecm:htmlProofing-contentOfLinks = 'true'">
          <xsl:apply-templates select="ref" mode="#current"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates select="(verbalization,ref)[1]" mode="#current"/>
        </xsl:otherwise>
      </xsl:choose>
    </li>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Affichage du contenu des &lt;referenceNode&gt; en fonction du paramètre <xd:ref name="ecm:htmlProofing-contentOfLinks" type="parameter">$ecm:htmlProofing-contentOfLinks</xd:ref>).</xd:desc>
  </xd:doc>
  <xsl:template match="referenceNode/ref[$ecm:htmlProofing-contentOfLinks = 'true']" priority="1" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <!-- Pour récupération de l'EE via Web Service. -->
    <xsl:variable name="doc.uri" select="concat($ecm:ee.url.begin,@xf:targetResId)" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="doc-available($doc.uri)">
        <!-- Lance la transformation sur l'EE accrochée. On ne conserve que le contenu du <body> généré. -->
        <xsl:variable name="xfRes:htmlProofing.text" as="document-node()">
          <xsl:document>
            <xsl:apply-templates select="doc($doc.uri)" mode="xfRes:htmlProofing.text">
              <!-- Les marges pour l'annotation ont déjà (éventuellement) été ajoutées à l'arbre -->
              <xsl:with-param name="ecm:htmlProofing-margin" tunnel="yes" select="'false'"/>
            </xsl:apply-templates>
          </xsl:document>
        </xsl:variable>
        <!-- Insertion du contenu. -->
        <xsl:copy-of select="$xfRes:htmlProofing.text/html:html/html:body/node()"/>
      </xsl:when>
      <xsl:otherwise>
        <!-- FALLBACK en cas d'EE introuvable -->
        <xsl:copy>
          <p class="ecm_error">[ERROR] EE <xsl:value-of select="$doc.uri"/> is not available !</p>
          <p class="ecm_debug">
            <xsl:choose>
              <xsl:when test="unparsed-text-available($doc.uri)">
                <p>unparsed-text-available :</p>
                <textarea rows="10" cols="50">
                  <xsl:value-of select="unparsed-text($doc.uri)"/>
                </textarea>
              </xsl:when>
              <xsl:otherwise>
                <p>unparsed-text is not available</p>
              </xsl:otherwise>
            </xsl:choose>
          </p>
        </xsl:copy>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Affichage de la cible d'un &lt;referenceNode&gt; (pas dans un lien).</xd:desc>
  </xd:doc>
  <xsl:template match="referenceNode/ref" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <p>
      <xsl:value-of select="(@xf:targetResId,@xf:idRef,'[ERROR] No verbalization or id found')[1]"/>
    </p>
  </xsl:template>

  <xd:doc>
    <xd:desc>Affichage de la verbalisation d'un &lt;referenceNode&gt; (pas dans un lien).</xd:desc>
  </xd:doc>
  <xsl:template match="referenceNode/verbalization" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <p>
      <xsl:apply-templates select="*/node()" mode="#current"/>
    </p>
  </xsl:template>

</xsl:stylesheet>