<?xml version="1.0" encoding="utf-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron"
        xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
        xmlns:xs="http://www.w3.org/2001/XMLSchema"
        xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
        xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
        xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
        xmlns:saxon="http://saxon.sf.net/" 
        queryBinding="xslt3" 
        id="validation_EE_revues">

  <title>Schematron for EFL - TEE Revue</title>

  <!-- Include specific XSL function -->
  <!--<xsl:include href="xslLib:/xslLib/els-common.xsl"/>-->
  <xsl:import href="EFL_TEE_tree.control.xsl"/>

  <ns prefix="xsl" uri="http://www.w3.org/1999/XSL/Transform"/>
  <ns prefix="xs" uri="http://www.w3.org/2001/XMLSchema"/>
  <ns prefix="saxon" uri="http://saxon.sf.net/"/>
  <ns prefix="xf" uri="http://www.lefebvre-sarrut.eu/ns/xmlfirst"/>
  <ns prefix="els" uri="http://www.lefebvre-sarrut.eu/ns/els"/>

  <!-- 
    Check the table of content existence on the first &lt;structuralNode&gt; element found into the EE.
    Context : All TEE revues except "EFL_TEE_acp"
  -->
  <pattern id="tocChecking">
    <rule context="/xf:editorialEntity[not(@code='EFL_TEE_acp') and not(@code='EFL_TEE_quotidienne') and not(@code='EFL_TEE_newsletter')]/xf:body/xf:structuralNode[1]" role="error">
      <assert
        test="xf:getMetaReferenceItemRef(xf:getMeta(.,'EFL_META_marqueurStructure')) = ('sommaire')">          
        Une revue doit avoir toujours un sommaire.
      </assert>
    </rule>
  </pattern>

  <!-- Duplicate tests from tree4_checkMarqueurs.xsl from CXML project -->
  <!-- 1/ not converted : tests about number of produitCx => now the EE contains ONLY ONE TEE code -->
  <!-- 2/ check marqueurs from referential file : actually we do not have access to XML referential file to make a checking -->

  <xd:doc>
    <xd:desc>
      <xd:p>Check if a &lt;structuralNode&gt; (level 1) element contains a meta "marqueur" (CXML : tree4_checkMarqueurs.xsl).</xd:p>
    </xd:desc>
  </xd:doc>
  <pattern id="marqueursCheckingStructuralNodeLevel1">
    <rule context="/xf:editorialEntity/xf:body/xf:structuralNode" role="error">
      <assert test="xf:hasMetaMarqueur(.) = true()">
        Ce niveau "<xsl:value-of select="string-join(xf:title, '')"/>" ne contient pas de meta marqueur.
      </assert>
    </rule>
  </pattern>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Check if a &lt;structuralNode&gt; (level 2) element contains a meta "marqueur" (CXML : tree4_checkMarqueurs.xsl).</xd:p>
    </xd:desc>
  </xd:doc>
  <pattern id="marqueursCheckingStructuralNodeLevel2">
    <rule context="/xf:editorialEntity/xf:body/xf:structuralNode/xf:structuralNode" role="error">
      <assert test="xf:hasMetaMarqueur(.) = true()">
        Ce niveau "<xsl:value-of select="string-join(xf:title, '')"/>" ne contient pas de meta marqueur.
      </assert>
    </rule>
  </pattern>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Check "corpus" for &lt;xf:body&gt; (CXML : tree4_checkMarqueurs.xsl).</xd:p>
    </xd:desc>
  </xd:doc>
  <pattern id="marqueursCheckingForBody">
    <rule context="/xf:editorialEntity" role="error">
      <report test="count(xf:body/xf:structuralNode[xf:hasReferentialMeta(., 'corpus')]) = 0">
        La revue n'a pas de 'Corpus'. Veuillez vérifier les métadata de le revue (1er niveau).
      </report>
      <report test="count(xf:body/xf:structuralNode[xf:hasReferentialMeta(., 'corpus')]) > 1">
        La revue a plusieurs 'Corpus'. Veuillez vérifier les métadata de le revue (1er niveau).
      </report>
    </rule>
  </pattern>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Check if a "corpus" is referenced by the first level of a &lt;structuralNode&gt; (CXML : tree4_checkMarqueurs.xsl).</xd:p>
    </xd:desc>
  </xd:doc>
  <pattern id="marqueursCheckingCorpusForStructuralNode">
    <rule context="xf:structuralNode[xf:hasReferentialMeta(., 'corpus')]" role="error">
      <report test="ancestor::xf:structuralNode">
        Ce niveau "<xsl:value-of select="string-join(xf:title, '')"/>" est défini comme corpus alors qu'il ne le devrait pas.
      </report>
    </rule>
    <!-- FIXME : y mettre aussi le même test pour 'sommaire' et selon quelles conditions (BRDA)?-->
  </pattern>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Check "corpus" for &lt;xf:body&gt; - only for BRDA (CXML : tree4_checkMarqueurs.xsl).</xd:p>
    </xd:desc>
  </xd:doc>
  <pattern id="marqueursChekcingSommaireForBRDA">
    <rule context="/xf:editorialEntity[xf:isCodeTee(., 'EFL_TEE_brda')]" role="error">
      <report test="count(xf:body/xf:structuralNode[xf:hasReferentialMeta(., 'sommaire')]) = 0">
        La revue BRDA n'a pas de 'Sommaire'. Veuillez vérifier les métadata de le revue (1er niveau).
      </report>
      <report test="count(xf:body/xf:structuralNode[xf:hasReferentialMeta(., 'sommaire')]) > 1">
        La revue BRDA a plusieurs 'Sommaire'. Veuillez vérifier les métadata de le revue (1er niveau).
      </report>
    </rule>
  </pattern>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Checking sublevel of "sommaire" - only for BRDA (CXML : tree4_checkMarqueurs.xsl)</xd:p>
    </xd:desc>
  </xd:doc>
  <pattern id="marqueursCheckingForSommaire">
    <rule context="xf:structuralNode[xf:isCodeTee(., 'EFL_TEE_brda')][xf:hasReferentialMeta(., 'sommaire')]" role="error">
      <report test="not(xf:structuralNode[xf:hasReferentialMeta(., 'aLaUne')])">
        Dans la revue, le sommaire ne contient pas niveau 'aLaUne'.
      </report>
      <report test="not(xf:structuralNode[xf:hasReferentialMeta(., 'lireAussi')])">
        Dans la revue, le sommaire ne contient pas niveau 'lireAussi'.
      </report>
<!--      <report test="count(xf:structuralNode[xf:hasReferentialMeta(., 'aLaUne')]) > 1">
        Dans la revue, le sommaire contient plusieurs niveaux 'aLaUne'.
      </report>
      <report test="count(xf:structuralNode[xf:hasReferentialMeta(., 'lireAussi')])> 1">
        Dans la revue, le sommaire contient plusieurs 'lireAussi'.
      </report>-->
    </rule>
  </pattern>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Checking "aLaUne" is first sublevel of "sommaire" - only for BRDA (CXML : tree4_checkMarqueurs.xsl)</xd:p>
    </xd:desc>
  </xd:doc>
  <pattern id="marqueursOrderALaUne">
    <rule context="xf:structuralNode[xf:isCodeTee(., 'EFL_TEE_brda')][xf:hasReferentialMeta(., 'sommaire')]/xf:structuralNode[xf:hasReferentialMeta(., 'aLaUne')]"  role="error">
      <report test="preceding-sibling::xf:structuralNode">
        Dans la revue, le niveau "aLaUne" devrait être le premier niveau du "sommaire".
      </report>
    </rule>
  </pattern>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Checking 'EFL_META_numeroOrdre' meta (CXML : meta8_checkMetadocument.xsl)</xd:p>
      <xd:p>Add the TEE code context : not(@code='EFL_TEE_quotidienne') and not(@code='EFL_TEE_newsletter')</xd:p>
      <xd:p>JIRA FLAS 262 - 235</xd:p>
    </xd:desc>
  </xd:doc>
  <pattern id="checkNumeroOrdre">
    <rule context="/xf:editorialEntity[not(@code='EFL_TEE_quotidienne') and not(@code='EFL_TEE_newsletter')]//xf:meta[@code='EFL_META_numeroOrdre']" role="warning">
      <report test="not(matches(xf:value, '^[0-9]{2}(-[0-9]{2})?$'))">
        La méta "numero d'ordre" '<xsl:value-of select="."/>' n'est pas valide (format attendu : 08, 12, 14-15, 08-09 etc.)        
      </report>
    </rule>
  </pattern>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Checking the value of 'EFL_META_anneePublication' (JIRA FLAS-235)</xd:p>
    </xd:desc>
  </xd:doc>
  <pattern id="checkAnneePublication">
    <rule context="/xf:editorialEntity[not(@code='EFL_TEE_quotidienne') and not(@code='EFL_TEE_newsletter')]//xf:meta[@code='EFL_META_anneePublication']/xf:value" role="error">
      <xsl:variable name="minYear" select="2000" as="xs:integer"/>
      <xsl:variable name="maxYear" select="2099" as="xs:integer"/>
      <report test="not(els:isNumber(.))">
        Veuillez modifier en un nombre la  valeur "<xsl:value-of select="."/>" de la méta "Année de Publication".
      </report>
      <report test="(number(.) &lt; $minYear) or (number(.) &gt; $maxYear)">
        Veuillez modifier la  valeur "<xsl:value-of select="."/>" de la méta "Année de Publication" en un nombre compris entre "<xsl:value-of select="$minYear"/>" et "<xsl:value-of select="$maxYear"/>".
      </report>
    </rule>
  </pattern>
  
</schema>
