<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
                xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
                xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
                xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
                xmlns:html="http://www.w3.org/1999/xhtml"
                xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/efl/infoCommentaire"
                xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
                exclude-result-prefixes="#all"
                version="3.0">
  
  
  <!--+==========================================================+
      | INIT.                                                    |
      +==========================================================+-->
  
  <!-- IMPORTS / INCLUDES -->
  <xsl:import href="../../xslLib/xslLib/els-common.xsl"/>
  <xsl:include href="_common/cxml2html.xsl"/>
  
  <!-- PARAMS / VARIABLES -->
  <xsl:param name="xfRes:infoCommentaire_systemTitleNames" select="('titre', 'titreObjet', 'titreUne', 'titreSommaire')" as="xs:string+"/>
  <xsl:param name="xfRes:infoCommentaire_systemAbstractNames" select="('resume', 'accroche', 'introduction')" as="xs:string+"/>
  
  <xd:doc>
    <xd:desc>
      <xd:p>XSL used for generation of calculable meta data.</xd:p>
    </xd:desc>
  </xd:doc>
  
  <xd:doc>
    <xd:desc>Modeless launch.</xd:desc>
  </xd:doc>
  <xsl:template match="/" priority="-1">
    <xsl:apply-templates mode="xfRes:EFL_TEE_infoCommentaire.getCalculatedMetatada"/>
  </xsl:template>
  
  
  <!--+==========================================================+
      | MAIN MODE                                                |
      | xfRes:EFL_TEE_infoCommentaire.getCalculatedMetatada      |
      +==========================================================+-->
  
  <xd:doc>
    <xd:desc>
      <xd:p>Spécific calculated meta data for TEE infoCommentaire.</xd:p>
      <xd:p>The input data is an editorialEntity.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="xf:editorialEntity" mode="xfRes:EFL_TEE_infoCommentaire.getCalculatedMetatada">
    <metadata>
      <!-- systemTitle -->
      <xsl:call-template name="xfRes:EFL_TEE_infoCommentaire.makeSystemTitleMeta">
        <xsl:with-param name="titres" select="xf:body/xf:contentNode/xf:content/infoCommentaire/*[local-name() = $xfRes:infoCommentaire_systemTitleNames]"/>
      </xsl:call-template>
      <!-- systemAbstract -->
      <xsl:call-template name="xfRes:EFL_TEE_infoCommentaire.makeSystemAbstractMeta">
        <xsl:with-param name="abstracts" select="xf:body/xf:contentNode/xf:content/infoCommentaire/microContenu/*[local-name() = $xfRes:infoCommentaire_systemAbstractNames]"/>
        <!-- FLAS-247 / Ne pas prendre en compte les blocRefDoc se trouvant dans des sourceReproduite (c'est une redite du blocRefDoc principal) -->
        <xsl:with-param name="allRefDocs" select="xf:body/xf:contentNode/xf:content/infoCommentaire//blocRefDoc[not(ancestor::sourceReproduite)]//refDoc"/>
      </xsl:call-template>
      <!-- EFL_META_blocRefDoc -->
      <xsl:call-template name="xfRes:EFL_TEE_infoCommentaire.makeBlocRefDocMeta">
        <xsl:with-param name="refDocs" select="xf:body/xf:contentNode/xf:content/infoCommentaire//blocRefDoc[not(ancestor::sourceReproduite)]//refDoc"/>
      </xsl:call-template>
      <!-- EFL_META_blocRattach -->
      <xsl:call-template name="xfRes:EFL_TEE_infoCommentaire.makeBlocRattachMeta">
        <!-- FLAS-247 / Seulement les 2 premiers rMemN -->
        <xsl:with-param name="renv" select="(xf:body/xf:contentNode/xf:content/infoCommentaire//blocRattach//rMemN)[position() &lt; 3]"/>
      </xsl:call-template>
    </metadata>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Spécific calculated meta data for TEE infoCommentaire.</xd:p>
      <xd:p>The input data is an invalid HTML file from the editor, but some metadata must be calculated anyway.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="html:body" mode="xfRes:EFL_TEE_infoCommentaire.getCalculatedMetatada">
    <metadata>
      <!-- systemTitle
           The meta is calculated from the html:div with a "title" class in the content -->
      <xsl:call-template name="xfRes:EFL_TEE_infoCommentaire.makeSystemTitleMeta">
        <xsl:with-param name="titres" select="html:div[els:hasClass(.,$xfRes:infoCommentaire_systemTitleNames)]"/>
      </xsl:call-template>
      <!-- systemAbstract
           The meta is calculated from the "abstract" html:div in the content + the external links qualified as refDoc -->
      <xsl:call-template name="xfRes:EFL_TEE_infoCommentaire.makeSystemAbstractMeta">
        <xsl:with-param name="abstracts" select="html:div[els:hasClass(.,$xfRes:infoCommentaire_systemAbstractNames)]"/>
        <!-- FLAS-247 / Ne pas prendre en compte les blocRefDoc se trouvant dans des sourceReproduite (c'est une redite du blocRefDoc principal) -->
        <xsl:with-param name="allRefDocs" select="descendant::html:div[els:hasClass(.,'blocRefDoc')][not(ancestor::html:div[els:hasClass(.,'sourceReproduite')])]//html:a[els:hasClass(.,('refDocJp','refDocTexte'))]"/>
      </xsl:call-template>
      <!-- EFL_META_blocRefDoc -->
      <xsl:call-template name="xfRes:EFL_TEE_infoCommentaire.makeBlocRefDocMeta">
        <xsl:with-param name="refDocs" select="descendant::html:div[els:hasClass(.,'blocRefDoc')][not(ancestor::html:div[els:hasClass(.,'sourceReproduite')])]//html:a[els:hasClass(.,('refDocJp','refDocTexte'))]"/>
      </xsl:call-template>
      <!-- EFL_META_blocRattach -->
      <xsl:call-template name="xfRes:EFL_TEE_infoCommentaire.makeBlocRattachMeta">
        <!-- FLAS-247 / Seulement les 2 premiers rMemN -->
        <!-- /!\ Le "blocRattach" n'existe pas dans la version HTML, il est ajouté par le fix-up lors de la transformation HTML -> XML -->
        <xsl:with-param name="renv" select="(descendant::html:div[els:hasClass(.,'rattach')]//html:a[els:hasClass(.,'rMemN')])[position() &lt; 3]"/>
      </xsl:call-template>
    </metadata>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Spécific calculated meta data for TEE infoCommentaire.</xd:p>
      <xd:p>The input data is an invalid HTML file from the editor, but some metadata must be calculated anyway.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="html:head" mode="xfRes:EFL_TEE_infoCommentaire.getCalculatedMetatada"/>
  
  
  <!--+==========================================================+
      | NAMED TEMPLATES / FUNCTIONS                              |
      +==========================================================+-->
  
  <xd:doc>
    <xd:desc>"systemTitle" meta generation.</xd:desc>
    <xd:param name="titres">[REQUIRED][element()*] Title elements to use to create the meta.</xd:param>
  </xd:doc>
  <xsl:template name="xfRes:EFL_TEE_infoCommentaire.makeSystemTitleMeta">
    <xsl:param name="titres" required="yes" as="element()*"/>
    <xsl:variable name="flashId" select="(@xf:id,'UNKNOWN')[1]" as="xs:string"/>
    <meta code="systemTitle" as="element(html:div)">
      <value as="element(html:div)">
        <div xmlns="http://www.w3.org/1999/xhtml">
          <xsl:choose>
            <xsl:when test="count($titres) = 0">
              <xsl:message>[INFO][XF-ECM-RES] System meta "title" undefined for xf:id=<xsl:value-of select="$flashId"/></xsl:message>
            </xsl:when>
            <xsl:otherwise>
              <!-- Switch according to $titres[1] namespace-uri -->
              <xsl:choose>
                <xsl:when test="$titres[1]/namespace-uri() = 'http://www.w3.org/1999/xhtml'">
                  <xsl:apply-templates select="$titres[1]/node()" mode="xfRes:EFL_TEE_infoCommentaire.simplifyHTMLForCalculatedMetadata"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:apply-templates select="$titres[1]/node()" mode="xfRes:cxml2html_systemTitle"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:otherwise>
          </xsl:choose>
        </div>
      </value>
    </meta>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>"systemAbstract" meta generation.</xd:desc>
    <xd:param name="abstracts">[REQUIRED][element()*] Abstract elements to use to create the meta.</xd:param>
    <xd:param name="allRefDocs">[REQUIRED][element()*] Refdoc elements to use to create the meta.</xd:param>
  </xd:doc>
  <xsl:template name="xfRes:EFL_TEE_infoCommentaire.makeSystemAbstractMeta">
    <xsl:param name="abstracts" required="yes" as="element()*"/>
    <xsl:param name="allRefDocs" required="yes" as="element()*"/>
    <xsl:variable name="flashId" select="(@xf:id,'UNKNOWN')[1]" as="xs:string"/>
    <meta code="systemAbstract" as="element(html:div)">
      <value as="element(html:div)">
        <div xmlns="http://www.w3.org/1999/xhtml">
          <xsl:choose>
            <xsl:when test="count($abstracts) = 0">
              <xsl:message>[INFO][XF-ECM-RES] System meta "abstract" undefined for xf:id=<xsl:value-of select="$flashId"/></xsl:message>
            </xsl:when>
            <xsl:otherwise>
              <!-- Switch according to $abstracts[1] namespace-uri -->
              <xsl:choose>
                <xsl:when test="$abstracts[1]/namespace-uri() = 'http://www.w3.org/1999/xhtml'">
                  <xsl:apply-templates select="$abstracts[1]/node()" mode="xfRes:EFL_TEE_infoCommentaire.simplifyHTMLForCalculatedMetadata"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:apply-templates select="$abstracts[1]/node()" mode="xfRes:cxml2html_systemAbstract"/>
                </xsl:otherwise>
              </xsl:choose>
              <!-- Ajout des refDoc principales (même traitement / affichage que pour la métadonnée EFL_META_blocRefDoc -->
              <xsl:if test="count($allRefDocs) > 0">
                <xsl:for-each select="$allRefDocs">
                  <xsl:apply-templates select="." mode="xfRes:EFL_TEE_infoCommentaire.refDocGeneratedMeta"/>
                  <xsl:if test="position() != last()">
                    <span> - </span>
                  </xsl:if>
                </xsl:for-each>
              </xsl:if>
            </xsl:otherwise>
          </xsl:choose>
        </div>
      </value>
    </meta>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>"EFL_META_blocRefDoc" meta generation.</xd:desc>
    <xd:param name="refDocs">[REQUIRED][element()*] refDoc elements to use to create the meta.</xd:param>
  </xd:doc>
  <xsl:template name="xfRes:EFL_TEE_infoCommentaire.makeBlocRefDocMeta">
    <xsl:param name="refDocs" required="yes" as="element()*"/>
    <!-- Même conversion des refDoc que pour la méta "systemAbstract" -->
    <meta code="EFL_META_blocRefDoc" as="element(html:div)">
      <value as="element(html:div)">
        <div xmlns="http://www.w3.org/1999/xhtml">
          <!-- FLAS-247 / Ne pas prendre en compte les blocRefDoc se trouvant dans des sourceReproduite (c'est une redite du blocRefDoc principal) -->
          <xsl:for-each select="$refDocs">
            <p>
              <xsl:apply-templates select="." mode="xfRes:EFL_TEE_infoCommentaire.refDocGeneratedMeta"/>
            </p>
          </xsl:for-each>
        </div>
      </value>
    </meta>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>"EFL_META_blocRattach" meta generation.</xd:desc>
    <xd:param name="renv">[REQUIRED][element()*] rMemN elements to use to create the meta.</xd:param>
  </xd:doc>
  <xsl:template name="xfRes:EFL_TEE_infoCommentaire.makeBlocRattachMeta">
    <xsl:param name="renv" required="yes" as="element()*"/>
    <meta code="EFL_META_blocRattach" as="element(html:div)">
      <value as="element(html:div)">
        <div xmlns="http://www.w3.org/1999/xhtml">
          <xsl:for-each select="$renv">
            <p>
              <xsl:apply-templates select="." mode="xfRes:EFL_TEE_infoCommentaire.rMemNGeneratedMeta"/>
            </p>
          </xsl:for-each>
        </div>
      </value>
    </meta>
  </xsl:template>
  

  <!--+==========================================================+
      | MODE xfRes:EFL_TEE_infoCommentaire.refDocGeneratedMeta   |
      | Match : <refDoc> + HTML <a class="refDoc">               |
      | Metas : systemAbstract + EFL_META_blocRefDoc             |
      +==========================================================+-->
  
  <xd:doc>
    <xd:desc>
      <xd:p>Manage &lt;refDoc&gt; / &lt;html:a class="refDoc"&gt; element for 'systemAbstract' and 'EFL_META_blocRefDoc' metas.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="refDoc | html:a[els:hasClass(.,('refDocJp','refDocTexte'))]" mode="xfRes:EFL_TEE_infoCommentaire.refDocGeneratedMeta">
    <span xmlns="http://www.w3.org/1999/xhtml">
      <!-- Switch according to current namespace-uri -->
      <xsl:choose>
        <xsl:when test="current()/namespace-uri() = 'http://www.w3.org/1999/xhtml'">
          <xsl:apply-templates mode="xfRes:EFL_TEE_infoCommentaire.simplifyHTMLForCalculatedMetadata"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates mode="xfRes:cxml2html"/>
        </xsl:otherwise>
      </xsl:choose>
    </span>
  </xsl:template>
  
  
  <!--+==========================================================+
      | MODE xfRes:EFL_TEE_infoCommentaire.rMemNGeneratedMeta    |
      | Match : <rMemN> + HTML <a class="rMemN">                 |
      | Metas : EFL_META_blocRattach                             |
      +==========================================================+-->
  
  <xd:doc>
    <xd:desc>
      <xd:p>Manage &lt;rMemN&gt; / &lt;html:a class="rMemN"&gt; element for 'EFL_META_blocRattach' meta.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="rMemN | html:a[@class = 'rMemN']" mode="xfRes:EFL_TEE_infoCommentaire.rMemNGeneratedMeta">
    <span xmlns="http://www.w3.org/1999/xhtml">
      <xsl:value-of select="upper-case((@nom,@data-xml-nom)[1])"/>
      <xsl:if test="(@mille,@data-xml-mille)[1]">
        <xsl:value-of select="' '"/>
        <xsl:value-of select="(@mille,@data-xml-mille)[1]"/>
      </xsl:if>
      <xsl:if test="(@num,@data-xml-num)[1]">
        <xsl:value-of select="' n° '"/>
        <xsl:value-of select="(@num,@data-xml-num)[1]"/>
      </xsl:if>
    </span>
  </xsl:template>
  

  <!--+==========================================================+
      | MODES xfRes:cxml2html_systemTitle +                      |
      |       xfRes:cxml2html_systemAbstract                     |
      | Match : CXML elements (-> cxml2html conversion)          |
      | Metas : systemTitle + systemAbstract                     |
      +==========================================================+-->
  
  <!-- TO DO : Les noms de mode xfRes:cxml2html_systemTitle / xfRes:cxml2html_systemAbstract est conservé au cas où il serait utilisé ailleurs (si cette XSL est importée)
               Voir s'il est possible de les harmoniser avec les autres modes sans impact -->

  <xd:doc>
    <xd:desc>
      <xd:p>Other element in 'xfRes:cxml2html_systemTitle' + 'xfRes:cxml2html_systemAbstract' mode.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="*" mode="xfRes:cxml2html_systemTitle xfRes:cxml2html_systemAbstract">
    <xsl:apply-templates select="." mode="xfRes:cxml2html"/>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Convert &lt;refDoc&gt; for html view into 'systemAbtract' metadata.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="refDoc" mode="xfRes:cxml2html_systemAbstract">
    <xsl:apply-templates mode="xfRes:cxml2html"/>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Convert &lt;refDocEnrichie&gt; for html view  into 'systemAbtract' metadata.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="refDocEnrichie" mode="xfRes:cxml2html_systemAbstract">
    <xsl:apply-templates mode="xfRes:cxml2html"/>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Convert &lt;refDocReprise&gt; for html view  into 'systemAbtract' metadata.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="refDocReprise" mode="xfRes:cxml2html_systemAbstract">
    <xsl:apply-templates mode="xfRes:cxml2html"/>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Convert &lt;renvoiAutrui&gt; for html view  into 'systemAbtract' metadata.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="renvoiAutrui" mode="xfRes:cxml2html_systemAbstract">
    <xsl:apply-templates mode="xfRes:cxml2html"/>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Convert &lt;renvoiAutruiTransition&gt; for html view  into 'systemAbtract' metadata.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="renvoiAutruiTransition" mode="xfRes:cxml2html_systemAbstract">
    <xsl:apply-templates mode="xfRes:cxml2html"/>
  </xsl:template>
  
  
  <!--=======================================================================+
      | MODE xfRes:EFL_TEE_infoCommentaire.simplifyHTMLForCalculatedMetadata |
      | Match : HTML elements (-> HTML file from the editor)                 |
      | -> Simplification of HTML elements from an editor HTML file          |
      | Metas : systemTitle + systemAbstract                                 |
      +======================================================================+-->
  
  <xd:doc>
    <xd:desc>Default copy - HTML elements stay unchanged, attributes are not kept.</xd:desc>
  </xd:doc>
  <xsl:template match="html:*" mode="xfRes:EFL_TEE_infoCommentaire.simplifyHTMLForCalculatedMetadata">
    <xsl:copy>
      <xsl:apply-templates mode="#current"/>
    </xsl:copy>
  </xsl:template>

  <xd:doc>
    <xd:desc>
      <xd:p>Manage &lt;html:a class="refDoc"&gt; for 'systemAbtract' meta.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="html:a[els:hasClass(.,('refDocJp','refDocTexte'))]" mode="xfRes:EFL_TEE_infoCommentaire.simplifyHTMLForCalculatedMetadata">
    <xsl:apply-templates mode="#current"/>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Convert &lt;html:* class="refDocEnrichie"&gt; for html view  into 'systemAbtract' metadata.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="html:*[els:hasClass(.,'refDocEnrichie')]" mode="xfRes:EFL_TEE_infoCommentaire.simplifyHTMLForCalculatedMetadata">
    <xsl:apply-templates mode="#current"/>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Convert &lt;renvoiAutrui&gt; for html view  into 'systemAbtract' metadata.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="html:a[els:hasClass(.,'renvoiAutrui')]" mode="xfRes:EFL_TEE_infoCommentaire.simplifyHTMLForCalculatedMetadata">
    <xsl:apply-templates mode="#current"/>
  </xsl:template>
  
</xsl:stylesheet>