<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns:ecm="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm"
  xmlns:xfe="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all"
  xmlns="http://www.w3.org/1999/xhtml"
  version="3.0">
  
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Génère une prévisualition HTML (vue dite d'épreuvage) de l'EE texte.</xd:p>
      <xd:p>Importe la transformation utilisée pour l'éditeur XML (xfe-XML2HTML.xsl) avec fichier de configuration customisé pour remonter quelques informations supplémentaires.</xd:p>
    </xd:desc>
  </xd:doc>
  
  <xsl:import href="EFL_TEE_uiCommentaire.htmlSimplePreview.xsl"/>
  <xsl:import href="../_common/extractMetaAsHtml.xsl"/>
  <xsl:import href="../_common/xf-common.xsl"/>
  
  <!-- PARAMS -->
  <xsl:param name="xfe:css.name" select="'EFL_TEE_uiCommentaire.htmlProofing.css'" as="xs:string"/>
  
  <!--+==================================================+
      | INIT.                                            |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>Lancement sans mode.</xd:desc>
  </xd:doc>
  <xsl:template match="/">
    <xsl:apply-templates select="." mode="xfRes:htmlProofing.text"/>
  </xsl:template>
  
  
  <!--+==================================================+
      | MAIN / MODE xfRes:htmlProofing.text              |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>Switch en fonction des paramètres donnés par l'ECM.</xd:desc>
  </xd:doc>
  <xsl:template match="/" mode="xfRes:htmlProofing.text">
    <!-- 1/ Application d'une preview HTML (transformation directe de l'éditeur). -->
    <xsl:variable name="xfRes:htmlProofing.text.content" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="." mode="xfe:xml2html"/>
      </xsl:document>
    </xsl:variable>
    <!-- 2/ Récupération des métadonnées en HTML, selon le paramètre $ecm:htmlProofing-metadata -->
    <xsl:variable name="xfRes:htmlProofing.text.meta" as="document-node()?">
      <xsl:if test="$ecm:htmlProofing-metadata = 'true'">
        <xsl:document>
          <xsl:apply-templates select="." mode="xfRes:extractMeta"/>
        </xsl:document>
      </xsl:if>
    </xsl:variable>
    <!-- 3/ Adaptations spécifiques du contenu. -->
    <xsl:apply-templates select="$xfRes:htmlProofing.text.content" mode="xfRes:htmlProofing.text.adaptContent">
      <xsl:with-param name="xfRes:htmlProofing.text.meta" tunnel="yes" select="$xfRes:htmlProofing.text.meta"/>
    </xsl:apply-templates>
  </xsl:template>
  
  
  <!--+==================================================+
      | MODE xfRes:htmlProofing.text.adaptContent        |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>Ajout d'une classe pour désactiver les marques de révision en fonction du paramètre <xd:ref name="ecm:htmlProofing-revisedMode" type="parameter">$ecm:htmlProofing-revisedMode</xd:ref>.</xd:desc>
  </xd:doc>
  <xsl:template match="html:ins | html:del" mode="xfRes:htmlProofing.text.adaptContent">
    <xsl:choose>
      <xsl:when test="$ecm:htmlProofing-revisedMode = 'false'">
        <xsl:copy>
          <xsl:apply-templates select="@* except @class" mode="#current"/>
          <xsl:attribute name="class" select="concat(@class,' ecm-noRevisionMarks')"/>
          <xsl:apply-templates select="node()" mode="#current"/>
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Gestion du &lt;body&gt; pour :</xd:p>
      <xd:ul>
        <xd:li>- l'insertion des métadonnées de l'EE ;</xd:li>
        <xd:li>- générer la prévisualisation avec ou sans marge.</xd:li>
      </xd:ul>
    </xd:desc>
    <xd:param name="xfRes:htmlProofing.text.meta">[REQUIRED][TUNNEL][document-node()] Métadonnées de l'EE (éventuellement), générées au format HTML.</xd:param>
    <xd:param name="ecm:htmlProofing-margin">[OPTIONAL][TUNNEL][xs:string] Ajout de marges pour l'annotation.</xd:param>
  </xd:doc>
  <xsl:template match="html:body" mode="xfRes:htmlProofing.text.adaptContent">
    <!-- Les métadonnées converties en HTML, uniquement si $ecm:htmlProofing-metadata = 'true'. -->
    <xsl:param name="xfRes:htmlProofing.text.meta" tunnel="yes" required="yes" as="document-node()?"/>
    <!-- Paramètre fourni en tunnel par la prévisualisation PROOFING de l'arbre ou hérité des paramètres fournis par l'ECM. -->
    <xsl:param name="ecm:htmlProofing-margin" tunnel="yes" required="no" select="$ecm:htmlProofing-margin" as="xs:string"/>
    <!-- Wrap dans un <div> conteneur. -->
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <!-- Ajout d'un <div> wrapper sous <body> pour gérer les marges, cf. $ecm:htmlProofing-margin -->
      <xsl:variable name="ecm-margin" select="if ($ecm:htmlProofing-margin = 'true') then (' ecm-margin') else ('')" as="xs:string"/>
      <div class="htmlProofing-text-container{$ecm-margin}">
        <!-- Insertion des métadonnées. -->
        <xsl:if test="$xfRes:htmlProofing.text.meta/html:div/html:table/html:tr">
          <xsl:apply-templates select="$xfRes:htmlProofing.text.meta" mode="#current"/>
        </xsl:if>
        <div class="ecm_content">
          <xsl:apply-templates select="node()" mode="#current"/>
        </div>
      </div>
    </xsl:copy>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Ajout de l'URL du serveur au @src d'une image pour qu'elle soit affichable dans la prévisualisation.</xd:p>
      <xd:p>Ex. : /api/media/original/f5fd9640f-232b-4c87-93c3-dcc80331bdf1 -> http://10.16.14.5:8282/ecm-core/api/media/original/f5fd9640f-232b-4c87-93c3-dcc80331bdf1.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="html:img/@src" mode="xfRes:htmlProofing.text.adaptContent">
    <xsl:attribute name="{name()}" select="concat($ecm:img.url.begin,.)"/>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Ajout de l'URL du serveur au @href d'un lien résolu pour qu'il puisse être utilisé dans la prévisualisation.</xd:p>
      <xd:p>Ex. : f5fd9640f-232b-4c87-93c3-dcc80331bdf1 -> http://10.16.14.5:8282/#/editorialEntity/f5fd9640f-232b-4c87-93c3-dcc80331bdf1.</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="html:a[els:hasClass(.,('refDocJp','refDocTexte','renvoiAutrui','rMemN','rPratN','rEflN'))]
                             [not(els:is-empty-or-whitespace(@href))]
                             /@href"
                mode="xfRes:htmlProofing.text.adaptContent">
    <xsl:attribute name="{name()}" select="ecm:getEEpreviewUrl(.)"/>
  </xsl:template>
  
  
  <xd:doc>
    <xd:desc>Suppression des attributs @href vides dans les liens.</xd:desc>
  </xd:doc>
  <xsl:template match="html:a[els:is-empty-or-whitespace(@href)]/@href" mode="xfRes:htmlProofing.text.adaptContent"/>
  
  <xd:doc>
    <xd:desc>Copie par défaut en mode xfRes:htmlProofing.text.adaptContent.</xd:desc>
  </xd:doc>
  <xsl:template match="@* | node()" mode="xfRes:htmlProofing.text.adaptContent">
    <xsl:copy>
      <xsl:apply-templates select="@* | node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
</xsl:stylesheet>