<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns:xfe="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/efl/infoCommentaire"
  exclude-result-prefixes="#all"
  xmlns="http://www.w3.org/1999/xhtml"
  version="3.0">
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Génère une vue HTML de prévisualisation de l'EE.</xd:p>
      <xd:p>Il s'agit de la vue utilisée dans le panneau de droite de la fiche de l'EE et lors de la prévisualisation d'une EE dans les résultats de recherche ou les arbres.</xd:p>
      <xd:p>Un second step xfRes:htmlSimplePreview.adaptContent permet de modifier le contenu issu de la transformation XFE pour l'adapter aux besoins de la prévisualisation ECM.</xd:p>
    </xd:desc>
  </xd:doc>
  
  <xsl:import href="../../xfe/xfe-XML2HTML.xsl"/>
  
  <!-- PARAMS -->
  <!-- Paramètre obligatoire pour xfe:xml2html -->
  <xsl:param name="xfe:conf.uri" select="resolve-uri('_common/EFL_TEE_uiCommentaire.htmlPreview.xfe-conf.xml',static-base-uri())" as="xs:string"/>
  <!--<xsl:param name="xfe:conf.uri" select="'xfe:/xfe/conf/efl/infoCommentaire/xfe-conf.xml'" as="xs:string"/> : en attente FE-123-->
  <xsl:param name="xfe:addXmlModelPi" select="false()" as="xs:boolean"/>
  <xsl:param name="xfe:addHTML5DOCTYPE" select="false()" as="xs:boolean"/>
  <xsl:param name="xfe:debug" select="false()" as="xs:boolean"/>
  <xsl:param name="xfe:fillHeadTitle" select="true()" as="xs:boolean"/>
  <xsl:param name="xfe:allowUnValidXML" select="true()" as="xs:boolean"/>
  <xsl:param name="xfe:css.name" select="'EFL_TEE_uiCommentaire.htmlPreview.css'" as="xs:string"/>
  
  <!-- Lien note / appel de note -->
  <xsl:key name="xfRes:getNoteFromId" match="html:div[els:hasClass(.,'xfe_note')]" use="@id"/>
  
  <!-- Copie par défaut pour le mode xfRes:htmlSimplePreview.adaptContent -->
  <xsl:mode name="xfRes:htmlSimplePreview.adaptContent" on-no-match="shallow-copy"/>
  
  <!--+==================================================+
      | INIT.                                            |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>Lancement sans mode.</xd:desc>
  </xd:doc>
  <xsl:template match="/">
    <xsl:apply-templates select="." mode="xfRes:htmlSimplePreview"/>
  </xsl:template>
  
  
  <!--+==================================================+
      | MAIN                                             |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>Lancement de la transformation en deux steps.</xd:desc>
  </xd:doc>
  <xsl:template match="/" mode="xfRes:htmlSimplePreview">
    <xsl:message>[INFO] Prévisualisation HTML de l'EE : <xsl:value-of select="base-uri()"/></xsl:message>
    <!-- 1/ Application d'une preview HTML (transformation directe de l'éditeur). -->
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="." mode="xfe:xml2html"/>
      </xsl:document>
    </xsl:variable>
    <!-- 2/ Adaptations spécifiques du contenu. -->
    <xsl:apply-templates select="$step" mode="xfRes:htmlSimplePreview.adaptContent"/>
  </xsl:template>
  
  
  <!--+==================================================+
      | MODE xfRes:htmlSimplePreview.adaptContent        |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>
      <xd:p>Pose un attribut @data-ecm-noteNum sur les appels de note, qui contient le numéro à afficher selon la position de la note liée.</xd:p>
      <xd:p>Il existe des cas où une note peut avoir plusieurs appels. Dans ce cas, tous les appels doivent afficher le même numéro.</xd:p>
      <xd:p>FIXME: A priori impossible à faire avec de simples compteurs CSS, faire ça via du JS plutôt ?</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="html:a[els:hasClass(.,'xfe_noteRef')]" mode="xfRes:htmlSimplePreview.adaptContent">
    <xsl:copy copy-namespaces="no">
      <xsl:apply-templates select="@*" mode="#current"/>
      <xsl:attribute name="data-ecm-noteNum" select="count(key('xfRes:getNoteFromId',substring-after(@href,'#'))/preceding-sibling::html:div[els:hasClass(.,'xfe_note')])+1"/>
      <xsl:apply-templates select="node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
</xsl:stylesheet>