<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all"
  version="2.0">
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Part of xf-common.xsl of xf-lib project.</xd:p>
      <xd:p>FIXME : when the CP protocol is fully available for Schematron in ECM, this file could be removed from this Schematron.</xd:p>
      <xd:p>(See FLAS-425)</xd:p>
    </xd:desc>
  </xd:doc>

  <!-- KEYS -->
  <!-- Toutes les relations, identifiées par leur ID (cas d'usage : obtenir la relation à partir d'un renvoiAutrui, pour identifier la cible) -->
  <xsl:key name="xf:getRelationById" match="xf:relation" use="@xf:id"/>

  <xd:doc>
    <xd:desc>
      <xd:p>Signature à 2 arguments de la fonction xf:hasMeta($e as element(), $codes as xs:string+, $condition as xs:string).</xd:p>
      <xd:p>Détermine si un élément donné (<xd:ref name="e" type="parameter">$e</xd:ref>) possède au moins une métadonnée dont le code correspond à une des valeurs de <xd:ref name="codes" type="parameter">$codes</xd:ref> ($condition = 'any').</xd:p>
      <xd:p>Si l'élément ne peut pas posséder de métadonnées selon le schéma FLASH (bloc &lt;xf:metadata&gt;), renvoie systématiquement false().</xd:p>
    </xd:desc>
    <xd:param name="e">[element()] Elément (&lt;xf:*&gt;) à partir duquel les métadonnées seront interrogées.</xd:param>
    <xd:param name="codes">[xs:string+] Code(s) de la/des métadonnée(s) à tester.</xd:param>
    <xd:return>[xs:boolean?] Le résultat du test.</xd:return>
  </xd:doc>
  <xsl:function name="xf:hasMeta" as="xs:boolean?">
    <xsl:param name="e" as="element()"/>
    <xsl:param name="codes" as="xs:string+"/>
    <xsl:sequence select="xf:hasMeta($e,$codes,'any')"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Détermine, selon <xd:ref name="condition" type="parameter">$condition</xd:ref>, si un élément donné (<xd:ref name="e" type="parameter">$e</xd:ref>) possède :</xd:p>
      <xd:ul>
        <xd:li>au moins une métadonnée dont le code correspond à une des valeurs de <xd:ref name="codes" type="parameter">$codes</xd:ref> (<xd:ref name="condition" type="parameter">$condition</xd:ref> = 'any').</xd:li>
        <xd:li>ou une métadonnée pour chaque valeur de <xd:ref name="codes" type="parameter">$codes</xd:ref> (<xd:ref name="condition" type="parameter">$condition</xd:ref> = 'all').</xd:li>
      </xd:ul>
      <xd:p>Si l'élément ne peut pas posséder de métadonnées selon le schéma FLASH (bloc &lt;xf:metadata&gt;), renvoie systématiquement false().</xd:p>
    </xd:desc>
    <xd:param name="e">[element()] Elément (&lt;xf:*&gt;) à partir duquel les métadonnées seront interrogées.</xd:param>
    <xd:param name="codes">[xs:string+] Code(s) de la/des métadonnée(s) à tester.</xd:param>
    <xd:param name="condition">[xs:string] Condition du test (valeurs 'any' ou 'all').</xd:param>
    <xd:return>[xs:boolean?] Le résultat du test.</xd:return>
  </xd:doc>
  <xsl:function name="xf:hasMeta" as="xs:boolean?">
    <xsl:param name="e" as="element()"/>
    <xsl:param name="codes" as="xs:string+"/>
    <xsl:param name="condition" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="not($condition = ('any','all'))">
        <!--        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="$els:currentResName"/>
          <xsl:with-param name="level" select="'error'"/>
          <xsl:with-param name="code" select="'FUNCTION_paramKO'"/>
          <xsl:with-param name="markup" select="false()"/>
          <xsl:with-param name="xpathContext" select="$e"/>
          <xsl:with-param name="description">[xf:hasMeta] La valeur '<xsl:value-of select="$condition"/>' n'est pas autorisée pour le paramètre $condition.</xsl:with-param>
        </xsl:call-template>-->
        <xsl:message>[xf:hasMeta] La valeur '<xsl:value-of select="$condition"/>' n'est pas autorisée pour le paramètre $condition.</xsl:message>
      </xsl:when>
      <xsl:when test="not($e/self::xf:editorialEntity or $e/self::xf:structuralNode or $e/self::xf:referenceNode or $e/self::xf:queryNode or $e/self::xf:contentNode or $e/self::xf:relation or $e/self::xf:referenceItem)">
        <xsl:sequence select="false()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select="if ($condition = 'any')
          then ($e/xf:metadata/xf:meta/@code = $codes)
          else (every $code in $codes satisfies xf:hasMeta($e,$code,'any'))"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne la/les "valeur(s) textuelle(s)" d'une métadonnée (cf. xf:meta/xf:value).</xd:p>
      <xd:p>Renvoie un ou plusieurs xs:string* (après normalize-space()) correspondant à tous les enfants &lt;xf:value&gt;.</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(xf:meta)] La métadonnée dont la "valeur textuelle" doit être calculée.</xd:param>
    <xd:return>[xs:string*] Un ou plusieurs xs:string (si la métadonnée est multivaluée).</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMetaValue" as="xs:string*">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <xsl:sequence select="xf:getMetaContent($meta,'value')/normalize-space(.)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne le/les ID(s) de référentiel(s) (xf:meta/xf:value/xf:ref[@xf:refType = 'xf:referenceItemRef']/@idRef) d'une métadonnée.</xd:p>
      <xd:p>Renvoie un ou plusieurs xs:string* (après normalize-space()) correspondant à toutes les valeurs des attributs xf:value/xf:ref/@idRef.</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(xf:meta)] La métadonnée dont l'ID de référentiel doit être calculé.</xd:param>
    <xd:return>[xs:string*] Un ou plusieurs xs:string (si la métadonnée est multivaluée).</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMetaReferenceItemRef" as="xs:string*">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <xsl:sequence select="xf:getMetaContent($meta,'referenceItemRef')/@idRef"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne un contenu de métadonnée. En fonction de <xd:ref name="type" type="parameter">$type</xd:ref>, récupère :</xd:p>
      <xd:ul>
        <xd:li>tous les enfants xf:value ;</xd:li>
        <xd:li>tous les enfants xf:verbalization ;</xd:li>
        <xd:li>tous les enfants xf:value/xf:ref (entrées de référentiels, relations, etc.).</xd:li>
      </xd:ul>
      <xd:p>Renvoie un ou plusieurs item()*.</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(xf:meta)] La métadonnée dont le contenu doit être calculé.</xd:param>
    <xd:param name="type">[xs:string] Le type de contenu à récupérer.</xd:param>
    <xd:return>[item()*] Le contenu de la métadonnée (plusieurs item() si celle-ci est multivaluée).</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMetaContent" as="item()*">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <xsl:param name="type" as="xs:string"/>
    <xsl:variable name="tmp.sequence" as="item()*">
      <!-- Possibilité d'adapter la manière de récupérer le contenu en fonction de $type -->
      <xsl:choose>
        <xsl:when test="$type = 'value'">
          <xsl:sequence select="$meta/xf:value"/>
        </xsl:when>
        <xsl:when test="$type = 'verbalization'">
          <!-- TO DO : Est-ce que cela a réellement une utilité ? -->
          <xsl:sequence select="$meta/xf:verbalization"/>
        </xsl:when>
        <xsl:when test="$type = 'referenceItemRef'">
          <xsl:sequence select="$meta/xf:value/xf:ref[@xf:refType = 'xf:referenceItemRef']"/>
        </xsl:when>
        <xsl:when test="$type = 'relationRef'">
          <!-- TO DO : La valeur 'xf:relation' est erronée - à corriger dans SAS + ECM -->
          <xsl:sequence select="$meta/xf:value/xf:ref[@xf:refType = ('xf:relationRef','xf:relation')]"/>
        </xsl:when>
        <xsl:otherwise>
          <!--          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="$els:currentResName"/>
            <xsl:with-param name="level" select="'error'"/>
            <xsl:with-param name="code" select="'FUNCTION_paramKO'"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="xpathContext" select="$meta"/>
            <xsl:with-param name="description">[xf:getMetaContent] Valeur inconnue pour le paramètre $type : <xsl:value-of select="$type"/>.</xsl:with-param>
          </xsl:call-template>-->
          <xsl:message>[xf:getMetaContent] Valeur inconnue pour le paramètre $type : <xsl:value-of select="$type"/>.</xsl:message>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="count($tmp.sequence) &gt; 1">
      <!--      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="$els:currentResName"/>
        <xsl:with-param name="level" select="'debug'"/>
        <xsl:with-param name="code" select="'FUNCTION_genericMessage'"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="xpathContext" select="$meta"/>
        <xsl:with-param name="description">[xf:getMetaContent] La métadonnée '<xsl:value-of select="$meta/@code"/>' est multivaluée.</xsl:with-param>
      </xsl:call-template>-->
      <xsl:message>[xf:getMetaContent] La métadonnée '<xsl:value-of select="$meta/@code"/>' est multivaluée.</xsl:message>
    </xsl:if>
    <xsl:sequence select="$tmp.sequence"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Signature à 2 arguments de la fonction xf:getMeta($e as element(), $metaCode as xs:string, $verbose as xs:boolean).</xd:p>
      <xd:p>Retourne la métadonnée (&lt;xf:meta&gt;) présente sous l'élément <xd:ref name="e" type="parameter">$e</xd:ref> et dont le code est <xd:ref name="metaCode" type="parameter">$metaCode</xd:ref>.</xd:p>
      <xd:p>En fonction de <xd:ref name="e" type="parameter">$e</xd:ref>, renvoie aussi bien une métadonnée d'editorialEntity, de referenceNode, de relation, etc.</xd:p>
      <xd:p>Par défaut, les messages d'erreur en cas de métadonnée introuvable par exemple sont désactivés.</xd:p>
    </xd:desc>
    <xd:param name="e">[element()] Elément (&lt;xf:*&gt;) à partir duquel les métadonnées seront interrogées.</xd:param>
    <xd:param name="metaCode">[xs:string] Code de la métadonnée (attribut @code de &lt;xf:meta&gt;).</xd:param>
    <xd:return>[element(xf:meta)*] La métadonnée identifiée, ou les métadonnées (si plusieurs occurrences pour un même code).</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMeta" as="element(xf:meta)*">
    <xsl:param name="e" as="element()"/>
    <xsl:param name="metaCode" as="xs:string"/>
    <xsl:sequence select="xf:getMeta($e,$metaCode,false())"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne la métadonnée (&lt;xf:meta&gt;) présente sous l'élément <xd:ref name="e" type="parameter">$e</xd:ref> et dont le code est <xd:ref name="metaCode" type="parameter">$metaCode</xd:ref>.</xd:p>
      <xd:p>En fonction de <xd:ref name="e" type="parameter">$e</xd:ref>, renvoie aussi bien une métadonnée d'editorialEntity, de referenceNode, de relation, etc.</xd:p>
      <xd:p>/!\ C'est une erreur dans les données entrantes d'avoir plusieurs occurrences d'une métadonnée avec le même code, mais en cours de traitement il peut arriver qu'on "duplique" une métadonnée (même code, sous un même élément).</xd:p>
      <xd:p>/!\ Une alerte est levée si <xd:ref name="verbose" type="parameter">$verbose</xd:ref> et que plusieurs métadonnées sont trouvées (ou aucune) ; si c'est un comportement voulu, on peut désactiver <xd:ref name="verbose" type="parameter">$verbose</xd:ref> lors de l'appel de la fonction.</xd:p>
    </xd:desc>
    <xd:param name="e">[element()] Elément (&lt;xf:*&gt;) à partir duquel les métadonnées seront interrogées.</xd:param>
    <xd:param name="metaCode">[xs:string] Code de la métadonnée (attribut @code de &lt;xf:meta&gt;).</xd:param>
    <xd:param name="verbose">[xs:boolean] Génère ou non des messages d'erreur.</xd:param>
    <xd:return>[element(xf:meta)*] La métadonnée identifiée, ou les métadonnées (si plusieurs occurrences pour un même code).</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMeta" as="element(xf:meta)*">
    <xsl:param name="e" as="element()"/>
    <xsl:param name="metaCode" as="xs:string"/>
    <xsl:param name="verbose" as="xs:boolean"/>
    <xsl:variable name="meta" select="$e/xf:metadata/xf:meta[@code = $metaCode]" as="element(xf:meta)*"/>
    <xsl:choose>
      <xsl:when test="not($e/self::xf:editorialEntity or $e/self::xf:structuralNode or $e/self::xf:referenceNode or $e/self::xf:queryNode or $e/self::xf:contentNode or $e/self::xf:relation or $e/self::xf:referenceItem)">
        <xsl:if test="$verbose = true()">
          <!--          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="$els:currentResName"/>
            <xsl:with-param name="level" select="'error'"/>
            <xsl:with-param name="code" select="'FUNCTION_paramKO'"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="xpathContext" select="$e"/>
            <xsl:with-param name="description">[xf:getMeta] Les métadonnées ne sont pas autorisées sous l'élément : <xsl:value-of select="name($e)"/>.</xsl:with-param>
          </xsl:call-template>-->
          <xsl:message>[xf:getMeta] Les métadonnées ne sont pas autorisées sous l'élément : <xsl:value-of select="name($e)"/>.</xsl:message>
        </xsl:if>
        <!-- Ne renvoie rien -->
      </xsl:when>      
      <xsl:when test="count($meta) = 0">
        <xsl:if test="$verbose = true()">
          <!--          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="$els:currentResName"/>
            <xsl:with-param name="level" select="'warning'"/>
            <xsl:with-param name="code" select="'META_notFound'"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="xpathContext" select="$e"/>
            <xsl:with-param name="description">[xf:getMeta] Aucune métadonnée trouvée avec pour code : <xsl:value-of select="$metaCode"/>.</xsl:with-param>
          </xsl:call-template>-->
          <xsl:message>[xf:getMeta] Aucune métadonnée trouvée avec pour code : <xsl:value-of select="$metaCode"/>.</xsl:message>
          <!-- Ne renvoie rien -->
        </xsl:if>
      </xsl:when>
      <xsl:when test="count($meta) gt 1">
        <xsl:if test="$verbose = true()">
          <!--          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="$els:currentResName"/>
            <xsl:with-param name="level" select="'warning'"/>
            <xsl:with-param name="code" select="'META_multiple'"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="xpathContext" select="$e"/>
            <xsl:with-param name="description">[xf:getMeta] <xsl:value-of select="count($meta)"/> métadonnées trouvées avec pour code : <xsl:value-of select="$metaCode"/>.</xsl:with-param>
          </xsl:call-template>-->
          <xsl:message>[xf:getMeta] <xsl:value-of select="count($meta)"/> métadonnées trouvées avec pour code : <xsl:value-of select="$metaCode"/>.</xsl:message>
        </xsl:if>
        <!-- /!\ Renvoie plusieurs métadonnées -->
        <xsl:sequence select="$meta"/>
      </xsl:when>
      <xsl:otherwise>
        <!-- Renvoie une seule métadonnée -->
        <xsl:sequence select="$meta"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne la valeur castée en xs:dateTime (xf:meta/xf:value[@as = 'xs:dateTime']) d'une métadonnée.</xd:p>
      <xd:p>Alerte si plusieurs xf:value[@as = 'xs:dateTime'] sont détectées, ou si la valeur n'est pas une instance valide de xs:dateTime.</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(xf:meta)] La métadonnée dont la valeur castée en xs:dateTime doit être renvoyée.</xd:param>
    <xd:return>[xs:boolean?] La valeur castée en xs:dateTime de la métadonnée.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMetaDateTimeValue" as="xs:dateTime?">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <xsl:if test="$meta/xf:value[@as = 'xs:dateTime'][2]">
<!--      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="$els:currentResName"/>
        <xsl:with-param name="level" select="'error'"/>
        <xsl:with-param name="code" select="'FUNCTION_paramKO'"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="xpathContext" select="$meta"/>
        <xsl:with-param name="description">[xf:getMetaDateTimeValue] Plusieurs valeurs xs:dateTime ont été détectées pour la métadonnée : '<xsl:value-of select="$meta/@code"/>'.</xsl:with-param>
      </xsl:call-template>-->
      <xsl:message>[xf:getMetaDateTimeValue] Plusieurs valeurs xs:dateTime ont été détectées pour la métadonnée : '<xsl:value-of select="$meta/@code"/>'.</xsl:message>
    </xsl:if>
    <!-- TODO : Mettre une alerte supplémentaire si $meta est empty ? -->
    <xsl:if test="not(empty($meta))">
      <!-- Tentative de casting de la méta en xs:dateTime, erreur + empty-sequence si impossible -->
      <xsl:choose>
        <xsl:when test="$meta/xf:value[@as = 'xs:dateTime'][1] castable as xs:dateTime">
          <xsl:sequence select="$meta/xf:value[@as = 'xs:dateTime'][1] cast as xs:dateTime"/>
        </xsl:when>
        <xsl:otherwise>
          <!--<xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="$els:currentResName"/>
            <xsl:with-param name="level" select="'error'"/>
            <xsl:with-param name="code" select="'FUNCTION_paramKO'"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="xpathContext" select="$meta"/>
            <xsl:with-param name="description">[xf:getMetaDateTimeValue] La valeur de la métadonnée '<xsl:value-of select="$meta/@code"/>' n'est pas une instance valide de xs:dateTime.</xsl:with-param>
          </xsl:call-template>-->
          <xsl:message>[xf:getMetaDateTimeValue] La valeur de la métadonnée '<xsl:value-of select="$meta/@code"/>' n'est pas une instance valide de xs:dateTime.</xsl:message>
          <!-- Ne retourne rien -->
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Détermine si un lien (un élément lié à une relation pointant vers une cible interne/externe - ex. : renvoiAutrui, refDoc, renvoi, etc.) est résolu.</xd:p>
      <xd:p>Si c'est le cas, les attributs @xf:targetResId, @xf:idRef/@idRef de sa relation (portés par xf:ref) contiennent des IDs.</xd:p>
      <xd:p>Si ce n'est pas le cas, ils contiennent une query, sous la forme "{query:...}".</xd:p>
      <xd:p>TODO : Fonction à nettoyer quand l'ECM saura gérer les queries.</xd:p>
    </xd:desc>
    <xd:param name="e">[element()] L'élément de lien à tester.</xd:param>
    <xd:return>[xs:boolean] Le résultat du test de résolution du lien.</xd:return>
  </xd:doc>
  <xsl:function name="xf:isResolvedLink" as="xs:boolean">
    <xsl:param name="e" as="element()"/>
    <xsl:variable name="top" select="$e/ancestor::xf:editorialEntity[1]" as="element(xf:editorialEntity)"/>
    <!-- Relation (éventuelle) associée à l'élément de lien -->
    <xsl:variable name="relation" select="key('xf:getRelationById',$e/@xf:idRef,$top)" as="element(xf:relation)?"/>
    <xsl:choose>
      <!-- Pas de relation associée
           TODO : A supprimer ? Pas un cas réel ? -->
      <xsl:when test="not($relation)">
        <xsl:sequence select="false()"/>
      </xsl:when>
      <!-- Les attributs de xf:ref sont vides
           TODO : A supprimer ? Pas un cas réel ? -->
      <xsl:when test="xf:getTargetIdRef($relation/xf:ref) = ('','/')">
        <xsl:sequence select="false()"/>
      </xsl:when>
      <!-- xf:ref/@xf:targetResId = '{historicalId:xxx}' -->
      <xsl:when test="$relation/xf:ref/xf:isHistoricalIdRef(@xf:targetResId)">
        <xsl:sequence select="false()"/>
      </xsl:when>
      <xsl:otherwise>
        <!-- Si le lien n'est pas résolu, les attributs de xf:ref contiennent une query sous la forme "{query:}" -->
        <xsl:sequence select="not(some $ref in ($relation/xf:ref/(@xf:targetResId,@xf:idRef,@idRef)) satisfies xf:isQueryRef($ref))"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Fonction utilitaire qui retourne une représentation unifiée d'un ID cible de lien FLASH ou NON-FLASH (ou ID/IDREF de ciblage).</xd:p>
      <xd:p>Prend en entrée un élément de référence (xf:ref) et concatène les attributs @xf:targetResId et @xf:idRef/@idRef.</xd:p>
      <xd:p>Le passage par cette fonction permet de comparer facilement des liens lorsque les deux attributs de ciblage sont utilisés, de créer des clés basées sur les cibles, etc.</xd:p>
    </xd:desc>
    <xd:param name="ref">[element(xf:ref)?] Un élément de référence xf:ref.</xd:param>
    <xd:return>[xs:string?] L'IDREF de ciblage, concaténation de @xf:targetResId et @xf:idRef/@idRef.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getTargetIdRef" as="xs:string">
    <xsl:param name="ref" as="element(xf:ref)?"/>
    <xsl:sequence select="xf:makeTargetId($ref/@xf:targetResId,($ref/@xf:idRef,$ref/@idRef)[1])"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Fonction utilitaire, qui concatène deux valeurs dans le but d'obtenir une représentation unifiée d'un ID cible (ou ID de ciblage).</xd:p>
      <xd:p>Les deux arguments sont typiquement des valeurs d'attributs xf:ref/@xf:targetResId / xf:ref/@xf:idRef (ou @idRef).</xd:p>
    </xd:desc>
    <xd:param name="targetResId">[xs:string?] Un ID de base, correspondant à un @xf:targetResId.</xd:param>
    <xd:param name="idRef">[xs:string?] Un ID à concaténer, correspondant à un @xf:idRef/@idRef.</xd:param>
    <xd:return>[xs:string?] L'ID de ciblage (ou une séquence vide, si les deux arguments sont des séquences vides).</xd:return>
  </xd:doc>
  <xsl:function name="xf:makeTargetId" as="xs:string">
    <xsl:param name="targetResId" as="xs:string?"/>
    <xsl:param name="idRef" as="xs:string?"/>
    <xsl:sequence select="string-join(($targetResId,$idRef),'/')"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Détermine si une référence donnée (<xd:ref name="ref" type="parameter">$ref</xd:ref>) fait référence à un ID historique.</xd:p>
      <xd:p>Typiquement, <xd:ref name="ref" type="parameter">$ref</xd:ref> est la valeur d'un attribut xf:ref/@xf:targetResId.</xd:p>
    </xd:desc>
    <xd:param name="ref">[xs:string?] Une référence (la valeur d'un attribut @xf:targetResId par exemple).</xd:param>
    <xd:return>[xs:boolean] Le résultat du test.</xd:return>
  </xd:doc>
  <xsl:function name="xf:isHistoricalIdRef" as="xs:boolean">
    <xsl:param name="ref" as="xs:string?"/>
    <xsl:sequence select="starts-with(normalize-space($ref),'{historicalId:')"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Détermine si une référence donnée (<xd:ref name="ref" type="parameter">$ref</xd:ref>) est une query, par opposition à un ID résolu.</xd:p>
      <xd:p>Typiquement, <xd:ref name="ref" type="parameter">$ref</xd:ref> est la valeur d'un attribut xf:ref/@xf:targetResId, xf:ref/@xf:idRef ou xf:ref/@idRef.</xd:p>
    </xd:desc>
    <xd:param name="ref">[xs:string?] Une référence (la valeur d'un attribut @xf:targetResId par exemple).</xd:param>
    <xd:return>[xs:boolean] Le résultat du test.</xd:return>
  </xd:doc>
  <xsl:function name="xf:isQueryRef" as="xs:boolean">
    <xsl:param name="ref" as="xs:string?"/>
    <xsl:sequence select="starts-with(normalize-space($ref),'{query:')"/>
  </xsl:function>
  
  <!-- *****************************************************************************************************************
  END OF functions get from xf-common.xsl of xf-lib project.
  ***************************************************************************************************************** -->
</xsl:stylesheet>