<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns:xfGuidanceEL="http://www.lefebvre-sarrut.eu/ns/xmlfirst/guidance/el"
  xmlns:xfGuidanceEL-QG="http://www.lefebvre-sarrut.eu/ns/els/guidance/quickguide"
  xmlns:ecm="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm"
  xmlns:xfe="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor"
  xmlns="http://www.w3.org/1999/xhtml"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all"
  version="3.0">
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Generate a HTML proofing for EE type "EL_TEE_arbreThemeGuidance"</xd:p>
    </xd:desc>
  </xd:doc>

  <xsl:import href="xf-lib:/xf-lib/xsl/_common/xf_resolve-referenceNodes.xsl"/>
  <xsl:import href="xf-lib:/xf-lib/xsl/el/xf-el-guidance_aggregate-quickguides.xsl"/>
  <xsl:import href="../../_common/TEE_tree.default.htmlFullPreview.xsl"/>
  <xsl:import href="../../els/_common/els-content.common.htmlPreview.xsl"/>
  
  <xsl:param name="xfRes:css.uri" select="xfRes:getCssRelativeUri($xfRes:editorialEntity.code, $xfRes:baseUri, 'ecm.htmlProofing.style')" as="xs:string?"/>
  <xsl:param name="xfRes:EL_TEE_arbreThemeGuidance.htmlProofing_skipAggregation" select="false()" as="xs:boolean"/>
  <xsl:param name="xfRes:EL_TEE_arbreThemeGuidance.htmlProofing_aggregateOnly" select="false()" as="xs:boolean"/>
  
  <!--indent="no" is important for xspec diff--> 
  <xsl:output method="xhtml" indent="no"/>
  
  <!--==================================================-->
  <!--INIT-->
  <!--==================================================-->
  
  <xsl:template match="/">
    <xsl:apply-templates select="." mode="xfRes:EL_TEE_arbreThemeGuidance.htmlProofing"/>
  </xsl:template>
  
  <!--==================================================-->
  <!--MAIN-->
  <!--==================================================-->
  
  <!--Driver-->
  <xsl:template match="/" mode="xfRes:EL_TEE_arbreThemeGuidance.htmlProofing">
    <!--resolve referencenNode-->
    <xsl:variable name="step" as="document-node()">
      <xsl:choose>
        <xsl:when test="$xfRes:EL_TEE_arbreThemeGuidance.htmlProofing_skipAggregation">
          <xsl:sequence select="."/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:document>
            <xsl:apply-templates select="." mode="xf:resolve-referenceNodes.main"/>
          </xsl:document>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="$xfRes:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('EL_TEE_arbreThemeGuidance.htmlProofing.step1.xml', $xfRes:log.uri)" as="xs:anyURI"/>
      <xsl:message>[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:message>
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <xsl:choose>
      <xsl:when test="not($xfRes:EL_TEE_arbreThemeGuidance.htmlProofing_aggregateOnly)">
        <!--Ajout des quickguide dans la Fiche Conseil-->
        <xsl:variable name="step" as="document-node()">
          <xsl:document>
            <xsl:apply-templates select="$step" mode="xfGuidanceEL:aggregate-quickguide.main"/>
          </xsl:document>
        </xsl:variable>
        <xsl:if test="$xfRes:debug">
          <xsl:variable name="step.log.uri" select="resolve-uri('EL_TEE_arbreThemeGuidance.htmlProofing.step2.xml', $xfRes:log.uri)" as="xs:anyURI"/>
          <xsl:message>[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:message>
          <xsl:result-document href="{$step.log.uri}">
            <xsl:sequence select="$step"/>
          </xsl:result-document>
        </xsl:if>
        <!--Adaptation du XML-->
        <xsl:variable name="step" as="document-node()">
          <xsl:document>
            <xsl:apply-templates select="$step" mode="xfRes:EL_TEE_arbreThemeGuidance.htmlProofing-step3"/>
          </xsl:document>
        </xsl:variable>
        <xsl:if test="$xfRes:debug">
          <xsl:variable name="step.log.uri" select="resolve-uri('EL_TEE_arbreThemeGuidance.htmlProofing.step3.xml', $xfRes:log.uri)" as="xs:anyURI"/>
          <xsl:message>[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:message>
          <xsl:result-document href="{$step.log.uri}">
            <xsl:sequence select="$step"/>
          </xsl:result-document>
        </xsl:if>
        <!--Transform to HTML-->
        <!--Use the simple preview of this tree, referenceNode template will be overrided to transform the referenced EE to html-->
        <xsl:variable name="step" as="document-node()">
          <xsl:document>
            <xsl:apply-templates select="$step" mode="xfRes:TEE_tree.default.htmlFullPreview"/>
          </xsl:document>
        </xsl:variable>
        <xsl:if test="$xfRes:debug">
          <xsl:variable name="step.log.uri" select="resolve-uri('EL_TEE_arbreThemeGuidance.htmlProofing.step4.xml', $xfRes:log.uri)" as="xs:anyURI"/>
          <xsl:message>[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:message>
          <xsl:result-document href="{$step.log.uri}">
            <xsl:sequence select="$step"/>
          </xsl:result-document>
        </xsl:if>
        <!--FINALY-->
        <xsl:sequence select="$step"/>
      </xsl:when>
      <xsl:otherwise>
        <!--FINALY-->
        <xsl:sequence select="$step"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!--=============================-->
  <!--STEP-1 : resolve-->
  <!--=============================-->
  
  <!--cf. import xf_resolve-referenceNodes.xsl-->

  <!--OVERRIDE functions-->
  
  <xsl:function name="xf:checkReferenceNodeResolution.mainEE" as="xs:boolean">
    <xsl:param name="editorialEntity" as="element(editorialEntity)"/>
    <!--By default true-->
    <xsl:sequence select="$editorialEntity/@code = ('EL_TEE_arbreGuidance', 'EL_TEE_arbreThemeGuidance')"/>
  </xsl:function>
  
  <!--Surcharge pour guidance EL-->
  <xsl:function name="xf:checkReferenceNodeResolution.beforeLoadingEE" as="xs:string">
    <xsl:param name="referenceNode" as="element(referenceNode)"/>
    <xsl:param name="resolved.ee.current.codes" as="xs:string*"/>
    <xsl:choose>
      <xsl:when test="$referenceNode/@active = 'false'">
        <xsl:sequence select="'do-not-load-not-active'"/>
      </xsl:when>
      <xsl:when test="deep-equal($resolved.ee.current.codes, ('EL_TEE_arbreThemeGuidance'))">
        <xsl:sequence select="'load'"/>
      </xsl:when>
      <xsl:when test="deep-equal($resolved.ee.current.codes, ('EL_TEE_arbreThemeGuidance', 'EL_TEE_pageGuidance')) 
        and $referenceNode/ancestor::structuralNode[xfGuidanceEL:is-KN.ficheConseil(.)]">
        <xsl:sequence select="'load'"/>
      </xsl:when>
      <xsl:otherwise>
        <!--Par défaut on ne charge pas le contenu-->
        <xsl:sequence select="'do-not-load'"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="xf:checkReferenceNodeResolution.AfterLoadingEE" as="xs:string">
    <xsl:param name="editorialEntity" as="element(editorialEntity)"/>
    <xsl:param name="resolved.ee.parent.codes" as="xs:string*"/>
    <!--By default resolve every EE if not archived-->
    <xsl:choose>
      <xsl:when test="$editorialEntity/@archived = 'true'">
        <xsl:sequence select="'do-not-resolve'"/>
      </xsl:when>
      <xsl:when test="deep-equal($resolved.ee.parent.codes, ('EL_TEE_arbreThemeGuidance')) and $editorialEntity/@code = 'EL_TEE_pageGuidance'">
        <xsl:sequence select="'resolve'"/>
      </xsl:when>
      <xsl:when test="deep-equal($resolved.ee.parent.codes, ('EL_TEE_arbreThemeGuidance', 'EL_TEE_pageGuidance')) and $editorialEntity/@code = 'EL_TEE_practiceNoteAssemblee'">
        <xsl:sequence select="'resolve'"/>
      </xsl:when>
      <xsl:when test="deep-equal($resolved.ee.parent.codes, ('EL_TEE_arbreThemeGuidance', 'EL_TEE_pageGuidance')) and $editorialEntity/@code = 'EL_TEE_practiceNoteRedigee'">
        <xsl:sequence select="'do-not-resolve'"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select="'unexpected'"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <!--=============================-->
  <!--STEP-2 : aggregate QuickGuide -->
  <!--=============================-->
  
  <!--cf. import-->
  
  <!--=============================-->
  <!--STEP-3 : adapt xml content  -->
  <!--=============================-->

  <xsl:template match="structuralNode[@xf:type = 'QG']/title/html:div" mode="xfRes:EL_TEE_arbreThemeGuidance.htmlProofing-step3">
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <a href="#{../../referenceNode/ref/@xf:targetResId}">
        <xsl:choose>
          <xsl:when test="count(*) = 1  and html:p">
            <xsl:apply-templates select="html:p/node()" mode="#current"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates select="node()" mode="#current"/>
          </xsl:otherwise>
        </xsl:choose>
      </a>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="structuralNode[@xf:type='QG']/referenceNode" mode="xfRes:EL_TEE_arbreThemeGuidance.htmlProofing-step3"/>
  
  <xsl:template match="structuralNode[xfGuidanceEL:is-KN.actualites(.)]" mode="xfRes:EL_TEE_arbreThemeGuidance.htmlProofing-step3">
    <xsl:copy>
      <xsl:apply-templates select="@* | node()" mode="#current"/>
      <li>
        <strong>Thème de l'actualité EL</strong>
        <xsl:text>&#160;: </xsl:text>
        <xsl:value-of select="ancestor::editorialEntity[1]/xf:getMeta(., 'EL_META_themeActualite')/xf:getMetaVerbalization(.)"/>
      </li>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xfRes:EL_TEE_arbreThemeGuidance.htmlProofing-step3">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>

  <!--=============================-->
  <!--STEP-3 : transform to HTML   -->
  <!--=============================-->
  
  <!--FIXME : this could be added as dafault behaviour in TEE_tree.default.htmlSimplePreview.xsl-->
  <xsl:template match="editorialEntity/metadata/meta[@code = 'systemTitle']" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <div class="systemTitle level-{count(ancestor::editorialEntity)}">
      <xsl:variable name="editorialEntity" select="../../self::editorialEntity" as="element(editorialEntity)"/>
      <xsl:apply-templates select="$editorialEntity/@xf:id" mode="#current"/>
      <xsl:apply-templates select="value/html:div/node()" mode="#current"/>
    </div>
    <!--<div class="systemTitle level-{count(ancestor::editorialEntity)}">
      <xsl:variable name="editorialEntity" select="../../self::editorialEntity" as="element(editorialEntity)"/>
      <xsl:apply-templates select="$editorialEntity/@xf:id" mode="#current"/>
      <xsl:variable name="referenceNode" select="$editorialEntity/parent::ref/parent::referenceNode" as="element(referenceNode)?"/>
      <xsl:choose>
        <xsl:when test="exists($referenceNode)">
          <xsl:variable name="referenceNode.systemTitle.value" select="xf:getMeta($referenceNode, 'systemTitle')/xf:getMetaValueContent(.)" as="item()*"/>
          <xsl:choose>
            <xsl:when test="not(els:is-empty-or-whitespace($referenceNode.systemTitle.value))">
              <xsl:sequence select="$referenceNode.systemTitle.value/node()"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="value/html:div/node()" mode="#current"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates select="value/html:div/node()" mode="#current"/>
        </xsl:otherwise>
      </xsl:choose>
    </div>-->
  </xsl:template>
  
  <!--Ne pas afficher les structuralNode KN vides-->
  <xsl:template match="structuralNode[xfGuidanceEL:is-KN(.)][count(* except (metadata | title)) = 0]" mode="xfRes:TEE_tree.default.htmlSimplePreview" priority="1"/>
  
  <!--Override referenceNode template of the original simple preview of this tree-->
  <xsl:template match="referenceNode[ref/editorialEntity]" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <!--DO NOT APPLY VERBALIZATION HERE-->
    <li class="referenceNode resolved">
      <xsl:apply-templates select="@xf:id" mode="#current"/>
      <xsl:apply-templates select="ref/editorialEntity" mode="#current"/>
    </li>
  </xsl:template>
  
  <xsl:template match="contentNode/content/xfGuidanceEL-QG:article[@class='els-content guidance-quickguide']" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:apply-templates select="." mode="xfRes:els-content.common.htmlSimplePreview"/>
  </xsl:template>
  
  <xsl:template match="contentNode[normalize-space(content) = '']" mode="xfRes:TEE_tree.default.htmlSimplePreview"/>
  
  <xd:doc>
    <xd:desc>Example:
      <els:log level="info" code="RESOLVE.EE-EL_TEE_practiceNoteAssemblee.UNEXPECTED" xsltName="xf-common.xsl" element="editorialEntity">
        <!--EL_TEE_arbreThemeGuidance > EL_TEE_pageGuidance  : Unexpected resolution of EE " EL_TEE_practiceNoteAssemblee " here-->
      </els:log>
    </xd:desc>
  </xd:doc>
  <xsl:template match="els:log" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <div class="ecm_error">
      <p style="color:inherit;">
        <xsl:apply-templates mode="#current"/>
      </p>
    </div>
  </xsl:template>
  <xsl:template match="els:log/comment()" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:value-of select="."/>
  </xsl:template>
  
  
  
</xsl:stylesheet>