<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:ecm="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns:xfGuidanceEL="http://www.lefebvre-sarrut.eu/ns/xmlfirst/guidance/el"
  xmlns="http://www.w3.org/1999/xhtml"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all"
  version="3.0">

	<xsl:import href="../../_common/TEE_tree.default.htmlSimplePreview.xsl"/>
  <xsl:import href="../../els/_common/els-content.common.htmlPreview.xsl"/>
  <xsl:import href="xf-lib:/xf-lib/xsl/el/xf-el-guidance.xsl"/>
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Generate a HTML simple preview for "EL_TEE_pageGuidance"</xd:p>
    </xd:desc>
  </xd:doc>

  <xsl:output method="xhtml" indent="no"/>

  <xsl:param name="xfRes:baseUri" select="base-uri()" as="xs:anyURI"/>
  
  <!--===========================================================-->
  <!--INIT-->
  <!--===========================================================-->
  
  <xsl:template match="/">
    <xsl:apply-templates select="." mode="xfRes:EL_TEE_pageGuidance.htmlSimplePreview"/>
  </xsl:template>
  
  
  <!--===========================================================-->
  <!--MAIN-->
  <!--===========================================================-->
  
  <!--main driver-->
  <xsl:template match="/" mode="xfRes:EL_TEE_pageGuidance.htmlSimplePreview">
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="." mode="xfRes:TEE_tree.default.htmlSimplePreview"/>
      </xsl:document>
    </xsl:variable>
    <!--Au step2 on a besoin de l'EE pour récupérer "ELS_META_titreEE"-->
    <xsl:variable name="editorialEntity" select="editorialEntity" as="element()"/>
    <xsl:apply-templates select="$step" mode="xfRes:EL_TEE_pageGuidance.htmlSimplePreview.step2">
      <xsl:with-param name="editorialEntity" select="$editorialEntity" as="element()" tunnel="yes"/>
    </xsl:apply-templates>
  </xsl:template>
  
  <!--===========================================================-->
  <!--OVERIDE IMPORT-->
  <!--===========================================================-->
  
  <xsl:template match="structuralNode[xfGuidanceEL:is-sujetsLies(.)]" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:variable name="title.div" as="node()*">
      <xsl:apply-templates select="title/html:div" mode="#current"/>
    </xsl:variable>
    <div class="pg-related-topics">
      <span class="pg-related-topics-title"><xsl:value-of select="concat($title.div, ' : ')" /></span>
      <ul class="pg-related-topics-list">
        <xsl:apply-templates select="referenceNode/(verbalization, ref)[1]" mode="xfRes:EL_TEE_pageGuidance.htmlSimplePreview"/>
      </ul>
    </div>
  </xsl:template>
  
  <xsl:template match="referenceNode/verbalization" mode="xfRes:EL_TEE_pageGuidance.htmlSimplePreview">
     <li class="pg-related-topics-item">
      <xsl:apply-templates select="." mode="xfRes:TEE_tree.default.htmlSimplePreview" />
    </li>
  </xsl:template>
  
  <xsl:template match="referenceNode/verbalization" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:variable name="ref" select="preceding-sibling::ref" as="element(ref)*"/>
    <xsl:variable name="referenceNode.systemTitle.div" select="../metadata/meta[@code = 'systemTitle']/value[@as = 'element(html:div)']/html:div[1]" as="element(html:div)?"/>
    <a href="{ecm:getEEpreviewUrl(($ref/@xf:targetResId, $ref/@xf:idRef)[1])}" target="_blank">
      <xsl:choose>
        <xsl:when test="not(matches(string($referenceNode.systemTitle.div), concat('^', $els:regAnySpace,'*$')))">
          <xsl:copy-of select="$referenceNode.systemTitle.div/node()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="normalize-space(.)" />
        </xsl:otherwise>
      </xsl:choose>
    </a>
  </xsl:template>

  <xsl:template match="referenceNode" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <li class="pg-tab-list-item">
      <xsl:apply-templates select="(verbalization, ref)[1]" mode="#current"/>
      <xsl:if test="not(ancestor::structuralNode[@xf:type='KN']/metadata/meta[@code='ELS_META_referentielKN']/value/ref/@idRef='ACTU'
        or ancestor::structuralNode[@xf:type='KN']/metadata/meta[@code='ELS_META_referentielKN']/value/ref/@idRef='FICHE_CONSEIL')">
        <span class="pg-tab-list-typedoc"><xsl:value-of select="ancestor::structuralNode[xfGuidanceEL:is-KN(.)]/xf:getMeta(., 'ELS_META_referentielKN')/xf:getMetaVerbalization(.)"/></span>
      </xsl:if>
    </li>
  </xsl:template>
  
  <!--<xsl:template match="structuralNode[@xf:type='KN']" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:variable name="title.div" as="node()*">
      <xsl:apply-templates select="title/html:div" mode="#current"/>
    </xsl:variable>
    <div class="pg-knowledge-navigator-container">
      <xsl:call-template name="xfRes:EL_TEE_pageGuidance.createTabs">
        <xsl:with-param name="currentId" select="@xf:id" />
      </xsl:call-template>
      <xsl:if test="* except (metadata | title)">
        <ul class="pg-tab-list">
          <xsl:apply-templates select="* except (metadata | title)" mode="#current"/>
        </ul>
      </xsl:if>
      <xsl:if test="metadata/meta[@code='ELS_META_referentielKN']/value/ref/@idRef='ACTU'">
        <div class="pg-titre-actu">
          <xsl:apply-templates select="ancestor::editorialEntity/metadata/meta[@code='EL_META_themeActualite']/verbalization" mode="#current"/>
        </div>
        <span class="pg-tab-list-typedoc">Thème de l'actualité EL</span>
      </xsl:if>
    </div>
  </xsl:template>-->
  
  <xsl:template match="structuralNode[@xf:type='KN'][referenceNode or xfGuidanceEL:is-KN.actualites(.)]" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:variable name="title.div" as="node()*">
      <xsl:apply-templates select="title/html:div" mode="#current"/>
    </xsl:variable>
    <div class="pg-knowledge-navigator-container">
      <xsl:call-template name="xfRes:EL_TEE_pageGuidance.createTabs">
        <xsl:with-param name="currentId" select="@xf:id"/>
      </xsl:call-template>
      <xsl:if test="* except (metadata | title)">
        <ul class="pg-tab-list">
          <xsl:apply-templates select="* except (metadata | title)" mode="#current"/>
        </ul>
      </xsl:if>
      <xsl:if test="xfGuidanceEL:is-KN.actualites(.)">
        <div class="pg-titre-actu">
          <xsl:apply-templates select="ancestor::editorialEntity/metadata/meta[@code='EL_META_themeActualite']/verbalization" mode="#current"/>
        </div>
        <span class="pg-tab-list-typedoc">Thème de l'actualité EL</span>
      </xsl:if>
    </div>
  </xsl:template>
  
  <xsl:template match="structuralNode[@xf:type='KN'][not(referenceNode or xfGuidanceEL:is-KN.actualites(.))]" mode="xfRes:TEE_tree.default.htmlSimplePreview"/>
  
  <xsl:template match="body" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:variable name="titre" as="xs:string?" select="../metadata/meta[@code='EL_META_typePageGuidance']/value/text()"/>
    <!--<body>-->
      <h1 class="pg-titre-entete"><xsl:value-of select="concat('Guidance - ', $titre)" /></h1>
      <h1 class="pg-titre-page"><xsl:value-of select="normalize-space(../metadata/meta[@code='ELS_META_titreEE'])" /></h1>
      <xsl:apply-templates select="../metadata" mode="xfRes:EL_TEE_pageGuidance.htmlSimplePreview" />
      <xsl:apply-templates select="contentNode" mode="#current"/>
      <xsl:apply-templates select="structuralNode[@xf:type='Sujets liés']" mode="#current"/>
      <xsl:apply-templates select="structuralNode[not(@xf:type='Sujets liés')]" mode="#current"/>
    <!--</body>-->
  </xsl:template>
  
  <xsl:template match="metadata" mode="xfRes:EL_TEE_pageGuidance.htmlSimplePreview">
    <section class="pg-metadatas">
      <xsl:apply-templates select="meta[@group='IDENTIFICATION' and not(@code='ELS_META_titreEE' or @code='EL_META_typePageGuidance')]" mode="#current" />
    </section>
  </xsl:template>
  
  <xsl:template match="meta" mode="xfRes:EL_TEE_pageGuidance.htmlSimplePreview">
    <!--<xsl:variable name="pg-meta-valeur" select="value" as="node()"/>-->
    <xsl:variable name="pg-meta-valeur" select="(verbalization, value)[1]" as="node()"/>
    <div class="pg-meta">
      <span class="pg-meta-titre"><xsl:value-of select="@label"/></span>
      <span class="pg-meta-valeur"><xsl:value-of select="normalize-space($pg-meta-valeur)"/></span>
    </div>
  </xsl:template>
  
  <!--Surcharge du mode "xfRes:TEE_tree.default.htmlSimplePreview" 
    pour afficher les contentNode (quickguide)-->
  <xsl:template match="contentNode" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:apply-templates mode="#current"/>
  </xsl:template>
  
  <xsl:template match="contentNode/content/*" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:apply-templates select="." mode="xfRes:els-content.common.htmlSimplePreview"/>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xfRes:EL_TEE_pageGuidance.htmlSimplePreview.step2">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template name="xfRes:EL_TEE_pageGuidance.createTabs">
    <xsl:param name="currentId" as="xs:string" />
    <div class="pg-knowledge-navigator">
      <nav>
        <xsl:for-each select="//structuralNode[@xf:type='KN'][referenceNode or xfGuidanceEL:is-KN.actualites(.)]">
          <a>
            <xsl:choose>
              <xsl:when test="@xf:id = $currentId">
                <xsl:attribute name="id" select="$currentId" />
                <xsl:attribute name="class" select="'pg-knowledge-item active'" />
              </xsl:when>
              <xsl:otherwise>
                <xsl:attribute name="href" select="concat('#', @xf:id)" />
                <xsl:attribute name="class" select="'pg-knowledge-item'" />
              </xsl:otherwise>
            </xsl:choose>
            <span class="pg-knowledge-label"><xsl:value-of select="normalize-space(title)"/></span>
          </a>
        </xsl:for-each>
      </nav>
    </div>
  </xsl:template>
  
 </xsl:stylesheet>