<?xml version="1.0" encoding="utf-8"?>
<!--<schema xmlns="http://www.ascc.net/xml/schematron">-->
<schema
  xmlns="http://purl.oclc.org/dsdl/schematron"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:saxon="http://saxon.sf.net/"
  xmlns:h="http://www.w3.org/1999/xhtml"
  queryBinding="xslt2"
  id="validation_el_pageGuidance">

  <title>Schematron for EL PageGuidance TEE</title>

  <ns prefix="xsl" uri="http://www.w3.org/1999/XSL/Transform"/>
  <ns prefix="xs" uri="http://www.w3.org/2001/XMLSchema"/>
  <ns prefix="saxon" uri="http://saxon.sf.net/"/>
  <ns prefix="xf" uri="http://www.lefebvre-sarrut.eu/ns/xmlfirst"/>
  <ns prefix="h" uri="http://www.w3.org/1999/xhtml"/>


  <let name="els-quick-guide-namespace" value="'http://www.lefebvre-sarrut.eu/ns/els/guidance/quickguide'"/>
  <let name="refKN" value="'FICHE_CONSEIL'" />

  <!--==============================================-->
  <!--CONTROLE METAS -->
  <!--==============================================-->

  <pattern id="metadata">
    <rule context="/xf:editorialEntity/xf:metadata/xf:meta[@code = 'ELS_META_titreEE']/xf:value/*">
      <assert test="local-name() = 'div' and namespace-uri() = 'http://www.w3.org/1999/xhtml'" role="error">
        La méta "ELS_META_titreEE" doit être de type "html:div"
      </assert>
    </rule>
    <rule context="/xf:editorialEntity/xf:metadata">
      <let name="title-length" value="string-length(./xf:meta[@code = 'ELS_META_titreEE'])"/>
      <report test="$title-length > 200" role="warning">
        Le titre est trop long : <value-of select="$title-length"/> caractères (max = 200)
      </report>
    </rule>
    <rule context="/xf:editorialEntity/xf:body">
      <let name="noeuds" value="string-join(./xf:structuralNode[@xf:type='KN' and not(./xf:metadata/xf:meta[@code='ELS_META_referentielKN'])]/xf:title/h:div/text(), ', ')" />
      <report test="count(./xf:structuralNode[@xf:type='KN' and not(./xf:metadata/xf:meta[@code='ELS_META_referentielKN'])]) > 0" role="warning">
        Veuillez renseigner la métadonnée obligatoire "Référentiel KN" pour le(s) onglet(s) <value-of select="$noeuds"/>
      </report>
    </rule>
  </pattern>

  <!--==============================================-->
  <!--CONTROLE DU CONTENU (Fiche conseil)-->
  <!--==============================================-->
  <pattern id="controle-fiche-conseil">
    <rule context="/xf:editorialEntity[xf:metadata/xf:meta[@code='EL_META_typePageGuidance']/xf:value='Page Tâche']/xf:body/xf:structuralNode[xf:metadata/xf:meta[@code='ELS_META_referentielKN' and xf:value/xf:ref/@idRef=$refKN]]">
      <assert test="xf:referenceNode" role="warning">
        L'onglet "Fiche conseil" de la Page Guidance doit contenir au moins une Practice Note
      </assert>
    </rule>
    <rule context="/xf:editorialEntity/xf:body">
      <assert test="xf:structuralNode/xf:metadata/xf:meta[@code='ELS_META_referentielKN' and xf:value/xf:ref/@idRef=$refKN]" role="warning">
        La Page Tâche doit avoir une Fiche Conseil
      </assert>
    </rule>
  </pattern>

  <!--==============================================-->
  <!--CONTROLE DU CONTENU (Liste)-->
  <!--==============================================-->
  <!--pattern id="controle-liste">
    <rule context="/xf:editorialEntity/xf:body/xf:structuralNode[xf:metadata/xf:meta[@code='ELS_META_referentielKN' and xf:value/xf:ref/@idRef!=$refKN]]">
      <assert test="xf:referenceNode" role="warning">
         Les onglets contenant des listes de ressources doivent avoir au moins une ressource référencée
      </assert>
    </rule>
  </pattern-->

  <!--==============================================-->
  <!--CONTROLE DU CONTENU (Noeud accrochage)-->
  <!--==============================================-->
  <pattern id="controle-noeud-accrochage">
    <rule context="/xf:editorialEntity[xf:metadata/xf:meta[@code='EL_META_typePageGuidance']/xf:value='Page Tâche']/xf:body">
      <report test="xf:referenceNode" role="warning">
        La Page Tâche ne doit pas contenir de nœud d'accrochage à la racine
      </report>
    </rule>
    <rule context="/xf:editorialEntity[xf:metadata/xf:meta[@code='EL_META_typePageGuidance']/xf:value='Page d''orientation']/xf:body">
      <report test="xf:referenceNode" role="warning">
        La Page Orientation ne doit pas contenir de nœud d'accrochage à la racine
      </report>
    </rule>
  </pattern>

  <!--==============================================-->
  <!--CONTROLE DU CONTENU (QUICKGUIDE)-->
  <!--==============================================-->
  <pattern id="quickguide-obligatoire">
    <rule context="/xf:editorialEntity[xf:metadata/xf:meta[@code='EL_META_typePageGuidance']/xf:value='Page Tâche']/xf:body">
      <assert test="xf:contentNode/xf:content and string-length(normalize-space(xf:contentNode/xf:content))>0" role="error">
        Le quickguide est obligatoire.
      </assert>
    </rule>
  </pattern>
  <pattern id="controle-quickguide">
    <rule context="/xf:editorialEntity/xf:body/xf:contentNode/xf:content/*">
      <let name="current-node-namespace" value="namespace-uri(.)"/>
      <let name="contenu" value="replace(normalize-space(.),'&#x0A;', '')"/>
      <let name="content-length" value="string-length($contenu)"/>
      <assert test="local-name() = 'article'" role="error">
        L'élément sous xf:contentNode/xf:content doit être "article"
      </assert>
      <assert test="$current-node-namespace = $els-quick-guide-namespace" role="error">
        Le namespace sous xf:contentNode/xf:content doit être "<value-of select="$els-quick-guide-namespace"/>" (au lieu de <value-of select="$current-node-namespace"/>)
      </assert>
      <report test="$content-length > 1000" role="warning">
        Le quickguide est trop long : <value-of select="$content-length"/> caractères (max = 1000).
      </report>
    </rule>
  </pattern>

  <!--==============================================-->
  <!--CONTROLE DE L'UNICITE DES ONGLETS-->
  <!--==============================================-->
  <pattern abstract="true" id="controle-onglet">
    <rule context="/xf:editorialEntity[xf:metadata/xf:meta[@code='EL_META_typePageGuidance']/xf:value='Page Tâche']/xf:body">
      <let name="titrePage" value="string(../xf:metadata/xf:meta[@code='systemTitle'])" />
      <report test="count(./xf:structuralNode[xf:metadata/xf:meta[@code='ELS_META_referentielKN' and xf:value/xf:ref/@idRef=$ongletID]]) > 1" role="error">
        Il y a plusieurs onglets de type <xsl:value-of select="$ongletType"/> dans la Page Tâche <xsl:value-of select="$titrePage"/>
      </report>
    </rule>
    <rule context="/xf:editorialEntity[xf:metadata/xf:meta[@code='EL_META_typePageGuidance']/xf:value='Page d''orientation']/xf:body">
      <let name="titrePage" value="string(../xf:metadata/xf:meta[@code='systemTitle'])" />
      <report test="count(./xf:structuralNode[xf:metadata/xf:meta[@code='ELS_META_referentielKN' and xf:value/xf:ref/@idRef=$ongletID]]) > 1" role="error">
        Il y a plusieurs onglets de type <xsl:value-of select="$ongletType"/> dans la Page Orientation <xsl:value-of select="$titrePage"/>
      </report>
    </rule>
  </pattern>

  <pattern is-a="controle-onglet" id="controle-ACTU">
    <param name="ongletID" value="'ACTU'" />
    <param name="ongletType" value="'Actualité'" />
  </pattern>
  <pattern is-a="controle-onglet" id="controle-CHECK_LIST">
    <param name="ongletID" value="'CHECK_LIST'" />
    <param name="ongletType" value="'Check-list'" />
  </pattern>
  <pattern is-a="controle-onglet" id="controle-FICHE_CONSEIL">
    <param name="ongletID" value="'FICHE_CONSEIL'" />
    <param name="ongletType" value="'Fiche Conseil'" />
  </pattern>
  <pattern is-a="controle-onglet" id="controle-FLOWCHART">
    <param name="ongletID" value="'FLOWCHART'" />
    <param name="ongletType" value="'Flowchart'" />
  </pattern>
  <pattern is-a="controle-onglet" id="controle-MODELE">
    <param name="ongletID" value="'MODELE'" />
    <param name="ongletType" value="'Modèle'" />
  </pattern>
  <pattern is-a="controle-onglet" id="controle-OUTILS_ET_PROC">
    <param name="ongletID" value="'OUTILS_ET_PROC'" />
    <param name="ongletType" value="'Outils et Procédures'" />
  </pattern>
  <pattern is-a="controle-onglet" id="controle-PAEPL">
    <param name="ongletID" value="'PAEPL'" />
    <param name="ongletType" value="'Pour aller plus loin'" />
  </pattern>
  <pattern is-a="controle-onglet" id="controle-SUJETS_LIES">
    <param name="ongletID" value="'SUJETS_LIES'" />
    <param name="ongletType" value="'Sujets liés'" />
  </pattern>
  <pattern is-a="controle-onglet" id="controle-TAB_RECAP">
    <param name="ongletID" value="'TAB_RECAP'" />
    <param name="ongletType" value="'Tableaux récapitulatif'" />
  </pattern>

  <!--==============================================-->
  <!--CONTROLE DES SURCHARGES DE TITRE              -->
  <!-- Attention avant de décommenter:              -->
  <!-- si plusieurs referenceNode dans le structuralNode on a une erreur: cf. SIEGEL-462 -->
  <!--==============================================-->
  <!--pattern id="surcharge-titre">
    <rule context="/xf:editorialEntity/xf:body/xf:structuralNode[xf:metadata/xf:meta[@code='ELS_META_referentielKN' and xf:value/xf:ref/@idRef='MODELE']]">
      <let name="title-length" value="string-length(./xf:referenceNode/xf:metadata/xf:meta[@code = 'systemTitle'])"/>
      <let name="modele" value="normalize-space(./xf:referenceNode/xf:verbalization)" />
      <report test="$title-length > 200" role="error">
        Le titre de surcharge du modèle "<value-of select="$modele"/>" est trop long : <value-of select="$title-length"/> caractères (max = 200)
      </report>
    </rule>
  </pattern-->

  <!--==============================================-->
  <!--CONTROLE DE NON RECURSIVITE DE L'ACCROCHE DES TP-->
  <!--==============================================-->
  <pattern id="accroche-idem">
    <rule context="/xf:editorialEntity/xf:body/xf:structuralNode[xf:metadata/xf:meta[@code='ELS_META_referentielKN']]/xf:referenceNode">
      <let name="titrePage" value="normalize-space(string(ancestor::xf:editorialEntity/xf:metadata/xf:meta[@code='systemTitle']))" />
      <report test="./xf:ref[@xf:targetResId = ancestor::xf:editorialEntity/@xf:id]" role="error">
        Votre Page Tâche "<xsl:value-of select="$titrePage"/>" ne peut pas s'accrocher elle-même
      </report>
    </rule>
  </pattern>

  <!--==============================================-->
  <!--CONTROLE DE RETOUR CHARIOT DANS LES TITRES    -->
  <!--==============================================-->
  <!--pattern id="tit-ret-chariot">
    <rule context="/xf:editorialEntity/xf:metadata/xf:meta[@code = 'ELS_META_titreEE']/xf:value/h:div">
      <let name="titrePage" value="normalize-space(string(.))" />
      <report test="contains(., '&#x0A;') or ./h:p or ./h:br" role="warning">
        Le titre "<xsl:value-of select="$titrePage"/>" comporte un retour-chariot
      </report>
    </rule>
  </pattern-->

  <!--=========================================-->
  <!--CONTROLE DES ONGLETS DANS UNE TOPIC PAGE -->
  <!--=========================================-->
  <pattern id="controle-onglet-type">
    <rule context="/xf:editorialEntity[xf:metadata/xf:meta[@code='EL_META_typePageGuidance']/xf:value='Page Tâche']/xf:body">
      <let name="structuralNode.sansTypage" value="xf:structuralNode[not(exists(xf:metadata/xf:meta[@code='ELS_META_referentielKN']) or @xf:type)]" />
      <assert test="count($structuralNode.sansTypage) = 0" role="warning">
        Tous les onglets de la Page Tâche doivent être typés. Merci de compléter les métadonnées obligatoires et le type du ou des onglets : <xsl:value-of select="$structuralNode.sansTypage/xf:title" separator=", "/>
      </assert>
    </rule>
    <rule context="/xf:editorialEntity[xf:metadata/xf:meta[@code='EL_META_typePageGuidance']/xf:value='Page d''orientation']/xf:body">
      <let name="structuralNode.sansTypage" value="xf:structuralNode[not(exists(xf:metadata/xf:meta[@code='ELS_META_referentielKN']) or @xf:type)]" />
      <assert test="count($structuralNode.sansTypage) = 0" role="warning">
        Tous les onglets de la Page Orientation doivent être typés. Merci de compléter les métadonnées obligatoires et le type du ou des onglets : <xsl:value-of select="$structuralNode.sansTypage/xf:title" separator=", "/>
      </assert>
    </rule>
  </pattern>
</schema>