<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns:ecm="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm"
  xmlns:xfe="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor"
  xmlns:fichesGB_FP="http://www.lefebvre-sarrut.eu/ns/el/fichesGB_FP"
  xmlns="http://www.w3.org/1999/xhtml"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all"
  version="3.0">
  
  <xsl:import href="xslLib:/xslLib/els-common.xsl"/>
  
  <!--==================================================-->
  <!--INIT-->
  <!--==================================================-->
  
  <xsl:template match="/">
    <xsl:apply-templates select="." mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-surcharges-xml"/>
  </xsl:template>
  
  <!--==================================================-->
  <!--MAIN-->
  <!--==================================================-->
  
  <!--=====================-->
  <!--Surcharge Tifp-->
  <!--=====================-->
  
  <!--SIEGEL-167 : surcharge de Tifp en utilisant le systemTitle du referenceNode si celui-ci est renseigné--> 
  <xsl:template match="referenceNode/ref/editorialEntity/body/contentNode/content/fichesGB_FP:FP/fichesGB_FP:Tifp[1]" mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-surcharges-xml">
    <xsl:variable name="referenceNode.systemTitle.div" select="ancestor::referenceNode[1]/metadata/meta[@code = 'systemTitle']/value[@as = 'element(html:div)']/html:div[1]" as="element(html:div)?"/>
    <xsl:choose>
      <xsl:when test="not(matches(string($referenceNode.systemTitle.div), concat('^', $els:regAnySpace,'*$')))">
        <div class="Tifp">
          <xsl:message use-when="false()">[DEBUG] SIEGEL-167.1 : surcharge Tifp</xsl:message>
          <xsl:copy-of select="$referenceNode.systemTitle.div/node()"/>
        </div>
      </xsl:when>
      <xsl:otherwise>
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!--=====================-->
  <!--Surcharge OBS-->
  <!--=====================-->
  
  <!--SIEGEL-167 : Si EL_META_conserveObservation=false sur le referenceNode, alors :
      dans l'editorialEntity référencée il faut remplacer le contenu "body/contentNode/content/FP/OBS[1]" 
      par le contenu de la méta EL_META_surchargeObservation si elle existe, sinon on supprime cet élément.
      ERRATUM : Lors du remplacement :
        1) Si OBS n'existe pas : 
          - Si PAPL exite : ajouter "OBS de surcharge" avant PAPL[last()]
          - Si PAPL n'existe pas : ajouter "OBS de surcharge" en dernier élément sous FP  
        2) Si EL_META_surchargeObservation contient un titre de l'observation (1er <p> avec <strong>) alors il vient remplacer OBS/Tiobs
          (NB : dans tous les cas on remplace tout ou rien) 
  -->

  <!--CAS 1 : surcharge demandée + pas OBS ni PAPL : ajout OBS fin de FP-->
  <xsl:template 
    match="referenceNode/ref/editorialEntity/body/contentNode/content/fichesGB_FP:FP
    [xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.get-EL_META_conserveObservation(.) = 'false']
    [not(fichesGB_FP:OBS)][not(fichesGB_FP:PAPL)]
    /*[last()]" priority="1"
    mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-surcharges-xml">
    <xsl:next-match/>
    <xsl:message use-when="false()">[DEBUG] SIEGEL-167.2 : CAS 1</xsl:message>
    <xsl:call-template name="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.makeObservationSurcharge"/>
  </xsl:template>
  
  <!--CAS 2 : surcharge demandée + OBS existe : on le remplace (par rien éventuellement)-->
  <xsl:template match="referenceNode/ref/editorialEntity/body/contentNode/content/fichesGB_FP:FP/fichesGB_FP:OBS[1]
    [xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.get-EL_META_conserveObservation(.) = 'false']" 
    mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-surcharges-xml">
    <xsl:message use-when="false()">[DEBUG] SIEGEL-167.2 : CAS 2</xsl:message>
    <xsl:call-template name="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.makeObservationSurcharge"/>
  </xsl:template>
  
  <!--CAS 3 : surcharge demandée + OBS n'existe pas mais PAPL existe : ajout OBS avant 1er PAPL-->
  <xsl:template match="referenceNode/ref/editorialEntity/body/contentNode/content/fichesGB_FP:FP[not(fichesGB_FP:OBS)]/fichesGB_FP:PAPL[1]
    [xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.get-EL_META_conserveObservation(.) = 'false']" 
    mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-surcharges-xml" priority="1">
    <xsl:message use-when="false()">[DEBUG] SIEGEL-167.2 : CAS 3</xsl:message>
    <xsl:call-template name="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.makeObservationSurcharge"/>
    <xsl:next-match/>
  </xsl:template>
  
  <!--WARNING : si Tiobs est directement sous FP et non dans OBS-->
  <xsl:template match="referenceNode/ref/editorialEntity/body/contentNode/content/fichesGB_FP:FP/fichesGB_FP:Tiobs
    [xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.get-EL_META_conserveObservation(.) = 'false']"
    mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-surcharges-xml">
    <xsl:message use-when="false()">[DEBUG] SIEGEL-167.2 : Warning</xsl:message>
    <div class="ecm_error">
      <p style="color:inherit;">[ERROR] Le Tiobs ci-dessous est mal placé, il devrait être à l'intérieur de OBS, la surcharge méta de l'observation ne sera pas correcte</p>
    </div>
    <xsl:next-match/>
  </xsl:template>
  
  <xsl:function name="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.get-EL_META_surchargeObservation" as="element(html:div)?">
    <xsl:param name="e" as="element()"/>
    <xsl:sequence select="$e/ancestor::referenceNode[1]/metadata/meta[@code = 'EL_META_surchargeObservation']/value[@as = 'element(html:div)']/html:div"/>
  </xsl:function>
  
  <xsl:function name="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.get-EL_META_conserveObservation" as="xs:string?">
    <xsl:param name="e" as="element()"/>
    <xsl:sequence select="$e/ancestor::referenceNode[1]/metadata/meta[@code = 'EL_META_conserveObservation']/value[@as = 'xs:boolean']/text()[1]"/>
  </xsl:function>
  
  <xsl:template name="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.makeObservationSurcharge">
    <xsl:param name="EL_META_surchargeObservation.content" select="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.get-EL_META_surchargeObservation(.)/node()[not(self::text()[els:is-empty-or-whitespace(.)])]" as="item()*"/>
    <xsl:if test="count($EL_META_surchargeObservation.content) != 0">
      <OBS xmlns="http://www.lefebvre-sarrut.eu/ns/el/fichesGB_FP">
        <xsl:variable name="Tiobs" select="$EL_META_surchargeObservation.content/self::*[xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.isHtmlTiobs(.)]" as="element()?"/>
        <xsl:if test="exists($Tiobs)">
          <Tiobs>
            <xsl:sequence select="$Tiobs/html:strong/node()"/>
          </Tiobs>
        </xsl:if>
        <xsl:sequence select="$EL_META_surchargeObservation.content except $Tiobs"/>
      </OBS>
    </xsl:if>
  </xsl:template>
  
  <xsl:function name="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.isHtmlTiobs" as="xs:boolean">
    <xsl:param name="e" as="element()"/>
    <xsl:sequence select="boolean($e/self::html:p[1][not(text()[normalize-space(.) != ''])][count(*)= 1 and html:strong])"/>
  </xsl:function>
  
  <!--=====================-->
  <!--Surcharge Resume-->
  <!--=====================-->

  <!--SIEGEL-167 : Si EL_META_conserveResume=false sur le referenceNode, alors :
      dans l'editorialEntity référencée il faut remplacer le contenu "body/contentNode/content/FP/INTROFP[1]" 
      par le contenu de la méta EL_META_surchargeResume si elle existe, sinon on supprime cet élément.
      ERRATUM : si INTROFP n'existe pas et qu'il y a une surcharge, on le rajoute sous Tifp  
  -->

  <xsl:template 
    match="referenceNode/ref/editorialEntity/body/contentNode/content/fichesGB_FP:FP
    [xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.get-EL_META_conserveResume(.) = 'false']
    [not(fichesGB_FP:INTROFP)]
    /fichesGB_FP:Tifp[1]" priority="1"
    mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-surcharges-xml">
    <xsl:next-match/>
    <xsl:message use-when="false()">[DEBUG] SIEGEL-167.3 : surcharge résumé - CAS 1</xsl:message>
    <xsl:call-template name="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.makeResumeSurcharge"/>
  </xsl:template>
  
  <xsl:template match="referenceNode/ref/editorialEntity/body/contentNode/content/fichesGB_FP:FP/fichesGB_FP:INTROFP[1]
    [xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.get-EL_META_conserveResume(.) = 'false']" 
    mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-surcharges-xml">
    <xsl:call-template name="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.makeResumeSurcharge"/>
    <xsl:message use-when="false()">[DEBUG] SIEGEL-167.3 : surcharge résumé - CAS 2</xsl:message>
  </xsl:template>
  
  <xsl:function name="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.get-EL_META_conserveResume" as="xs:string?">
    <xsl:param name="e" as="element()"/>
    <xsl:sequence select="$e/ancestor::referenceNode[1]/metadata/meta[@code = 'EL_META_conserveResume']/value[@as = 'xs:boolean']/text()[1]"/>
  </xsl:function>
  
  <xsl:function name="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.get-EL_META_surchargeResume" as="element(html:div)?">
    <xsl:param name="e" as="element()"/>
    <xsl:sequence select="$e/ancestor::referenceNode[1]/metadata/meta[@code = 'EL_META_surchargeResume']/value[@as = 'element(html:div)']/html:div"/>
  </xsl:function>
  
  <xsl:template name="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.makeResumeSurcharge">
    <xsl:param name="EL_META_surchargeResume.content" select="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing.get-EL_META_surchargeResume(.)/node()[not(self::text()[els:is-empty-or-whitespace(.)])]" as="item()*"/>
    <xsl:if test="count($EL_META_surchargeResume.content) != 0">
      <INTROFP xmlns="http://www.lefebvre-sarrut.eu/ns/el/fichesGB_FP">
        <xsl:sequence select="$EL_META_surchargeResume.content"/>
      </INTROFP>
    </xsl:if>
  </xsl:template>
  
  <!--=====================-->
  <!--Surcharge PAPL-->
  <!--=====================-->
  
  <!--SIEGEL-167 : Si EL_META_conservePAPL=false sur le referenceNode, alors :
      dans l'editorialEntity référencée on supprime tous les "body/contentNode/content/FP/PAPL" 
   -->
  <xsl:template match="referenceNode/ref/editorialEntity/body/contentNode/content/fichesGB_FP:FP/fichesGB_FP:PAPL" mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-surcharges-xml">
    <xsl:variable name="EL_META_conservePAPL" select="ancestor::referenceNode[1]/metadata/meta[@code = 'EL_META_conservePAPL']/value[@as = 'xs:boolean']/text()[1]" as="xs:string?"/>
    <xsl:choose>
      <xsl:when test="$EL_META_conservePAPL = 'false'">
        <xsl:message use-when="false()">[DEBUG] SIEGEL-167.4 : EL_META_conservePAPL=false</xsl:message>
        <!--PAPL deleted -->
      </xsl:when>
      <xsl:otherwise>
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!--=====================-->
  <!--Surcharge REFPRINC-->
  <!--=====================-->
  
  <!--SIEGEL-167 : Si EL_META_conserveRefPrincipale=false sur le referenceNode, alors :
      dans l'editorialEntity référencée on supprime tous les "body/contentNode/content/FP/REFPRINC" 
   -->
  <xsl:template match="referenceNode/ref/editorialEntity/body/contentNode/content/fichesGB_FP:FP/fichesGB_FP:REFPRINC" 
    mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-surcharges-xml">
    <xsl:variable name="EL_META_conserveRefPrincipale" select="ancestor::referenceNode[1]/metadata/meta[@code = 'EL_META_conserveRefPrincipale']/value[@as = 'xs:boolean']/text()[1]" as="xs:string?"/>
    <xsl:choose>
      <xsl:when test="$EL_META_conserveRefPrincipale = 'false'">
        <xsl:message use-when="false()">[DEBUG] SIEGEL-167.5 : EL_META_conserveRefPrincipale=false</xsl:message>
        <!--REFPRINC deleted-->
      </xsl:when>
      <xsl:otherwise>
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-surcharges-xml">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>