<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns:ecm="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm"
  xmlns:xfe="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor"
  xmlns:fichesGB_FP="http://www.lefebvre-sarrut.eu/ns/el/fichesGB_FP"
  xmlns="http://www.w3.org/1999/xhtml"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all"
  version="3.0">
  
  <xsl:import href="EL_TEE_practiceNoteAssemblee.htmlSimplePreview.xsl"/>
  <xsl:import href="EL_TEE_practiceNoteAssemblee.htmlProofing.surcharges-xml.xsl"/>
  <xsl:import href="../fichePratique.htmlSimplePreview.xsl"/>
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Generate a HTML proofing for EE type "EL_TEE_practiceNoteAssemblee"</xd:p>
    </xd:desc>
  </xd:doc>
  
  <xsl:param name="xfRes:debug" select="false()" as="xs:boolean"/>
  <xsl:param name="xfRes:log.uri" select="resolve-uri('log/', base-uri(/))" as="xs:string"/>
  <xsl:param name="xfRes:css.uri" select="'../../../../main/web-resources/css/EL_TEE_practiceNoteAssemblee.htmlProofing.css'" as="xs:string"/>
  <xsl:param name="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing_skipAggregation" select="false()" as="xs:boolean"/>
  
  <!--indent="no" is important for xspec diff--> 
  <xsl:output method="xhtml" indent="no"/>
  
  <!--+==================================================+
      | INIT.                                            |
      +==================================================+-->
  
  <xsl:template match="/">
    <xsl:apply-templates select="." mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing"/>
  </xsl:template>
  
  <!--==================================================-->
  <!--MAIN-->
  <!--==================================================-->
  
  <!--Driver-->
  <xsl:template match="/" mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing">
    <!--resolve referencenNode-->
    <xsl:variable name="step" as="document-node()">
      <xsl:choose>
        <xsl:when test="$xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing_skipAggregation">
          <xsl:sequence select="."/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:document>
            <xsl:apply-templates select="." mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-resolve"/>
          </xsl:document>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="$xfRes:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('EL_TEE_practiceNoteAssemblee.htmlProofing.step1.xml', $xfRes:log.uri)" as="xs:anyURI"/>
      <xsl:message>[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:message>
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <!--Surcharge XML-->
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="$step" mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-surcharges-xml"/>
      </xsl:document>
    </xsl:variable>
    <xsl:if test="$xfRes:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('EL_TEE_practiceNoteAssemblee.htmlProofing.step2.xml', $xfRes:log.uri)" as="xs:anyURI"/>
      <xsl:message>[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:message>
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <!--Transform to HTML-->
    <!--Use the simple preview of this tree, referenceNode template will be overrided to transform the referenced EE to html-->
    <xsl:apply-templates select="$step" mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlSimplePreview"/>
  </xsl:template>
  
  <!--=============================-->
  <!--STEP-1 : resolve-->
  <!--=============================-->

  <xsl:template match="referenceNode/ref" mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-resolve">
    <xsl:variable name="doc.uri" select="concat($ecm:ee.url.begin, @xf:targetResId)" as="xs:string"/>
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <xsl:choose>
        <xsl:when test="doc-available($doc.uri)">
          <xsl:apply-templates select="doc($doc.uri)" mode="#current"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy>
            <html:p class="ecm_error">[ERROR] EE <xsl:value-of select="$doc.uri"/> is not available !</html:p>
            <html:p class="ecm_debug">
              <xsl:choose>
                <xsl:when test="unparsed-text-available($doc.uri)">
                  <p>unparsed-text-available :</p>
                  <textarea rows="10" cols="50">
                    <xsl:value-of select="unparsed-text($doc.uri)"/>
                  </textarea>
                </xsl:when>
                <xsl:otherwise>
                  <p>unparsed-text is not available</p>
                </xsl:otherwise>
              </xsl:choose>
            </html:p>
          </xsl:copy>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlProofing-resolve">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--=============================-->
  <!--STEP-2 : surcharge XML -->
  <!--=============================-->
  
  <!--cf. import-->

  <!--=============================-->
  <!--STEP-3 : transform to HTML   -->
  <!--=============================-->
  
  <!--
    mode xfRes:EL_TEE_practiceNoteAssemblee.htmlSimplePreview => EL_TEE_practiceNoteAssemblee.htmlSimplePreview.xsl
    EL_TEE_practiceNoteAssemblee.htmlSimplePreview.xsl => is calling mode "xfRes:TEE_tree.default.htmlSimplePreview"
    So overriden rules will be applied on this mode
  --> 
  
  
  <xsl:template match="editorialEntity[@code = 'EL_TEE_fichePratique']" mode="xfRes:TEE_tree.default.htmlSimplePreview" priority="1">
    <xsl:variable name="editorialEntity.doc" as="document-node()">
      <xsl:document>
        <xsl:sequence select="."/>
      </xsl:document>
    </xsl:variable>
    <xsl:apply-templates select="$editorialEntity.doc" mode="xfRes:EL_TEE_fichePratique.htmlSimplePreview"/>
  </xsl:template>
  
  <xsl:template match="editorialEntity[@code != 'EL_TEE_practiceNoteAssemblee']" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:variable name="editorialEntity.doc" as="document-node()">
      <xsl:document>
        <xsl:sequence select="."/>
      </xsl:document>
    </xsl:variable>
    <p class="ecm_error">EE code="<xsl:value-of select="$editorialEntity.doc/editorialEntity/@code"/>" unexpected here</p>
    <xsl:apply-templates select="$editorialEntity.doc" mode="xfRes:TEE_text.default.htmlSimplePreview"/>
  </xsl:template>
  
  <!--Override referenceNode template of the original simple preview of this tree-->
  <xsl:template match="referenceNode" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <li class="referenceNode">
      <xsl:choose>
        <xsl:when test="ref/editorialEntity">
          <xsl:variable name="referenceNode.ee.html" as="element()*">
            <xsl:apply-templates select="ref/editorialEntity" mode="#current"/>
          </xsl:variable>
          <div class="ecm-aggregate">
            <xsl:choose>
              <xsl:when test="$referenceNode.ee.html//html:body">
                <xsl:variable name="class" select="$referenceNode.ee.html//html:body/@class" as="xs:string?"/>
                <xsl:if test="normalize-space($class) != ''">
                  <xsl:attribute name="class" select="concat('ecm-aggregate ', $class)"/>
                </xsl:if>
                <xsl:sequence select="$referenceNode.ee.html//html:body/*"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:sequence select="$referenceNode.ee.html/html:div/*"/>
              </xsl:otherwise>
            </xsl:choose>
          </div>
        </xsl:when>
        <xsl:otherwise>
          <!--display resolution error message-->
          <xsl:apply-templates select="ref/*" mode="#current"/>
          <xsl:apply-templates select="(verbalization, ref)[1]" mode="#current"/>
        </xsl:otherwise>
      </xsl:choose>
    </li>
  </xsl:template>
  
  <xsl:template match="html:*" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:element name="{local-name()}">
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates mode="#current"/>
    </xsl:element>
  </xsl:template>

  <!--Override XFE conversion : keep html as is-->
  <!--Some html chunk from html meta has been added to the content on step 2-->
  
  <xsl:template match="html:*" mode="xfe:xml2html_annotations xfe:xml2html_main">
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
</xsl:stylesheet>