<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:ecm="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns="http://www.w3.org/1999/xhtml"
  xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all"
  version="3.0">

  <!--<xsl:import href="_common/el2html.xsl"/>-->
	<xsl:import href="../../_common/TEE_tree.default.htmlSimplePreview.xsl"/>
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Generate a HTML simple preview for EE type "EL_TEE_practiceNoteAssemblee"</xd:p>
      <xd:p>Import de "default tree preview", qui va être surchargée ici pour ajouter le titre depuis la méta ELS_META_titreEE</xd:p>
    </xd:desc>
  </xd:doc>
  
  <!--indent="no" is important for xspec diff--> 
  <xsl:output method="xhtml" indent="no"/>
  
  <!--base-uri est perdu à cause du xsl:document dans le step 1, on surcharge sa valeur ici-->
  <xsl:param name="xfRes:baseUri" select="base-uri()" as="xs:anyURI"/>
  
  <!--==================================================-->
  <!--INIT-->
  <!--==================================================-->

  <!--No modes for standalone execution-->
  <xsl:template match="/">
    <xsl:apply-templates select="." mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlSimplePreview"/>
  </xsl:template>
  
  <!--==================================================-->
  <!--MAIN-->
  <!--==================================================-->
  
  <xsl:template match="/" mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlSimplePreview">
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="." mode="xfRes:TEE_tree.default.htmlSimplePreview"/>
      </xsl:document>
    </xsl:variable>
    <!--Au step2 on a besoin de l'EE pour récupérer "ELS_META_titreEE"-->
    <xsl:variable name="editorialEntity" select="editorialEntity" as="element()"/>
    <xsl:apply-templates select="$step" mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlSimplePreview.step2">
      <xsl:with-param name="editorialEntity" select="$editorialEntity" as="element()" tunnel="yes"/>
    </xsl:apply-templates>
  </xsl:template>
  
  <!--======-->
  <!-- MODE xfRes:TEE_tree.default.htmlSimplePreview-->
  <!--======-->
  
  <!--cf. import-->
  
  <!--Surcharge sur contentNode-->
  <xsl:template match="contentNode/content/*" mode="xfRes:TEE_tree.default.htmlSimplePreview">
    <xsl:apply-templates select="node()" mode="#current"/>
  </xsl:template>

  <!--======-->
  <!-- MODE xfRes:EL_TEE_practiceNoteAssemblee.htmlSimplePreview.step2-->
  <!--======-->
  
  <!--Ajout du titre de la fiche au début du body-->
  <xsl:template match="html:body/*[1]" mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlSimplePreview.step2" priority="1">
    <xsl:param name="editorialEntity" as="element()" tunnel="yes"/>
    <xsl:variable name="ELS_META_titreEE" select="$editorialEntity/xf:getMeta(., 'ELS_META_titreEE')[1]" as="element()?"/>
    <xsl:if test="count($ELS_META_titreEE) = 1">
      <h1 class="Titrepa">
        <xsl:apply-templates select="$ELS_META_titreEE/value/html:div/node()" mode="#current"/>
      </h1>
    </xsl:if>
    <xsl:next-match/>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xfRes:EL_TEE_practiceNoteAssemblee.htmlSimplePreview.step2">
    <xsl:copy copy-namespaces="no">
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
 </xsl:stylesheet>