<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns:xfe="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:xslLib="http://www.lefebvre-sarrut.eu/ns/els/xslLib"
  xpath-default-namespace="http://www.w3.org/1999/xhtml"
  exclude-result-prefixes="#all"
  xmlns="http://www.w3.org/1999/xhtml"
  version="3.0">
  
 
  <!--+==================================================+
      | INIT.                                            |
      +==================================================+-->
  
  <!-- IMPORTS -->
  <xsl:import href="xfe:/xfe/xsl/xfe-XML2HTML.xsl"/>
  <xsl:import href="el_injectionJsTable.xsl"/>
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Génère une vue HTML de prévisualisation de l'EE.</xd:p>
      <xd:p>Il s'agit de la vue utilisée dans le panneau de droite de la fiche de l'EE et lors de la prévisualisation d'une EE dans les résultats de recherche ou les arbres.</xd:p>
      <xd:p>Importe la transformation utilisée pour l'éditeur XML (xfe-XML2HTML.xsl) avec fichier de configuration customisé pour remonter quelques informations supplémentaires.</xd:p>
    </xd:desc>
  </xd:doc>
  
  <!-- PARAMS -->
  <!-- Paramètre obligatoire pour xfe:xml2html -->
  <xsl:param name="xfe:conf.uri" select="resolve-uri('_common/el_FICHE_FPRO.htmlPreview.xfe-conf.xml',static-base-uri())" as="xs:string"/>
  <xsl:param name="xfe:addXmlModelPi" select="false()" as="xs:boolean"/>
  <xsl:param name="xfe:addHTML5DOCTYPE" select="false()" as="xs:boolean"/>
  <xsl:param name="xfe:debug" select="false()" as="xs:boolean"/>
  <xsl:param name="xfe:addXfeLabel" select="false()" as="xs:boolean"/>
  <xsl:param name="xfe:fillHeadTitle" select="true()" as="xs:boolean"/>
  <xsl:param name="xfe:allowUnValidXML" select="true()" as="xs:boolean"/>

  <!--CSS-->
  <xsl:param name="xfe:css.path" select="'../../../../main/web-resources/css/'" as="xs:string"/>
  <xsl:param name="xfe:css.name" select="'EL_TEE_fichePro.css'" as="xs:string"/>
  
  <!--CALS-->
  <!--Pas de style sur les bordures ou autre, sera géré en css par type de tableau-->
  <xsl:param name="xslLib:cals2html.use-style-insteadOf-class" select="false()" as="xs:boolean"/>
  
  <!--+==================================================+
      | INIT                                             |
      +==================================================+-->  
  <!-- LANCEMENT SANS MODE -->
  <xsl:template match="/">
    <xsl:apply-templates select="." mode="xfRes:htmlSimplePreview"/>
  </xsl:template>
  
  <!--+==================================================+
      | MAIN                                             |
      +==================================================+-->
  <xsl:template match="/" mode="xfRes:htmlSimplePreview">
    <xsl:message>[INFO] Prévisualisation HTML de l'EE : <xsl:value-of select="base-uri()"/></xsl:message>
    
    <!-- Application de la transfoHTML pour l'éditeur XML -->
    <xsl:variable name="step" select="." as="document-node()"/>
    
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="$step" mode="xfe:xml2html"/>
      </xsl:document>
    </xsl:variable>
    
    <!-- modification Patch Adapt HTML --> 
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="$step" mode="xfRes:EL_TEE_fichePro.htmlSimplePreview.adaptHTML"/>
      </xsl:document>
    </xsl:variable>
    
    <!--FINALY-->
    <xsl:apply-templates select="$step" mode="injectJS"/>
  </xsl:template>
  
  <!--============================================-->
  <!--xfRes:EL_TEE_fichePratique.adaptHTML-->
  <!--============================================-->
  <!-- Wrapping des elements groupes -->
  <xsl:template match="*[not(self::ul)][li]" mode="xfRes:EL_TEE_fichePro.htmlSimplePreview.adaptHTML" priority="1">
    <xsl:message select="concat('node : ', name())"/>       
    <xsl:variable name="wrapper" as="element()"><ul/></xsl:variable>
    <xsl:apply-templates select="els:wrap-elements-adjacent-by-names(., 'li', $wrapper)" mode="#current"/>
  </xsl:template>
  
  <xsl:template match="*[not(self::footer)][aside]" mode="xfRes:EL_TEE_fichePro.htmlSimplePreview.adaptHTML">
    <xsl:variable name="wrapper" as="element()"><footer/></xsl:variable>
    <xsl:apply-templates select="els:wrap-elements-adjacent-by-names(., 'aside', $wrapper)" mode="#current"/>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xfRes:EL_TEE_fichePro.htmlSimplePreview.adaptHTML">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
</xsl:stylesheet>