<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:h="http://www.w3.org/1999/xhtml"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
  xmlns:xfe="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor"
  xmlns:juri="http://www.els.fr/ns/juri"
  xmlns="http://www.w3.org/1999/xhtml"
  xpath-default-namespace="http://www.els.fr/ns/juri" 
  exclude-result-prefixes="#all" 
  version="3.0"
  >
 
  <!--==========================================-->
  <!-- INIT -->
  <!--==========================================-->
  
  <!-- IMPORTS -->
  <xsl:import href="../_common/xf-common.htmlPreview.xsl"/>

  <!--indent="no" is important for xspec diff--> 
	<xsl:output method="xhtml" indent="no"/>

  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Génère une vue HTML de prévisualisation de l'EE.</xd:p>
      <xd:p>Il s'agit de la vue utilisée dans le panneau de droite de la fiche de l'EE et lors de la prévisualisation d'une EE dans les résultats de recherche ou les arbres.</xd:p>
    </xd:desc>
  </xd:doc>
  
  <!-- PARAMS -->
  <xsl:param name="xfe:css.name" select="els:getFileName($xfRes:css.uri)" as="xs:string"/>
  <xsl:param name="xfe:css.path" select="els:getFolderPath($xfRes:css.uri)" as="xs:string"/>  

	<xsl:template match="/">
		<xsl:apply-templates select="." mode="xfRes:htmlSimplePreview"/>		
	</xsl:template>
	
  <!--==========================================-->
  <!-- MAIN -->
  <!--==========================================-->
  
	<xsl:template match="/" mode="xfRes:htmlSimplePreview">
		<xsl:message>[INFO] Prévisualisation HTML de l'EE : <xsl:value-of select="base-uri()"/></xsl:message>
		<xsl:apply-templates mode="#current"/>
	</xsl:template>
  
  <xsl:template match="/xf:editorialEntity" mode="xfRes:htmlSimplePreview">
    <html>
      <xsl:call-template name="xfRes:htmlPreview.makeHtmlHead"/>
      <xsl:apply-templates mode="#current"/>
    </html>
  </xsl:template>
  
  <xsl:template match="/xf:editorialEntity/xf:metadata" mode="xfRes:htmlSimplePreview"/>
		
  <xsl:template match="/xf:editorialEntity/xf:body" mode="xfRes:htmlSimplePreview">
		<body>
			<div id="wrapper">
				<div class="text file-container" style="display:block;">
					<div class="doc">
						<xsl:apply-templates mode="#current"/>
					</div>
				</div>
			</div>
		</body>
	</xsl:template>

  <!--==========================================-->
  <!-- juri2html -->
  <!--==========================================-->
  
  <xsl:template match="MetaJuri | MetaDoc | MetaFlux" mode="xfRes:htmlSimplePreview">
    <!-- Méta non affichées -->
  </xsl:template>
  
	<xsl:template match="P" mode="xfRes:htmlSimplePreview">
		<p>
			<xsl:apply-templates mode="#current"/>
		</p>
	</xsl:template>

  <xsl:template match="Personne" mode="xfRes:htmlSimplePreview">
    <span class="nompartie show-text">
		  <xsl:apply-templates mode="#current"/>
    </span>
  </xsl:template>

  <xsl:template match="Personne/Texte" mode="xfRes:htmlSimplePreview">
    <i class="nompartie-texte">
      <xsl:apply-templates mode="#current"/>
    </i>
  </xsl:template>

  <xsl:template match="Personne/TexteAnonymise" mode="xfRes:htmlSimplePreview">
    <b class="nompartie-texte-anonymise">
		  <xsl:apply-templates mode="#current"/>
    </b>
  </xsl:template>

  <xsl:template match="Adresse" mode="xfRes:htmlSimplePreview">
    <span class="adresse">
		  <xsl:apply-templates mode="#current"/>
    </span>
  </xsl:template>

</xsl:stylesheet>