<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:fn="http://www.w3.org/2005/xpath-functions"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:functx="http://www.functx.com"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:xslLib="http://www.lefebvre-sarrut.eu/ns/els/xslLib"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  exclude-result-prefixes="#all"
  version="3.0">
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Ensemble de variables / fonctions / templates utilitaires permettant de manipuler des données FLASH (toutes maisons confondus).</xd:p>
      <xd:p>L'idée est de ne mettre dans cette XSL que des fonctions / templates pouvant être utilisés pour tous les documents FLASH</xd:p>
    </xd:desc>
  </xd:doc>

  <!-- IMPORTS -->
  <xsl:import href="../../xslLib/xslLib/els-common.xsl"/>
  <xsl:import href="../../xslLib/xslLib/els-log.xsl"/>
  <xsl:import href="../../xslLib/xslLib/xml2json/xjson2json.xsl"/>
  
  <!--A schema to validate xml query parameter of xf:makeQueryIdAttributeValue(), need a schema aware processor-->
  <xsl:import-schema schema-location="../../grammars/query/query.xsd"/>
  
  <!-- VARIABLES -->
  <!-- Nom de la XSL, pour els:log appelés directement depuis cette xsl-->
  <xsl:param name="els:currentResName" required="no" select="els:getFileName(static-base-uri())" as="xs:string"/>
  
  <!--dalloz, efl, el...-->
  <xsl:param name="xf:company" select="'els'" as="xs:string"/>
  
  <!-- KEYS -->
  <!-- Toutes les relations, identifiées par leur ID (cas d'usage : obtenir la relation à partir d'un renvoiAutrui, pour identifier la cible) -->
  <xsl:key name="xf:getRelationById" match="xf:relation" use="@xf:id"/>
  <!-- Toutes les EE, identifiées par leur ID -->
  <xsl:key name="xf:getEEById" match="xf:editorialEntity" use="@xf:id"/>
  
  <!--+==================================================+
      | FONCTIONS / Informations diverses sur les EE     |
      +==================================================+-->
  
  <!--INTERFACE : this function has to be overrided by the calling xslt-->
  <xsl:function name="xf:getEEdocByFlashId" as="document-node()">
    <xsl:param name="id" as="xs:string"/>
    <xsl:param name="targetResVersion" as="xs:string?"/>
    <xsl:document>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="$els:currentResName" as="xs:string"/>
        <xsl:with-param name="xpathContext" select="''" as="xs:string"/>
        <xsl:with-param name="level" select="'error'" as="xs:string"/>
        <xsl:with-param name="code" select="'function_getEEdocByFlashId-missingImplementation'" as="xs:string"/>
        <xsl:with-param name="description">Calling xf:getEEdocByFlashId('<xsl:value-of select="$id"/>','<xsl:value-of select="$targetResVersion"/>') : Please override this function in your own XSLT !</xsl:with-param>
      </xsl:call-template>
    </xsl:document>
  </xsl:function>
  
  <!--INTERFACE : this function has to be overrided by the calling xslt-->
  <xsl:function name="xf:getEEdocByFlashId" as="document-node()">
    <xsl:param name="id" as="xs:string"/>
    <xsl:document>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="$els:currentResName" as="xs:string"/>
        <xsl:with-param name="xpathContext" select="''" as="xs:string"/>
        <xsl:with-param name="level" select="'error'" as="xs:string"/>
        <xsl:with-param name="code" select="'function_getEEdocByFlashId-missingImplementation'" as="xs:string"/>
        <xsl:with-param name="description" as="xs:string">Calling xf:getEEdocByFlashId('<xsl:value-of select="$id"/>') : Please override this function in your own XSLT !</xsl:with-param>
      </xsl:call-template>
    </xsl:document>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Récupère le code TEE de l'EE <xd:ref name="ee" type="parameter">$ee</xd:ref> (attribut @code de /xf:editorialEntity - toute EE).</xd:p>
    </xd:desc>
    <xd:param name="ee">[element(xf:editorialEntity)?] Une EE quelconque.</xd:param>
    <xd:return>[xs:string] La valeur du code TEE, ou une chaîne vide si non-trouvée.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getTEE" as="xs:string">
    <xsl:param name="ee" as="element(xf:editorialEntity)?"/>
    <xsl:sequence select="($ee/@code,'')[1]"/>
  </xsl:function>
  
  <!--+==================================================+
      | FUNCTIONS / FLASH Relations                      |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>
      <xd:p>Generate an unique relation id (especially usefull for xspec as replacement of generate-id())-</xd:p>
    </xd:desc>
  </xd:doc>
  <!-- don't use as="xs:ID" which is not compatible with gauloisPipe, use as="xs:string" instead-->
  <!--FIXME : when used in SAS where multiple EE are allowed, the count could be contextualized to the current EE 
    (but then need to add EE count, so it remains unique in the whole SAS document) : necessary ?-->
  <xsl:function name="xf:generateRelationId" as="xs:string">
    <xsl:param name="e" as="element()"/>
    <xsl:variable name="count" as="xs:integer">
      <xsl:number select="$e" level="any"/>
    </xsl:variable>
    <xsl:value-of select="concat('relation-', local-name($e), '-', $count)"/>
  </xsl:function>
  
  <!--2 arg signature of xf:makeHistoricalIdAttributeValue-->
  <xsl:function name="xf:makeHistoricalIdAttributeValue" as="xs:string">
    <xsl:param name="e" as="element()?"/>
    <xsl:param name="historicalId" as="xs:string?"/>
    <xsl:sequence select="xf:makeHistoricalIdAttributeValue($e, $historicalId, $xf:company)"/>
  </xsl:function>
  
  <!--Flash use its own id (UUID) but its need a mapping to the historical ID, so the persistence of ID remains the same during import/export-->
  <!--This function makes an AVT (Attribute Value Template) to say Flash to replace it with the mapped hitorical ID-->
  <xsl:function name="xf:makeHistoricalIdAttributeValue" as="xs:string">
    <xsl:param name="e" as="element()?"/>
    <xsl:param name="historicalId" as="xs:string?"/>
    <xsl:param name="company" as="xs:string?"/>
    <xsl:choose>
      <xsl:when test="empty($historicalId)">
        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
          <xsl:with-param name="level" select="'error'"/>
          <xsl:with-param name="code" select="'xf:makeHistoricalIdAttributeValue_IS-EMPTY'"/>
          <xsl:with-param name="alert" select="true()"/>
          <xsl:with-param name="markup" select="false()"/>
          <xsl:with-param name="xpathContext" select="$e"/>
          <xsl:with-param name="description">$historicalId is empty <xsl:if test="not(empty($e))"> at <xsl:value-of select="els:get-xpath($e)"/></xsl:if></xsl:with-param>      
        </xsl:call-template>
        <xsl:sequence select="''"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select="concat('{historicalId:', if(not(($company, 'els')[1] = ('', 'els'))) then(concat($company, '_')) else() ,  $historicalId , '}')"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>FE-117 : a function to generate a json AVT query from target attributes of a link.</xd:p>
      <xd:p>When the ID is unknown one might give a "flash query" instead as an AVT (Attribute Value Template)</xd:p>
      <xd:p>Flash will try to resolve the query (during import, edition or just regularly in the life time of the editorial entity)</xd:p>
    </xd:desc>
    <xd:param name="e">The current element link (only usefull for logging messages)</xd:param>
    <xd:param name="query-as-xml.element">
      <xd:p>An xml representation of the query that must be valid according to query.xsd</xd:p>
      <xd:p>cf. query.xsd for documentation and query.sample.xml</xd:p>
    </xd:param>
    <xd:return>an JSON string representing the query understandable by the ECM, actually the AVT string looks like {query:jsonString}</xd:return>
  </xd:doc>
  <xsl:function name="xf:makeQueryIdAttributeValue" as="xs:string">
    <xsl:param name="e" as="element()?"/>
    <xsl:param name="query.xml" as="element(xf:query)?"/>
    <xsl:param name="fullText" as="xs:string"/>
    <!--it the query is empty, make a new one so we can add fullText inside-->
    <xsl:variable name="query.xml" as="element(xf:query)">
      <xsl:choose>
        <xsl:when test="empty($query.xml)">
          <query xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:sequence select="$query.xml"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="query.xml.document" as="document-node(schema-element(xf:query))">
      <xsl:document validation="strict">
        <xsl:sequence select="$query.xml"/>
      </xsl:document>
    </xsl:variable>
    <xsl:variable name="query.xml.normalized" as="element(xf:query)">
      <xsl:apply-templates select="$query.xml.document" mode="xf:normalizeXmlQuery">
        <xsl:with-param name="fullText" select="$fullText" as="xs:string" tunnel="yes"/>
      </xsl:apply-templates>
    </xsl:variable>
    <xsl:variable name="query.xjson" as="element(fn:map)">
      <xsl:apply-templates select="$query.xml.normalized" mode="xf:xmlQueryToXjson"/>
    </xsl:variable>
    <xsl:value-of select="concat('{query:', xslLib:xjson2json($query.xjson), '}')"/>
  </xsl:function>
  
  <xd:p>2 args signature : default fullText is the string value of the current element</xd:p>
  <xsl:function name="xf:makeQueryIdAttributeValue" as="xs:string">
    <xsl:param name="e" as="element()?"/>
    <xsl:param name="query.xml" as="element(xf:query)?"/>
    <xsl:variable name="e.stringValue" select="$e" as="xs:string"/>
    <xsl:sequence select="xf:makeQueryIdAttributeValue($e, $query.xml, normalize-space($e.stringValue))"/>
  </xsl:function>
  
  <!--================================-->
  <!--mode xf:normalizeXmlQuery-->
  <!--================================-->
  
  <xsl:template match="xf:query" mode="xf:normalizeXmlQuery">
    <xsl:param name="fullText" as="xs:string" tunnel="yes"/>
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <!--Add fullText if necessary-->
      <xsl:if test="count(xf:metadata/xf:meta[@code != 'systemTee']) = 0 and not(xf:fullText)">
        <fullText xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst">
          <xsl:sequence select="$fullText"/>
        </fullText> 
      </xsl:if>
      <xsl:apply-templates select="node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="xf:metadata" mode="xf:normalizeXmlQuery">
    <xsl:variable name="meta.systemTee" select="xf:meta[@code = 'systemTee'][xf:value[normalize-space(.) != '']]" as="element(xf:meta)*"/>
    <xsl:if test="count($meta.systemTee) = 0">
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())" as="xs:string"/>
        <xsl:with-param name="code" select="'xf:normalizeXmlQuery_systemTeeMissing'" as="xs:string"/>
        <xsl:with-param name="level" select="'fatal'" as="xs:string"/>
        <xsl:with-param name="alert" select="true()" as="xs:boolean"/>
        <xsl:with-param name="markup" select="false()" as="xs:boolean"/>
        <xsl:with-param name="description" as="xs:string">[FATAL] query must define at least one meta "systemTee"</xsl:with-param>
      </xsl:call-template>
      <xsl:message terminate="yes"/>
    </xsl:if>
    <xsl:apply-templates select="." mode="xf:normalizeMetadata"/>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xf:normalizeXmlQuery">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--================================-->
  <!--mode xf:xmlQueryToXjson-->
  <!--================================-->
  
  <xsl:template match="xf:query" mode="xf:xmlQueryToXjson">
    <fn:map>
      <xsl:apply-templates mode="#current"/>
    </fn:map>
  </xsl:template>
  
  <xsl:template match="xf:fullText" mode="xf:xmlQueryToXjson">
    <fn:string key="fullText">
      <xsl:apply-templates mode="#current"/>
    </fn:string>
  </xsl:template>
  
  <xsl:template match="xf:metadata" mode="xf:xmlQueryToXjson">
    <fn:array key="metadata">
      <xsl:apply-templates mode="#current">
        <!--sort to keep order with XSpec-->
        <!--<xsl:sort select="@code"/>-->
      </xsl:apply-templates>
    </fn:array>
  </xsl:template>
  
  <xsl:template match="xf:meta" mode="xf:xmlQueryToXjson">
    <fn:map>
      <fn:string key="code">
        <xsl:value-of select="@code"/>
      </fn:string>
      <fn:array key="values">
        <xsl:apply-templates mode="#current"/>
      </fn:array>
    </fn:map>
  </xsl:template>
  
  <xsl:template match="xf:value" mode="xf:xmlQueryToXjson">
    <fn:string>
      <xsl:value-of select="."/>
    </fn:string>
  </xsl:template>
  
  <!--+==================================================+
      | Normalize Flash Metadata                         |
      +==================================================+-->
  
  <!--Group metadata with same code so they become multivalued meta-->
  <xsl:template match="xf:metadata" mode="xf:normalizeMetadata">
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <xsl:for-each-group select="xf:meta[xf:value[* or normalize-space(.) != '']]" group-by="self::xf:meta/@code">
        <meta code="{current-grouping-key()}" xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst">
          <xsl:apply-templates select="current-group()[1]/@*" mode="#current"/>
          <xsl:apply-templates select="current-group()/node()" mode="#current"/>
        </meta>
      </xsl:for-each-group>
    </xsl:copy>
  </xsl:template>
  
  <!--Removes duplicated values from a multivalued meta -->
  <xsl:template match="xf:meta" mode="xf:normalizeMetadata">
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <xsl:apply-templates select="functx:distinct-deep(*)" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xf:normalizeMetadata">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--+==================================================+
      | FONCTIONS / Manipulation de métadonnées FLASH    |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>
      <xd:p>Détermine, selon <xd:ref name="condition" type="parameter">$condition</xd:ref>, si un élément donné (<xd:ref name="e" type="parameter">$e</xd:ref>) possède :</xd:p>
      <xd:ul>
        <xd:li>au moins une métadonnée dont le code correspond à une des valeurs de <xd:ref name="codes" type="parameter">$codes</xd:ref> (<xd:ref name="condition" type="parameter">$condition</xd:ref> = 'any').</xd:li>
        <xd:li>ou une métadonnée pour chaque valeur de <xd:ref name="codes" type="parameter">$codes</xd:ref> (<xd:ref name="condition" type="parameter">$condition</xd:ref> = 'all').</xd:li>
      </xd:ul>
      <xd:p>Si l'élément ne peut pas posséder de métadonnées selon le schéma FLASH (bloc &lt;xf:metadata&gt;), renvoie systématiquement false().</xd:p>
    </xd:desc>
    <xd:param name="e">[element()] Elément (&lt;xf:*&gt;) à partir duquel les métadonnées seront interrogées.</xd:param>
    <xd:param name="codes">[xs:string+] Code(s) de la/des métadonnée(s) à tester.</xd:param>
    <xd:param name="condition">[xs:string] Condition du test (valeurs 'any' ou 'all').</xd:param>
    <xd:return>[xs:boolean?] Le résultat du test.</xd:return>
  </xd:doc>
  <xsl:function name="xf:hasMeta" as="xs:boolean?">
    <xsl:param name="e" as="element()"/>
    <xsl:param name="codes" as="xs:string+"/>
    <xsl:param name="condition" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="not($condition = ('any','all'))">
        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="$els:currentResName"/>
          <xsl:with-param name="level" select="'error'"/>
          <xsl:with-param name="code" select="'FUNCTION_paramKO'"/>
          <xsl:with-param name="markup" select="false()"/>
          <xsl:with-param name="xpathContext" select="$e"/>
          <xsl:with-param name="description">[xf:hasMeta] La valeur '<xsl:value-of select="$condition"/>' n'est pas autorisée pour le paramètre $condition.</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="not($e/self::xf:editorialEntity or $e/self::xf:structuralNode or $e/self::xf:referenceNode or $e/self::xf:queryNode or $e/self::xf:contentNode or $e/self::xf:relation or $e/self::xf:referenceItem)">
        <xsl:sequence select="false()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select="if ($condition = 'any')
                              then ($e/xf:metadata/xf:meta/@code = $codes)
                              else (every $code in $codes satisfies xf:hasMeta($e,$code,'any'))"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Signature à 2 arguments de la fonction xf:hasMeta($e as element(), $codes as xs:string+, $condition as xs:string).</xd:p>
      <xd:p>Détermine si un élément donné (<xd:ref name="e" type="parameter">$e</xd:ref>) possède au moins une métadonnée dont le code correspond à une des valeurs de <xd:ref name="codes" type="parameter">$codes</xd:ref> ($condition = 'any').</xd:p>
      <xd:p>Si l'élément ne peut pas posséder de métadonnées selon le schéma FLASH (bloc &lt;xf:metadata&gt;), renvoie systématiquement false().</xd:p>
    </xd:desc>
    <xd:param name="e">[element()] Elément (&lt;xf:*&gt;) à partir duquel les métadonnées seront interrogées.</xd:param>
    <xd:param name="codes">[xs:string+] Code(s) de la/des métadonnée(s) à tester.</xd:param>
    <xd:return>[xs:boolean?] Le résultat du test.</xd:return>
  </xd:doc>
  <xsl:function name="xf:hasMeta" as="xs:boolean?">
    <xsl:param name="e" as="element()"/>
    <xsl:param name="codes" as="xs:string+"/>
    <xsl:sequence select="xf:hasMeta($e,$codes,'any')"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>Indique si une métadonnée est de type HTML, à partir de la valeur de son attribut @as.</xd:desc>
    <xd:param name="meta">[element(xf:meta)?] La métadonnée à tester.</xd:param>
    <xd:return>[xs:boolean] Le résultat du test indiquant si la métadonnée est de type HTML.</xd:return>
  </xd:doc>
  <xsl:function name="xf:isHTMLMeta" as="xs:boolean">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <xsl:sequence select="matches($meta/@as,'^element\(html:')"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Signature à 2 arguments de la fonction xf:getMeta($e as element(), $metaCode as xs:string, $verbose as xs:boolean).</xd:p>
      <xd:p>Retourne la métadonnée (&lt;xf:meta&gt;) présente sous l'élément <xd:ref name="e" type="parameter">$e</xd:ref> et dont le code est <xd:ref name="metaCode" type="parameter">$metaCode</xd:ref>.</xd:p>
      <xd:p>En fonction de <xd:ref name="e" type="parameter">$e</xd:ref>, renvoie aussi bien une métadonnée d'editorialEntity, de referenceNode, de relation, etc.</xd:p>
      <xd:p>Par défaut, les messages d'erreur en cas de métadonnée introuvable par exemple sont désactivés.</xd:p>
    </xd:desc>
    <xd:param name="e">[element()] Elément (&lt;xf:*&gt;) à partir duquel les métadonnées seront interrogées.</xd:param>
    <xd:param name="metaCode">[xs:string] Code de la métadonnée (attribut @code de &lt;xf:meta&gt;).</xd:param>
    <xd:return>[element(xf:meta)*] La métadonnée identifiée, ou les métadonnées (si plusieurs occurrences pour un même code).</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMeta" as="element(xf:meta)*">
    <xsl:param name="e" as="element()"/>
    <xsl:param name="metaCode" as="xs:string"/>
    <xsl:sequence select="xf:getMeta($e,$metaCode,false())"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne la métadonnée (&lt;xf:meta&gt;) présente sous l'élément <xd:ref name="e" type="parameter">$e</xd:ref> et dont le code est <xd:ref name="metaCode" type="parameter">$metaCode</xd:ref>.</xd:p>
      <xd:p>En fonction de <xd:ref name="e" type="parameter">$e</xd:ref>, renvoie aussi bien une métadonnée d'editorialEntity, de referenceNode, de relation, etc.</xd:p>
      <xd:p>/!\ C'est une erreur dans les données entrantes d'avoir plusieurs occurrences d'une métadonnée avec le même code, mais en cours de traitement il peut arriver qu'on "duplique" une métadonnée (même code, sous un même élément).</xd:p>
      <xd:p>/!\ Une alerte est levée si <xd:ref name="verbose" type="parameter">$verbose</xd:ref> et que plusieurs métadonnées sont trouvées (ou aucune) ; si c'est un comportement voulu, on peut désactiver <xd:ref name="verbose" type="parameter">$verbose</xd:ref> lors de l'appel de la fonction.</xd:p>
    </xd:desc>
    <xd:param name="e">[element()] Elément (&lt;xf:*&gt;) à partir duquel les métadonnées seront interrogées.</xd:param>
    <xd:param name="metaCode">[xs:string] Code de la métadonnée (attribut @code de &lt;xf:meta&gt;).</xd:param>
    <xd:param name="verbose">[xs:boolean] Génère ou non des messages d'erreur.</xd:param>
    <xd:return>[element(xf:meta)*] La métadonnée identifiée, ou les métadonnées (si plusieurs occurrences pour un même code).</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMeta" as="element(xf:meta)*">
    <xsl:param name="e" as="element()"/>
    <xsl:param name="metaCode" as="xs:string"/>
    <xsl:param name="verbose" as="xs:boolean"/>
    <xsl:variable name="meta" select="$e/xf:metadata/xf:meta[@code = $metaCode]" as="element(xf:meta)*"/>
    <xsl:choose>
      <xsl:when test="not($e/self::xf:editorialEntity or $e/self::xf:structuralNode or $e/self::xf:referenceNode or $e/self::xf:queryNode or $e/self::xf:contentNode or $e/self::xf:relation or $e/self::xf:referenceItem)">
        <xsl:if test="$verbose = true()">
          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="$els:currentResName"/>
            <xsl:with-param name="level" select="'error'"/>
            <xsl:with-param name="code" select="'FUNCTION_paramKO'"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="xpathContext" select="$e"/>
            <xsl:with-param name="description">[xf:getMeta] Les métadonnées ne sont pas autorisées sous l'élément : <xsl:value-of select="name($e)"/>.</xsl:with-param>
          </xsl:call-template>
        </xsl:if>
        <!-- Ne renvoie rien -->
      </xsl:when>      
      <xsl:when test="count($meta) = 0">
        <xsl:if test="$verbose = true()">
          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="$els:currentResName"/>
            <xsl:with-param name="level" select="'warning'"/>
            <xsl:with-param name="code" select="'META_notFound'"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="xpathContext" select="$e"/>
            <xsl:with-param name="description">[xf:getMeta] Aucune métadonnée trouvée avec pour code : <xsl:value-of select="$metaCode"/>.</xsl:with-param>
          </xsl:call-template>
          <!-- Ne renvoie rien -->
        </xsl:if>
      </xsl:when>
      <xsl:when test="count($meta) gt 1">
        <xsl:if test="$verbose = true()">
          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="$els:currentResName"/>
            <xsl:with-param name="level" select="'warning'"/>
            <xsl:with-param name="code" select="'META_multiple'"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="xpathContext" select="$e"/>
            <xsl:with-param name="description">[xf:getMeta] <xsl:value-of select="count($meta)"/> métadonnées trouvées avec pour code : <xsl:value-of select="$metaCode"/>.</xsl:with-param>
          </xsl:call-template>
        </xsl:if>
        <!-- /!\ Renvoie plusieurs métadonnées -->
        <xsl:sequence select="$meta"/>
      </xsl:when>
      <xsl:otherwise>
        <!-- Renvoie une seule métadonnée -->
        <xsl:sequence select="$meta"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne la valeur booléenne (xf:meta[@as = 'xs:boolean']/xf:value) d'une métadonnée.</xd:p>
      <xd:p>Alerte si plusieurs xf:value booléennes sont détectées.</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(xf:meta)] La métadonnée dont la valeur booléenne doit être renvoyée.</xd:param>
    <xd:return>[xs:boolean?] La valeur booléenne de la métadonnée.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMetaBooleanValue" as="xs:boolean?">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <xsl:if test="$meta/xf:value[@as = 'xs:boolean'][2]">
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="$els:currentResName"/>
        <xsl:with-param name="level" select="'error'"/>
        <xsl:with-param name="code" select="'FUNCTION_paramKO'"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="xpathContext" select="$meta"/>
        <xsl:with-param name="description">[xf:getMetaBooleanValue] Plusieurs valeurs booléennes ont été détectées pour la métadonnée : '<xsl:value-of select="$meta/@code"/>'.</xsl:with-param>
      </xsl:call-template>
    </xsl:if>
    <!-- TODO : Mettre une alerte supplémentaire si $meta est empty ? -->
    <xsl:if test="not(empty($meta))">
      <xsl:sequence select="$meta/xf:value[@as = 'xs:boolean'][1] cast as xs:boolean"/>
    </xsl:if>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne la valeur castée en xs:dateTime (xf:meta/xf:value[@as = 'xs:dateTime']) d'une métadonnée.</xd:p>
      <xd:p>Alerte si plusieurs xf:value[@as = 'xs:dateTime'] sont détectées, ou si la valeur n'est pas une instance valide de xs:dateTime.</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(xf:meta)] La métadonnée dont la valeur castée en xs:dateTime doit être renvoyée.</xd:param>
    <xd:return>[xs:boolean?] La valeur castée en xs:dateTime de la métadonnée.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMetaDateTimeValue" as="xs:dateTime?">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <xsl:if test="$meta/xf:value[@as = 'xs:dateTime'][2]">
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="$els:currentResName"/>
        <xsl:with-param name="level" select="'error'"/>
        <xsl:with-param name="code" select="'FUNCTION_paramKO'"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="xpathContext" select="$meta"/>
        <xsl:with-param name="description">[xf:getMetaDateTimeValue] Plusieurs valeurs xs:dateTime ont été détectées pour la métadonnée : '<xsl:value-of select="$meta/@code"/>'.</xsl:with-param>
      </xsl:call-template>
    </xsl:if>
    <!-- TODO : Mettre une alerte supplémentaire si $meta est empty ? -->
    <xsl:if test="not(empty($meta))">
      <!-- Tentative de casting de la méta en xs:dateTime, erreur + empty-sequence si impossible -->
      <xsl:choose>
        <xsl:when test="$meta/xf:value[@as = 'xs:dateTime'][1] castable as xs:dateTime">
          <xsl:sequence select="$meta/xf:value[@as = 'xs:dateTime'][1] cast as xs:dateTime"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="$els:currentResName"/>
            <xsl:with-param name="level" select="'error'"/>
            <xsl:with-param name="code" select="'FUNCTION_paramKO'"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="xpathContext" select="$meta"/>
            <xsl:with-param name="description">[xf:getMetaDateTimeValue] La valeur de la métadonnée '<xsl:value-of select="$meta/@code"/>' n'est pas une instance valide de xs:dateTime.</xsl:with-param>
          </xsl:call-template>
          <!-- Ne retourne rien -->
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne la/les "valeur(s) textuelle(s)" d'une métadonnée (cf. xf:meta/xf:value).</xd:p>
      <xd:p>Renvoie un ou plusieurs xs:string* (après normalize-space()) correspondant à tous les enfants &lt;xf:value&gt;.</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(xf:meta)] La métadonnée dont la "valeur textuelle" doit être calculée.</xd:param>
    <xd:return>[xs:string*] Un ou plusieurs xs:string (si la métadonnée est multivaluée).</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMetaValue" as="xs:string*">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <xsl:sequence select="xf:getMetaContent($meta,'value')/normalize-space(.)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne le contenu des enfants &lt;xf:value&gt; d'une métadonnée :</xd:p>
      <xd:ul>
        <xd:li>si le contenu est une data simple (ex. : xs:string), renvoie celui-ci après normalize-space() ;</xd:li>
        <xd:li>si le contenu est de type xf:* ou element(), renvoie les éléments enfants ;</xd:li>
        <xd:li>sinon, renvoi le contenu inchangé.</xd:li>
      </xd:ul>
      <xd:p>Se base sur la présence d'un attribut @as sur xf:value, ou déduit le traitement à appliquer du contenu.</xd:p>
      <xd:p>Fait appel à la fonction xf:normalizeContentInMeta($elt as element()*).</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(xf:meta)] La métadonnée dont le contenu des xf:value doit être restitué.</xd:param>
    <xd:return>[item()*] Le contenu des xf:value de la métadonnée.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMetaValueContent" as="item()*">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <!-- Récupération du (ou des) enfant(s) xf:value de la métadonnée -->
    <xsl:variable name="value" select="xf:getMetaContent($meta,'value')" as="element(xf:value)*"/>
    <!-- Normalisation des valeurs textuelles, contenu brut sinon -->
    <xsl:sequence select="xf:normalizeContentInMeta($value)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne la/les "valeur(s) textuelle(s)" de la verbalisation d'une métadonnée référentielle (cf. xf:meta/xf:verbalization).</xd:p>
      <xd:p>Renvoie un ou plusieurs xs:string* (après normalize-space()) correspondant à tous les enfants &lt;xf:verbalization&gt;.</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(xf:meta)] La métadonnée référentielle dont la "valeur textuelle" de la verbalisation doit être calculée.</xd:param>
    <xd:return>[xs:string*] Un ou plusieurs xs:string (si la métadonnée est multivaluée).</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMetaVerbalization" as="xs:string*">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <xsl:sequence select="xf:getMetaContent($meta,'verbalization')/normalize-space(.)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne le/les ID(s) pointant vers une relation (xf:meta/xf:value/xf:ref[@xf:refType = 'xf:relationRef']/@xf:idRef) d'une métadonnée.</xd:p>
      <xd:p>Renvoie un ou plusieurs xs:string* (après normalize-space()) correspondant à toutes les valeurs des attributs xf:value/xf:ref/@xf:idRef.</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(xf:meta)] La métadonnée dont l'ID de relation doit être calculé.</xd:param>
    <xd:return>[xs:string*] Un ou plusieurs xs:string (si la métadonnée est multivaluée).</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMetaRelationRef" as="xs:string*">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <xsl:sequence select="xf:getMetaContent($meta,'relationRef')/@xf:idRef"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne le/les ID(s) de référentiel(s) (xf:meta/xf:value/xf:ref[@xf:refType = 'xf:referenceItemRef']/@idRef) d'une métadonnée.</xd:p>
      <xd:p>Renvoie un ou plusieurs xs:string* (après normalize-space()) correspondant à toutes les valeurs des attributs xf:value/xf:ref/@idRef.</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(xf:meta)] La métadonnée dont l'ID de référentiel doit être calculé.</xd:param>
    <xd:return>[xs:string*] Un ou plusieurs xs:string (si la métadonnée est multivaluée).</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMetaReferenceItemRef" as="xs:string*">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <xsl:sequence select="xf:getMetaContent($meta,'referenceItemRef')/@idRef"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne un contenu de métadonnée. En fonction de <xd:ref name="type" type="parameter">$type</xd:ref>, récupère :</xd:p>
      <xd:ul>
        <xd:li>tous les enfants xf:value ;</xd:li>
        <xd:li>tous les enfants xf:verbalization ;</xd:li>
        <xd:li>tous les enfants xf:value/xf:ref (entrées de référentiels, relations, etc.).</xd:li>
      </xd:ul>
      <xd:p>Renvoie un ou plusieurs item()*.</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(xf:meta)] La métadonnée dont le contenu doit être calculé.</xd:param>
    <xd:param name="type">[xs:string] Le type de contenu à récupérer.</xd:param>
    <xd:return>[item()*] Le contenu de la métadonnée (plusieurs item() si celle-ci est multivaluée).</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMetaContent" as="item()*">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <xsl:param name="type" as="xs:string"/>
    <xsl:variable name="tmp.sequence" as="item()*">
      <!-- Possibilité d'adapter la manière de récupérer le contenu en fonction de $type -->
      <xsl:choose>
        <xsl:when test="$type = 'value'">
          <xsl:sequence select="$meta/xf:value"/>
        </xsl:when>
        <xsl:when test="$type = 'verbalization'">
          <!-- TODO : Est-ce que cela a réellement une utilité ? -->
          <xsl:sequence select="$meta/xf:verbalization"/>
        </xsl:when>
        <xsl:when test="$type = 'referenceItemRef'">
          <xsl:sequence select="$meta/xf:value/xf:ref[@xf:refType = 'xf:referenceItemRef']"/>
        </xsl:when>
        <xsl:when test="$type = 'relationRef'">
          <!-- TODO : La valeur 'xf:relation' est erronée - à corriger dans SAS + ECM -->
          <xsl:sequence select="$meta/xf:value/xf:ref[@xf:refType = ('xf:relationRef','xf:relation')]"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="$els:currentResName"/>
            <xsl:with-param name="level" select="'error'"/>
            <xsl:with-param name="code" select="'FUNCTION_paramKO'"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="xpathContext" select="$meta"/>
            <xsl:with-param name="description">[xf:getMetaContent] Valeur inconnue pour le paramètre $type : <xsl:value-of select="$type"/>.</xsl:with-param>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="count($tmp.sequence) &gt; 1">
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="$els:currentResName"/>
        <xsl:with-param name="level" select="'debug'"/>
        <xsl:with-param name="code" select="'FUNCTION_genericMessage'"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="xpathContext" select="$meta"/>
        <xsl:with-param name="description">[xf:getMetaContent] La métadonnée '<xsl:value-of select="$meta/@code"/>' est multivaluée.</xsl:with-param>
      </xsl:call-template>
    </xsl:if>
    <xsl:sequence select="$tmp.sequence"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne un / plusieurs éléments xf:value d'une métadonnée donnée (<xd:ref name="meta" type="parameter">$meta</xd:ref>), s'ils possèdent un enfant xf:field dont le nom est <xd:ref name="fieldName" type="parameter">$fieldName</xd:ref> et la valeur est <xd:ref name="fieldValue" type="parameter">$fieldValue</xd:ref>.</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(xf:meta)?] Une métadonnée.</xd:param>
    <xd:param name="fieldName">[xs:string] Le nom du champ (xf:field/@name) souhaité.</xd:param>
    <xd:param name="fieldValue">[xs:string] La valeur attendue pour le champ (contenu textuel de xf:field).</xd:param>
    <xd:return>[element(xf:value)*] Le ou les éléments xf:value qui contiennent le champ recherché.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMetaContentFromField" as="element(xf:value)*">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <xsl:param name="fieldName" as="xs:string"/>
    <xsl:param name="fieldValue" as="xs:string"/>
    <xsl:sequence select="xf:getMetaContent($meta,'value')[matches(@as,'^xf:field')][xf:hasField(.,$fieldName,$fieldValue)]"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne un / plusieurs éléments xf:value d'une métadonnée donnée (<xd:ref name="meta" type="parameter">$meta</xd:ref>), s'ils possèdent un enfant xf:field dont le nom est <xd:ref name="fieldName" type="parameter">$fieldName</xd:ref>.</xd:p>
    </xd:desc>
    <xd:param name="meta">[element(xf:meta)?] Une métadonnée.</xd:param>
    <xd:param name="fieldName">[xs:string] Le nom du champ (xf:field/@name) souhaité.</xd:param>
    <xd:return>[element(xf:value)*] Le ou les éléments xf:value qui contiennent le champ recherché.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMetaContentFromField" as="element(xf:value)*">
    <xsl:param name="meta" as="element(xf:meta)?"/>
    <xsl:param name="fieldName" as="xs:string"/>
    <xsl:sequence select="xf:getMetaContent($meta,'value')[matches(@as,'^xf:field')][xf:hasField(.,$fieldName)]"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne l'élément xf:field d'un xf:value donné, dont l'attribut @name correspond à la valeur de <xd:ref name="fieldName" type="parameter">$fieldName</xd:ref>.</xd:p>
    </xd:desc>
    <xd:param name="elt">[element(xf:value)?] Un élément xf:value (enfant de xf:meta, typiquement).</xd:param>
    <xd:param name="fieldName">[xs:string] Le nom du champ (xf:field/@name).</xd:param>
    <xd:return>[element(xf:field)] L'enfant xf:field correspondant.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getField" as="element(xf:field)?">
    <xsl:param name="elt" as="element(xf:value)?"/>
    <xsl:param name="fieldName" as="xs:string"/>
    <xsl:variable name="field" select="$elt/xf:field[@name = $fieldName]" as="element(xf:field)*"/>
    <!-- Erreur si plusieurs xf:field identifiés -->
    <xsl:if test="count($field) &gt; 1">
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="$els:currentResName"/>
        <xsl:with-param name="level" select="'error'"/>
        <xsl:with-param name="code" select="'mutipleFieldsFound'"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="xpathContext" select="$elt"/>
        <xsl:with-param name="description">[xf:getField] Plusieurs éléments xf:field ont été trouvés avec pour nom : "<xsl:value-of select="$fieldName"/>" ; seul le 1er sera retourné.</xsl:with-param>
      </xsl:call-template>
    </xsl:if>
    <xsl:sequence select="$field[1]"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne le contenu de l'élément xf:field d'un xf:value donné, dont l'attribut @name correspond à la valeur de <xd:ref name="fieldName" type="parameter">$fieldName</xd:ref>.</xd:p>
    </xd:desc>
    <xd:param name="elt">[element(xf:value)?] Un élément xf:value (enfant de xf:meta, typiquement).</xd:param>
    <xd:param name="fieldName">[xs:string] Le nom du champ (xf:field/@name).</xd:param>
    <xd:return>[item()*] Le contenu (noeud HTML, data, etc.) de l'enfant xf:field correspondant.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getFieldContent" as="item()*">
    <xsl:param name="elt" as="element(xf:value)?"/>
    <xsl:param name="fieldName" as="xs:string"/>
    <!-- Obtient l'élément xf:field et retourne son contenu normalisé
         /!\ Peut être un élément HTML ou du data -->
    <xsl:sequence select="xf:getField($elt,$fieldName)/xf:normalizeContentInMeta(.)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Détermine si un élément xf:value donné possède un enfant xf:field dont l'attribut @name correspond à la valeur de <xd:ref name="fieldName" type="parameter">$fieldName</xd:ref>.</xd:p>
    </xd:desc>
    <xd:param name="elt">[element(xf:value)?] Un élément xf:value (enfant de xf:meta, typiquement).</xd:param>
    <xd:param name="fieldName">[xs:string] Le nom du champ (xf:field/@name).</xd:param>
    <xd:return>[xs:boolean] Le résultat du test déterminant la présence de l'enfant xf:field.</xd:return>
  </xd:doc>
  <xsl:function name="xf:hasField" as="xs:boolean">
    <xsl:param name="elt" as="element(xf:value)?"/>
    <xsl:param name="fieldName" as="xs:string"/>
    <xsl:sequence select="$elt/xf:field/@name = $fieldName"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Détermine si un élément xf:value donné possède un enfant xf:field dont le nom est <xd:ref name="fieldName" type="parameter">$fieldName</xd:ref> et la valeur est <xd:ref name="fieldValue" type="parameter">$fieldValue</xd:ref>.</xd:p>
    </xd:desc>
    <xd:param name="elt">[element(xf:value)?] Un élément xf:value (enfant de xf:meta, typiquement).</xd:param>
    <xd:param name="fieldName">[xs:string] Le nom du champ (xf:field/@name).</xd:param>
    <xd:param name="fieldValue">[xs:string] La valeur attendue pour le champ (contenu textuel de xf:field).</xd:param>
    <xd:return>[xs:boolean] Le résultat du test déterminant la présence de l'enfant xf:field.</xd:return>
  </xd:doc>
  <xsl:function name="xf:hasField" as="xs:boolean">
    <xsl:param name="elt" as="element(xf:value)?"/>
    <xsl:param name="fieldName" as="xs:string"/>
    <xsl:param name="fieldValue" as="xs:string"/>
    <xsl:sequence select="$elt/xf:field[@name = $fieldName] = $fieldValue"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne le contenu d'un ou plusieurs éléments &lt;xf:value&gt; / &lt;xf:field&gt; d'une métadonnée normalisé, selon les règles :</xd:p>
      <xd:ul>
        <xd:li>si le contenu est une data simple (ex. : xs:string), renvoie celui-ci après normalize-space() ;</xd:li>
        <xd:li>si le contenu est de type xf:* ou element(), renvoie les éléments enfants ;</xd:li>
        <xd:li>sinon, renvoi le contenu inchangé.</xd:li>
      </xd:ul>
      <xd:p>Se base sur la présence d'un attribut @as sur xf:value, ou déduit le traitement à appliquer du contenu.</xd:p>
    </xd:desc>
    <xd:param name="elt">[element()*] Le(s) élément(s) &lt;xf:value&gt; / &lt;xf:field&gt; dont le contenu doit être retourné normalisé.</xd:param>
    <xd:return>[item()*] Le contenu normalisé.</xd:return>
  </xd:doc>
  <xsl:function name="xf:normalizeContentInMeta" as="item()*">
    <xsl:param name="elt" as="element()*"/> <!-- xf:value ou xf:field -->
    <xsl:for-each select="$elt">
      <xsl:variable name="as" select="@as" as="xs:string?"/>
      <xsl:choose>
        <!-- Traitement de normalisation induit par @as -->
        <xsl:when test="$as">
          <xsl:sequence select="if (matches($as,'^xs:')) then (normalize-space(.))
                                else if (matches($as,'^(xf:|element\()')) then (./*)
                                else (.)"/>
        </xsl:when>
        <!-- Traitement de normalisation déduit du contenu -->
        <xsl:otherwise>
          <xsl:sequence select="if (.[*]) then (./*) else (normalize-space(.))"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne la verbalisation (xf:verbalization) associée à un élément xf:ref donné (<xd:ref name="ref" type="parameter">$ref</xd:ref>).</xd:p>
      <xd:p>/!\ Retourne aussi bien une verbalisation liées à un xf:ref dans une métadonnée ou dans un xf:referenceNode.</xd:p>
    </xd:desc>
    <xd:param name="ref">[element(xf:ref)] L'élément xf:ref dont la verbalisation doit être retrounée.</xd:param>
    <xd:return>[element(xf:verbalization)?] L'élément de verbalisation associé au xf:ref (s'il existe).</xd:return>
  </xd:doc>
  <xsl:function name="xf:getRefVerbalization" as="element(xf:verbalization)?">
    <xsl:param name="ref" as="element(xf:ref)?"/>
    <xsl:sequence select="($ref/parent::xf:value/following-sibling::*[1][self::xf:verbalization],
                           $ref[parent::xf:referenceNode]/following-sibling::*[1][self::xf:verbalization])[1]"/>
  </xsl:function>
  
  
  <!--+==================================================+
      | FONCTIONS / Manipulation de référentiels FLASH   |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne une entrée de référentiel.</xd:p>
      <xd:p>Interroge le référentiel FLASH <xd:ref name="refXML" type="parameter">$refXML</xd:ref> et retourne l'entrée dont l'ID est <xd:ref name="entryId" type="parameter">$entryId</xd:ref>.</xd:p>
    </xd:desc>
    <xd:param name="refXML">[element(xf:referenceTable)] Le référentiel XML (format FLASH).</xd:param>
    <xd:param name="entryId">[xs:string] L'ID de l'entrée de référentiel.</xd:param>
    <xd:return>[element(xf:referenceItem)?] L'entrée de référentiel, si elle a été trouvée.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getRefEntry" as="element(xf:referenceItem)?">
    <xsl:param name="refXML" as="element(xf:referenceTable)"/>
    <xsl:param name="entryId" as="xs:string"/>
    <!-- Obtention de l'entrée de référentiel -->
    <!-- TODO : A améliorer ? Utilisation de @archived, alerte si l'entrée n'est pas trouvée, etc. -->
    <xsl:sequence select="$refXML//xf:referenceItem[@id = $entryId]"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne une métadonnée liée à une entrée de référentiel.</xd:p>
      <xd:p>Retourne la métadonnée dont le code est <xd:ref name="code" type="parameter">$code</xd:ref> pour l'entrée <xd:ref name="refEntry" type="parameter">$refEntry</xd:ref>.</xd:p>
    </xd:desc>
    <xd:param name="refEntry">[element(xf:referenceItem)?] Une entrée de référentiel (format FLASH).</xd:param>
    <xd:param name="code">[xs:string] Le code de la métadonnée à retourner (exemples : "libelleComplet", "libelleAbrege", "CXML_Id", etc.).</xd:param>
    <xd:return>[element(xf:meta)?] La métadonnée, si elle a été trouvée.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getRefEntryMeta" as="element(xf:meta)?">
    <xsl:param name="refEntry" as="element(xf:referenceItem)?"/>
    <xsl:param name="code" as="xs:string"/>
    <xsl:if test="$refEntry">
      <!-- Obtention de la métadonnée correspondant à $code (si elle existe) -->
      <xsl:sequence select="xf:getMeta($refEntry,$code)"/>
    </xsl:if>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Signature à 3 arguments de la fonction xf:getRefEntryMeta($refEntry as element(xf:referenceItem), $code as xs:string)</xd:p>
      <xd:p>Retourne une métadonnée liée à une entrée de référentiel.</xd:p>
      <xd:p>Interroge le référentiel FLASH <xd:ref name="refXML" type="parameter">$refXML</xd:ref> et retourne la métadonnée dont le code est <xd:ref name="code" type="parameter">$code</xd:ref> pour l'entrée <xd:ref name="entryId" type="parameter">$entryId</xd:ref>.</xd:p>
    </xd:desc>
    <xd:param name="refXML">[element(xf:referenceTable)] Le référentiel XML (format FLASH).</xd:param>
    <xd:param name="entryId">[xs:string] L'ID de l'entrée de référentiel.</xd:param>
    <xd:param name="code">[xs:string] Le code de la métadonnée à retourner (exemples : "libelleComplet", "libelleAbrege", "CXML_Id", etc.).</xd:param>
    <xd:return>[element(xf:meta)?] La métadonnée, si elle a été trouvée.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getRefEntryMeta" as="element(xf:meta)?">
    <xsl:param name="refXML" as="element(xf:referenceTable)"/>
    <xsl:param name="entryId" as="xs:string"/>
    <xsl:param name="code" as="xs:string"/>
    <!-- Obtention de l'entrée de référentiel -->
    <xsl:variable name="referenceItem" select="xf:getRefEntry($refXML,$entryId)" as="element(xf:referenceItem)?"/>
    <!-- Obtention de la métadonnée correspondant à $code (si elle existe) -->
    <xsl:sequence select="xf:getRefEntryMeta($referenceItem,$code)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne le contenu d'une métadonnée liée à une entrée de référentiel.</xd:p>
      <xd:p>Retourne le contenu de la métadonnée dont le code est <xd:ref name="code" type="parameter">$code</xd:ref> pour l'entrée <xd:ref name="refEntry" type="parameter">$refEntry</xd:ref>.</xd:p>
      <xd:p>Fait appel à la fonction xf:getMetaContent($meta as element(xf:meta?), $type as xs:string) as item()*.</xd:p>
    </xd:desc>
    <xd:param name="refEntry">[element(xf:referenceItem)?] Une entrée de référentiel (format FLASH).</xd:param>
    <xd:param name="code">[xs:string] Le code de la métadonnée à retourner (exemples : "libelleComplet", "libelleAbrege", "CXML_Id", etc.).</xd:param>
    <xd:param name="type">[xs:string] Le type de contenu à récupérer.</xd:param>
    <xd:return>[item()*] Le contenu de la métadonnée, si elle a été trouvée.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getRefEntryMetaContent" as="item()*">
    <xsl:param name="refEntry" as="element(xf:referenceItem)?"/>
    <xsl:param name="code" as="xs:string"/>
    <xsl:param name="type" as="xs:string"/>
    <!-- Obtention de la métadonnée correspondant à $code (si elle existe) -->
    <xsl:variable name="referenceItemMeta" select="xf:getRefEntryMeta($refEntry,$code)" as="element(xf:meta)?"/>
    <!-- Récupération du contenu de la métadonnée (si elle existe et selon $type) -->
    <xsl:sequence select="xf:getMetaContent($referenceItemMeta,$type)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Signature à 4 arguments de la fonction xf:getRefEntryMetaValue($refEntry as element(xf:referenceItem), $code as xs:string)</xd:p>
      <xd:p>Retourne la valeur textuelle d'une métadonnée liée à une entrée de référentiel.</xd:p>
      <xd:p>Interroge le référentiel FLASH <xd:ref name="refXML" type="parameter">$refXML</xd:ref> et retourne la valeur de la métadonnée dont le code est <xd:ref name="code" type="parameter">$code</xd:ref> pour l'entrée <xd:ref name="entryId" type="parameter">$entryId</xd:ref>.</xd:p>
      <xd:p>Fait appel à la fonction xf:getMetaContent($meta as element(xf:meta?), $type as xs:string) as item()*.</xd:p>
    </xd:desc>
    <xd:param name="refXML">[element(xf:referenceTable)] Le référentiel XML (format FLASH).</xd:param>
    <xd:param name="entryId">[xs:string] L'ID de l'entrée de référentiel.</xd:param>
    <xd:param name="code">[xs:string] Le code de la métadonnée à retourner (exemples : "libelleComplet", "libelleAbrege", "CXML_Id", etc.).</xd:param>
    <xd:param name="type">[xs:string] Le type de contenu à récupérer.</xd:param>
    <xd:return>[item()*] Le contenu de la métadonnée, si elle a été trouvée.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getRefEntryMetaContent" as="item()*">
    <xsl:param name="refXML" as="element(xf:referenceTable)"/>
    <xsl:param name="entryId" as="xs:string"/>
    <xsl:param name="code" as="xs:string"/>
    <xsl:param name="type" as="xs:string"/>
    <!-- Obtention de l'entrée de référentiel -->
    <xsl:variable name="referenceItem" select="xf:getRefEntry($refXML,$entryId)" as="element(xf:referenceItem)?"/>
    <!-- Récupération du contenu de la métadonnée (si elle existe et selon $type) -->
    <xsl:sequence select="xf:getRefEntryMetaContent($referenceItem,$code,$type)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne la valeur textuelle d'une métadonnée liée à une entrée de référentiel.</xd:p>
      <xd:p>Retourne la valeur de la métadonnée dont le code est <xd:ref name="code" type="parameter">$code</xd:ref> pour l'entrée <xd:ref name="refEntry" type="parameter">$refEntry</xd:ref>.</xd:p>
    </xd:desc>
    <xd:param name="refEntry">[element(xf:referenceItem)?] Une entrée de référentiel (format FLASH).</xd:param>
    <xd:param name="code">[xs:string] Le code de la métadonnée à retourner (exemples : "libelleComplet", "libelleAbrege", "CXML_Id", etc.).</xd:param>
    <xd:return>[xs:string*] La valeur (textuelle) de la métadonnée, si elle a été trouvée.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getRefEntryMetaValue" as="xs:string*">
    <xsl:param name="refEntry" as="element(xf:referenceItem)?"/>
    <xsl:param name="code" as="xs:string"/>
    <!-- Obtention de la métadonnée correspondant à $code (si elle existe) -->
    <xsl:variable name="referenceItemMeta" select="xf:getRefEntryMeta($refEntry,$code)" as="element(xf:meta)?"/>
    <!-- Récupération de la valeur de la métadonnée (si elle existe) -->
    <xsl:sequence select="xf:getMetaValue($referenceItemMeta)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Signature à 3 arguments de la fonction xf:getRefEntryMetaValue($refEntry as element(xf:referenceItem), $code as xs:string)</xd:p>
      <xd:p>Retourne la valeur textuelle d'une métadonnée liée à une entrée de référentiel.</xd:p>
      <xd:p>Interroge le référentiel FLASH <xd:ref name="refXML" type="parameter">$refXML</xd:ref> et retourne la valeur de la métadonnée dont le code est <xd:ref name="code" type="parameter">$code</xd:ref> pour l'entrée <xd:ref name="entryId" type="parameter">$entryId</xd:ref>.</xd:p>
    </xd:desc>
    <xd:param name="refXML">[element(xf:referenceTable)] Le référentiel XML (format FLASH).</xd:param>
    <xd:param name="entryId">[xs:string] L'ID de l'entrée de référentiel.</xd:param>
    <xd:param name="code">[xs:string] Le code de la métadonnée à retourner (exemples : "libelleComplet", "libelleAbrege", "CXML_Id", etc.).</xd:param>
    <xd:return>[xs:string*] La valeur (textuelle) de la métadonnée, si elle a été trouvée.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getRefEntryMetaValue" as="xs:string*">
    <xsl:param name="refXML" as="element(xf:referenceTable)"/>
    <xsl:param name="entryId" as="xs:string"/>
    <xsl:param name="code" as="xs:string"/>
    <!-- Obtention de l'entrée de référentiel -->
    <xsl:variable name="referenceItem" select="xf:getRefEntry($refXML,$entryId)" as="element(xf:referenceItem)?"/>
    <!-- Récupération de la valeur de la métadonnée (si elle existe) -->
    <xsl:sequence select="xf:getRefEntryMetaValue($referenceItem,$code)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne le contenu de l'enfant &lt;xf:value&gt; d'une métadonnée liée à une entrée de référentiel.</xd:p>
      <xd:p>Retrouve la métadonnée dont le code est <xd:ref name="code" type="parameter">$code</xd:ref> pour l'entrée <xd:ref name="refEntry" type="parameter">$refEntry</xd:ref>.</xd:p>
      <xd:p>Fait appel à la fonction xf:getMetaValueContent($meta as element(xf:meta?)) as item()*.</xd:p>
    </xd:desc>
    <xd:param name="refEntry">[element(xf:referenceItem)?] Une entrée de référentiel (format FLASH).</xd:param>
    <xd:param name="code">[xs:string] Le code de la métadonnée à retourner (exemples : "libelleComplet", "libelleAbrege", "CXML_Id", etc.).</xd:param>
    <xd:return>[item()*] Le contenu de l'élément &lt;xf:value&gt; de la métadonnée, si elle a été trouvée.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getRefEntryMetaValueContent" as="item()*">
    <xsl:param name="refEntry" as="element(xf:referenceItem)?"/>
    <xsl:param name="code" as="xs:string"/>
    <!-- Obtention de la métadonnée correspondant à $code (si elle existe) -->
    <xsl:variable name="referenceItemMeta" select="xf:getRefEntryMeta($refEntry,$code)" as="element(xf:meta)?"/>
    <!-- Récupération de la valeur de la métadonnée (si elle existe) -->
    <xsl:sequence select="xf:getMetaValueContent($referenceItemMeta)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Signature à 3 arguments de la fonction xf:getRefEntryMetaValueContent($refEntry as element(xf:referenceItem), $code as xs:string)</xd:p>
      <xd:p>Retourne le contenu de l'enfant &lt;xf:value&gt; d'une métadonnée liée à une entrée de référentiel.</xd:p>
      <xd:p>Interroge le référentiel FLASH <xd:ref name="refXML" type="parameter">$refXML</xd:ref> et retrouve la métadonnée dont le code est <xd:ref name="code" type="parameter">$code</xd:ref> pour l'entrée <xd:ref name="entryId" type="parameter">$entryId</xd:ref>.</xd:p>
    </xd:desc>
    <xd:param name="refXML">[element(xf:referenceTable)] Le référentiel XML (format FLASH).</xd:param>
    <xd:param name="entryId">[xs:string] L'ID de l'entrée de référentiel.</xd:param>
    <xd:param name="code">[xs:string] Le code de la métadonnée à retourner (exemples : "libelleComplet", "libelleAbrege", "CXML_Id", etc.).</xd:param>
    <xd:return>[item()*] Le contenu de l'élément &lt;xf:value&gt; de la métadonnée, si elle a été trouvée.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getRefEntryMetaValueContent" as="item()*">
    <xsl:param name="refXML" as="element(xf:referenceTable)"/>
    <xsl:param name="entryId" as="xs:string"/>
    <xsl:param name="code" as="xs:string"/>
    <!-- Obtention de l'entrée de référentiel -->
    <xsl:variable name="referenceItem" select="xf:getRefEntry($refXML,$entryId)" as="element(xf:referenceItem)?"/>
    <!-- Récupération du contenu de la métadonnée (si elle existe) -->
    <xsl:sequence select="xf:getRefEntryMetaValueContent($referenceItem,$code)"/>
  </xsl:function>
  
  
  <!--+==================================================+
      | FONCTIONS / Fonctions liées aux ID / liens FLASH |
      | -> Normalisation des ID cibles des liens         |
      | -> Test de la résolution des liens               |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>
      <xd:p>Fonction utilitaire qui retourne une représentation unifiée d'un ID cible de lien FLASH (ou ID/IDREF de ciblage).</xd:p>
      <xd:p>Prend en entrée un élément de référence (xf:ref) et concatène les attributs @xf:targetResId et @xf:idRef.</xd:p>
      <xd:p>Le passage par cette fonction permet de comparer facilement des liens lorsque les deux attributs de ciblage sont utilisés, de créer des clés basées sur les cibles, etc.</xd:p>
    </xd:desc>
    <xd:param name="ref">[element(xf:ref)?] Un élément de référence xf:ref.</xd:param>
    <xd:return>[xs:string?] L'IDREF de ciblage, concaténation de @xf:targetResId et @xf:idRef.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getFlashTargetIdRef" as="xs:string">
    <xsl:param name="ref" as="element(xf:ref)?"/>
    <xsl:sequence select="xf:makeTargetId($ref/@xf:targetResId,$ref/@xf:idRef)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Fonction utilitaire qui retourne une représentation unifiée d'un ID cible de lien NON-FLASH (ou ID/IDREF de ciblage).</xd:p>
      <xd:p>Prend en entrée un élément de référence (xf:ref) et concatène les attributs @xf:targetResId et @idRef.</xd:p>
      <xd:p>Le passage par cette fonction permet de comparer facilement des liens lorsque les deux attributs de ciblage sont utilisés, de créer des clés basées sur les cibles, etc.</xd:p>
    </xd:desc>
    <xd:param name="ref">[element(xf:ref)?] Un élément de référence xf:ref.</xd:param>
    <xd:return>[xs:string?] L'IDREF de ciblage, concaténation de @xf:targetResId et @idRef.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getNonFlashTargetIdRef" as="xs:string">
    <xsl:param name="ref" as="element(xf:ref)?"/>
    <xsl:sequence select="xf:makeTargetId($ref/@xf:targetResId,$ref/@idRef)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Fonction utilitaire qui retourne une représentation unifiée d'un ID cible de lien FLASH ou NON-FLASH (ou ID/IDREF de ciblage).</xd:p>
      <xd:p>Prend en entrée un élément de référence (xf:ref) et concatène les attributs @xf:targetResId et @xf:idRef/@idRef.</xd:p>
      <xd:p>Le passage par cette fonction permet de comparer facilement des liens lorsque les deux attributs de ciblage sont utilisés, de créer des clés basées sur les cibles, etc.</xd:p>
    </xd:desc>
    <xd:param name="ref">[element(xf:ref)?] Un élément de référence xf:ref.</xd:param>
    <xd:return>[xs:string?] L'IDREF de ciblage, concaténation de @xf:targetResId et @xf:idRef/@idRef.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getTargetIdRef" as="xs:string">
    <xsl:param name="ref" as="element(xf:ref)?"/>
    <xsl:sequence select="xf:makeTargetId($ref/@xf:targetResId,($ref/@xf:idRef,$ref/@idRef)[1])"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Fonction utilitaire, qui concatène deux valeurs dans le but d'obtenir une représentation unifiée d'un ID cible (ou ID de ciblage).</xd:p>
      <xd:p>Les deux arguments sont typiquement des valeurs d'attributs xf:ref/@xf:targetResId / xf:ref/@xf:idRef (ou @idRef).</xd:p>
    </xd:desc>
    <xd:param name="targetResId">[xs:string?] Un ID de base, correspondant à un @xf:targetResId.</xd:param>
    <xd:param name="idRef">[xs:string?] Un ID à concaténer, correspondant à un @xf:idRef/@idRef.</xd:param>
    <xd:return>[xs:string?] L'ID de ciblage (ou une séquence vide, si les deux arguments sont des séquences vides).</xd:return>
  </xd:doc>
  <xsl:function name="xf:makeTargetId" as="xs:string">
    <xsl:param name="targetResId" as="xs:string?"/>
    <xsl:param name="idRef" as="xs:string?"/>
    <xsl:sequence select="string-join(($targetResId,$idRef),'/')"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Calcule "l'ID de ciblage" d'un élément donné, obtenu à partir de l'attribut @xf:id de l'EE parente et de l'attribut @xf:id ou @id de l'élément courant.</xd:p>
      <xd:p>Cet ID calculé permettra de déterminer si l'élément est la cible d'un lien (xf:ref -> @xf:targetResId + @xf:idRef/@idRef).</xd:p>
    </xd:desc>
    <xd:param name="elt">[element()] L'élément courant, dont on souhaite déterminer l'ID de ciblage.</xd:param>
    <xd:return>[xs:string] L'ID de ciblage calculé de l'élément.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getElementTargetId" as="xs:string">
    <xsl:param name="elt" as="element()"/>
    <!-- ID de l'EE -->
    <xsl:variable name="targetResId" select="$elt/ancestor-or-self::xf:editorialEntity[1]/@xf:id" as="xs:string?"/>
    <!-- ID de l'élément (s'il n'est pas une EE) - generate-id() si pas de valeur, pour pouvoir utiliser une indexation via des keys (-> type 'getElementByTargetId') -->
    <xsl:variable name="id" select="$elt[not(self::xf:editorialEntity)]/(@xf:id,@id,generate-id())[1]" as="xs:string?"/>
    <!-- Obtention de l'ID de ciblage de l'élément -->
    <xsl:sequence select="xf:makeTargetId($targetResId,$id)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne la partie correspondant au @xf:targetResId depuis un ID de ciblage.</xd:p>
    </xd:desc>
    <xd:param name="targetId">[xs:string] L'ID de ciblage complet (@xf:targetResId + @xf:idRef ou @idRef).</xd:param>
    <xd:return>[xs:string] La partie de <xd:ref name="targetId" type="parameter">$targetId</xd:ref> correspondant à un attribut de ciblage @xf:targetResId.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getTargetResIdFromTargetId" as="xs:string?">
    <xsl:param name="targetId" as="xs:string"/>
    <xsl:sequence select="tokenize($targetId,'/')[1]"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne la partie correspondant au @xf:idRef / @idRef depuis un ID de ciblage.</xd:p>
    </xd:desc>
    <xd:param name="targetId">[xs:string] L'ID de ciblage complet (@xf:targetResId + @xf:idRef ou @idRef).</xd:param>
    <xd:return>[xs:string] La partie de <xd:ref name="targetId" type="parameter">$targetId</xd:ref> correspondant à un attribut de ciblage @xf:idRef / @idRef.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getIdFromTargetId" as="xs:string?">
    <xsl:param name="targetId" as="xs:string"/>
    <xsl:sequence select="tokenize($targetId,'/')[2]"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Détermine si un lien (un élément lié à une relation pointant vers une cible interne/externe - ex. : renvoiAutrui, refDoc, renvoi, etc.) est résolu.</xd:p>
      <xd:p>Si c'est le cas, les attributs @xf:targetResId, @xf:idRef/@idRef de sa relation (portés par xf:ref) contiennent des IDs.</xd:p>
      <xd:p>Si ce n'est pas le cas, ils contiennent une query, sous la forme "{query:...}".</xd:p>
      <xd:p>TODO : Fonction à nettoyer quand l'ECM saura gérer les queries.</xd:p>
    </xd:desc>
    <xd:param name="e">[element()] L'élément de lien à tester.</xd:param>
    <xd:return>[xs:boolean] Le résultat du test de résolution du lien.</xd:return>
  </xd:doc>
  <xsl:function name="xf:isResolvedLink" as="xs:boolean">
    <xsl:param name="e" as="element()"/>
    <xsl:variable name="top" select="$e/ancestor::xf:editorialEntity[1]" as="element(xf:editorialEntity)"/>
    <!-- Relation (éventuelle) associée à l'élément de lien -->
    <xsl:variable name="relation" select="key('xf:getRelationById',$e/@xf:idRef,$top)" as="element(xf:relation)?"/>
    <xsl:choose>
      <!-- Pas de relation associée
           TODO : A supprimer ? Pas un cas réel ? -->
      <xsl:when test="not($relation)">
        <xsl:sequence select="false()"/>
      </xsl:when>
      <!-- Les attributs de xf:ref sont vides
           TODO : A supprimer ? Pas un cas réel ? -->
      <xsl:when test="xf:getTargetIdRef($relation/xf:ref) = ('','/')">
        <xsl:sequence select="false()"/>
      </xsl:when>
      <!-- xf:ref/@xf:targetResId = '{historicalId:xxx}' -->
      <xsl:when test="$relation/xf:ref/xf:isHistoricalIdRef(@xf:targetResId)">
        <xsl:sequence select="false()"/>
      </xsl:when>
      <xsl:otherwise>
        <!-- Si le lien n'est pas résolu, les attributs de xf:ref contiennent une query sous la forme "{query:}" -->
        <xsl:sequence select="not(some $ref in ($relation/xf:ref/(@xf:targetResId,@xf:idRef,@idRef)) satisfies xf:isQueryRef($ref))"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Détermine si une référence donnée (<xd:ref name="ref" type="parameter">$ref</xd:ref>) est une query, par opposition à un ID résolu.</xd:p>
      <xd:p>Typiquement, <xd:ref name="ref" type="parameter">$ref</xd:ref> est la valeur d'un attribut xf:ref/@xf:targetResId, xf:ref/@xf:idRef ou xf:ref/@idRef.</xd:p>
    </xd:desc>
    <xd:param name="ref">[xs:string?] Une référence (la valeur d'un attribut @xf:targetResId par exemple).</xd:param>
    <xd:return>[xs:boolean] Le résultat du test.</xd:return>
  </xd:doc>
  <xsl:function name="xf:isQueryRef" as="xs:boolean">
    <xsl:param name="ref" as="xs:string?"/>
    <xsl:sequence select="starts-with(normalize-space($ref),'{query:')"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Détermine si une référence donnée (<xd:ref name="ref" type="parameter">$ref</xd:ref>) fait référence à un ID historique.</xd:p>
      <xd:p>Typiquement, <xd:ref name="ref" type="parameter">$ref</xd:ref> est la valeur d'un attribut xf:ref/@xf:targetResId.</xd:p>
    </xd:desc>
    <xd:param name="ref">[xs:string?] Une référence (la valeur d'un attribut @xf:targetResId par exemple).</xd:param>
    <xd:return>[xs:boolean] Le résultat du test.</xd:return>
  </xd:doc>
  <xsl:function name="xf:isHistoricalIdRef" as="xs:boolean">
    <xsl:param name="ref" as="xs:string?"/>
    <xsl:sequence select="starts-with(normalize-space($ref),'{historicalId:')"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne la valeur d'un ID historique (typiquement construit depuis la transformation SAS et stocké dans un attribut @xf:historicalId dans l'ECM), en tronquant un éventuel préfixe.</xd:p>
    </xd:desc>
    <xd:param name="prefix">[xs:string] Le préfixe à tronquer (ex. : "efl_").</xd:param>
    <xd:param name="historicalId">[xs:string?] L'ID historique tel que stocké dans l'ECM (ex. : valeur d'un @xf:historicalId).</xd:param>
    <xd:return>[xs:string?] La valeur de l'ID historique après troncature.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getHistoricalIdValue" as="xs:string?">
    <xsl:param name="prefix" as="xs:string"/>
    <xsl:param name="historicalId" as="xs:string?"/>
    <xsl:sequence select="if (starts-with($historicalId,$prefix)) then (substring-after($historicalId,$prefix)) else ($historicalId)"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Retourne le nom d'un fichier média, tel qu'appelé dans un attribut @xf:targetResId.</xd:p>
      <xd:p>L'ECM sérialise le nom du fichier sous la forme : @xf:targetResId="{filename:xxx} - où "xxx" est un nom de fichier avec son extension.</xd:p>
      <xd:p>Ce genre de notation est utilisé dans les métadonnées de type média, par exemple.</xd:p>
    </xd:desc>
    <xd:param name="targetResIdVal">[xs:string?] La valeur de l'attribut @targetResId.</xd:param>
    <xd:return>[xs:string?] Le nom du fichier.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getMediaFileNameFromTargetResId" as="xs:string?">
    <xsl:param name="targetResIdVal" as="xs:string?"/>
    <xsl:sequence select="replace($targetResIdVal,'^\{filename:(.+?)\}$','$1')"/>
  </xsl:function>

</xsl:stylesheet>