<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:rng="http://relaxng.org/ns/structure/1.0"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:xslLib="http://www.lefebvre-sarrut.eu/ns/els/xslLib"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:xf-lib="http://www.lefebvre-sarrut.eu/ns/xmlfirst/lib"
  xmlns:xfe="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns:html2xml="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor/html2xsl"
  xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xpath-default-namespace="http://www.w3.org/1999/xhtml"
  exclude-result-prefixes="#all"
  version="3.0">
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Generic XSLT to convert a edited contentNode (HTML) back to XML</xd:p>
      <xd:p>BEWARE : if any specific fixup need to be applied so the xml is valid, then do not call this XSLT directly, call a new one that import this one</xd:p>
    </xd:desc>
  </xd:doc>
  
  <xsl:import href="xfe-common.xsl"/>
  <xsl:import href="xfe-HTML2XML_clean.xsl"/>
  <xsl:import href="xslLib:/xslLib/html2cals.xsl"/>
  <!--xf-makeRelations.xsl should be included by the specific xfe-HTML2XML.xsl driver, but we import it here as fallback--> 
  <xsl:import href="xf-lib:/xf-lib/xsl/_common/xf-makeRelations.xsl"/>
  
  <xsl:key name="getElementByName" match="*" use="local-name()"/>
  
  <xsl:output method="xml" indent="no" omit-xml-declaration="yes"/>
  
  <xsl:param name="xslLib:html2cals.cals.ns.uri" select="$xfe:ns.uri" as="xs:string"/>
  <xsl:param name="xfe:wrapWithFictionalEE" select="true()" as="xs:boolean"/>
  <xsl:param name="xfe:addXmlModelPi" select="true()" as="xs:boolean"/>
  <xsl:param name="xfe:cleanHtmlBefore" select="false()" as="xs:boolean"/>
  
  <xsl:variable name="xfe:ns.uri" select="rng:getRootNamespaceUri($xfe:srng.grammar.normalized)" as="xs:string"/>
  <xsl:variable name="xfe:resolveSingleLink" select="(count(/*) = 1) and count(/a) = 1" as="xs:boolean"/>
  
  <!--================================-->
  <!--INIT-->
  <!--================================-->
  
  <xsl:template match="/">
    <xsl:apply-templates select="." mode="xfe:html2xml"/>
  </xsl:template>
  
  <!--================================-->
  <!--MAIN-->
  <!--================================-->
  
  <xsl:template match="/" mode="xfe:html2xml">
    <xsl:call-template name="els:log">
      <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
      <xsl:with-param name="code" select="$els:log.level.code"/>
      <xsl:with-param name="level" select="'info'"/>
      <xsl:with-param name="alert" select="true()"/>
      <xsl:with-param name="markup" select="false()"/>
      <xsl:with-param name="description">Applying xfe-HTML2XML.xsl on <xsl:value-of select="els:getFileName(base-uri(.))"/> [params : xfe:wrapWithFictionalEE=<xsl:value-of select="$xfe:wrapWithFictionalEE"/>, xfe:addXmlModelPi=<xsl:value-of select="$xfe:addXmlModelPi"/>, xfe:cleanHtmlBefore=<xsl:value-of select="$xfe:cleanHtmlBefore"/>]</xsl:with-param>
    </xsl:call-template>
    <!--STEP 0 : clean before if necessary (need for local tests) : theses steps will be hard process on the platform-->
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:choose>
         <xsl:when test="$xfe:cleanHtmlBefore">
           <xsl:variable name="step" as="document-node()">
             <xsl:document>
               <xsl:apply-templates select="." mode="xfe:html2xml_clean"/>
             </xsl:document>
           </xsl:variable>
           <xsl:apply-templates select="$step" mode="xfe:html2xml_clean.specific"/>
         </xsl:when>
         <xsl:otherwise>
           <xsl:sequence select="."/>
         </xsl:otherwise>
       </xsl:choose>
      </xsl:document>
    </xsl:variable>
    <xsl:if test="$xfe:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('html2xml.step0.xml', $log.uri)" as="xs:anyURI"/>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template>
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <!--STEP 1 : main-->
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:choose>
          <xsl:when test="$xfe:resolveSingleLink">
            <xsl:apply-templates select="$step/a" mode="xfe:html2xml_main"/>
          </xsl:when>
          <xsl:when test="$xfe:srng.grammar.rootIsSingle">
            <xsl:apply-templates select="$step/html/body" mode="xfe:html2xml_main"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates select="$step/html/body/div[1]" mode="xfe:html2xml_main"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:document>
    </xsl:variable>
    <xsl:if test="$xfe:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('html2xml.step1.xml', $log.uri)" as="xs:anyURI"/>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template>
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <!--STEP 2 : xhtml2cals -->
    <xsl:variable name="step" as="document-node()">
      <xsl:choose>
        <!--FIXME : some model may have both html and cals table : we need a specific class for each so we can choose how to convert it back-->
        <xsl:when test="$xfe:resolveSingleLink or (html2xml:has-no-cals-table() and count(key('getElementByName', 'table')) != 0)">
          <xsl:sequence select="$step"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:document>
            <xsl:apply-templates select="$step" mode="xslLib:xhtml2cals">
              <xsl:with-param name="ns.uri" select="$xfe:ns.uri"/>
            </xsl:apply-templates>
          </xsl:document>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="$xfe:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('html2xml.step2.xml', $log.uri)" as="xs:anyURI"/>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template>
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <!--STEP 3 : Cals Table Fixup-->
    <xsl:variable name="step" as="document-node()">
      <xsl:choose>
        <xsl:when test="$xfe:resolveSingleLink or (html2xml:has-no-cals-table() and count(key('getElementByName', 'table')) != 0)">
          <xsl:sequence select="$step"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:document>
            <xsl:apply-templates select="$step" mode="xfe:html2xml.calsTable-fixup"/>
          </xsl:document>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="$xfe:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('html2xml.step3.xml', $log.uri)" as="xs:anyURI"/>      
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template>      
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <!--STEP 4 : XML Fixup-->
    <xsl:variable name="step" as="document-node()">
      <xsl:choose>
        <!--Unactive mode fixup when resolving a single link-->
        <xsl:when test="$xfe:resolveSingleLink">
          <xsl:sequence select="$step"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:document>
            <xsl:apply-templates select="$step" mode="xfe:html2xml_xmlFixup"/>
          </xsl:document>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="$xfe:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('html2xml.step4.xml', $log.uri)" as="xs:anyURI"/>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template> 
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <!--STEP5 : CLEAN -->
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="$step" mode="html2xml:clean"/>
      </xsl:document>
    </xsl:variable>
    <xsl:if test="$xfe:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('html2xml.step5.xml', $log.uri)" as="xs:anyURI"/>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template>
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <!--FINALY-->
    <xsl:choose>
      <!--when resolving a single link, one must wrap within a fictional EE so the relations is resolved properly-->
      <xsl:when test="$xfe:wrapWithFictionalEE or $xfe:resolveSingleLink">
        <xsl:variable name="step" as="document-node()">
          <xsl:document>
            <editorialEntity 
              xmlns="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
              xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst">
              <metadata/>
              <body>
                <contentNode>
                  <metadata/>
                  <content>
                    <xsl:sequence select="$step"/>
                  </content>
                </contentNode>
              </body>
              <relations/>
            </editorialEntity>
          </xsl:document>
        </xsl:variable>
        <xsl:if test="$xfe:addXmlModelPi and not($xfe:resolveSingleLink)">
          <xsl:processing-instruction name="xml-model">
            <xsl:text>href="xf-models:/xf-models/editorialEntity/editorialEntity.nvdl" type="application/xml" schematypens="http://purl.oclc.org/dsdl/nvdl/ns/structure/1.0"</xsl:text>
          </xsl:processing-instruction>
        </xsl:if>
        <!--STEP : makeRelations-->
        <xsl:variable name="step" as="document-node()">
          <xsl:document>
            <!--This mode implies to import the specific makeRelations XSLT-->
            <xsl:apply-templates select="$step" mode="xf:makeRelations"/>
          </xsl:document>
        </xsl:variable>
        <xsl:choose>
          <xsl:when test="$xfe:resolveSingleLink">
            <xsl:value-of select="$step/xf:editorialEntity/xf:relations/xf:relation/xf:ref/@xf:targetResId"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates select="$step" mode="els:fixNamespaceDeclarations">
              <xsl:with-param name="namespaces" as="element(els:namespace)*">
                <els:namespace name="xf" uri="http://www.lefebvre-sarrut.eu/ns/xmlfirst"/>
              </xsl:with-param>
            </xsl:apply-templates>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:if test="$xfe:addXmlModelPi">
          <xsl:processing-instruction name="xml-model">
            <xsl:text>href="</xsl:text>
            <xsl:value-of select="els:getRelativePath(els:getFolderPath(base-uri()), $xfe:srng.uri)"/>
            <xsl:text>" type="application/xml" schematypens="http://relaxng.org/ns/structure/1.0"</xsl:text>
          </xsl:processing-instruction>
        </xsl:if>
        <xsl:apply-templates select="$step" mode="els:fixNamespaceDeclarations">
          <xsl:with-param name="namespaces" as="element(els:namespace)*">
            <els:namespace name="xf" uri="http://www.lefebvre-sarrut.eu/ns/xmlfirst"/>
            <els:namespace name="xf-lib" uri="http://www.lefebvre-sarrut.eu/ns/xmlfirst/lib"/>
          </xsl:with-param>
        </xsl:apply-templates>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!--DEBUG-->
  <!--<xsl:template match="/" mode="xfe:html2xml">
    <debugs><xsl:apply-templates mode="xfe:html2xml_main.DEBUG"/></debugs>
  </xsl:template>
  <xsl:template match="*[@class='xfe_title-5']" mode="xfe:html2xml_main.DEBUG">
    <match select="{els:displayNode(.)}" label="xfe:getConfMappingFromHTML(.)"><xsl:copy-of select="xfe:getConfMappingFromHTML(.)"/></match>
  </xsl:template>
  <xsl:template match="text()" mode="xfe:html2xml_main.DEBUG"/>-->
  
  <!--================================-->
  <!--STEP 1 : HTML2XML-->
  <!--================================-->
  
  <xsl:template match="table | colgroup | col | thead | tbody | tfoot | tr | td | th" mode="xfe:html2xml_main">
    <xsl:choose>
      <xsl:when test="html2xml:has-no-cals-table()">
        <xsl:next-match/>
      </xsl:when>
      <xsl:otherwise>
        <!--Tableau CALS: on les copie pour le moment, ils seront transformés après dans un autre step--> 
        <xsl:copy>
          <xsl:copy-of select="@*"/>
          <xsl:apply-templates mode="#current"/>
        </xsl:copy>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="*" mode="xfe:html2xml_main">
    <xsl:element name="{xfe:getXmlELementName(.)}" namespace="{$xfe:ns.uri}">
      <xsl:call-template name="html2xml:makeXmlAttributes"/>
      <xsl:apply-templates mode="#current"/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="ins | del" mode="xfe:html2xml_main">
    <xsl:processing-instruction name="xfe_{local-name(.)}_start">
      <xsl:for-each select="@*">
        <xsl:value-of select="name()"/>
        <xsl:text>="</xsl:text>
        <xsl:value-of select="."/>
        <xsl:text>"</xsl:text>
        <xsl:if test="not(position() = last())">
          <xsl:text> </xsl:text>
        </xsl:if>
      </xsl:for-each>
    </xsl:processing-instruction>
    <xsl:apply-templates mode="#current"/>
    <xsl:processing-instruction name="xfe_{local-name(.)}_end"/>
  </xsl:template>
  
  <xsl:template match="span[els:hasClass(., 'tinyMce_annotation')]" mode="xfe:html2xml_main">
    <xsl:processing-instruction name="xfe_comment_start">
      <xsl:for-each select="@*">
        <xsl:value-of select="name()"/>
        <xsl:text>="</xsl:text>
        <xsl:value-of select="."/>
        <xsl:text>"</xsl:text>
        <xsl:if test="not(position() = last())">
          <xsl:text> </xsl:text>
        </xsl:if>
      </xsl:for-each>
    </xsl:processing-instruction>
    <xsl:apply-templates mode="#current"/>
    <xsl:processing-instruction name="xfe_comment_end"/>
  </xsl:template>
  
  <xsl:template name="html2xml:makeXmlAttributes">
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="name(.) = 'id'">
          <!--les @id sont les seuls attributs qu'on conserve entre xml et html-->
          <xsl:copy-of select="."/>
        </xsl:when>
        <xsl:when test="name(.) = concat($xfe:dataXfeAttribute.prefix, 'label')"/>
        <!--Links-->
        <xsl:when test="parent::a and name(.) = 'href'">
          <xsl:choose>
            <!--Internal link "#" to anchor-->
            <xsl:when test="html2xml:is-internalLinkHref(.)">
              <!--When final format is XHTML keep the "href"-->
              <!--When final format is XML : we don't know the final XML attribut's name (e.g. @REF) : use xf:href 
                  XML FIXUP must resolve this attribute name according to the current schema -->
              <!--FIXME : use another prefix like "xfe-tmp:internal-ref" ?-->
              <xsl:attribute name="{if($xfe:srng.is-xhtml) then('href') else('xf:href')}" select="."/>
            </xsl:when>
            <!--Flash link : should be resolved--> 
            <xsl:when test="html2xml:is-externalLinkHref(parent::*)">
              <xsl:attribute name="xf-lib:xf_targetResId" select="html2xml:normalize-externalLinkHref(.)"/>
            </xsl:when>
            <xsl:when test="html2xml:is-multimediaLink(parent::*)">
              <xsl:variable name="idRefAttributeName" select="replace(xfe:getConfMappingFromHTML(parent::*)/@type, '^xfe:multimediaLink\(@(\w+)\)$', '$1')" as="xs:string"/>
              <xsl:attribute name="{$idRefAttributeName}" select="."/>
            </xsl:when>
            <!-- TODO
            <xsl:when test="html2xml:is-webLinkHref(parent::*)">
              <xsl:attribute name="{if($xfe:srng.is-xhtml) then('href') else('xf:href')}" select="."/>
            </xsl:when>-->
            <!--Otherwise let's consider it as web link "http://..."-->
            <xsl:otherwise>
              <!--FIXME : use another prefix like "xfe-tmp:web-ref" ?-->
              <xsl:attribute name="{if($xfe:srng.is-xhtml) then('href') else('xf:href')}" select="."/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:when test="starts-with(local-name(.), $xfe:dataXmlAttribute.prefix)">
          <xsl:attribute name="{substring-after(local-name(), $xfe:dataXmlAttribute.prefix)}" select="."/>
        </xsl:when>
        <!--data-xfe-idRef => xf-lib:idRef
            data-xfe-xf_idRef => xf-lib:xf_idRef-->
        <xsl:when test="local-name(.) = concat($xfe:dataXfeAttribute.prefix, 'idRef') or local-name(.) = concat($xfe:dataXfeAttribute.prefix, 'xf_idRef') ">
          <xsl:attribute name="{concat('xf-lib:', substring-after(local-name(), $xfe:dataXfeAttribute.prefix))}" select="."/>
        </xsl:when>
        <!--data-xfe-listType => xf:listType-->
        <xsl:when test="starts-with(local-name(.), $xfe:dataXfeAttribute.prefix)">
          <xsl:attribute name="{concat('xf:', substring-after(local-name(), $xfe:dataXfeAttribute.prefix))}" select="."/>
        </xsl:when>
        <xsl:when test="$xfe:srng.is-xhtml">
          <xsl:variable name="attribute.value" as="xs:string*">
            <xsl:for-each select="tokenize(., '\s+')">
              <xsl:if test="not(starts-with(., 'xfe_'))">
                <xsl:value-of select="."/>
               </xsl:if>
            </xsl:for-each>
          </xsl:variable>
          <xsl:if test="count($attribute.value) != 0">
            <xsl:attribute name="{name()}" select="string-join($attribute.value, ' ')"/>
          </xsl:if>
          <xsl:variable name="xfe.attribute.value" as="xs:string*">
            <xsl:for-each select="tokenize(., '\s+')">
              <xsl:if test="starts-with(., 'xfe_')">
                <xsl:value-of select="."/>
              </xsl:if>
            </xsl:for-each>
          </xsl:variable>
          <xsl:if test="count($xfe.attribute.value) != 0">
            <xsl:attribute name="{concat('html:', local-name(.))}" select="string-join($xfe.attribute.value, ' ')"/>
          </xsl:if>
        </xsl:when>
        <xsl:when test="not(contains(name(), ':'))">
          <xsl:attribute name="{concat('html:', local-name(.))}" select="."/>
        </xsl:when>
      </xsl:choose>
    </xsl:for-each>
  </xsl:template>
  
  <!--====================================-->
  <!-- STEP 2 : xhtml2cals -->
  <!--====================================-->
  
  <!--cf. xhtml2cals.xsl-->
  
  <!--====================================-->
  <!-- STEP 3 : CALS Tables Fixup -->
  <!--====================================-->
  
 <!-- Because xhtml2cals generate one cals table for one html table, and because cals table are always nested in a div class="blocTableau", 
     we get this structure at this point : 
     <blocTableau attributes="already converted to xml by xml2html_main">
       <SomeXMLlikeTitle>...</SomeXMLlikeTitle>
       <table attributes="already converted to cals by xhtml2cals">...</table>
       <table attributes="already converted to cals by xhtml2cals">...</table>
     </blocTableau>
     Here we get the Cals structure OK : blocTableau will be converted to table by inheriting cals attribute from child table
     and child table will be deleted -->
  
  <xsl:template match="*:blocTableau" mode="xfe:html2xml.calsTable-fixup">
    <xsl:element name="table" namespace="{$xfe:ns.uri}">
      <xsl:copy-of select="@*"/>
      <xsl:copy-of select="(.//*:table)[1]/(@frame|@colsep|@rowsep)"/>
      <xsl:apply-templates mode="#current"/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="*:blocTableau/*:table" mode="xfe:html2xml.calsTable-fixup">
    <xsl:apply-templates select="*" mode="#current"/>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xfe:html2xml.calsTable-fixup">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--====================================-->
  <!-- STEP 4 : xfe:html2xml_xmlFixup -->
  <!--====================================-->
  
  <!--This mode should be defined in a specific XSLT imported by the main driver-->
  
  <!--====================================-->
  <!-- STEP 5 : clean -->
  <!--====================================-->
  
  <xsl:template match="@xfe:* | @html:*" mode="html2xml:clean"/>
  
  <!--Suppression dans les PI des pseudo attributs xfe:* et html:-->
  <xsl:template match="processing-instruction('xfe_del_start') | processing-instruction('xfe_ins_start') | processing-instruction('xfe_comment_start')" mode="html2xml:clean">
    <xsl:variable name="regex" select="concat('(xfe|html):\w+=', $els:dquot, '.*?', $els:dquot)" as="xs:string"/>
    <xsl:processing-instruction name="{name(.)}" select="normalize-space(replace(., $regex, ''))"/>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="html2xml:clean">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--====================================-->
  <!-- STEP 5 : xf:makeRelations -->
  <!--====================================-->
  
  <!--This mode should be defined in a specific XSLT imported by the main driver-->
  
  <!--================================-->
  <!--CLEAN SPECIFIC FALLBACK-->
  <!--================================-->
  
  <xsl:template match="/" mode="xfe:html2xml_clean.specific" priority="-1">
    <xsl:call-template name="els:log">
      <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
      <xsl:with-param name="code" select="$els:log.level.code"/>
      <xsl:with-param name="level" select="'warning'"/>
      <xsl:with-param name="alert" select="true()"/>
      <xsl:with-param name="markup" select="false()"/>
      <xsl:with-param name="description">[WARNING] No match "/" in mode="xfe:html2xml_clean.specific", if you need it please add it to your own xslt</xsl:with-param>
    </xsl:call-template>
    <xsl:next-match/>
  </xsl:template>
  
  <!--default copy for xfe:html2xml_clean.specific-->
  <xsl:template match="node() | @*" mode="xfe:html2xml_clean.specific" priority="-1">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--================================-->
  <!--XMLFIXUP SPECIFIC FALLBACK-->
  <!--================================-->
  
  <xsl:template match="/" mode="xfe:html2xml_xmlFixup" priority="-1">
    <xsl:call-template name="els:log">
      <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
      <xsl:with-param name="code" select="$els:log.level.code"/>
      <xsl:with-param name="level" select="'warning'"/>
      <xsl:with-param name="alert" select="true()"/>
      <xsl:with-param name="markup" select="false()"/>
      <xsl:with-param name="description">[WARNING] No match "/" in mode="xfe:html2xml_xmlFixup", if you need it please add it to your own xslt</xsl:with-param>
    </xsl:call-template>    
    <xsl:next-match/>
  </xsl:template>
  
  <!--default copy for xfe:html2xml_clean.specific-->
  <xsl:template match="node() | @*" mode="xfe:html2xml_xmlFixup" priority="-1">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--================================-->
  <!--COMMON-->
  <!--================================-->  
  
  <!--FIXME : fonction mal nommée et/ou à remplacer ?-->
  <xsl:function name="html2xml:has-no-cals-table" as="xs:boolean">
    <xsl:sequence select="count($xfe:conf.normalized/*:mapping/*:element[@type = 'cals:table']) = 0"/>
  </xsl:function>
  
  <xsl:function name="html2xml:is-externalLinkHref" as="xs:boolean">
    <xsl:param name="e" as="element(html:a)"/>
    <xsl:sequence select="starts-with(xfe:getConfMappingFromHTML($e)/@type, 'xfe:externalLinkType')"/>
  </xsl:function>
  
  <xsl:function name="html2xml:is-multimediaLink" as="xs:boolean">
    <xsl:param name="e" as="element(html:a)"/>
    <xsl:sequence select="starts-with(xfe:getConfMappingFromHTML($e)/@type, 'xfe:multimediaLink')"/>
  </xsl:function>
  
  <xsl:function name="html2xml:is-internalLinkHref" as="xs:boolean">
    <xsl:param name="href" as="attribute(href)"/>
    <xsl:sequence select="starts-with($href, '#')"/>
  </xsl:function>
  
  <!--ex : 
    <a class="refDoc" href="http://localhost:3000/#/editorialEntity/4e3d261b-8749-4348-ad6d-c60ff402d3df">blablabla</a>
    =>
    <a class="refDoc" href="4e3d261b-8749-4348-ad6d-c60ff402d3df">blablabla</a>
  -->
  <xsl:function name="html2xml:normalize-externalLinkHref" as="xs:string">
    <xsl:param name="href" as="attribute(href)"/>
    <xsl:choose>
      <xsl:when test="contains($href, $xfe:ecm.ee.url.path)">
        <xsl:value-of select="substring-after($href, $xfe:ecm.ee.url.path)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$href"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
</xsl:stylesheet>