<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns="http://www.w3.org/1999/xhtml"
  xmlns:saxon="http://saxon.sf.net/"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:rng="http://relaxng.org/ns/structure/1.0"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:xfe="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor"
  xmlns:xslLib="http://www.lefebvre-sarrut.eu/ns/els/xslLib"
  xmlns:cals="http://docs.oasis-open.org/ns/oasis-exchange/table"
  xmlns:html="http://www.w3.org/1999/xhtml"
  exclude-result-prefixes="#all"
  version="3.0">
  
  <xsl:import href="xfe-common.xsl"/>
  <xsl:import href="../xslLib/xslLib/cals2html.xsl"/>
  <xsl:import href="../xslLib/xslLib/xml2simpleHtml.xsl"/>
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Conversion XML vers HTML d'un contentNode pour XFE</xd:p>
    </xd:desc>
  </xd:doc>
  
  <!--CALS2HTML params-->
  <!--FE-124 : don't use "colgroup" in html table-->
  <xsl:param name="xslLib:cals2html.compute-column-width-within-colgroup" select="false()" as="xs:boolean"/>
  <xsl:param name="xslLib:cals2html-keep-unmatched-attributes" select="true()" as="xs:boolean"/>
  <xsl:param name="xslLib:cals2html-unmatched-attributes-prefix" select="$xfe:dataXmlAttribute.prefix" as="xs:string"/>
  <!--When the EE has multiple contentNode, get the id of the one to be edited-->
  <xsl:param name="xfe:contentNodeId" select="''" as="xs:string"/>
  <xsl:param name="xfe:generateCompleteHtmlStructure" select="true()" as="xs:boolean"/>
  <xsl:param name="xfe:addHTML5DOCTYPE" select="true()" as="xs:boolean"/>
  <xsl:param name="xfe:addXmlModelPi" select="true()" as="xs:boolean"/>
  <xsl:param name="xfe:fillHeadTitle" select="false()" as="xs:boolean"/><!--FE-105 : pas de contenu dans le titre-->
  <xsl:param name="xfe:addXfeLabel" select="true()" as="xs:boolean"/>
  <!--param to prevent throwing a fatal error when unexpected xml element found-->
  <xsl:param name="xfe:allowUnValidXML" select="if($xfe:conf.hasSchema) then(false()) else(true())" as="xs:boolean"/>
  
  <!--Le doctype xhtml ajoute des méta spécifiques qui ne sont pas reproduites dans xspec (qui ignore vraissemblablement xsl:output)-->
  <!--FE-130 : we have to force omit-xml-declaration="yes" because it's not possible to add a parameter for this (except using xsl:result-document which is not the case here)--> 
  <xsl:output method="xhtml" indent="no" omit-xml-declaration="yes"/>
     
  <!--Le vrai DOCTYPE html5 serait avec 
  method="html"
  version="5.0"
  Mais cela pose des problème de balise non fermée (http://markmail.org/message/jaajiozvbbhqeipz)
  => cf. ajout manuel du DOCTYPE HTML5
  -->
  
  <!--================================-->
  <!--INIT-->
  <!--================================-->
  
  <xsl:template match="/">
    <xsl:apply-templates select="." mode="xfe:xml2html"/>
  </xsl:template>
  
  <!--================================-->
  <!--MAIN-->
  <!--================================-->
  
  <xsl:template match="/" mode="xfe:xml2html" priority="1">
    <!--Overiding xfe:contentNodeId param so we can change it by calling this template only-->
    <xsl:param name="xfe:contentNodeId" select="$xfe:contentNodeId" as="xs:string"/>
    <xsl:variable name="contentNodes" select=".//xf:contentNode" as="element()*"/>
    <xsl:choose>
      <xsl:when test="not(xf:editorialEntity)">
        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
          <xsl:with-param name="code" select="$els:log.level.code"/>
          <xsl:with-param name="alert" select="true()"/>
          <xsl:with-param name="level" select="'error'"/>
          <xsl:with-param name="markup" select="false()"/>
          <xsl:with-param name="description">[ERROR][XFE] Document must be an editorialEntity, but we continue for test</xsl:with-param>
        </xsl:call-template>
        <xsl:next-match/>
      </xsl:when>
      <xsl:when test="count($contentNodes) = 0">
        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
          <xsl:with-param name="code" select="$els:log.level.code"/>
          <xsl:with-param name="alert" select="true()"/>
          <xsl:with-param name="level" select="'fatal'"/>
          <xsl:with-param name="markup" select="false()"/>
          <xsl:with-param name="description">[FATAL][XFE] No contentNode found in this EE</xsl:with-param>
        </xsl:call-template>
        <xsl:message terminate="yes"/>
      </xsl:when>
      <xsl:when test="count($contentNodes) = 1">
        <xsl:apply-templates select="$contentNodes[1]/xf:content/*" mode="#current"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="contentNode" select="$contentNodes[@xf:id = $xfe:contentNodeId]" as="element()?"/>
        <xsl:choose>
          <xsl:when test="count($contentNode) = 0">
            <xsl:call-template name="els:log">
              <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
              <xsl:with-param name="code" select="$els:log.level.code"/>
              <xsl:with-param name="alert" select="true()"/>
              <xsl:with-param name="level" select="'fatal'"/>
              <xsl:with-param name="markup" select="false()"/>
              <xsl:with-param name="description">[FATAL][XFE] No contentNode with id="<xsl:value-of select="$xfe:contentNodeId"/>" found in this EE</xsl:with-param>
            </xsl:call-template>
            <xsl:message terminate="yes"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates select="$contentNode/xf:content/*" mode="#current"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!--On garde la possibilité d'avoir un contentNode en entrée (/) pour que les sample continuent de fonctionner, même si ce n'est pas la cible dans l'ECM (on aura toujours une EE en entrée)-->
  <!--Cela permets de travailler ici sur des samples valides selon le modèle srng de la conf-->
  <xsl:template match="/ | xf:content/*" mode="xfe:xml2html">
    <xsl:call-template name="els:log">
      <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
      <xsl:with-param name="code" select="$els:log.level.code"/>
      <xsl:with-param name="alert" select="true()"/>
      <xsl:with-param name="markup" select="false()"/>
      <xsl:with-param name="description">[INFO] xfe:conf.uri=<xsl:value-of select="$xfe:conf.uri"/></xsl:with-param>
    </xsl:call-template>
    <xsl:call-template name="els:log">
      <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
      <xsl:with-param name="code" select="$els:log.level.code"/>
      <xsl:with-param name="alert" select="true()"/>
      <xsl:with-param name="markup" select="false()"/>
      <xsl:with-param name="description">[INFO] xfe:debug=<xsl:value-of select="$xfe:debug"/><xsl:if test="not($xfe:debug)"> (les log de la transformation ne seront pas écrits sur le filesytem)</xsl:if></xsl:with-param>
    </xsl:call-template>
    <xsl:variable name="root" as="document-node()">
      <xsl:document>
        <xsl:sequence select="."/>
      </xsl:document>
    </xsl:variable>
    <xsl:call-template name="xfe:log.normalizeConf"/>
    <!--<xsl:copy-of select="."/>-->
    <!--STEP 0-->
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:choose>
          <xsl:when test="parent::xf:content">
            <xsl:apply-templates select="." mode="xfe:xml2html_resolveRelation"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:copy-of select="."/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:document>
    </xsl:variable>
    <xsl:if test="$xfe:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('XML2HTML.step0_xml2html_resolveRelation.log.xml', $log.uri)" as="xs:anyURI"/>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template>
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <!--STEP 1-->
    <xsl:variable name="step" as="document-node()">
      <xsl:choose>
        <xsl:when test="$xfe:conf.hasSchema">
          <xsl:document>
            <xsl:apply-templates select="$step" mode="xfe:xml2html_annotations"/>
          </xsl:document>
        </xsl:when>
        <xsl:otherwise>
          <xsl:sequence select="$step"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="$xfe:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('XML2HTML.step1_xml2html_annotation.log.xml', $log.uri)" as="xs:anyURI"/>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template>
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <!--STEP 2-->
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="$step" mode="xfe:xml2html_main"/>
      </xsl:document>
    </xsl:variable>
    <xsl:if test="$xfe:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('XML2HTML.step1_xml2html_main.log.xml', $log.uri)" as="xs:anyURI"/>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template>
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <!--STEP 3-->
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="$step" mode="xslLib:cals2html"/>
      </xsl:document>
    </xsl:variable>
    <xsl:if test="$xfe:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('XML2HTML.step2_cals2html.log.xml', $log.uri)" as="xs:anyURI"/>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template>
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <!--STEP 4-->
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="$step" mode="xfe:xml2html_step3_moveInlineNotes"/>
      </xsl:document>
    </xsl:variable>
    <xsl:if test="$xfe:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('XML2HTML.step3_moveInlineNotes.log.xml', $log.uri)" as="xs:anyURI"/>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template>
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <!--STEP 5-->
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="$step" mode="xslLib:xml2html_customizeCals2html"/>
      </xsl:document>
    </xsl:variable>
    <xsl:if test="$xfe:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('XML2HTML.step4_customizeCals2html.log.xml', $log.uri)" as="xs:anyURI"/>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template>
      <xsl:result-document href="{$step.log.uri}">
        <xsl:sequence select="$step"/>
      </xsl:result-document>
    </xsl:if>
    <!--FINALY-->
    <xsl:choose>
      <xsl:when test="$xfe:generateCompleteHtmlStructure">
        <xsl:variable name="xfe:srng.name" select="els:getFileName($xfe:conf.normalized/*:srngXmlmodel/@href, false())" as="xs:string"/>
        <xsl:if test="$xfe:addXmlModelPi and $xfe:conf.hasSchema">
          <xsl:processing-instruction name="xml-model">
        <xsl:text>href="</xsl:text>
        <xsl:value-of select="els:getRelativePath(els:getFolderPath(base-uri()), concat(els:getFolderPath($xfe:srng.uri), '/', $xfe:srng.name, '.html.rng'))"/>
        <xsl:text>" type="application/xml" schematypens="http://relaxng.org/ns/structure/1.0"</xsl:text>
      </xsl:processing-instruction>
        </xsl:if>
        <!--Puisqu'on veut éviter des problème de balise non fermées à la sérialisation en local, 
            on ajoute ce doctype pour html5 avec une vielle méthode-->
        <xsl:if test="$xfe:addHTML5DOCTYPE">
          <xsl:text disable-output-escaping='yes'>&lt;!DOCTYPE html></xsl:text>
        </xsl:if>
        <html>
          <head>
            <!--<meta charset="UTF-8"/> FIXME : pour html5-->
            <title>
              <xsl:if test="$xfe:fillHeadTitle">
                <xsl:value-of select="els:evaluate-xpath($xfe:conf.normalized/*:title/@xpath, $root/*)"/>
              </xsl:if>
            </title>
            <xsl:variable name="css.path" select="($xfe:css.path, $xfe:conf.normalized/*:css/@path, '')[1]" as="xs:string"/>
            <xsl:variable name="css.path.rel" select="if (els:isAbsoluteUri($css.path)) then (els:getRelativePath(els:getFolderPath(base-uri()),$css.path)) else ($css.path)" as="xs:string"/>
            <xsl:variable name="css.name" select="($xfe:css.name, $xfe:conf.normalized/*:css/@name, concat($xfe:srng.name, '.css'))[1]" as="xs:string"/>
            <link type="text/css" rel="Stylesheet" href="{concat($css.path.rel, if(normalize-space($css.path.rel) != '' and not(ends-with($css.path.rel, '/'))) then('/') else(''), $css.name)}" />
            <!--FIXME : impossible de distinguer les tableaux cals ou html natif, de toute façon pour l'édition mieux vaut ne pas mettre de css ici 
                mais tout mettre en attribut (@style si besoin), servira notamment à la rétro-conv HTML2XML-->
            <!--<link type="text/css" rel="Stylesheet" href="../../cals2html.css"/>-->
            <!--<xsl:result-document href="{resolve-uri('cals2html.css', base-uri(.))}" format="els:text">
              <xsl:copy-of select="unparsed-text('xslLib:/cals2html.css', 'utf-8')"/>
            </xsl:result-document>-->
          </head>
          <xsl:apply-templates select="$step" mode="xfe:xml2html_finaly"/>
        </html>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="$step" mode="xfe:xml2html_finaly"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!--================================================================-->
  <!-- STEP 0 : resolve relation-->
  <!--================================================================-->
  
  <!--Replace the xf:idRef target by the one targeted in the relation-->
  <xsl:template match="xf:content//*[@xf:idRef[count(key('getRelationByXfId', .)) = 1]]/@xf:idRef" mode="xfe:xml2html_resolveRelation">
    <xsl:variable name="relation" select="key('getRelationByXfId', .)" as="element(xf:relation)"/>
    <xsl:choose>
      <xsl:when test="$relation/xf:ref/@xf:targetResId">
        <!--exclude unresolved relation {query:...} : don't add href in this case-->
        <xsl:if test="$relation/xf:ref/@xf:targetResId[not(starts-with(., '{'))]">
          <xsl:attribute name="html:href" select="$relation/xf:ref/@xf:targetResId"/>
          <xsl:if test="$relation/xf:ref/@idRef">
            <xsl:attribute name="{concat('html:', $xfe:dataAttribute.prefix, 'xfe-idRef')}" select="$relation/xf:ref/@idRef"/>
          </xsl:if>
          <xsl:if test="$relation/xf:ref/@xf:idRef">
            <xsl:attribute name="{concat('html:', $xfe:dataAttribute.prefix, 'xfe-xf_idRef')}" select="$relation/xf:ref/@xf:idRef"/>
          </xsl:if>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <!--temp : ancien schéma-->
        <!--<xsl:attribute name="html:href" select="$relation/xf:ref/@xf:idRef"/>-->
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!--Make an empty href if the relation is not resolved
  Unusefull : if the relation is not resolved then don't make a link now, if the element is an externalLink then the href will be initialized later-->
  <!--<xsl:template match="xf:content//*[@xf:idRef[count(key('getRelationByXfId', .)) != 1]]/@xf:idRef" mode="xfe:xml2html_resolveRelation">
    <xsl:message>[WARNING] <xsl:value-of select="count(key('getRelationByXfId', .))"/> relation(s) found for xf:idRef="<xsl:value-of select="."/>". The link will not be resolved</xsl:message>
    <xsl:attribute name="html:href" select="''"/>
  </xsl:template>-->
  
  <xsl:template match="node() | @*" mode="xfe:xml2html_resolveRelation">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--================================================================-->
  <!-- STEP 1 : mode xml2html_annotations-->
  <!--================================================================-->
  
  <xsl:template match="*" mode="xfe:xml2html_annotations">
    <xsl:param name="rngParentElement" as="element(rng:element)?"/>
    <!-- A predicate on attributes to restrict the SRNG data model search -->
    <xsl:param name="predicates" required="no" as="xs:string*">
      <xsl:for-each select="@*[not(contains(name(), ':'))]">
        <xsl:value-of separator="">
          <xsl:text>@</xsl:text>
          <xsl:value-of select="name()"/>
          <xsl:text>='</xsl:text>
          <xsl:value-of select="."/>
          <xsl:text>'</xsl:text>
        </xsl:value-of>
      </xsl:for-each>
    </xsl:param>
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <xsl:variable name="rngCurrentElement" as="element(rng:element)?">
        <xsl:choose>
          <xsl:when test="self::* is root(.)/*">
            <xsl:sequence select="rng:getSRNGdataModelFromXpath(concat('/', local-name(.)), $xfe:srng.grammar.normalized)"/>
          </xsl:when>
          <xsl:when test="count($rngParentElement) = 0">
            <!--nothing : allow unvalid elements-->
          </xsl:when>
          <xsl:otherwise>
            <xsl:sequence select="rng:getSRNGdataModelReccurse($rngParentElement, local-name(.), $predicates, true())"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <xsl:if test="count($rngCurrentElement) != 0">
        <xsl:attribute name="xfe:srng-define" select="$rngCurrentElement/parent::rng:define/@name"/>
        <xsl:attribute name="xfe:confElement.xpath" select="xfe:getConfMappingElementsFromXML(., $rngCurrentElement/parent::rng:define/@name)/els:get-xpath(.)"/>
      </xsl:if>
      <xsl:apply-templates select="node()" mode="#current">
        <xsl:with-param name="rngParentElement" select="$rngCurrentElement"/>
      </xsl:apply-templates>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xfe:xml2html_annotations" priority="-1">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--================================================================-->
  <!-- STEP 2 : mode xml2html_main avec gestion tableaux-->
  <!--================================================================-->
  
  <!--On ajoute le body ici car on va avoir besoin d'une racine pour ajouter le bloc xfe_notes sous le body-->
  <!--Si la racine n'est pas unique, on ajoute le body, la racine du xml deviendra un div (cf. ci-après), sinon c'est la racine xml qui fera office de body-->
  <xsl:template match="/*[not($xfe:srng.grammar.rootIsSingle)]" mode="xfe:xml2html_main" priority="1">
    <body>
      <xsl:next-match/>
    </body>
  </xsl:template>
  
  <!--+ passage en mode addCalsNamespace ou keepAsHtml--> 
  
  <xsl:template match="*[xfe:getConfMappingElementFromXML(.)/@type = 'cals:table']" mode="xfe:xml2html_main">
    <xsl:apply-templates select="." mode="xfe:xml2html_addCalsNamespace"/>
  </xsl:template>
  
  <xsl:template match="*[xfe:getConfMappingElementFromXML(.)/@type = 'html:table']" mode="xfe:xml2html_main">
    <xsl:apply-templates select="." mode="xfe:xml2html_keepAsHtml"/>
  </xsl:template>
  
  <!--=================================-->
  <!--Mode addCalsNamespace-->
  <!--=================================-->
  
  <xsl:template 
    match="*:table | *:tgroup | *:colspec | *:spanspec | *:thead | *:tbody | *:tfoot | *:row 
    | *:TABLE | *:TGROUP | *:COLSPEC | *:SPANSPEC | *:THEAD | *:TBODY | *:TFOOT | *:ROW 
    | *:Table | *:Tgroup | *:Colspec | *:Spanspec | *:Thead | *:Tbody | *:Tfoot | *:Row" 
         mode="xfe:xml2html_addCalsNamespace">
    <!--table/titre-->
    <xsl:element name="cals:{lower-case(local-name())}">
      <xsl:for-each select="@*">
        <xsl:attribute name="{lower-case(name(.))}" select="."/>
      </xsl:for-each>
      <xsl:if test="lower-case(local-name()) = 'table'">
        <xsl:call-template name="xfe:addXfeLabel"/>
      </xsl:if>
      <xsl:apply-templates mode="#current"/>
    </xsl:element>
  </xsl:template>
  
  <!--titre de tableau cals : on repasse dans le mode main-->
  <!--<xsl:template match="*:table/*:titre | *:TABLE/*:TITRE | *:Table/*:Titre " mode="xfe:xml2html_addCalsNamespace">
    <xsl:apply-templates select="." mode="xfe:xml2html_main"/>
  </xsl:template>-->
  
  <!--Pour les autres éléments qui ne seraient pas du cals on repasse dans le mode step1-->
  <xsl:template match="*" mode="xfe:xml2html_addCalsNamespace">
    <xsl:apply-templates select="." mode="xfe:xml2html_main"/>
  </xsl:template>
  
  <xsl:template match="*:entry | *:ENTRY | *:Entry" mode="xfe:xml2html_addCalsNamespace">
    <xsl:element name="cals:{lower-case(local-name())}">
      <xsl:for-each select="@*">
        <xsl:attribute name="{lower-case(name(.))}" select="."/>
      </xsl:for-each>
      <!--On repasse par le mode normal-->
      <xsl:apply-templates mode="xfe:xml2html_main"/>
    </xsl:element>
  </xsl:template>
  
  <!--=================================-->
  <!--Mode keepAsHtml-->
  <!--=================================-->
  
  <xsl:template match="*:table | *:table//* | *:TABLE | *:TABLE//*" mode="xfe:xml2html_keepAsHtml">
    <!--table/titre-->
    <xsl:element name="{lower-case(local-name())}">
      <xsl:copy-of select="@*"/>
      <xsl:if test="lower-case(local-name(.)) = 'table'">
        <xsl:call-template name="xfe:addXfeLabel"/>
      </xsl:if>
      <xsl:apply-templates mode="#current"/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="*:td | *:TD | *:th | *:TH | *:caption | *:CAPTION" mode="xfe:xml2html_keepAsHtml" priority="1">
    <xsl:element name="{lower-case(local-name())}">
      <xsl:copy-of select="@*"/>
      <!--On repasse par le mode normal-->
      <xsl:apply-templates mode="xfe:xml2html_main"/>
    </xsl:element>
  </xsl:template>
  
  <!--=================================-->
  <!-- Mode xfe:xml2html_main default -->
  <!--=================================-->
  
  <!--DEBUG-->
  <!--<xsl:template match="e">
    <xsl:variable name="e.dataModel" select="rng:getSRNGdataModelFromXmlElement(., $srng.grammar)" as="element(rng:element)"/>
    <xsl:variable name="e.parent.dataModel" select="rng:getSRNGdataModelFromXmlElement(./parent::*, $srng.grammar)" as="element(rng:element)"/>
    <parentDataModel>
      <xsl:copy-of select="$e.parent.dataModel"/>
    </parentDataModel>
    <DataModel>
      <xsl:copy-of select="$e.dataModel"/>
    </DataModel>
    <xsl:variable name="ref" select="$e.parent.dataModel//rng:ref[rng:getDefine(.)/rng:element is $e.dataModel]" as="element(rng:ref)"/>
    <inline><xsl:value-of select="exists($ref/ancestor::rng:interleave/rng:text)"/></inline>
    <isInline><xsl:value-of select="rng:isInline($e.dataModel, $e.parent.dataModel)"/></isInline>
    <!-\-<xsl:message><xsl:value-of select="local-name()"/> : <xsl:value-of select="xf:getHtmlElementName(.)"/></xsl:message>-\->
  </xsl:template>-->
  
  <xsl:template match="*[$xfe:conf.hasSchema][count(@xfe:srng-define) = 0]" mode="xfe:xml2html_main" priority="1">
    <xsl:variable name="msg" as="xs:string*">
      <xsl:text>Element &lt;</xsl:text>
      <xsl:value-of select="local-name(.)"/>
      <xsl:if test="@*"><xsl:for-each select="@*"><xsl:value-of select="concat(' ', name(), '=&quot;', ., '&quot;')"/></xsl:for-each></xsl:if>
      <xsl:text>> not expected here by the schema</xsl:text>
    </xsl:variable>
    <xsl:variable name="msg" select="string-join($msg, '')" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="$xfe:allowUnValidXML">
        <div class="xfe-error" style="background-color:red; color:white; padding:0.5em; font-family: monospace;">
          <h3 style="margin-bottom:0">ERROR : <xsl:value-of select="$msg"/></h3>
          <p>xpath : <xsl:value-of select="els:get-xpath(.)"/><br/>xfe:srng.uri : <xsl:value-of select="$xfe:srng.uri"/></p>
          <div>
            <xsl:apply-templates select="." mode="els:xml2simpleHtml"/>
          </div>
        </div>
        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
          <xsl:with-param name="code" select="$els:log.level.code"/>
          <xsl:with-param name="level" select="'warning'"/>
          <xsl:with-param name="alert" select="true()"/>
          <xsl:with-param name="markup" select="false()"/>
          <xsl:with-param name="description">[WARNING] <xsl:value-of select="$msg"/></xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
          <xsl:with-param name="code" select="$els:log.level.code"/>
          <xsl:with-param name="level" select="'fatal'"/>
          <xsl:with-param name="alert" select="true()"/>
          <xsl:with-param name="markup" select="false()"/>
          <xsl:with-param name="description">[FATAL] <xsl:value-of select="$msg"/></xsl:with-param>
        </xsl:call-template>
        <xsl:message terminate="yes"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="*[ancestor::*][not($xfe:conf.hasSchema)][count(xfe:getConfMappingElementFromXML(.)) = 0]" mode="xfe:xml2html_main" priority="1">
    <xsl:variable name="msg" as="xs:string*">
      <xsl:text>Element &lt;</xsl:text>
      <xsl:value-of select="local-name(.)"/>
      <xsl:if test="@*"><xsl:for-each select="@*"><xsl:value-of select="concat(' ', name(), '=&quot;', ., '&quot;')"/></xsl:for-each></xsl:if>
      <xsl:text>> is not mapped in the conf. </xsl:text>
      <xsl:value-of select="els:get-xpath(.)"/>
    </xsl:variable>
    <xsl:call-template name="els:log">
      <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
      <xsl:with-param name="code" select="$els:log.level.code"/>
      <xsl:with-param name="level" select="'warning'"/>
      <xsl:with-param name="alert" select="true()"/>
      <xsl:with-param name="markup" select="false()"/>
      <xsl:with-param name="description">[WARNING] <xsl:value-of select="$msg"/> (it might be a namespace prefix problem, because evaluation is based on the root default namespace)</xsl:with-param>
    </xsl:call-template>
    <h3 class="xfe-error" style="margin-bottom:0; background-color:red; color:white; padding:0.5em; font-family: monospace;">ERROR : <xsl:value-of select="$msg" separator=""/></h3>
    <xsl:apply-templates mode="#current"/>
  </xsl:template>
  
  <xsl:template match="*" mode="xfe:xml2html_main">
    <xsl:variable name="self" select="self::*" as="element()"/>
    <xsl:variable name="confMappingElement" select="xfe:getConfMappingElementFromXML(.)" as="element()?"/>
    <xsl:variable name="isXMLrootElement" select="not(ancestor::*)" as="xs:boolean"/>
    <xsl:if test="$confMappingElement/@error">
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="level" select="'error'"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description"><xsl:value-of select="$confMappingElement/@error"/> (<xsl:value-of select="els:get-xpath(.)"/>)</xsl:with-param>
      </xsl:call-template>
    </xsl:if>
    <xsl:variable name="isVoid" select="normalize-space($confMappingElement/@html-name) = 'void'" as="xs:boolean"/>
    <!--<xsl:variable name="e.dataModel" select="if (not($isVoid) and $xfe:conf.hasSchema) then (rng:getSRNGdataModelFromXmlElement(., $xfe:srng.grammar.normalized)) else ()" as="element(rng:element)?"/>-->
    <xsl:variable name="e.dataModel" select="if (not($isVoid) and $xfe:conf.hasSchema) then (key('rng:getDefineByName', @xfe:srng-define, $xfe:srng.grammar.normalized)/rng:element[1]) else ()" as="element(rng:element)?"/>
    <xsl:variable name="htmlElementName" as="xs:string?">
      <xsl:choose>
        <xsl:when test="$isVoid">
          <!--ne pas sortir d'élément ici-->
        </xsl:when>
        <!--l'élément est surchargé dans la conf-->
        <xsl:when test="normalize-space($confMappingElement/@html-name) != ''">
          <xsl:value-of select="$confMappingElement/@html-name"/>
        </xsl:when>
        <!--La racine sera toujours un div ou un body -->
        <xsl:when test="$isXMLrootElement">
          <xsl:choose>
            <xsl:when test="$xfe:srng.grammar.rootIsSingle">
              <xsl:text>body</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>div</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:when test="$xfe:srng.is-xhtml">
          <xsl:value-of select="local-name(.)"/>
        </xsl:when>
        <xsl:when test="not($xfe:conf.hasSchema)">
          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
            <xsl:with-param name="code" select="$els:log.level.code"/>
            <xsl:with-param name="alert" select="true()"/>
            <xsl:with-param name="level" select="'error'"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="description">[FATAL] Unable to find the html name for <xsl:value-of select="els:displayNode(.)"/></xsl:with-param>
          </xsl:call-template>
          <xsl:message terminate="yes"/>
        </xsl:when>
        <xsl:otherwise>
          <!--default inline or block ? we need to check the element data-model in srng schema-->
          <!--<xsl:variable name="e.dataModel" select="rng:getSRNGdataModelFromXmlElement(., $xfe:srng.grammar.normalized)" as="element(rng:element)"/>-->
          <xsl:variable name="e.parent.dataModel" select="rng:getSRNGdataModelFromXmlElement(parent::*, $xfe:srng.grammar.normalized)" as="element(rng:element)"/>
          <!--<xsl:value-of select="els:displayNode($e.dataModel/parent::rng:define)"/>-->
          <xsl:choose>
            <xsl:when test="rng:isInline($e.dataModel, $e.parent.dataModel)">
              <xsl:text>span</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>div</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="$isVoid">
        <xsl:apply-templates mode="#current"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:element name="{$htmlElementName}">
          <xsl:variable name="xmlNameAsClass" select="not(matches($confMappingElement/@html-xmlNameAsClass, '(false|0)'))" as="xs:boolean"/>
          <xsl:variable name="class" as="xs:string*">
            <xsl:if test="$xmlNameAsClass and not($xfe:srng.is-xhtml)">
              <xsl:value-of select="local-name()"/>
            </xsl:if>
            <xsl:if test="$confMappingElement/@html-addClass != ''">
              <xsl:value-of select="$confMappingElement/@html-addClass"/>
            </xsl:if>
            <xsl:if test="$xfe:srng.is-xhtml and @class and not($confMappingElement/@html-ignoreXmlAttributes = 'class')">
              <xsl:value-of select="@class"/>
            </xsl:if>
          </xsl:variable>
          <xsl:call-template name="xfe:addXfeLabel">
            <xsl:with-param name="confMappingElement" select="$confMappingElement" as="element()?"/>
          </xsl:call-template>
          <xsl:if test="string-join($class, ' ') != ''">
            <xsl:attribute name="class" select="string-join($class, ' ')"/>
          </xsl:if>
          <xsl:if test="starts-with($confMappingElement/@type, 'xfe:list(')">
            <xsl:choose>
              <xsl:when test="matches($confMappingElement/@type, 'xfe:list\(.*?\)')">
                <xsl:variable name="listTypeValue.xpath" select="replace($confMappingElement/@type, '^xfe:list\((.*?)\)$', '$1')" as="xs:string"/>
                <xsl:attribute name="{concat($xfe:dataAttribute.prefix, 'xfe-listType')}">
                  <!--<xsl:message>evaluate(<xsl:value-of select="$listTypeValue.xpath"/>, .)=<xsl:value-of select="els:evaluate-xpath($listTypeValue.xpath, .)"/></xsl:message>-->
                  <xsl:if test="normalize-space($listTypeValue.xpath) != ''">
                    <xsl:value-of select="normalize-space(els:evaluate-xpath($listTypeValue.xpath, .))"/>
                  </xsl:if>
                </xsl:attribute>
              </xsl:when>
              <xsl:otherwise>                
                <xsl:call-template name="els:log">
                  <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
                  <xsl:with-param name="code" select="$els:log.level.code"/>
                  <xsl:with-param name="alert" select="true()"/>
                  <xsl:with-param name="level" select="'error'"/>
                  <xsl:with-param name="markup" select="false()"/>
                  <xsl:with-param name="description">[ERROR] @type xfe:list doit être de la forme xfe:list(expressionXpath)</xsl:with-param>
                </xsl:call-template>
                <xsl:message terminate="yes"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:if>
          <xsl:variable name="htmlAttributes" select="$confMappingElement/@*[starts-with(local-name(.), 'html-attribute-')]" as="attribute()*"/>
          <xsl:if test="count($htmlAttributes) != 0">
            <xsl:for-each select="$htmlAttributes">
              <xsl:attribute name="{substring-after(local-name(.), 'html-attribute-')}" select="els:evaluate-xpath(., $self)"/>
            </xsl:for-each>
          </xsl:if>
          <xsl:choose>
            <!--FIXME : ce choose pourrait être mis plus haut mais il faudra gérer les impact sur le html.rng et sur la css, pour l'instant on laisse ici-->
            <!--Common case :-->
            <xsl:when test="not($isXMLrootElement and $xfe:srng.grammar.rootIsSingle)">
              <xsl:for-each select="@* except (@xfe:*|@xf:*|@html:*|(@id|@ID|@Id)[1])">
                <xsl:if test="not(name() = tokenize($confMappingElement/@html-ignoreXmlAttributes, '\s+'))
                  and not($xfe:srng.is-xhtml and name()= 'class')">
                  <xsl:attribute name="{concat(if ($xfe:srng.is-xhtml) then '' else $xfe:dataXmlAttribute.prefix, local-name(.))}">
                    <xsl:choose>
                      <!-- Convert canonical boolean attribute values to integer values (0 / 1) because TinyMCE doesn't handle true / false -->
                      <xsl:when test="rng:getAttributeDataType($e.dataModel,name(.)) = 'boolean'">
                        <xsl:value-of select="els:convertAtomicValueToIntegerBooleanValue(.)"/>
                      </xsl:when>
                      <xsl:otherwise>
                        <xsl:value-of select="."/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:attribute>
                </xsl:if>
              </xsl:for-each>
              <!--Empty href attribute for external links-->
              <!--DESACTIVATION TEMPORAIRE cf. FE-136 / cf. FE-131 : ces attributs vides seront ajouté au clean générique-->
              <!--FIXME : attention cela rend le html invalide par rapport au schéma html.rng => c'est temporaire
              Une autre solution serait de rendre le href optionel dans le schéma ?-->
              <!--<xsl:if test="starts-with($confMappingElement/@type, 'xfe:externalLinkType') and $htmlElementName = 'a'">
                <xsl:attribute name="href" select="''"/>
              </xsl:if>-->
              <!--This href will be override by the resolved relation (in case it has one) that has been calculated before with @html:href-->
              <xsl:for-each select="@html:*">
                <xsl:attribute name="{local-name(.)}" select="."/>
              </xsl:for-each>
              <!--<xsl:for-each select="@xf:refType">
                <xsl:attribute name="{concat($dataXmlAttribute.prefix, 'xf_', local-name(.))}" select="."/>
              </xsl:for-each>-->
            </xsl:when>
            <!--From here we are on the single root element-->
            <xsl:otherwise>
              <xsl:if test="$xfe:conf.hasSchema">
                <xsl:if test="@* except @xfe:*">
                  <xsl:call-template name="els:log">
                    <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
                    <xsl:with-param name="code" select="$els:log.level.code"/>
                    <xsl:with-param name="alert" select="true()"/>
                    <xsl:with-param name="level" select="'error'"/>
                    <xsl:with-param name="markup" select="false()"/>
                    <xsl:with-param name="description">[ERROR] the root element of a single root grammar should not have attributes because the editor will not deal with them : remove attribute(s) <xsl:value-of select="(@* except @xfe:*)/name(.)" separator=", "/></xsl:with-param>
                  </xsl:call-template>
                </xsl:if>
                <xsl:if test="count($confMappingElement)">
                  <xsl:call-template name="els:log">
                    <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
                    <xsl:with-param name="code" select="$els:log.level.code"/>
                    <xsl:with-param name="alert" select="true()"/>
                    <xsl:with-param name="level" select="'error'"/>
                    <xsl:with-param name="markup" select="false()"/>
                    <xsl:with-param name="description">[ERROR] Conf mapping element on the root element of a single root grammar will be ignored</xsl:with-param>
                  </xsl:call-template>
                </xsl:if>
              </xsl:if>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:if test="@id | @ID | @Id">
            <xsl:attribute name="id" select="(@id | @ID | @Id)[1]"/>
          </xsl:if>
          <xsl:if test="normalize-space($confMappingElement/@type) = 'xfe:inlineNote'">
            <xsl:attribute name="xfe_type" select="$confMappingElement/@type"/>
          </xsl:if>
          <xsl:call-template name="xfe:revisionPiTohtml"/>
          <!--<xsl:apply-templates mode="#current"/>-->
        </xsl:element>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="xfe:addXfeLabel">
    <xsl:param name="confMappingElement" select="xfe:getConfMappingElementFromXML(.)" as="element()?"/>
    <xsl:if test="$xfe:addXfeLabel">
      <!--<xsl:attribute name="{$dataAttribute.prefix}xfe-label" ... Error with saxon9.7, cf. http://markmail.org/thread/xoajcp6uemrmkp6q -->
      <xsl:attribute name="{concat($xfe:dataAttribute.prefix, 'xfe-label')}" select="($confMappingElement/@label, local-name(.))[1]"/>
    </xsl:if>
  </xsl:template>
  
  <xsl:template name="xfe:revisionPiTohtml">
    <xsl:for-each-group select="node()" group-starting-with="processing-instruction('xfe_del_start')">
      <xsl:choose>
        <xsl:when test="current-group()[1]/self::processing-instruction('xfe_del_start')">
          <xsl:variable name="piStart" select="current-group()[1]" as="processing-instruction()"/>
          <xsl:for-each-group select="current-group()" group-ending-with="processing-instruction('xfe_del_end')">
            <xsl:choose>
              <xsl:when test="current-group()[last()]/self::processing-instruction('xfe_del_end')">
                <del>
                  <xsl:copy-of select="els:pseudoAttributes2xml($piStart, $els:dquot)"/>
                  <xsl:apply-templates select="current-group()" mode="xfe:xml2html_main"/>
                </del>
              </xsl:when>
              <xsl:otherwise>
                <xsl:call-template name="xfe:insPiTohtml"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:for-each-group>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="xfe:insPiTohtml"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each-group>
  </xsl:template>
  
  <xsl:template name="xfe:insPiTohtml">
    <xsl:for-each-group select="current-group()" group-starting-with="processing-instruction('xfe_ins_start')">
      <xsl:choose>
        <xsl:when test="current-group()[1]/self::processing-instruction('xfe_ins_start')">
          <xsl:variable name="piStart" select="current-group()[1]" as="processing-instruction()"/>
          <xsl:for-each-group select="current-group()" group-ending-with="processing-instruction('xfe_ins_end')">
            <xsl:choose>
              <xsl:when test="current-group()[last()]/self::processing-instruction('xfe_ins_end')">
                <ins>
                  <xsl:copy-of select="els:pseudoAttributes2xml($piStart, $els:dquot)"/>
                  <xsl:apply-templates select="current-group()" mode="xfe:xml2html_main"/>
                </ins>
              </xsl:when>
              <xsl:otherwise>
                <xsl:call-template name="xfe:commentPiTohtml"/>
                <!--<xsl:apply-templates select="current-group()" mode="xfe:xml2html_main"/>-->
              </xsl:otherwise>
            </xsl:choose>
          </xsl:for-each-group>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="xfe:commentPiTohtml"/>
          <!--<xsl:apply-templates select="current-group()" mode="xfe:xml2html_main"/>-->
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each-group>
  </xsl:template>
  
  <xsl:template name="xfe:commentPiTohtml">
    <xsl:for-each-group select="current-group()" group-starting-with="processing-instruction('xfe_comment_start')">
      <xsl:choose>
        <xsl:when test="current-group()[1]/self::processing-instruction('xfe_comment_start')">
          <xsl:variable name="piStart" select="current-group()[1]" as="processing-instruction()"/>
          <xsl:for-each-group select="current-group()" group-ending-with="processing-instruction('xfe_comment_end')">
            <xsl:choose>
              <xsl:when test="current-group()[last()]/self::processing-instruction('xfe_comment_end')">
                <span class="tinyMce_annotation">
                  <xsl:copy-of select="els:pseudoAttributes2xml($piStart, $els:dquot)"/>
                  <xsl:apply-templates select="current-group()" mode="xfe:xml2html_main"/>
                </span>
              </xsl:when>
              <xsl:otherwise>
                <xsl:apply-templates select="current-group()" mode="xfe:xml2html_main"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:for-each-group>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates select="current-group()" mode="xfe:xml2html_main"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each-group>
  </xsl:template>
  
  <!--<xsl:template match="processing-instruction('xfe_comment')" mode="xfe:xml2html_main">
    <span class="tinyMce_annotation">
      <xsl:copy-of select="els:pseudoAttributes2xml(., $els:dquot)"/>
    </span>
  </xsl:template>-->
  
  <!--================================================================-->
  <!-- STEP 3 : xslLib:cals2html -->
  <!--================================================================-->
  <!--cf. xslLib:/cals2html.xsl -->
  
  <!--Override cals2html mode for xfe annotation attributes-->
  <xsl:template match="@xfe:*" mode="xslLib:cals2html.attributes"/>
  
  <!--Consider @security attribute is never usefull for XFE--> 
  <xsl:template match="@security" mode="xslLib:cals2html.attributes"/>
  
  <xsl:template match="@*[name() = concat($xfe:dataAttribute.prefix, 'xfe-label')]" mode="xslLib:cals2html.attributes">
    <xsl:copy-of select="."/>
  </xsl:template>
  
  <!--================================================================-->
  <!-- STEP 4 : moveInlineNotes -->
  <!--================================================================-->
  <!--suppression du contenu de la note en ligne qui est mise dans un div "xfe_notes" plus bas-->
  
  <!--On vide la note en line de son contenu-->
  <xsl:template match="*[@xfe_type = 'xfe:inlineNote']" mode="xfe:xml2html_step3_moveInlineNotes">
    <xsl:copy>
      <xsl:apply-templates select="@* except @xfe_type" mode="#current"/>
      <xsl:attribute name="href" select="concat('#' , xfe:generateInlineNoteId(.))"/>
    </xsl:copy>
  </xsl:template>
  
  <!--Ajout d'un conteneur pour les notes après la racine-->
  <xsl:template match="/html:body[$xfe:conf.normalized//*[@type = 'xfe:inlineNote']]" mode="xfe:xml2html_step3_moveInlineNotes">
    <xsl:call-template name="els:log">
      <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
      <xsl:with-param name="code" select="$els:log.level.code"/>
      <xsl:with-param name="alert" select="true()"/>
      <xsl:with-param name="markup" select="false()"/>
      <xsl:with-param name="description">[INFO] Adding block "xfe_notes" inside html body</xsl:with-param>
    </xsl:call-template>
    <xsl:copy>
      <xsl:apply-templates select="@* | node()" mode="#current"/>
      <div class="xfe_notes">
        <xsl:apply-templates select="//*[@xfe_type = 'xfe:inlineNote']" mode="step3_copyInlineNoteContent"/>
      </div>
    </xsl:copy>
  </xsl:template>
  
  <!--Déplacement des notes dans ce conteneur-->
  <xsl:template match="*[@xfe_type = 'xfe:inlineNote']" mode="step3_copyInlineNoteContent">
    <div class="xfe_note" id="{xfe:generateInlineNoteId(.)}">
      <xsl:apply-templates select="node()" mode="#current"/>
    </div>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xfe:xml2html_step3_moveInlineNotes step3_copyInlineNoteContent">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:function name="xfe:generateInlineNoteId" as="xs:string">
    <xsl:param name="noteRef" as="element()"/>
    <xsl:sequence select="concat('xfe-note_', count($noteRef/preceding::*[@xfe_type = 'xfe:inlineNote']) + 1)"/>
  </xsl:function>
  
  <!--================================================================-->
  <!-- STEP 5 : xml2html_customizeCals2html -->
  <!--================================================================-->
  <!-- cf xslLib:/xslLib/cals2html.xsl -->
  <!-- On surcharge la transfo de la librairie -->
  
  <xsl:template match="html:div[els:hasClass(., 'cals_table')]/@class" mode="xslLib:xml2html_customizeCals2html">
    <xsl:attribute name="class" select="'blocTableau'"/>
    <xsl:attribute name="data-xfe-label" select="'blocTableau'"/>
  </xsl:template>
  
  <!--FIXME : mettre 1 param dans cals2html serait une autre solution-->
  <xsl:template match="@*[starts-with(name(), 'data-cals-')]" mode="xslLib:xml2html_customizeCals2html">
    <xsl:attribute name="{concat($xfe:dataXmlAttribute.prefix, substring-after(name(.), 'data-cals-'))}" select="."/>
  </xsl:template>
  
  <xsl:template match="node() | @*" mode="xslLib:xml2html_customizeCals2html">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--================================================================-->
  <!-- STEP 5 : xfe:xml2html_finaly -->
  <!--================================================================-->
  
  <!--FE-151 : FIXME trouver un mécanisme plus propre, on le mets ici car urgence-->
  <!-- => On rajoute simplement le contenu de $xfe:media.url.path à la valeur de @src qui devrait être du type EFL_TEE_uiCommentaire/chaineXML/XXXX.jpg-->
  <xsl:template match="html:img/@src" mode="xfe:xml2html_finaly">
    <xsl:attribute name="{local-name(.)}" select="concat($xfe:media.url.path, .)"/>
  </xsl:template>
  
  <!--Delete xfe attributes which have been added during processing-->
  <xsl:template match="@xfe:*" mode="xfe:xml2html_finaly"/>
  
  <xsl:template match="node() | @*" mode="xfe:xml2html_finaly">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
</xsl:stylesheet>   