<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:rng="http://relaxng.org/ns/structure/1.0"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  xmlns:xfe="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor"
  xmlns:html="http://www.w3.org/1999/xhtml"
  exclude-result-prefixes="#all"
  version="3.0">
  
  <xsl:import href="../xslLib/xslLib/rng-common.xsl"/>
  <xsl:import href="../xslLib/xslLib/els-log.xsl"/>
    
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Common function and templates for XFE</xd:p>
    </xd:desc>
  </xd:doc>
  
  <xsl:key name="getRelationByXfId" match="xf:relation" use="@xf:id"/>
  
  <!--FE-68 : This debug parameter can be used to desactivate logging to file system when this XSLT is imported : 
  XTDE1480: Cannot execute xsl:result-document while evaluating variable-->
  <xsl:param name="xfe:debug" select="true()" as="xs:boolean"/>
  
  <!--ECM path for externalLink to editorialEntity-->
  <xsl:param name="xfe:ecm.ee.url.path" select="'/#/editorialEntity/'" as="xs:string"/>
  <xsl:param name="xfe:media.url.path" select="'/api/media/original/'" as="xs:string"/>
  
  <!--Par défaut les log sont écrits à côté du xml-->
  <xsl:param name="LOG.URI" select="resolve-uri('log', base-uri(.))" as="xs:string"/>
  <xsl:variable name="log.uri" select="if(ends-with($LOG.URI, '/')) then ($LOG.URI) else(concat($LOG.URI, '/'))" as="xs:string"/>
  
  <!--Overriding css path and name of the conf file is possible-->
  <xsl:param name="xfe:css.name" as="xs:string?"/>
  <xsl:param name="xfe:css.path" as="xs:string?"/>
  
  <!--log level code for logging -->
  <xsl:param name="els:log.level.code" select="'xfe'" as="xs:string"/>
  
  <!--configuration file for XFE-->
  <xsl:param name="xfe:conf.uri" select="resolve-uri('xfe-conf.xml', base-uri(/))" as="xs:string"/>
  <xsl:variable name="xfe:conf.generate-missing-mapping-from-schema" select="false()" as="xs:boolean"/>
  <xsl:variable name="xfe:conf" as="element()">
    <xsl:choose>
      <xsl:when test="doc-available($xfe:conf.uri)">
        <xsl:sequence select="document($xfe:conf.uri)/*"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
          <xsl:with-param name="code" select="$els:log.level.code"/>
          <xsl:with-param name="alert" select="true()"/>
          <xsl:with-param name="level" select="'fatal'"/>
          <xsl:with-param name="markup" select="false()"/>
          <xsl:with-param name="description">document <xsl:value-of select="$xfe:conf.uri"/> is not available</xsl:with-param>
        </xsl:call-template>
        <xsl:message terminate="yes"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="xfe:conf.resolved.get-recursive-imports" as="document-node()">
    <xsl:document>
      <xsl:apply-templates select="$xfe:conf" mode="xfe:conf.resolved.get-recursive-imports"/>
    </xsl:document>
  </xsl:variable>
  <xsl:variable name="xfe:conf.resolved" as="document-node()">
    <xsl:document>
      <xsl:apply-templates select="$xfe:conf.resolved.get-recursive-imports" mode="xfe:resolve-conf"/>
    </xsl:document>
  </xsl:variable>
  <xsl:variable name="xfe:conf.hasSchema" select="exists($xfe:conf/*:srngXmlmodel/@href)" as="xs:boolean"/>
  <xsl:variable name="xfe:srng.href" select="$xfe:conf/*:srngXmlmodel/@href" as="xs:anyURI?"/>
  <xsl:variable name="xfe:srng.uri" select="resolve-uri($xfe:srng.href, $xfe:conf.uri)" as="xs:anyURI?"/>
  <xsl:variable name="xfe:srng" as="document-node()?">
    <xsl:if test="$xfe:conf.hasSchema">
      <xsl:choose>
        <xsl:when test="doc-available($xfe:srng.uri)">
          <xsl:sequence select="document($xfe:srng.uri)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
            <xsl:with-param name="code" select="$els:log.level.code"/>
            <xsl:with-param name="alert" select="true()"/>
            <xsl:with-param name="level" select="'fatal'"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="description">document <xsl:value-of select="$xfe:srng.uri"/> is not available</xsl:with-param>
          </xsl:call-template>
          <xsl:message terminate="yes"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:variable>
  <xsl:variable name="xfe:srng.normalized" as="document-node()?">
    <xsl:if test="$xfe:conf.hasSchema">
      <xsl:document>
        <xsl:apply-templates select="$xfe:srng" mode="xfe:srng_duplicateDefineByAttributeValue"/>
      </xsl:document>
    </xsl:if>
  </xsl:variable>
  <xsl:variable name="xfe:srng.grammar.normalized" select="$xfe:srng.normalized/rng:grammar" as="element(rng:grammar)?"/>
  <xsl:variable name="xfe:conf.normalized" as="element()?">
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="$xfe:conf.resolved" mode="normalizeConf.step1"/>
      </xsl:document>
    </xsl:variable>
    <xsl:variable name="step" as="document-node()">
      <xsl:document>
        <xsl:apply-templates select="$step" mode="normalizeConf.step2"/>
      </xsl:document>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="$xfe:conf.generate-missing-mapping-from-schema">
        <xsl:apply-templates select="$step" mode="normalizeConf.step3"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select="$step/*"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="xfe:srng.grammar.rootIsSingle" select="if($xfe:conf.hasSchema) then(rng:rootIsSingle($xfe:srng.grammar.normalized)) else(true())" as="xs:boolean"/>
  <xsl:variable name="xfe:srng.is-xhtml" select="($xfe:conf/*:srngXmlmodel/@is-xhtml, false())[1]" as="xs:boolean"/>
  <xsl:variable name="xfe:dataAttribute.prefix" select="'data-'" as="xs:string"/>
  <xsl:variable name="xfe:dataXmlAttribute.prefix" select="concat($xfe:dataAttribute.prefix, 'xml-')" as="xs:string"/>
  <xsl:variable name="xfe:dataXfeAttribute.prefix" select="concat($xfe:dataAttribute.prefix, 'xfe-')" as="xs:string"/>

  <!--=================================================-->
  <!--MODE xfe:conf.resolved.get-recursive-imports-->
  <!--=================================================-->
  
  <xsl:template match="/*:conf" mode="xfe:conf.resolved.get-recursive-imports">
    <xsl:param name="rel-uri" as="xs:string?"/>
    <xsl:variable name="uri" select="document-uri(/)" as="xs:anyURI"/>
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:if test="not(empty($rel-uri))">
        <xsl:attribute name="xml:base" select="$rel-uri"/>
      </xsl:if>
      <xsl:apply-templates mode="#current">
        <xsl:with-param name="caller.uri" select="$uri" tunnel="yes"/>
      </xsl:apply-templates>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="*:import" mode="xfe:conf.resolved.get-recursive-imports">
    <xsl:param name="caller.uri" as="xs:anyURI" tunnel="yes"/>
    <xsl:variable name="this.uri" select="resolve-uri(@href,base-uri(.))" as="xs:anyURI"/>
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:if test="$caller.uri != $this.uri">
        <xsl:choose>
          <xsl:when test="doc-available($this.uri)">
            <xsl:apply-templates select="document($this.uri)/*" mode="#current">
              <xsl:with-param name="rel-uri" select="@href"/>
            </xsl:apply-templates>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="els:log">
              <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
              <xsl:with-param name="code" select="$els:log.level.code"/>
              <xsl:with-param name="alert" select="true()"/>
              <xsl:with-param name="level" select="'fatal'"/>
              <xsl:with-param name="markup" select="false()"/>
              <xsl:with-param name="description">[FATAL] import href="<xsl:value-of select="@href"/>" is not available in <xsl:value-of select="base-uri(.)"/></xsl:with-param>
            </xsl:call-template>
            <xsl:message terminate="yes"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>
    </xsl:copy>
  </xsl:template>
  
  <!--copy template-->
  <xsl:template match="node() | @*" mode="xfe:conf.resolved.get-recursive-imports">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--=================================================-->
  <!--MODE xfe:normalizeConf-->
  <!--=================================================-->
  
  <!--Starting resolving conf at document-node()-->
  <xsl:template match="/" mode="xfe:resolve-conf">
    <xsl:choose>
      <xsl:when test="*:conf/*:import">
        <xsl:call-template name="xfe:resolve-conf">
          <xsl:with-param name="conf" select="*:conf" as="element()"/>
          <xsl:with-param name="level" select="count(descendant::*:import)" as="xs:integer"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select="."/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!--Recursiv template to resolve conf step by step, starting at deeper level and getting to 1st level of nested conf-->
  <xsl:template name="xfe:resolve-conf">
    <xsl:param name="conf" required="yes" as="element()"/>
    <xsl:param name="level" required="yes" as="xs:integer"/>
    <xsl:choose>
      <xsl:when test="$level = 0">
        <xsl:sequence select="$conf"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="conf.level-n" as="element()">
          <xsl:apply-templates select="$conf" mode="xfe:resolve-conf">
            <xsl:with-param name="level" select="$level" as="xs:integer" tunnel="yes"/>
          </xsl:apply-templates>
        </xsl:variable>
        <xsl:call-template name="xfe:resolve-conf">
          <xsl:with-param name="conf" select="$conf.level-n" as="element()"/>
          <xsl:with-param name="level" select="$level - 1" as="xs:integer"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!--From here the conf resolution works with elements (no document-node) after being called by the recursiv template name="xfe:resolve-conf"-->
  <xsl:template match="*:conf" mode="xfe:resolve-conf">
    <xsl:param name="level" as="xs:integer" tunnel="yes"/>
    <xsl:choose>
      <xsl:when test="count($level) = 0">
        <xsl:next-match/>
      </xsl:when>
      <xsl:when test="count(ancestor::*:import) = $level -1">
        <xsl:variable name="importedConf" select="*:import/*:conf" as="element()"/>
        <xsl:copy>
          <xsl:copy-of select="@*"/>
          <xsl:choose>
            <xsl:when test="*:srngXmlmodel">
              <xsl:apply-templates select="*:srngXmlmodel" mode="#current"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="$importedConf/*:srngXmlmodel" mode="xfe:resolve-conf.copy-importedConf"/>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:choose>
            <xsl:when test="*:title">
              <xsl:apply-templates select="*:title" mode="#current"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="$importedConf/*:title" mode="xfe:resolve-conf.copy-importedConf"/>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:choose>
            <xsl:when test="*:css">
              <xsl:apply-templates select="*:css" mode="#current"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="$importedConf/*:css" mode="xfe:resolve-conf.copy-importedConf"/>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:choose>
            <xsl:when test="*:mapping">
              <xsl:apply-templates select="*:mapping" mode="#current">
                <xsl:with-param name="importedConf" select="$importedConf" as="element()"/>
              </xsl:apply-templates>
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="$importedConf/*:mapping" mode="xfe:resolve-conf.copy-importedConf"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="*:mapping" mode="xfe:resolve-conf">
    <xsl:param name="importedConf" as="element()?"/>
    <xsl:variable name="self" select="." as="element()"/>
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <!--copy every child (*:element) of the original conf-->
      <xsl:apply-templates mode="#current"/>
      <!--copy every child (*:element) of the imported conf except those which are already defined in the original conf-->
      <xsl:for-each select="$importedConf/*:mapping/*:element">
        <xsl:if test="not(concat(@xpath, '_', @xpath-filter) = $self/*:element/concat(@xpath, '_', @xpath-filter))">
          <xsl:apply-templates select="." mode="xfe:resolve-conf.copy-importedConf"/>
        </xsl:if>
      </xsl:for-each>
    </xsl:copy>
  </xsl:template>
  
  <!--default copy-->
  <xsl:template match="node() | @*" mode="xfe:resolve-conf xfe:resolve-conf.copy-importedConf" priority="-1">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--===== STEP 1 =====-->
  <!--Gestion des @type-->
  
  <!--CALS Tables-->
  <xsl:template match="*:element[@type = 'cals:table']" mode="normalizeConf.step1">
    <xsl:variable name="element.name" select="tokenize(@xpath, '/')[last()]" as="xs:string"/>
    <xsl:variable name="nameCase" select="xfe:getTableCasePattern($element.name)" as="xs:string"/>
    <!--on estime que la casse sera la même sur tous les éléments du langage cals-->
    <!--<xsl:copy-of select="@xpath-filter"/> FIXME : ne s'applique pas au même nœud-->
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <!--FIXME : schematron : interdire @html-name/@html-addClass sur les table, FE-124 => car Tiny est configuré pour (?)-->
      <xsl:attribute name="html-name" select="'div'"/>
      <xsl:attribute name="html-addClass" select="'blocTableau'"/>
      <xsl:attribute name="label" select="(@label, xfe:getNameFromXpath(.))[1]"/>
      <xsl:attribute name="html-xmlNameAsClass" select="'false'"/>
    </xsl:copy>
    <element xpath="{@xpath}/{xfe:apply-case('tgroup', $nameCase)}"  html-name="table" type="cals:tgroup" html-xmlNameAsClass="false"/>
    <element xpath="{@xpath}/tgroup/{xfe:apply-case('colspec', $nameCase)}" html-name="col"   type="cals:colspec" html-xmlNameAsClass="false"/>
    <!--<element xpath="{@xpath}/{xfe:conditional-upper-case('spanspec', $nameCase)}" html-name="?"/>-->
    <element xpath="{@xpath}/{xfe:apply-case('tgroup', $nameCase)}/{xfe:apply-case('thead', $nameCase)}"   html-name="thead" type="cals:thead" html-xmlNameAsClass="false"/>
    <element xpath="{@xpath}/{xfe:apply-case('tgroup', $nameCase)}/{xfe:apply-case('tbody', $nameCase)}"   html-name="tbody" type="cals:tbody" html-xmlNameAsClass="false"/>
    <element xpath="{@xpath}/{xfe:apply-case('tgroup', $nameCase)}/{xfe:apply-case('tfoot', $nameCase)}"   html-name="tfoot" type="cals:tfoot" html-xmlNameAsClass="false"/>
    <element xpath="{@xpath}/{xfe:apply-case('tgroup', $nameCase)}//{xfe:apply-case('row', $nameCase)}"     html-name="tr"    type="cals:row" html-xmlNameAsClass="false"/>
    <element xpath="{@xpath}/{xfe:apply-case('tgroup', $nameCase)}/{xfe:apply-case('tbody', $nameCase)}/{xfe:apply-case('row', $nameCase)}/{xfe:apply-case('entry', $nameCase)}"   html-name="td"    type="cals:entry" html-xmlNameAsClass="false"/>
    <element xpath="{@xpath}/{xfe:apply-case('tgroup', $nameCase)}/{xfe:apply-case('thead', $nameCase)}/{xfe:apply-case('row', $nameCase)}/{xfe:apply-case('entry', $nameCase)}" html-name="th"    type="cals:entry" html-xmlNameAsClass="false"/>
    <!--xpath-filter="ancestor::thead" => génère ambiguité sur élément de surcharge, peut être ancestor::*:thead ?-->
  </xsl:template>
  
  <!--HTML Tables-->
  <xsl:template match="*:element[@type = 'html:table']" mode="normalizeConf.step1">
    <xsl:variable name="element.name" select="tokenize(@xpath, '/')[last()]" as="xs:string"/>
    <xsl:variable name="nameCase" select="xfe:getTableCasePattern($element.name)" as="xs:string"/>
    <!--on estime que la casse sera la même sur tous les éléments du langage cals-->
    <!--<xsl:copy-of select="@xpath-filter"/> FIXME : ne s'applique pas au même nœud-->
    <xsl:copy>
      <xsl:attribute name="html-name" select="'table'"/>
      <xsl:attribute name="html-xmlNameAsClass" select="'false'"/>
      <xsl:copy-of select="@*"/>
    </xsl:copy>
    <element xpath="{@xpath}/{xfe:apply-case('caption', $nameCase)}" html-name="caption" type="html:caption" html-xmlNameAsClass="false"/>
    <element xpath="{@xpath}/{xfe:apply-case('col', $nameCase)}"     html-name="col"     type="html:col"     html-xmlNameAsClass="false"/>
    <element xpath="{@xpath}/{xfe:apply-case('colgroup', $nameCase)}"     html-name="colgroup"     type="html:colgroup"     html-xmlNameAsClass="false"/>
    <element xpath="{@xpath}/{xfe:apply-case('thead', $nameCase)}"   html-name="thead"   type="html:thead"   html-xmlNameAsClass="false"/>
    <element xpath="{@xpath}/{xfe:apply-case('tbody', $nameCase)}"   html-name="tbody"   type="html:tbody"   html-xmlNameAsClass="false"/>
    <element xpath="{@xpath}/{xfe:apply-case('tfoot', $nameCase)}"   html-name="tfoot"   type="html:tfoot"   html-xmlNameAsClass="false"/>
    <element xpath="{@xpath}/{xfe:apply-case('tr', $nameCase)}"      html-name="tr"      type="html:tr"      html-xmlNameAsClass="false"/>
    <!--FIXME : attention le xpath table/td devient faux... et pourtant ça marche ???-->
    <element xpath="{@xpath}/{xfe:apply-case('td', $nameCase)}"      html-name="td"      type="html:td"      html-xmlNameAsClass="false"/>
    <element xpath="{@xpath}/{xfe:apply-case('th', $nameCase)}"      html-name="th"      type="html:th"      html-xmlNameAsClass="false"/>
  </xsl:template>
  
  <xsl:template match="*:element[starts-with(@type, 'xfe:title-')]" mode="normalizeConf.step1">
    <xsl:variable name="level" select="substring-after(@type, 'xfe:title-')" as="xs:string"/>
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <!--Enforce some attributes-->
      <!--cf. schematron conf-->
      <xsl:attribute name="html-addClass" select="normalize-space(concat('xfe_title-', $level, ' ', @html-addClass))"/>
      <!--Don't set @html-name : it could be set to h1, h2, etc. or div, depending on the xml-->
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="*:element[starts-with(@type, 'xfe:list(')]" mode="normalizeConf.step1">
    <xsl:copy>
      <xsl:attribute name="html-name" select="'ul'"/> <!--default value, can be overrided-->
      <xsl:apply-templates select="@* | node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="*:element[starts-with(@type, 'xfe:externalLinkType')]" mode="normalizeConf.step1">
    <xsl:copy>
      <!--Enforce some attributes-->
      <xsl:attribute name="html-name" select="'a'"/>
      <!--FIXME :to avoid impact : keep @html-name when specified-->
      <xsl:apply-templates select="@*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="*:element[@type = 'xfe:inlineNote']" mode="normalizeConf.step1">
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <!--Enforce some attributes-->
      <!--cf. schematron conf-->
      <xsl:attribute name="html-name" select="'a'"/>
      <xsl:attribute name="html-attribute-href">
        <xsl:text>''</xsl:text>
        <!--valeur sera générée dans XML2HTML (pour avoir des generate-id() homogènes), mais le modèle a besoin qu'on définisse la surcharge ici-->
      </xsl:attribute>
      <xsl:attribute name="html-addClass" select="normalize-space(concat('xfe_noteRef', ' ', @html-addClass))"/>
      <!--<xsl:attribute name="html-xmlNameAsClass" select="'false'"/> non car on pourrait mélanger les note inline et les autres à la reconstruction du XML-->
    </xsl:copy>
  </xsl:template>
  
  <!--FIXME ! ATTENTION le fait de remplacer les attribut id et idRef par des nouveaux attributs change leur type (et donc ceux des autres éléments html du même nom adjacents)
  => il faudrait plutôt avoir une instruction pour changer le nom d'un attribut :
  <attribute xpath="toto/@foo" html-name="bar"/>
  Sinon c'est surtout @IDNOT sur les note qui doit être transformé en @id, à généraliser (attribut qui sont des id mais n'ont pas le bon nom) hors de notes ?  -->
  
  <xsl:template match="*:element[starts-with(@type, 'xfe:noteRef(')]" mode="normalizeConf.step1">
    <xsl:choose>
      <xsl:when test="matches(@type, 'xfe:noteRef\(@\w+\)')">
        <xsl:variable name="idRefAttributeName" select="replace(@type, '^xfe:noteRef\(@(\w+)\)$', '$1')" as="xs:string"/>
        <xsl:copy>
          <xsl:apply-templates select="@*" mode="#current"/>
          <!--Enforce some attributes-->
          <!--cf. schematron conf-->
          <xsl:attribute name="html-name" select="'a'"/>
          <xsl:attribute name="html-attribute-href">
            <!--xpath évalué à ne pas mettre en select-->
            <xsl:text>concat('#', @</xsl:text>
            <xsl:value-of select="$idRefAttributeName"/>
            <xsl:text>)</xsl:text>
          </xsl:attribute>
          <xsl:attribute name="html-addClass" select="normalize-space(concat('xfe_noteRef', ' ', @html-addClass))"/>
          <xsl:attribute name="html-ignoreXmlAttributes" select="normalize-space(concat(@html-ignoreXmlAttributes, ' ', $idRefAttributeName))"/>
          <!--<xsl:attribute name="html-xmlNameAsClass" select="'false'"/> non car on pourrait mélanger les note inline et les autres à la reconstruction du XML-->
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
          <xsl:with-param name="code" select="$els:log.level.code"/>
          <xsl:with-param name="alert" select="true()"/>
          <xsl:with-param name="level" select="'error'"/>
          <xsl:with-param name="markup" select="false()"/>
          <xsl:with-param name="description">[ERROR] @type xfe:noteRef must include an attribute name used for idRef. it should look like xfe:noteRef(@idRefAttributeName)</xsl:with-param>          
        </xsl:call-template>
        <xsl:message terminate="yes"/>
      </xsl:otherwise>  
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="*:element[starts-with(@type, 'xfe:note(')]" mode="normalizeConf.step1">
    <xsl:choose>
      <xsl:when test="matches(@type, 'xfe:note\(@\w+\)')">
        <xsl:variable name="idAttributeName" select="replace(@type, '^xfe:note\(@(\w+)\)$', '$1')" as="xs:string"/>
        <xsl:copy>
          <xsl:apply-templates select="@*" mode="#current"/>
          <!--Enforce some attributes-->
          <!--cf. schematron conf-->
          <xsl:attribute name="html-attribute-id">
            <!--xpath évalué à ne pas mettre en select-->
            <xsl:text>@</xsl:text>
            <xsl:value-of select="$idAttributeName"/>
          </xsl:attribute>
          <xsl:attribute name="html-addClass" select="normalize-space(concat('xfe_note', ' ', @html-addClass))"/>
          <xsl:attribute name="html-ignoreXmlAttributes" select="normalize-space(concat(@html-ignoreXmlAttributes, ' ', $idAttributeName))"/>
          <!--<xsl:attribute name="html-xmlNameAsClass" select="'false'"/> non car on pourrait mélanger les note inline et les autres à la reconstruction du XML-->
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
          <xsl:with-param name="code" select="$els:log.level.code"/>
          <xsl:with-param name="alert" select="true()"/>
          <xsl:with-param name="level" select="'error'"/>
          <xsl:with-param name="markup" select="false()"/>          
          <xsl:with-param name="description">[ERROR] @type xfe:note must include an attribute name used for idRef. it should look like xfe:note(@idRefAttributeName)</xsl:with-param>
        </xsl:call-template>
        <xsl:message terminate="yes"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="*:element[starts-with(@type, 'xfe:multimediaLink(')]" mode="normalizeConf.step1">
    <xsl:choose>
      <xsl:when test="matches(@type, 'xfe:multimediaLink\(@\w+\)')">
        <xsl:variable name="idRefAttributeName" select="replace(@type, '^xfe:multimediaLink\(@(\w+)\)$', '$1')" as="xs:string"/>
        <xsl:copy>
          <xsl:apply-templates select="@*" mode="#current"/>
          <!--Enforce some attributes-->
          <!--cf. schematron conf-->
          <xsl:attribute name="html-name" select="'a'"/>
          <xsl:attribute name="html-attribute-href">
            <!--xpath évalué à ne pas mettre en select-->
            <xsl:text>concat('</xsl:text>
            <xsl:value-of select="$xfe:media.url.path"/>
            <xsl:text>', @</xsl:text>            
            <xsl:value-of select="$idRefAttributeName"/>
            <xsl:text>)</xsl:text>
          </xsl:attribute>
          <xsl:attribute name="html-addClass" select="normalize-space(concat('xfe_multimediaLink', ' ', @html-addClass))"/>
          <xsl:attribute name="html-ignoreXmlAttributes" select="normalize-space(concat(@html-ignoreXmlAttributes, ' ', $idRefAttributeName))"/>
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())" as="xs:string"/>
          <xsl:with-param name="code" select="$els:log.level.code" as="xs:string"/>
          <xsl:with-param name="alert" select="true()" as="xs:boolean"/>
          <xsl:with-param name="level" select="'error'" as="xs:string"/>
          <xsl:with-param name="markup" select="false()" as="xs:boolean"/>
          <xsl:with-param name="description">[ERROR] @type xfe:multimediaLink must include an attribute name used for idRef. it should look like xfe:multimediaLink(@idRefAttributeName)</xsl:with-param>
        </xsl:call-template>
        <xsl:message terminate="yes"/>
      </xsl:otherwise>  
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="*:element[starts-with(@type, 'xfe:webLink(')]" mode="normalizeConf.step1">
    <xsl:choose>
      <xsl:when test="matches(@type, 'xfe:webLink\(@\w+\)')">
        <xsl:variable name="idRefAttributeName" select="replace(@type, '^xfe:webLink\(@(\w+)\)$', '$1')" as="xs:string"/>
        <xsl:copy>
          <xsl:apply-templates select="@*" mode="#current"/>
          <!--Enforce some attributes-->
          <xsl:attribute name="html-name" select="'span'"/>
          <xsl:attribute name="html-attribute-href">
            <!--xpath évalué à ne pas mettre en select-->
            <xsl:text>@</xsl:text>            
            <xsl:value-of select="$idRefAttributeName"/>
          </xsl:attribute>
          <xsl:attribute name="html-addClass" select="normalize-space(concat('xfe_webLink', ' ', @html-addClass))"/>
          <xsl:attribute name="html-ignoreXmlAttributes" select="normalize-space(concat(@html-ignoreXmlAttributes, ' ', $idRefAttributeName))"/>
          <xsl:if test="not(@label)">
            <xsl:attribute name="label" select="'webLink'"/>
          </xsl:if>
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
          <xsl:with-param name="code" select="$els:log.level.code"/>
          <xsl:with-param name="alert" select="true()"/>
          <xsl:with-param name="level" select="'error'"/>
          <xsl:with-param name="markup" select="false()"/>
          <xsl:with-param name="description">[ERROR] @type xfe:webLink must include an attribute name used for href. it should look like xfe:webLink(@myRef)</xsl:with-param>
        </xsl:call-template>
        <xsl:message terminate="yes"/>
      </xsl:otherwise>  
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="*:element[@type = 'html:table-container']" mode="normalizeConf.step1">
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <xsl:attribute name="html-addClass" select="'blocTableau'"/>
      <xsl:attribute name="html-ignoreXmlAttributes" select="'class'"/>
    </xsl:copy>
  </xsl:template>
  
  <!--===== STEP 2 =====-->
  
  <!-- - Add @srng-defines attribute (helpfull for the mapping between the conf and the srng schema)
       - Add automatique @label when none is defined
       - When $xfe:is-xhtml is true : add some default attributes
  -->
  <xsl:template match="*:element" mode="normalizeConf.step2">
    <xsl:copy>
      <!--<xsl:if test="$xfe:srng.is-xhtml">
        <xsl:attribute name="html-name" select="xfe:getNameFromXpath(.)"/>
        <xsl:attribute name="html-xmlNameAsClass" select="'false'"/> 
        <!-\-<xsl:attribute name="html-ignoreXmlAttributes" select="'class'"/> : they all will be ignore in the xsl code-\->
      </xsl:if>-->
      <!--thoses added attributes will be overriden by already setted attributes-->
      <xsl:apply-templates select="@*" mode="#current"/>
      <xsl:attribute name="label" select="(@label, tokenize(@xpath, '/')[last()])[1]"/>
      <xsl:if test="$xfe:conf.hasSchema">
        <xsl:attribute name="srng-defines">
          <xsl:variable name="xpath.token" select="tokenize(@xpath, '/')" as="xs:string*"/>
          <xsl:choose>
            <!--@xpath is absolute-->
            <xsl:when test="starts-with(@xpath, '/') and not(starts-with(@xpath, '//'))">
              <xsl:value-of select="rng:getSRNGdataModelFromXpath(@xpath, $xfe:srng.grammar.normalized, @xpath-filter)/parent::rng:define/@name"/>
              <!--<xsl:value-of select="rng:getSRNGdataModelFromXpath(@xpath, $xfe:srng.grammar.normalized)/parent::rng:define/@name"/>-->
            </xsl:when>
            <xsl:otherwise>
              <xsl:variable name="xpath.elementName" select="xfe:getNameFromXpath(.)" as="xs:string"/>
              <xsl:variable name="rngDefinesByName" select="$xfe:srng.grammar.normalized//rng:define[rng:element[1][@name = $xpath.elementName]]" as="element(rng:define)*"/>
              <xsl:choose>
                <!--No define with this name in the schema : this is an error in the conf (cf. schematron "no-element-with-this-name-in-the-schema")-->
                <xsl:when test="count($rngDefinesByName) = 0">
                  <xsl:call-template name="els:log">
                    <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())" as="xs:string"/>
                    <xsl:with-param name="code" select="$els:log.level.code" as="xs:string"/>
                    <xsl:with-param name="alert" select="true()" as="xs:boolean"/>
                    <xsl:with-param name="level" select="'fatal'" as="xs:string"/>
                    <xsl:with-param name="markup" select="false()" as="xs:boolean"/>
                    <xsl:with-param name="description">[FATAL] 0 define pour <xsl:copy-of select="$xpath.token"/> (dans conf /element/@xpath='<xsl:value-of select="@xpath"/>')</xsl:with-param>
                  </xsl:call-template>
                  <xsl:message terminate="yes"/>
                </xsl:when>
                <!--Only one define with this name in the schema : it *must* be this one-->
                <xsl:when test="count($rngDefinesByName) = 1">
                  <xsl:value-of select="$rngDefinesByName/@name"/>
                </xsl:when>
                <!--The xpath is a single element name : at least select the defines with the same name -->
                <xsl:when test="count($xpath.token) = 1">
                  <!--If the xpath-filter is of kind @foo = 'bar', maybe there is such a static define in the schema (<attribute name="foo"><value>bar</value></attribute>)
                    (and no other same element's name with the same attribute's name which may have the same value)-->
                  <xsl:variable name="xpath-filter" select="normalize-space(@xpath-filter)" as="xs:string"/>
                  <!--Construct a fake element with fixed attribute found in each selected define so we can evaluate the xpath-filter and discriminate the unmatched ones-->  
                  <xsl:variable name="fake.elements" as="element()">
                    <fake-elements>
                      <xsl:for-each select="$rngDefinesByName">
                        <xsl:element name="{$xpath.token}">
                          <xsl:attribute name="rngDefineName" select="@name"/>
                          <xsl:for-each select="descendant::rng:attribute[count(*)=1][rng:value]">
                            <xsl:attribute name="{@name}" select="rng:value/text()"/>
                          </xsl:for-each>
                        </xsl:element>
                      </xsl:for-each>
                    </fake-elements>
                  </xsl:variable>
                  <xsl:variable name="xpathToEvaluate" select="concat('*', if($xpath-filter != '') then(concat('[', $xpath-filter,']')) else(''))" as="xs:string"/>
                  <!--<xsl:message>CONF-ELEMENT <xsl:copy-of select="."/></xsl:message>
                  <xsl:message><xsl:copy-of select="$fake.elements/*"/></xsl:message>
                  <xsl:message>xpathToEvaluate = <xsl:value-of select="$xpathToEvaluate"/></xsl:message>
                  <xsl:message>result: <xsl:sequence select="els:evaluate-xpath($xpathToEvaluate, $fake.elements)"/></xsl:message>-->
                  <xsl:variable name="rngDefineNames.xpath-filtered" select="els:evaluate-xpath($xpathToEvaluate, $fake.elements)/@rngDefineName" as="xs:string*"/>
                  <xsl:variable name="rngDefinesByName.xpath-filtered" select="$rngDefinesByName[@name = $rngDefineNames.xpath-filtered]" as="element()*"/>                      
                  <xsl:choose>
                    <xsl:when test="count($rngDefinesByName.xpath-filtered) = 0">
                      <xsl:value-of select="$rngDefinesByName/@name" separator=" "/>
                      <xsl:call-template name="els:log">
                        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())" as="xs:string"/>
                        <xsl:with-param name="code" select="$els:log.level.code" as="xs:string"/>
                        <xsl:with-param name="alert" select="true()" as="xs:boolean"/>
                        <xsl:with-param name="level" select="'error'" as="xs:string"/>
                        <xsl:with-param name="markup" select="false()" as="xs:boolean"/>
                        <xsl:with-param name="description">[ERROR] 0 define found for <xsl:copy-of select="$xpath.token"/> (in conf : /element/@xpath='<xsl:value-of select="@xpath"/>')</xsl:with-param>
                      </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                      <xsl:value-of select="$rngDefinesByName.xpath-filtered/@name" separator=" "/>
                    </xsl:otherwise>
                  </xsl:choose>
                </xsl:when>
                <xsl:otherwise>
                  <!--Solution temporaire sur le parent uniquement :-->
                  <xsl:variable name="xpath.parentName" select="$xpath.token[last() - 1]" as="xs:string"/>
                  <xsl:variable name="rngDefines.parent" select="$xfe:srng.grammar.normalized//rng:define[rng:element[1][@name = $xpath.parentName]]" as="element(rng:define)*"/>
                  <xsl:variable name="rngDefines.filter" as="element(rng:define)*">
                    <xsl:for-each select="$rngDefinesByName">
                      <xsl:variable name="rngDefine" select="." as="element(rng:define)"/>
                      <xsl:if test="$rngDefines.parent[rng:defineHasRefName(., $rngDefine/@name)]">
                        <xsl:sequence select="."/>
                      </xsl:if>
                    </xsl:for-each>
                  </xsl:variable>
                  <xsl:value-of select="$rngDefines.filter/@name" separator=" "/>
                  <!--<xsl:call-template name="filterConfAmbiguousRngDefinesOnRelativeXpath">
                  <xsl:with-param name="defines" select="$rngDefines"/>
                  <xsl:with-param name="xpath.token" select="$xpath.token"/>
                  <xsl:with-param name="xpath.token.position" select="index-of($xpath.token, last())"/>
                </xsl:call-template>-->
                </xsl:otherwise>
              </xsl:choose>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
      </xsl:if>
      <xsl:apply-templates mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!-- FIXME A FINIR !
    <xsl:template name="filterConfAmbiguousRngDefinesOnRelativeXpath">
    <xsl:param name="defines" required="yes" as="element(rng:define)*"/>
    <xsl:param name="xpath.tokens" required="yes" as="xs:string*"/>
    <xsl:param name="xpath.token.position" required="yes" as="xs:integer"/>
    <xsl:choose>
      <xsl:when test="count($defines) = 1">
        <xsl:sequence select="$defines"/>
      </xsl:when>
      <xsl:when test="$xpath.token.position = 0">
        <xsl:sequence select="$defines"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="current.xpath.name" select="$xpath.tokens[$xpath.token.position]" as="xs:string"/>
        <xsl:variable name="rngDefines" select="$srng.grammar//rng:define[rng:element[1][@name = $current.xpath.name]]" as="element(rng:define)*"/>
        <xsl:choose>
          <!-\-Il n'y a qu'un seul élément avec ce nom définit dans le modèle-\->
          <xsl:when test="count($rngDefines) = 1">
            
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="filterConfAmbiguousRngDefinesOnRelativeXpath">
              <xsl:with-param name="defines" select="$rngDefines"/>
              <xsl:with-param name="xpath.token" select="remove($xpath.tokens, last())"/>
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>-->
  
  <!--adapte les @match pour le rendre "absolu" (//) => de type "template rule"-->
  <xsl:template match="@xpath" mode="normalizeConf.step2">
    <xsl:attribute name="{name(.)}">
      <xsl:choose>
        <xsl:when test="starts-with(., '/')">
          <xsl:value-of select="."/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat('//', .)"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
  </xsl:template>
  
  <!--===== STEP 3 =====-->

  <xsl:template match="/*:conf/*:mapping" mode="normalizeConf.step3">
    <xsl:variable name="self" select="self::*" as="element()"/>
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <xsl:apply-templates select="node()" mode="#current"/>
      <xsl:comment>==============================</xsl:comment>
      <xsl:comment> GENERATED FROM SCHEMA</xsl:comment>
      <xsl:comment>==============================</xsl:comment>
      <xsl:apply-templates select="$xfe:srng/rng:grammar/rng:define" mode="normalizeConf.step3.iterate-srng">
        <xsl:with-param name="mapping" select="$self" as="element()"/>
      </xsl:apply-templates>
    </xsl:copy>
  </xsl:template>
  
  <!--Exclude root define(s)-->
  <xsl:template match="rng:define[@name = rng:getRootDefines(/rng:grammar)/@name]" mode="normalizeConf.step3.iterate-srng">
    <xsl:message use-when="false()"><xsl:value-of select="@name"/> is root define, non need to add it in the conf</xsl:message>
  </xsl:template>
  
  <xsl:template match="rng:define" mode="normalizeConf.step3.iterate-srng">
    <xsl:param name="mapping" as="element()"/>
    <xsl:variable name="rng:element" select="rng:element" as="element(rng:element)"/>
    <xsl:variable name="rng:element.name" select="rng:element/@name" as="xs:string"/>
    <!--FIXME : we only check for parents define here, we should actualy look at the full possible context and check against the conf specficied contexts-->
    <!--FIXME : cals table context is more precise than only parents : we have to exclude them too-->
    <xsl:variable name="parent.names" select="key('rng:getRefByName', @name)/ancestor::rng:define/rng:element[1]/@name" as="xs:string*"/>
    <xsl:variable name="unspecified-parent-context-in-conf" as="xs:string*">
      <xsl:choose>
        <xsl:when test="count($mapping/*:element[@xpath = concat('//', $rng:element/@name)]) != 0">
          <!--empty sequence-->
        </xsl:when>
        <xsl:otherwise>
          <xsl:variable name="conf.specified.contexts" select="$mapping/*:element[xfe:getNameFromXpath(.) = $rng:element.name]/@xpath" as="xs:string*"/>
          <xsl:for-each select="$parent.names">
            <xsl:variable name="context" select="concat('//', ., '/', $rng:element.name)" as="xs:string"/>
            <xsl:if test="not($context = $conf.specified.contexts)">
              <xsl:value-of select="."/>
            </xsl:if>
          </xsl:for-each>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="@name = 'infoCommentaire'">
      <xsl:message><xsl:value-of select="@name"/> : <xsl:value-of select="$unspecified-parent-context-in-conf"/></xsl:message>
    </xsl:if>
    <xsl:choose>
      <xsl:when test="count($unspecified-parent-context-in-conf) = 0"/>
      <xsl:otherwise>
        <xsl:variable name="html.default.name" as="xs:string">
          <xsl:choose>
            <xsl:when test="rng:isInlineOnly($rng:element, $unspecified-parent-context-in-conf)">
              <xsl:text>span</xsl:text>
            </xsl:when>
            <xsl:when test="rng:isBlockOnly($rng:element, $unspecified-parent-context-in-conf)">
              <xsl:text>div</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>spanOrdiv</xsl:text>
              <xsl:call-template name="els:log">
                <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
                <xsl:with-param name="code" select="$els:log.level.code"/>
                <xsl:with-param name="alert" select="true()"/>
                <xsl:with-param name="level" select="'fatal'"/>
                <xsl:with-param name="markup" select="false()"/>
                <xsl:with-param name="xpathContext" select="'_null'"/>
                <xsl:with-param name="description">[normalizeConf.step3.iterate-srng] Unable to add missing conf element for srng define "<xsl:value-of select="@name"/>". 
                  The element might be inline or block depending on the context. Please remove this ambiguity (see the documentation at main/conf/xfe-conf.md)</xsl:with-param>
              </xsl:call-template>
              <xsl:message terminate="yes"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <element xpath="{concat('//', $rng:element/@name)}" html-name="{$html.default.name}" srng-defines="{self::rng:define/@name}"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!--===== STEP common =====-->
  
  <xsl:template match="node() | @*" mode="normalizeConf.step1 normalizeConf.step2 normalizeConf.step3">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--=========================================-->
  <!--SRNG Grammar Normalized -->
  <!--=========================================-->
  
  <!--Because tinyMCE interface read the different ref/define to propose adding a node, 
    we duplicate some define for which we want the user don't have to make a choice in attributes values 
    but directly add the good item with a fixed attribute value-->
  
  <xsl:template match="rng:define" mode="xfe:srng_duplicateDefineByAttributeValue">
    <xsl:variable name="self" select="self::*" as="element()"/>
    <xsl:variable name="duplicate-define" select="xfe:srng_getConfDuplicatedDefine(.)" as="element()?"/>
      <!--<xsl:message> rng:define name= <xsl:value-of select="@name"/> - count duplicate-define = <xsl:value-of select="count($duplicate-define)"/>
    </xsl:message>-->
    <xsl:choose>
      <xsl:when test="count($duplicate-define) != 0">
        <xsl:variable name="attribute-name" select="$duplicate-define/@attribute-name" as="xs:string"/>
        <xsl:for-each select=".//rng:attribute[@name = $attribute-name]/rng:choice/rng:value">
          <xsl:variable name="value" select="." as="xs:string"/>
          <xsl:apply-templates select="$self" mode="xfe:srng_duplicateDefineByAttributeValue-copy">
            <xsl:with-param name="attribute-name" select="$attribute-name" tunnel="yes"/>
            <xsl:with-param name="value" select="$value" tunnel="yes"/>
          </xsl:apply-templates>
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="rng:define" mode="xfe:srng_duplicateDefineByAttributeValue-copy">
    <xsl:param name="value" as="xs:string" tunnel="yes"/>
    <xsl:copy copy-namespaces="no">
      <xsl:apply-templates select="@*" mode="#current"/>
      <xsl:attribute name="name" select="xfe:srng_duplicatedDefineRenameDef(@name, $value)"/>
      <xsl:apply-templates select="node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--Delete the choice on attribute value, because we will keep only one value-->
  <xsl:template match="rng:attribute/rng:choice" mode="xfe:srng_duplicateDefineByAttributeValue-copy">
    <xsl:param name="attribute-name" as="xs:string" tunnel="yes"/>
    <xsl:param name="value" as="xs:string" tunnel="yes"/>
    <xsl:choose>
      <xsl:when test="ancestor::rng:attribute[1]/@name = $attribute-name">
        <!--delete choice--> 
        <xsl:apply-templates mode="#current"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!--Delete other attributes values-->
  <xsl:template match="rng:attribute/rng:choice/rng:value" mode="xfe:srng_duplicateDefineByAttributeValue-copy">
    <xsl:param name="attribute-name" as="xs:string" tunnel="yes"/>
    <xsl:param name="value" as="xs:string" tunnel="yes"/>
    <xsl:choose>
      <xsl:when test="ancestor::rng:attribute[1]/@name = $attribute-name and . != $value">
        <!--delete other values--> 
      </xsl:when>
      <xsl:otherwise>
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!--Generate multiple ref where there duplicated define is referenced--> 
  <xsl:template match="rng:ref" mode="xfe:srng_duplicateDefineByAttributeValue xfe:srng_duplicateDefineByAttributeValue-copy">
    <xsl:variable name="self" select="self::*" as="element()"/>
    <xsl:variable name="define" select="rng:getDefine(.)" as="element(rng:define)"/>
    <xsl:variable name="duplicate-define" select="xfe:srng_getConfDuplicatedDefine($define)" as="element()?"/>
    <xsl:variable name="duplicated-ref" as="element(rng:ref)*">
      <xsl:for-each select="$define//rng:attribute[@name = $duplicate-define/@attribute-name]/rng:choice/rng:value">
        <ref name="{xfe:srng_duplicatedDefineRenameDef($self/@name, .)}" xmlns="http://relaxng.org/ns/structure/1.0"/>
      </xsl:for-each>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="count($duplicate-define) != 0">
        <xsl:choose>
          <!--No need to add a choice if the context is already a choice-->
          <xsl:when test="parent::rng:choice">
            <xsl:sequence select="$duplicated-ref"/>
          </xsl:when>
          <xsl:otherwise>
            <choice xmlns="http://relaxng.org/ns/structure/1.0">
              <xsl:sequence select="$duplicated-ref"/>
            </choice>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:function name="xfe:srng_duplicatedDefineRenameDef" as="xs:string">
    <xsl:param name="define.name" as="xs:string"/>
    <xsl:param name="attribute.value" as="xs:string"/>
    <xsl:value-of select="concat($define.name, '_', normalize-space(string-join(tokenize($attribute.value, '\s+'), '_')))"/>
  </xsl:function>
  
  <xsl:function name="xfe:srng_getConfDuplicatedDefine" as="element()?">
    <xsl:param name="define" as="element(rng:define)"/>
    <xsl:sequence select="$xfe:conf.resolved/*:conf/*:srngXmlmodel/*:duplicate-define
      [@element-name =  $define/rng:element/@name]
      [@attribute-name = $define//rng:attribute[count(*) = 1][rng:choice][count(rng:choice/rng:value) = count(rng:choice/*)]/@name]"/>
  </xsl:function>
  
  <!--default copy-->
  <xsl:template match="node() | @*" 
    mode="xfe:srng_duplicateDefineByAttributeValue
    xfe:srng_duplicateDefineByAttributeValue-copy">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--==================================================================-->
  <!--COMMON FUNCTIONS-->
  <!--==================================================================-->
  
  <!--=================================================-->
  <!--CONF/RNG functions-->
  <!--=================================================-->
  
  <!--Un template a appeler pour logger la conf normalisée-->
  <xsl:template name="xfe:log.normalizeConf">
    <xsl:if test="$xfe:debug">
      <xsl:variable name="step.log.uri" select="resolve-uri('conf.resolved.xml', $log.uri)" as="xs:anyURI"/>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template>
      <xsl:result-document href="{$step.log.uri}" format="els:xml">
        <xsl:sequence select="$xfe:conf.resolved"/>
      </xsl:result-document>
      <xsl:variable name="step.log.uri" select="resolve-uri('conf.normalized.xml', $log.uri)" as="xs:anyURI"/>
      <xsl:call-template name="els:log">
        <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
        <xsl:with-param name="code" select="$els:log.level.code"/>
        <xsl:with-param name="alert" select="true()"/>
        <xsl:with-param name="markup" select="false()"/>
        <xsl:with-param name="description">[INFO] writing <xsl:value-of select="$step.log.uri"/></xsl:with-param>
      </xsl:call-template>
      <xsl:result-document href="{$step.log.uri}" format="els:xml">
        <xsl:sequence select="$xfe:conf.normalized"/>
      </xsl:result-document>
      <xsl:if test="$xfe:conf.hasSchema">
        <xsl:variable name="step.log.uri" select="resolve-uri('srng.grammar.normalized.srng', $log.uri)" as="xs:anyURI"/>
        <xsl:result-document href="{$step.log.uri}" format="els:xml">
          <xsl:sequence select="$xfe:srng.grammar.normalized"/>
        </xsl:result-document>
      </xsl:if>
    </xsl:if>
  </xsl:template>
  
  <!--In conf.normalized.xml we added on each element an @srng-define attribute that lists all defines in the schema-->
  <!--FIXME : préfixe xfe: au lieu de xf:-->
  <xsl:function name="xf:getConfMappingElementsFromSrng" as="element()*">
    <xsl:param name="rngElement" as="element(rng:element)"/>
    <xsl:variable name="define" select="$rngElement/parent::rng:define" as="element(rng:define)"/>
    <xsl:sequence select="$xfe:conf.normalized/*:mapping/*:element[xfe:confElementHasRngDefine(., $define/@name)]"/>
  </xsl:function>
  
  <xsl:function name="xfe:getTableCasePattern" as="xs:string">
    <xsl:param name="elementName" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="upper-case($elementName) = $elementName">
        <xsl:value-of select="'uppercase'"/>
      </xsl:when>
      <xsl:when test="lower-case($elementName) = $elementName">
        <xsl:value-of select="'lowercase'"/>
      </xsl:when>
      <xsl:when test="upper-case(substring($elementName, 1, 1)) = substring($elementName, 1, 1) and lower-case(substring($elementName, 2)) = substring($elementName, 2)">
        <xsl:value-of select="'titlecase'"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="'error'"/>
        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
          <xsl:with-param name="code" select="$els:log.level.code"/>
          <xsl:with-param name="alert" select="true()"/>
          <xsl:with-param name="level" select="'fatal'"/>
          <xsl:with-param name="markup" select="false()"/>
          <xsl:with-param name="xpathContext" select="'_null'"/>
          <xsl:with-param name="description">[FATAL][xfe:getTableCasePattern] Impossible de déduire le modèle de nommage des tableaux (uppercase, lowercase ot titlecase)</xsl:with-param>
        </xsl:call-template>
        <xsl:message terminate="yes"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="xfe:apply-case" as="xs:string">
    <xsl:param name="string" as="xs:string"/>
    <xsl:param name="nameCase" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="$nameCase = 'uppercase'">
        <xsl:sequence select="upper-case($string)"/>
      </xsl:when>
      <xsl:when test="$nameCase = 'lowercase'">
        <xsl:sequence select="lower-case($string)"/>
      </xsl:when>
      <xsl:when test="$nameCase = 'titlecase'">
        <xsl:sequence select="concat(upper-case(substring($string, 1, 1)), lower-case(substring($string, 2)))"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select="$string"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <!--=================================================-->
  <!--XML2HTML functions-->
  <!--=================================================-->
  
  <!--1 arg signature-->
  <xsl:function name="xfe:getConfMappingElementsFromXML" as="element()*">
    <xsl:param name="e" as="element()"/>
    <xsl:choose>
      <xsl:when test="$xfe:conf.hasSchema">
        <xsl:variable name="e.define.name" as="xs:string">
          <xsl:choose>
            <xsl:when test="$e/@xfe:srng-define != ''">
              <xsl:value-of select="$e/@xfe:srng-define"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:variable name="e.dataModel" select="rng:getSRNGdataModelFromXmlElement($e, $xfe:srng.grammar.normalized)" as="element(rng:element)"/>
              <xsl:variable name="e.define" select="$e.dataModel/parent::rng:define" as="element(rng:define)"/>
              <xsl:value-of select="$e.define/@name"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:sequence select="xfe:getConfMappingElementsFromXML($e, $e.define.name)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select="xfe:getConfMappingElementsFromXML($e, ())"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Get all conf elements corresponding to the $e xml element instance : whether xpath based or sharing the same define in the schema</xd:p>
      <xd:p>Remember we add the schema define name to every conf elements</xd:p>
      <xd:p>Beware of the plural (elementS) in the name of the function, see the other function "xfe:getConfMappingElementFromXML" (which take the last element)  </xd:p>
    </xd:desc>
    <xd:param name="e">xml element in xml instance</xd:param>
    <xd:return>Zero or more conf elements corresponding</xd:return>
  </xd:doc>
  <xsl:function name="xfe:getConfMappingElementsFromXML" as="element()*">
    <xsl:param name="e" as="element()"/>
    <xsl:param name="e.define.name" as="xs:string?"/> <!--(so we can call this function for debug with the define's name)-->
    <!--Ici il faut faire attention car la conf défini du xpath ce qui n'est pas forcément basé sur le srng : 
    ex : si titre prévu en h1 sur infoCommentaire/titre mais que tableau/titre utilise la même définition srng, 
    alors il faut induire qu'un titre de tableau sera forcément un h1
    (sauf s'il est lui même redéfini dans la conf plus bas)-->
    <xsl:variable name="conf.elements"  as="element()*">
      <xsl:choose>
        <xsl:when test="$xfe:conf.hasSchema and empty($e.define.name)">
          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
            <xsl:with-param name="code" select="$els:log.level.code"/>
            <xsl:with-param name="alert" select="true()"/>
            <xsl:with-param name="level" select="'fatal'"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="description">[FATAL] Calling xfe:getConfMappingElementsFromXML with an empty sequence as 2nd agrument "e.define.name" is not allowed when $xfe:conf.hasSchema is true</xsl:with-param>
          </xsl:call-template>
          <xsl:message terminate="yes"/>
        </xsl:when>
        <xsl:when test="$xfe:conf.hasSchema">
          <xsl:sequence select="$xfe:conf.normalized/*:mapping/*:element[xfe:confElementHasRngDefine(., $e.define.name)]"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:sequence select="$xfe:conf.normalized/*:mapping/*:element"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="count($conf.elements) = 1">
        <xsl:sequence select="$conf.elements"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="conf.matching.elements" as="element()*">
          <xsl:for-each select="$conf.elements[tokenize(@xpath, '/')[last()] = local-name($e)]">
            <xsl:variable name="conf.element" select="." as="element()"/>
            <xsl:variable name="xpath-with-filter" select="if(@xpath-filter) then(concat(@xpath, '[', @xpath-filter,']')) else(@xpath)" as="xs:string"/>
            <xsl:variable name="xml.matching.elements" select="els:evaluate-xpath($xpath-with-filter, $e/root()/*)" as="element()*"/>
            <xsl:if test="generate-id($e) = $xml.matching.elements/generate-id(.)">
              <xsl:sequence select="$conf.element"/>
            </xsl:if>
          </xsl:for-each>
        </xsl:variable>
        <xsl:variable name="conf.matching.elements.theMorePreciseXpath" as="element()*" 
          select="$conf.matching.elements[ count(tokenize(@xpath, '/')) = max( for $e in $conf.matching.elements return count(tokenize($e/@xpath, '/')) )]"/>
        <xsl:choose>
          <!--There is one (or more) more precise xpath corresponding conf element--> 
          <xsl:when test="count($conf.matching.elements.theMorePreciseXpath) != 0">
            <xsl:sequence select="$conf.matching.elements.theMorePreciseXpath"/>
          </xsl:when>
          <xsl:when test="not($xfe:conf.hasSchema)">
            <!--<!-\-Only the the root may not be declared in the conf-\->
            <xsl:if test="$e/ancestor::*">
              <xsl:message terminate="yes">[FATAL] No element found in the conf for <xsl:value-of select="els:displayNode($e)"/></xsl:message>
            </xsl:if>-->
          </xsl:when>
          <!--Otherwise : look if there is a define corresponding element in the conf-->
          <xsl:otherwise>
            <!-- on vérifie qu'il n'y a pas de règles dans la conf qui s'applique au même élément par sa définition (mais pas par son xpath)-->
            <xsl:variable name="e.srng-define" select="$e.define.name" as="xs:string"/>
            <xsl:variable name="e.define" select="$xfe:srng.grammar.normalized//rng:define[@name = $e.srng-define]" as="element(rng:define)?"/>
            <xsl:variable name="conf.matchingDefine.elements" as="element()*">
              <!--If the xml File contains an unvalid element we must not through an error--> 
              <xsl:if test="count($e.define) != 0">
                <xsl:sequence select="$conf.elements[xfe:confElementHasRngDefine(., $e.define.name)]"/>
              </xsl:if>
            </xsl:variable>
            <xsl:if test="count($conf.matchingDefine.elements) != 0">
              <xsl:call-template name="els:log">
                <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
                <xsl:with-param name="code" select="$els:log.level.code"/>
                <xsl:with-param name="alert" select="true()"/>
                <xsl:with-param name="markup" select="false()"/>
                <xsl:with-param name="xpathContext" select="$e"/>
                <xsl:with-param name="description">[INFO] surcharge de conf induite sur <xsl:value-of select="els:displayNode($e)"/></xsl:with-param>
              </xsl:call-template>
              <!--<xsl:message use-when="false()">e.srng-define= <xsl:value-of select="$e.srng-define"/></xsl:message>
              <xsl:message use-when="false()">e.define =<xsl:value-of select="$e.define/@name"/></xsl:message>
              <xsl:message use-when="false()">count $conf.matchingDefine.elements = <xsl:value-of select="count($conf.matchingDefine.elements)"/></xsl:message>-->
              <xsl:if test="count($conf.matchingDefine.elements) gt 1">
                <xsl:call-template name="els:log">
                  <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
                  <xsl:with-param name="code" select="$els:log.level.code"/>
                  <xsl:with-param name="level" select="'warning'"/>
                  <xsl:with-param name="alert" select="true()"/>
                  <xsl:with-param name="markup" select="false()"/>
                  <xsl:with-param name="xpathContext" select="$e"/>
                  <xsl:with-param name="description">
                    <xsl:value-of select="concat('[WARNING] ', count($conf.matchingDefine.elements), ' elements de surcharge induite sur ', els:displayNode($e), ', on a pris le dernier ?')"/>
                    <xsl:value-of select="concat('&#10;  xpath : ', els:get-xpath($e))"/>
                    <xsl:for-each select="$conf.matchingDefine.elements">
                      <xsl:value-of select="concat('&#10;  ', els:displayNode($e))"/>
                    </xsl:for-each>
                 </xsl:with-param>
                </xsl:call-template>
              </xsl:if>
              <xsl:sequence select="$conf.matchingDefine.elements[last()]"/>
            </xsl:if>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="xfe:confElementHasRngDefine" as="xs:boolean">
    <xsl:param name="e" as="element()"/> <!--element dans la conf (sans namespace)-->
    <xsl:param name="defineName" as="xs:string"/>
    <xsl:sequence select="tokenize($e/@srng-defines, ' ') = $defineName"/>
  </xsl:function>
  
  <xsl:function name="xfe:getConfMappingElementFromXML" as="element()*">
    <xsl:param name="e" as="element()"/>
    <!--comme en css, on prend la dernière règle qui matche-->
    <xsl:sequence select="xfe:getConfMappingElementsFromXML($e)[last()]"/>
  </xsl:function>
  
  <!--=================================================-->
  <!--HTML2XML functions-->
  <!--=================================================-->
  
  <!--Get conf definition element from any html element : 
      - result can  be empty if the element is not defined in the conf.
      - if multiple definition found, falling back to the first occurence + xsl:message
  -->
  <!--FIXME : another method could be to get the define in the html.rng with a validator (msv.jar) and then just read the PI we have inserted-->
  <xsl:function name="xfe:getConfMappingFromHTML" as="element()?">
    <!--Assume 
          Schematron conf : 
            - pattern-every-xfe-type-that-infer-a-link-has-a-class-and-href-attributes-and-is-unique-for-same-xml-name
            - no-added-class-equal-xml-element-name
            - unique-html-name-when-no-class
          html.rng schema : 
            - class attribute are strict values (order specified)-->
    <xsl:param name="e" as="element()"/><!--html:*-->
    <!--xfe-XML2HTML always add the xml element's name at first position into @class (when it add it)-->
    <xsl:variable name="class1" select="tokenize($e/@class, '\s+')[1]" as="xs:string?"/>
    <xsl:variable name="name" select="local-name($e)" as="xs:string?"/>
    <xsl:choose>
      <!--============================-->
      <!--SPECIFIC-->
      <!--============================-->
      <!--=== TABLES ===-->
      <!--type="cals:table/html:table"-->
      <xsl:when test="$e/self::div/table">
        <!--TODO : get the conf <element type="cals:table or html:table"/> using the div class-->
      </xsl:when>
      <xsl:when test="local-name($e) = ('table' , 'caption', 'colgroup' , 'col' , 'thead' , 'tbody' , 'tfoot' , 'tr' , 'td' , 'th')
        or local-name($e) = ('tgroup', 'colspec', 'thead', 'tbody', 'tfoot', 'row', 'entry')">
        <!--schematron : no-mapping-for-table-elements-use-type-instead
        => nothing-->
      </xsl:when>
      <!--=== TITLE ===-->
      <!--type="xfe:title-xx" => <xx class="xfe_title-yy">-->
      <xsl:when test="some $class in tokenize($e/@class, $els:regAnySpace) satisfies (starts-with($class, 'xfe_title-'))">
        <xsl:variable name="xfe_title.class" select="(tokenize($e/@class, $els:regAnySpace)[starts-with(., 'xfe_title-')])[1]" as="xs:string"/>
        <xsl:variable name="conf.mappingElements" select="$xfe:conf.normalized/*:mapping/*:element[@html-addClass = $xfe_title.class]" as="element()*"/>
        <xsl:sequence select="xfe:applyCommonfiltersOnConfMappingElements($e, $conf.mappingElements)"/>
      </xsl:when>
      <!--=== LIST ===-->
      <!--type="xfe:list(xx)" => <ul/ol data-xfe-listType="...">-->
      <xsl:when test="$e/@*[local-name(.) = concat($xfe:dataXfeAttribute.prefix, 'listType')]">
        <!--schematron :
             - mapping-for-list-elements-use-type-instead
             - mapping-for-li-elements-but-no-xfe-list-type-defined
             - xfe-list-must-have-html-name-ul-or-ol
             - when-xfe-list-defined-li-mapping-must-be-defined
        -->
        <xsl:variable name="conf.mappingElements" 
          select="$xfe:conf.normalized/*:mapping/*:element[starts-with(@type, 'xfe:list(')][@html-name = $name]" as="element()*"/>
        <xsl:sequence select="xfe:applyCommonfiltersOnConfMappingElements($e, $conf.mappingElements)"/>
      </xsl:when>
      <!--should be ok with the default behaviour-->
      <!--<xsl:when test="$name = 'li'"/>-->
      <!--=== FOOTNOTES ===-->
      <!--
        type="xfe:noteRef(xx)" => <a class="[xmlName] xfe_noteRef [addClass]">
        type="xfe:inlineNote" => <a class="[xmlName] xfe_noteRef [addClass]">
        type="xfe:note(xx)" => <xx class="[xmlName] xfe_note [addClass]">
      -->
      <xsl:when test="els:hasClass($e, 'xfe_noteRef')">
        <xsl:variable name="conf.mappingElements" as="element()*"
          select="$xfe:conf.normalized/*:mapping/*:element[starts-with(@type, 'xfe:noteRef') or @type = 'xfe:inlineNote']" />
        <xsl:sequence select="xfe:applyCommonfiltersOnConfMappingElements($e, $conf.mappingElements)"/>
      </xsl:when>
      <xsl:when test="els:hasClass($e, 'xfe_note')"> 
        <xsl:variable name="conf.mappingElements" as="element()*"
          select="$xfe:conf.normalized/*:mapping/*:element[starts-with(@type, 'xfe:note(')]" />
        <xsl:sequence select="xfe:applyCommonfiltersOnConfMappingElements($e, $conf.mappingElements)"/>
      </xsl:when>
      <!--=== LINKS ===-->
      <xsl:when test="els:hasClass($e, 'xfe_note')"> 
        <xsl:variable name="conf.mappingElements" as="element()*"
          select="$xfe:conf.normalized/*:mapping/*:element[starts-with(@type, 'xfe:note(')]" />
        <xsl:sequence select="xfe:applyCommonfiltersOnConfMappingElements($e, $conf.mappingElements)"/>
      </xsl:when>
      <xsl:when test="els:hasClass($e, 'xfe_webLink')"> 
        <xsl:variable name="conf.mappingElements" as="element()*"
          select="$xfe:conf.normalized/*:mapping/*:element[starts-with(@type, 'xfe:webLink(')]" />
        <xsl:sequence select="xfe:applyCommonfiltersOnConfMappingElements($e, $conf.mappingElements)"/>
      </xsl:when>
      <!--type="xfe:externalLinkType" => <a class="[xmlName] [addClass]">-->
      <!--No way to discern those external link we have to deal with every links, so fallback to default behaviour (otherwise)-->
      <!--We sould alway get a confElement for links because of schematron rules :
            - every-xfe-type-that-infer-a-link-has-a-class-and-href-attributes-and-is-unique-for-same-xml-name
            - no-added-class-equal-xml-element-name
            - unique-html-name-when-no-class
      -->
      <!--============================-->
      <!--COMMON-->
      <!--============================-->
      <xsl:otherwise>
        <!--There is a class1 which should be the xml element's name-->
        <!--exclude any conf definitions that has already been processed-->
        <xsl:variable name="conf.mappingElements" select="$xfe:conf.normalized/*:mapping/*:element
          [not(@type = 'cals:table')]
          [not(@type = 'html:table')]
          [not(starts-with(@type, 'xfe:title-'))]
          [not(starts-with(@type, 'xfe:list('))]
          [not(starts-with(@type, 'xfe:noteRef('))]
          [not(starts-with(@type, 'xfe:note('))]
          [not(@type = 'inlineNote')]
          " as="element()*"/>
        <xsl:sequence select="xfe:applyCommonfiltersOnConfMappingElements($e, $conf.mappingElements)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <!--When multiple definition has been found in xfe:getConfMappingFromHTML(), 
    this function filters them with common rules based on conf attributes : 
      - @html-name
      - @html-xmlClassAsName
      - @html-addClass
  -->
  <xsl:function name="xfe:applyCommonfiltersOnConfMappingElements" as="element()?">
    <xsl:param name="e" as="element()"/><!--html element-->
    <xsl:param name="conf.mappingElements" as="element()*"/>
    <!--xfe-XML2HTML adds the xml element's name at first position into @class (when it add it)
        html.rng schema : 
            - class attribute are strict values (order specified)-->
    <xsl:variable name="class1" select="normalize-space(tokenize($e/@class, '\s+')[1][not(starts-with(., 'xfe_'))])" as="xs:string?"/>
    <xsl:variable name="name" select="local-name($e)" as="xs:string?"/>
    <xsl:message use-when="false()">[DEBUG][xfe:applyCommonfiltersOnConfMappingElements] e=<xsl:value-of select="els:displayNode($e)"/></xsl:message>
    <xsl:message use-when="false()">[DEBUG][xfe:applyCommonfiltersOnConfMappingElements] name=<xsl:value-of select="$name"/> class1=<xsl:value-of select="$class1"/></xsl:message>
    <xsl:message use-when="false()">[DEBUG][xfe:applyCommonfiltersOnConfMappingElements] count($conf.mappingElements) = <xsl:value-of select="count($conf.mappingElements)"/></xsl:message>
    <xsl:variable name="conf.mappingElements.filtered" as="element()*">
      <xsl:choose>
        <xsl:when test="count($conf.mappingElements) = 0">
          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
            <xsl:with-param name="code" select="$els:log.level.code"/>
            <xsl:with-param name="alert" select="true()"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="xpathContext" select="$e"/>
            <xsl:with-param name="description">[INFO][xfe:applyCommonfiltersOnConfMappingElements] No elements found in conf for html <xsl:value-of select="els:displayNode($e)"/>. </xsl:with-param>
          </xsl:call-template>
        </xsl:when>
        <xsl:when test="count($conf.mappingElements) = 1">
          <xsl:sequence select="$conf.mappingElements"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:variable name="conf.mappingElements.filteredByHtmlName" as="element()*">
            <xsl:choose>
              <!--default html-element name is calculated with the schema (inline or block)-->
              <xsl:when test="$name = ('span', 'div')">
                <!--FIXME : if a simple element declaration without html-name exists in the conf, then it would not always be the good one
                cf. conf/_test/xfe-conf.xml <element xpath="notes" label="footnotes"/> => we can actually not know -->
                <!--<xsl:sequence select="$conf.mappingElements[not(@html-name) or @html-name = $name]"/>-->
                <xsl:sequence select="$conf.mappingElements"/>
              </xsl:when>
              <!--if the name is not span or div, then it has been necessarily specified in the conf-->
              <xsl:otherwise>
                <xsl:sequence select="$conf.mappingElements[@html-name = $name]"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <xsl:message use-when="false()">[DEBUG][xfe:applyCommonfiltersOnConfMappingElements] count($conf.mappingElements.filteredByHtmlName) = <xsl:value-of select="count($conf.mappingElements.filteredByHtmlName)"/></xsl:message>
          <xsl:choose>
            <xsl:when test="count($conf.mappingElements.filteredByHtmlName) = 1">
              <xsl:sequence select="$conf.mappingElements.filteredByHtmlName"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:variable name="conf.mappingElements.filteredByClass" as="element()*">
                <xsl:choose>
                  <!--when $class1 is empty, then it has been necessarily specified in the conf with @html-xmlNameAsClass="false"-->
                  <xsl:when test="$class1 = ''">
                    <!-- Exclude all @html-addClass that doesn't match (= that are not added in @class) -->
                    <xsl:sequence
                      select="$conf.mappingElements.filteredByHtmlName[@html-xmlNameAsClass = 'false']
                                                                      [not(some $class in tokenize(@html-addClass, '\s+') satisfies not($class = tokenize($e/@class, '\s+')))]"/>
                  </xsl:when>
                  <!--when @class is not empty, it could be : 
                        - wether the @html-xmlNameAsClass 
                        - or any @html-addClass-->
                  <xsl:otherwise>
                    <xsl:variable name="conf.mappingElements.filteredByXmlNameAsClass" as="element()*"
                                  select="$conf.mappingElements.filteredByHtmlName[xfe:getNameFromXpath(.) = $class1]"/>
                       <!-- It could be an empty sequence : that means the element is not defined in the conf or that its name is mapped on a @html-addClass value -->
                    <xsl:choose>
                      <xsl:when test="count($conf.mappingElements.filteredByXmlNameAsClass) = 1">
                        <xsl:sequence select="$conf.mappingElements.filteredByXmlNameAsClass"/>
                      </xsl:when>
                      <xsl:when test="count($conf.mappingElements.filteredByXmlNameAsClass) &gt; 1">
                        <!-- filtering on @html-addClass-->
                        <xsl:variable name="conf.mappingElements.filteredByHtmlAddClass" as="element()*">
                          <xsl:choose>
                            <!--there is no @html-addClass left in the conf -->
                            <xsl:when test="count($conf.mappingElements.filteredByXmlNameAsClass[@html-addClass]) = 0">
                              <!--do not filter on it-->
                              <xsl:sequence select="$conf.mappingElements.filteredByXmlNameAsClass"/>
                            </xsl:when>
                            <!--there are added class that matches the current class-->
                            <xsl:when test="count($conf.mappingElements.filteredByXmlNameAsClass[tokenize(@html-addClass, '\s+') = tokenize($e/@class, '\s+')]) != 0">
                              <!--filter on it-->
                              <xsl:sequence select="$conf.mappingElements.filteredByXmlNameAsClass[tokenize(@html-addClass, '\s+') = tokenize($e/@class, '\s+')]"/>
                            </xsl:when>
                            <xsl:otherwise>
                              <!--keep the last filter active-->
                              <!--FIXME : we could exclude all @html-addClass that doesn't match-->
                              <xsl:sequence select="$conf.mappingElements.filteredByXmlNameAsClass"/>
                            </xsl:otherwise>
                          </xsl:choose>
                        </xsl:variable>
                        <xsl:sequence select="$conf.mappingElements.filteredByHtmlAddClass"/>
                      </xsl:when>
                      <xsl:otherwise>
                        <!-- filtering on @html-addClass on $conf.mappingElements.filteredByHtmlName-->
                        <xsl:variable name="conf.mappingElements.filteredByHtmlAddClass" as="element()*">
                          <xsl:choose>
                            <!--there are added class that matches the current class-->
                            <xsl:when test="count($conf.mappingElements.filteredByHtmlName[tokenize(@html-addClass, '\s+') = tokenize($e/@class, '\s+')]) != 0">
                              <!--filter on it-->
                              <xsl:sequence select="$conf.mappingElements.filteredByHtmlName[tokenize(@html-addClass, '\s+') = tokenize($e/@class, '\s+')]"/>
                            </xsl:when>
                            <xsl:otherwise>
                              <!--that means the element is not defined in the conf-->
                            </xsl:otherwise>
                          </xsl:choose>
                        </xsl:variable>
                        <xsl:sequence select="$conf.mappingElements.filteredByHtmlAddClass"/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:variable>
              <xsl:sequence select="$conf.mappingElements.filteredByClass"/>
              <xsl:message use-when="false()">[DEBUG][xfe:applyCommonfiltersOnConfMappingElements] count($conf.mappingElements.filteredByClass) = <xsl:value-of select="count($conf.mappingElements.filteredByClass)"/></xsl:message>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="count($conf.mappingElements.filtered) = 0">
        <!--<xsl:message>[INFO][xfe:applyCommonfiltersOnConfMappingElements] <xsl:value-of select="count($conf.mappingElements.filtered)"/> elements found in conf for html <xsl:value-of select="els:displayNode($e)"/>.</xsl:message>-->
      </xsl:when>
      <xsl:when test="count($conf.mappingElements.filtered) = 1">
        <xsl:sequence select="$conf.mappingElements.filtered"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
          <xsl:with-param name="code" select="$els:log.level.code"/>
          <xsl:with-param name="alert" select="true()"/>
          <xsl:with-param name="level" select="'warning'"/>
          <xsl:with-param name="markup" select="false()"/>
          <xsl:with-param name="xpathContext" select="$e"/>
          <xsl:with-param name="description">[WARNING][xfe:applyCommonfiltersOnConfMappingElements] <xsl:value-of select="count($conf.mappingElements.filtered)"/> elements found in conf for html <xsl:value-of select="els:displayNode($e)"/>. Falling back to first occurence</xsl:with-param>
        </xsl:call-template>
        <xsl:if test="count($conf.mappingElements.filtered) != 0">
          <xsl:call-template name="els:log">
            <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
            <xsl:with-param name="code" select="$els:log.level.code"/>
            <xsl:with-param name="alert" select="true()"/>
            <xsl:with-param name="level" select="'warning'"/>
            <xsl:with-param name="markup" select="false()"/>
            <xsl:with-param name="xpathContext" select="$e"/>
            <xsl:with-param name="description"><xsl:text>conf elements found :&#10;</xsl:text>               
              <xsl:for-each select="$conf.mappingElements.filtered">
                <xsl:text>     </xsl:text>
                <xsl:value-of select="els:displayNode(.)"/>
                <xsl:text>&#10;</xsl:text>
              </xsl:for-each>
            </xsl:with-param>
          </xsl:call-template>
          </xsl:if>
        <xsl:sequence select="$conf.mappingElements.filtered[1]"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="xfe:getXmlELementName" as="xs:string">
    <xsl:param name="e" as="element()"/> <!--élement html-->
    <xsl:variable name="result" as="xs:string">
      <xsl:choose>
         <!--This could be avoid because but we will add the good class while "specific-cleaning" so the html is always valid
         From the good class, the element name should be retrieve. 
         => But actually there are case where the good class doesn't allow to retrieve the good name 
         (ex : when xml <article> becomes <body>, we should be able to do the reverse <body> => <article>)
         => So keep this xsl:when here-->   
        <xsl:when test="$xfe:srng.grammar.rootIsSingle and $e/self::html:body">
          <xsl:value-of select="rng:getRootElementsName($xfe:srng.grammar.normalized)[1]"/>
        </xsl:when>
        <xsl:when test="$xfe:srng.is-xhtml and not(els:hasClass($e, 'xfe_webLink'))">
          <xsl:value-of select="local-name($e)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:variable name="conf.mappingElement" select="xfe:getConfMappingFromHTML($e)" as="element()*"/>
          <xsl:choose>
            <xsl:when test="count($conf.mappingElement) = 1">
              <xsl:value-of select="xfe:getNameFromXpath($conf.mappingElement)"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="tokenize($e/@class, $els:regAnySpace)[1]"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="normalize-space($result) != ''">
        <xsl:value-of select="$result"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:message terminate="yes">[FATAL][xfe:getXmlELementName] Unable to get the xml element's name for <xsl:value-of select="els:displayNode($e)"/> at <xsl:value-of select="els:get-xpath($e)"/></xsl:message>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <!--FIXME xslLib ? : but xpath must be a path type a/b/c (nothing else, no *, no predicates)-->
  <xsl:function name="xfe:getNameFromXpath" as="xs:string">
    <xsl:param name="e" as="element()"/>
    <xsl:value-of select="tokenize($e/@xpath, '/')[last()]"/>
  </xsl:function>
  
</xsl:stylesheet>