<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:saxon="http://saxon.sf.net/"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
	xmlns:rng="http://relaxng.org/ns/structure/1.0"
	xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
	xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
	xmlns:xfe="http://www.lefebvre-sarrut.eu/ns/xmlfirst/xmlEditor"
	xmlns:html="http://www.w3.org/1999/xhtml"
	xmlns="http://www.lefebvre-sarrut.eu/ns/el/fichesGB_FP"
	xpath-default-namespace="http://www.lefebvre-sarrut.eu/ns/dalloz/revues.poc"
	exclude-result-prefixes="#all"
	version="3.0">
  
  <xsl:import href="xslLib:/xslLib/els-common.xsl"/>
  <!--Cette xslt peut être exécutée de manière indépendante pour debogage-->
  <xsl:import href="../../xfe-common.xsl"/>
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>XSLT applied after HTML2XML to fix the XML according to the schema xf_GB_FP.srng</xd:p>
    </xd:desc>
  </xd:doc>
	
	<!--===================================================-->
	<!--INIT-->
	<!--===================================================-->
	
	<xsl:template match="/">
		  <xsl:apply-templates select="." mode="xfe:html2xml_xmlFixup"/>
	</xsl:template>

	<!--===================================================-->
	<!--MAIN-->
	<!--===================================================-->
  
  <xsl:template match="/" mode="xfe:html2xml_xmlFixup">
    <xsl:call-template name="els:log">
      <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
      <xsl:with-param name="code" select="$els:log.level.code"/>
      <xsl:with-param name="alert" select="true()"/>
      <xsl:with-param name="markup" select="false()"/>
      <xsl:with-param name="description">[INFO] xfe:html2xml_xmlFixup</xsl:with-param>
    </xsl:call-template>
    <xsl:variable name="step">
      <xsl:apply-templates mode="#current"/>
    </xsl:variable>
    <xsl:apply-templates select="$step" mode="xfe:html2xml_xmlFixup-attributes"/>
  </xsl:template>
  
  <xsl:template match="@id" mode="xfe:html2xml_xmlFixup">
    <xsl:attribute name="ID" select="."/>
  </xsl:template>
  
  <!--cf. FE-308-->
  <xsl:template match="entry/Al[starts-with(local-name(*[1]), 'Tet')]" mode="xfe:html2xml_xmlFixup">
    <xsl:apply-templates mode="#current"/>
  </xsl:template>
  
  <xsl:template match="table | tgroup | colspec | tbody | thead | row | entry" mode="xfe:html2xml_xmlFixup">
    <xsl:element name="{local-name(.)}">
      <xsl:apply-templates select="@* | node()" mode="#current"/>
    </xsl:element>
  </xsl:template>
  
  <!--Pertes des alignement car non géré par le modèle-->
  <xsl:template match="entry/@align" mode="xfe:html2xml_xmlFixup"/>
  
  <xsl:template match="LISTE/@xf:listType" mode="xfe:html2xml_xmlFixup-attributes">
    <xsl:choose>
      <xsl:when test=". = ''"/>
      <xsl:when test=". = '-'">
        <xsl:attribute name="TYPE" select="'TIRET'"/>
      </xsl:when>
      <xsl:when test=". = '•'">
        <xsl:attribute name="TYPE" select="'PUCE'"/>
      </xsl:when>
      <xsl:when test=". = '■'">
        <xsl:attribute name="TYPE" select="'CARRE'"/>
      </xsl:when>
      <xsl:when test=". = '*'">
        <xsl:attribute name="TYPE" select="'ETOILE'"/>
      </xsl:when>
      
      <xsl:when test=". = 'upper-roman'">
        <xsl:attribute name="NUM" select="'I'"/>
      </xsl:when>
      <xsl:when test=". = 'upper-alpha'">
        <xsl:attribute name="NUM" select="'A'"/>
      </xsl:when>
      <xsl:when test=". = 'decimal'">
        <xsl:attribute name="NUM" select="'1'"/>
      </xsl:when>
      <xsl:when test=". = 'lower-roman'">
        <xsl:attribute name="NUM" select="'i'"/>
      </xsl:when>
      <xsl:when test=". = 'lower-alpha'">
        <xsl:attribute name="NUM" select="'a'"/>
      </xsl:when>
      
      <xsl:when test=". = 'upper-roman )'">
        <xsl:attribute name="NUM" select="'I'"/>
        <xsl:attribute name="TYPE" select="'PARENTHESE'"/>
      </xsl:when>
      <xsl:when test=". = 'upper-alpha )'">
        <xsl:attribute name="NUM" select="'A'"/>
        <xsl:attribute name="TYPE" select="'PARENTHESE'"/>
      </xsl:when>
      <xsl:when test=". = 'decimal )'">
        <xsl:attribute name="NUM" select="'1'"/>
        <xsl:attribute name="TYPE" select="'PARENTHESE'"/>
      </xsl:when>
      <xsl:when test=". = 'lower-roman )'">
        <xsl:attribute name="NUM" select="'i'"/>
        <xsl:attribute name="TYPE" select="'PARENTHESE'"/>
      </xsl:when>
      <xsl:when test=". = 'lower-alpha )'">
        <xsl:attribute name="NUM" select="'a'"/>
        <xsl:attribute name="TYPE" select="'PARENTHESE'"/>
      </xsl:when>
      
      <xsl:when test=". = 'upper-roman .'">
        <xsl:attribute name="NUM" select="'I'"/>
        <xsl:attribute name="TYPE" select="'POINT'"/>
      </xsl:when>
      <xsl:when test=". = 'upper-alpha .'">
        <xsl:attribute name="NUM" select="'A'"/>
        <xsl:attribute name="TYPE" select="'POINT'"/>
      </xsl:when>
      <xsl:when test=". = 'decimal .'">
        <xsl:attribute name="NUM" select="'1'"/>
        <xsl:attribute name="TYPE" select="'POINT'"/>
      </xsl:when>
      <xsl:when test=". = 'lower-roman .'">
        <xsl:attribute name="NUM" select="'i'"/>
        <xsl:attribute name="TYPE" select="'POINT'"/>
      </xsl:when>
      <xsl:when test=". = 'lower-alpha .'">
        <xsl:attribute name="NUM" select="'a'"/>
        <xsl:attribute name="TYPE" select="'POINT'"/>
      </xsl:when>
      
      <xsl:when test=". = 'upper-roman .-'">
        <xsl:attribute name="NUM" select="'I'"/>
        <xsl:attribute name="TYPE" select="'POINT-TIRET'"/>
      </xsl:when>
      <xsl:when test=". = 'upper-alpha .-'">
        <xsl:attribute name="NUM" select="'A'"/>
        <xsl:attribute name="TYPE" select="'POINT-TIRET'"/>
      </xsl:when>
      <xsl:when test=". = 'decimal .-'">
        <xsl:attribute name="NUM" select="'1'"/>
        <xsl:attribute name="TYPE" select="'POINT-TIRET'"/>
      </xsl:when>
      <xsl:when test=". = 'lower-roman .-'">
        <xsl:attribute name="NUM" select="'i'"/>
        <xsl:attribute name="TYPE" select="'POINT-TIRET'"/>
      </xsl:when>
      <xsl:when test=". = 'lower-alpha .-'">
        <xsl:attribute name="NUM" select="'a'"/>
        <xsl:attribute name="TYPE" select="'POINT-TIRET'"/>
      </xsl:when>
      
      
      <xsl:when test=". = 'decimal °'">
        <xsl:attribute name="NUM" select="'1'"/>
        <xsl:attribute name="TYPE" select="'DEGRE'"/>
      </xsl:when>
      <xsl:when test=". = 'decimal °)'">
        <xsl:attribute name="NUM" select="'1'"/>
        <xsl:attribute name="TYPE" select="'DEGRE-PARENT'"/>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:call-template name="els:log">
          <xsl:with-param name="xsltName" select="els:getFileName(static-base-uri())"/>
          <xsl:with-param name="code" select="$els:log.level.code"/>
          <xsl:with-param name="alert" select="true()"/>
          <xsl:with-param name="level" select="'error'"/>
          <xsl:with-param name="markup" select="false()"/>
          <xsl:with-param name="description">[ERROR] format de liste "<xsl:value-of select="."/>" non reconnu</xsl:with-param>
        </xsl:call-template>          
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="TITRE/@html:class" mode="xfe:html2xml_xmlFixup-attributes">
    <xsl:choose>
      <xsl:when test=". = 'xfe_title-1'">
        <xsl:attribute name="NIVEAU" select="'0'"/>
      </xsl:when>
      <xsl:when test=". = 'xfe_title-2'">
        <xsl:attribute name="NIVEAU" select="'1'"/>
      </xsl:when>
      <xsl:when test=". = 'xfe_title-3'">
        <xsl:attribute name="NIVEAU" select="'2'"/>
      </xsl:when>
      <xsl:when test=". = 'xfe_title-4'">
        <xsl:attribute name="NIVEAU" select="'3'"/>
      </xsl:when>
      <xsl:when test=". = 'xfe_title-5'">
        <xsl:attribute name="NIVEAU" select="'4'"/>
      </xsl:when>
      <xsl:when test=". = 'xfe_title-6'">
        <xsl:attribute name="NIVEAU" select="'5'"/>
      </xsl:when>
      <xsl:when test=". = 'xfe_title-7'">
        <xsl:attribute name="NIVEAU" select="'6'"/>
      </xsl:when>
      <xsl:when test=". = 'xfe_title-8'">
        <xsl:attribute name="NIVEAU" select="'7'"/>
      </xsl:when>
      <xsl:when test=". = 'xfe_title-9'">
        <xsl:attribute name="NIVEAU" select="'8'"/>
      </xsl:when>
      <xsl:when test=". = 'xfe_title-10'">
        <xsl:attribute name="NIVEAU" select="'9'"/>
      </xsl:when>
      </xsl:choose>
  </xsl:template>
  
  <xsl:template match="TITRE/@html:style" mode="xfe:html2xml_xmlFixup-attributes">
    <xsl:choose>
      <xsl:when test=". = 'text-transform:uppercase;'">
        <xsl:attribute name="STYLE" select="'CAP'"/>
      </xsl:when>
      <xsl:when test=". = 'font-style:italic;'">
        <xsl:attribute name="STYLE" select="'ITAL'"/>
      </xsl:when>
      <xsl:when test=". = 'font-weight:bold;'">
        <xsl:attribute name="STYLE" select="'GRAS'"/>
      </xsl:when>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="FOOTNOTE" mode="xfe:html2xml_xmlFixup">
    <xsl:variable name="target" select="@xf:href"/>
    <xsl:choose>
      <xsl:when test="$target">
        <xsl:copy>
          <xsl:variable name="refered" select="ancestor::ARTICLE//xfe_note[@id=substring-after($target, '#')]"/>
          <xsl:if test="$refered">
            <xsl:copy-of select="$refered/node()"/>
          </xsl:if>
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise/>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="P[xfe_note]" mode="xfe:html2xml_xmlFixup"/>
  
  <!--===============================-->
  <!-- External Links : resolved -->
  <!--===============================-->
  
	<!--<xsl:template match="renvoiAutrui/@xf:targetResId" mode="xfe:html2xml_xmlFixup">
    <xsl:next-match/>
    <xsl:attribute name="xf:refType" select="'efl:renvoiAutrui'"/>
  </xsl:template>-->
  
	<!--===================================================-->
	<!--COMMON-->
	<!--===================================================-->
	
  <xsl:template match="node() | @*" mode="xfe:html2xml_xmlFixup xfe:html2xml_xmlFixup-attributes">
		<xsl:copy>
			<xsl:apply-templates select="node() | @*" mode="#current"/>
		</xsl:copy>
	</xsl:template>
	
</xsl:stylesheet>